// chnconv_dll.h


#ifndef CHNCONV_DLL_H
#define CHNCONV_DLL_H



////////////////////////////////////////////
//             Converting RTF             //
////////////////////////////////////////////



#define ESCAPE       '\\'
#define BRACE_START  '{'
#define BRACE_END    '}'
#define SEMICOLON    ';'


static const char *const HiddenTagList [] = {
   "info",
   "colortbl",
   "*",
   "pict",
   "shppict",
   "nonshppict",
   "datafield",
} ;

static const int HIDDEN_TAG_LIST_NELEM = NELEMOF ( HiddenTagList ) ;



typedef struct {
   int nCode ;
   const char *szName ;
} CHARACTER_NAME ;


static const CHARACTER_NAME RtfEscapeList [] = {
   { '\\',     "\\" },
   { '{',      "{" },
   { '}',      "}" },
} ;

static const int RTF_ESCAPE_LIST_NELEM = NELEMOF ( RtfEscapeList ) ;



static const char szRtfSignature [] = "{\\rtf" ;
static const char szTmSignature  [] = "\xFE" ;



static const char szGbGothic1       [] = "Hei" ;
static const char szGbGothic2       [] = "Gothic" ;
static const char szGbGothic3       [] = "\xBA\xDA" ;

static const char szBig5Gothic1     [] = "Hei" ;
static const char szBig5Gothic2     [] = "Gothic" ;
static const char szBig5Gothic3     [] = "\xB6\xC2" ;

static const char szCwHead       [] = "fa" ;
static const char szCwFantiHead  [] = "jf" ;
static const char szCwGothic     [] = "SVbN" ;
static const char szCw7Head      [] = "CW-GB" ;
static const char szCw7FantiHead [] = "CW-B5" ;
static const char szCw7Gothic    [] = "Gothic" ;

static const char szNhHead       [] = "mg" ;
static const char szNh3Head      [] = "NH3" ;
static const char szNhOptionHead1   [] = "NG" ;
static const char szNhOptionHead2   [] = "NB" ;
static const char szNhGothic     [] = "" ;

static const char szCwnHead      [] = "OMGB" ;
static const char szCwnFantiHead [] = "OMɑ" ;
static const char szCwnGothic    [] = "޼" ;

static const char szZqHead       [] = "FN" ;
static const char szZqFantiHead  [] = "FN" ;
static const char szZqGothic1    [] = "" ;
static const char szZqGothic2    [] = "" ;
static const char szZqGothic3    [] = "" ;

static const char szCxHead       [] = "CX " ;
static const char szCxGothic     [] = "S" ;

static const char szZjHead       [] = "ȑ" ;
static const char szZjFantiHead  [] = "ɑ" ;
static const char szZjGothic     [] = "޼" ;

static const char szHkscsName    [] = "HKSCS" ;



static const char szCwFont       [] = "fa " ;
static const char szCwGothicFont [] = "fa SVbN" ;
static const char szKgFont       [] = "jf ɑ̖" ;
static const char szKgGothicFont [] = "jf ɑ̃SVbN" ;

static const char szCw7Font       [] = "CW-GB Mincho" ;
static const char szCw7GothicFont [] = "CW-GB Gothic" ;
static const char szKg7Font       [] = "CW-B5 Mincho" ;
static const char szKg7GothicFont [] = "CW-B5 Gothic" ;

static const char szNhFont       [] = "mgȑ̑v" ;
static const char szNhGothicFont [] = "mgȑ̍" ;
static const char szNh3pFont     [] = "NH3" ;

static const char szCwnFont       [] = "OMGB" ;
static const char szCwnGothicFont [] = "OMGB޼" ;
static const char szCwnfFont       [] = "OMɑ̖" ;
static const char szCwnfGothicFont [] = "OMɑ̺޼" ;

static const char szZqFont       [] = "FNȑv" ;
static const char szZqGothicFont [] = "FNȍ" ;

static const char szCxFont       [] = "CX Pȃ~-R" ;
static const char szCxGothicFont [] = "CX PȃS-R" ;

static const char szZjFont       [] = "ȑ̖" ;
static const char szZjGothicFont [] = "ȑ̺޼" ;

static const char szJapaneseFont       [] = "lr " ;
static const char szJapaneseGothicFont [] = "lr SVbN" ;

static const char szCourierFont [] = DEFAULT_COURIER_FONT ;



////////////////////////////////////////////
//                TM->RTF                 //
////////////////////////////////////////////



static const char szRtfStartString  [] =  "{\\rtf1\\ansi\\deff0" ;
static const char szRtfEndString    [] = "\r\n}" ;
static const char szCharFormat      [] = "\\\'%02x" ;
static const char szFontNameFormat  [] = "{\\f%d\\%s\\fcharset%d %s;}" ;
static const char szParagraphTag    [] = "par" ;


#if 0
      cf0: black     |   |
      cf1: blue      |  b|
      cf2: red       |r  |
      cf3: purple    |r b|
      cf4: green     | g |
      cf5: cyan      | gb|
      cf6: yellow    |rg |
      cf7: white     |rgb|
#endif

// nColorMode
#define TM_COLOR_BLUE      1
#define TM_COLOR_RED       2
#define TM_COLOR_GREED     4

static const char szColorTable [] =
   "{\\colortbl;"
   "\\red0\\green0\\blue255;"          // blue
   "\\red255\\green0\\blue0;"          // red
   "\\red255\\green0\\blue255;"        // purple
   "\\red0\\green255\\blue0;"          // green
   "\\red0\\green255\\blue255;"        // cyan
   "\\red255\\green255\\blue0;"        // yellow
   "}\r\n" ;


// int nFontMode
#define TM_FONT_ITALIC     1
#define TM_FONT_BOLD       2
#define TM_FONT_COURIER    4

static const char *const FontModeArray [] = {
   "",
   "\\i",                  // TM_FONT_ITALIC
   "\\b",                  // TM_FONT_BOLD
   "\\i\\b",               // TM_FONT_ITALIC | TM_FONT_BOLD
} ;


// int nSizeMode
#define TM_SIZE_SUB        1
#define TM_SIZE_SUPER      2
#define TM_SIZE_LARGE      3

static const char *const SizeModeArray [] = {
   "",
   "\\sub",                // TM_SIZE_SUB
   "\\super",              // TM_SIZE_SUPER
} ;


// int nUnderLineMode
#define TM_ATTRIB_UNDERLINE         1
#define TM_ATTRIB_UNDERLINE_DOTTED  2
#define TM_ATTRIB_UNDERLINE_THICK   3
#define TM_ATTRIB_UNDERLINE_DOUBLE  4
#define TM_ATTRIB_UNDERLINE_WAVE    5
#define TM_ATTRIB_STROKE            6
#define TM_ATTRIB_STROKE_DOUBLE     7
#define TM_ATTRIB_SHADING_1         8
#define TM_ATTRIB_SHADING_2         9
#define TM_ATTRIB_SHADING_3         10
#define TM_ATTRIB_BORDER            11

static const char *const UnderLineModeArray [] = {
   "",
   "\\ul",                          // TM_ATTRIB_UNDERLINE
   "\\uld",                         // TM_ATTRIB_UNDERLINE_DOTTED
   "\\ulth",                        // TM_ATTRIB_UNDERLINE_THICK
   "\\uldb",                        // TM_ATTRIB_UNDERLINE_DOUBLE
   "\\ulwave",                      // TM_ATTRIB_UNDERLINE_WAVE
   "\\strike",                      // TM_ATTRIB_STROKE
   "\\striked1",                    // TM_ATTRIB_STROKE_DOUBLE
   "\\chshdng1500",                 // TM_ATTRIB_SHADING_1
   "\\chshdng2000",                 // TM_ATTRIB_SHADING_2
   "\\chshdng2500",                 // TM_ATTRIB_SHADING_3
   "\\chbrdr\\brdrs\\brdrw10",      // TM_ATTRIB_BORDER
} ;


// eNmCg̎ʋL
#define TM_IDENTIFIER    '_'


#pragma pack(1)
// eNmCg̍s
struct TMLINEINFO {
   unsigned char Start ;         // ˂ 0xFE
   unsigned char Attrib1 ;       // s
   unsigned char Attrib2 ;       // s
   unsigned char Dummy ;         // ˂ 0x00
   unsigned char Length ;        // s{R
   unsigned char LeftMargin ;    // }[W{S
   unsigned char RightMargin ;   // E}[W{S
} ;
#pragma pack()



////////////////////////////////////////////
//           eϊ֐̐錾             //
////////////////////////////////////////////



int Gb2big5 ( int C, unsigned long dwConversionMode ) ;
int Gb2cw ( int C, unsigned long dwConversionMode ) ;
int Gb2nh ( int C, unsigned long dwConversionMode ) ;
int Gb2cwn ( int C, unsigned long dwConversionMode ) ;
int Gb2zq ( int C, unsigned long dwConversionMode ) ;
int Gb2cx ( int C, unsigned long dwConversionMode ) ;
int Gb2zj ( int C, unsigned long dwConversionMode ) ;

int Big52gb ( int C, unsigned long dwConversionMode ) ;
int Big52cw ( int C, unsigned long dwConversionMode ) ;
int Big52nh ( int C, unsigned long dwConversionMode ) ;
int Big52cwn ( int C, unsigned long dwConversionMode ) ;
int Big52zq ( int C, unsigned long dwConversionMode ) ;
int Big52cx ( int C, unsigned long dwConversionMode ) ;
int Big52zj ( int C, unsigned long dwConversionMode ) ;

int Cw2gb ( int C, unsigned long dwConversionMode ) ;
int Cw2big5 ( int C, unsigned long dwConversionMode ) ;
int Cw2nh ( int C, unsigned long dwConversionMode ) ;
int Cw2cwn ( int C, unsigned long dwConversionMode ) ;
int Cw2zq ( int C, unsigned long dwConversionMode ) ;
int Cw2cx ( int C, unsigned long dwConversionMode ) ;
int Cw2zj ( int C, unsigned long dwConversionMode ) ;

int Nh2gb ( int C, unsigned long dwConversionMode ) ;
int Nh2big5 ( int C, unsigned long dwConversionMode ) ;
int Nh2cw ( int C, unsigned long dwConversionMode ) ;
int Nh2cwn ( int C, unsigned long dwConversionMode ) ;
int Nh2zq ( int C, unsigned long dwConversionMode ) ;
int Nh2cx ( int C, unsigned long dwConversionMode ) ;
int Nh2zj ( int C, unsigned long dwConversionMode ) ;

int Cwn2gb ( int C, unsigned long dwConversionMode ) ;
int Cwn2big5 ( int C, unsigned long dwConversionMode ) ;
int Cwn2cw ( int C, unsigned long dwConversionMode ) ;
int Cwn2nh ( int C, unsigned long dwConversionMode ) ;
int Cwn2zq ( int C, unsigned long dwConversionMode ) ;
int Cwn2cx ( int C, unsigned long dwConversionMode ) ;
int Cwn2zj ( int C, unsigned long dwConversionMode ) ;

int Zq2gb ( int C, unsigned long dwConversionMode ) ;
int Zq2big5 ( int C, unsigned long dwConversionMode ) ;
int Zq2cw ( int C, unsigned long dwConversionMode ) ;
int Zq2nh ( int C, unsigned long dwConversionMode ) ;
int Zq2cwn ( int C, unsigned long dwConversionMode ) ;
int Zq2cx ( int C, unsigned long dwConversionMode ) ;
int Zq2zj ( int C, unsigned long dwConversionMode ) ;

int Cx2gb ( int C, unsigned long dwConversionMode ) ;
int Cx2big5 ( int C, unsigned long dwConversionMode ) ;
int Cx2cw ( int C, unsigned long dwConversionMode ) ;
int Cx2nh ( int C, unsigned long dwConversionMode ) ;
int Cx2cwn ( int C, unsigned long dwConversionMode ) ;
int Cx2zq ( int C, unsigned long dwConversionMode ) ;
int Cx2zj ( int C, unsigned long dwConversionMode ) ;

int Zj2gb ( int C, unsigned long dwConversionMode ) ;
int Zj2big5 ( int C, unsigned long dwConversionMode ) ;
int Zj2cw ( int C, unsigned long dwConversionMode ) ;
int Zj2nh ( int C, unsigned long dwConversionMode ) ;
int Zj2cwn ( int C, unsigned long dwConversionMode ) ;
int Zj2zq ( int C, unsigned long dwConversionMode ) ;
int Zj2cx ( int C, unsigned long dwConversionMode ) ;

int Kg2gb ( int C, unsigned long dwConversionMode ) ;
int Kg2big5 ( int C, unsigned long dwConversionMode ) ;
int Cwnf2gb ( int C, unsigned long dwConversionMode ) ;
int Cwnf2big5 ( int C, unsigned long dwConversionMode ) ;
int Cwnf2cw ( int C, unsigned long dwConversionMode ) ;

int Tm2gb ( int C, unsigned long dwConversionMode ) ;
int Tm2big5 ( int C, unsigned long dwConversionMode ) ;
int Tm2cw ( int C, unsigned long dwConversionMode ) ;
int Tm2nh ( int C, unsigned long dwConversionMode ) ;
int Tm2cwn ( int C, unsigned long dwConversionMode ) ;
int Tm2zq ( int C, unsigned long dwConversionMode ) ;
int Tm2cx ( int C, unsigned long dwConversionMode ) ;
int Tm2zj ( int C, unsigned long dwConversionMode ) ;

int Tmf2gb ( int C, unsigned long dwConversionMode ) ;
int Tmf2big5 ( int C, unsigned long dwConversionMode ) ;

int ConvertChineseCharacterDummy ( int C, unsigned long dwConversionMode ) ;



#endif

