// chnconv_dll_old.cpp

#include <windows.h>
#include <stdio.h>
#include "msc.h"
#include "codepage.h"
#include "storechr.h"
#include "stream.h"
#include "stringex.h"

#include "conv_err.h"
#include "chnconv.h"



static volatile long nSemaphore ;            // NĂ 1
#define LockSemaphore()    InterlockedExchange ( & nSemaphore, 1 )   // if success, return 0
#define UnlockSemaphore()  InterlockedExchange ( & nSemaphore, 0 )

static CHNCONV_USERDEF *AllocCurrentUserDefinition ( int nSrcCodePage, int nDstCodePage, size_t *pLength ) ;



////////////////////////////////////////////
//               ֐ (1)               //
////////////////////////////////////////////



static CHNCONVEXINFO Info ;
static unsigned long dwConversionModeTm ;



int WINAPI ChnconvA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   if ( nSrcCodePage == CP_JAPANESE_TM ) dwConversionMode |= dwConversionModeTm ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_TM_NO_OUTPUT_RTFFORM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   CHNCONV_USERDEF *pSrcToDst = AllocCurrentUserDefinition ( nSrcCodePage, nDstCodePage, & Info.nNumberOfSrcToDst ) ;
   Info.pSrcToDst = pSrcToDst ;
   CHNCONV_USERDEF *pSrcToUc = AllocCurrentUserDefinition ( nSrcCodePage, CP_UTF16, & Info.nNumberOfSrcToUc ) ;
   Info.pSrcToUc = pSrcToUc ;

   int nResult = ChnconvExA ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, & Info ) ;

   free ( pSrcToDst ) ;
   free ( pSrcToUc ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI ChnconvW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   if ( nSrcCodePage == CP_JAPANESE_TM ) dwConversionMode |= dwConversionModeTm ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_TM_NO_OUTPUT_RTFFORM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   CHNCONV_USERDEF *pSrcToDst = AllocCurrentUserDefinition ( nSrcCodePage, nDstCodePage, & Info.nNumberOfSrcToDst ) ;
   Info.pSrcToDst = pSrcToDst ;
   CHNCONV_USERDEF *pSrcToUc = AllocCurrentUserDefinition ( nSrcCodePage, CP_UTF16, & Info.nNumberOfSrcToUc ) ;
   Info.pSrcToUc = pSrcToUc ;

   int nResult = ChnconvExW ( szSrcFileName, szDstFileName, nSrcCodePage, nDstCodePage, & Info ) ;

   free ( pSrcToDst ) ;
   free ( pSrcToUc ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI ChnconvString ( const char *szSrcBuffer, char *szDstBuffer, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   if ( nSrcCodePage == CP_JAPANESE_TM ) dwConversionMode |= dwConversionModeTm ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = ( dwConversionMode & CONVMODE_TM_NO_OUTPUT_RTFFORM ) ? RESERVEDMODE_NO_START_END : 0 ;
   Info.pReserved = (void*) & ReservedInfo ;

   CHNCONV_USERDEF *pSrcToDst = AllocCurrentUserDefinition ( nSrcCodePage, nDstCodePage, & Info.nNumberOfSrcToDst ) ;
   Info.pSrcToDst = pSrcToDst ;
   CHNCONV_USERDEF *pSrcToUc = AllocCurrentUserDefinition ( nSrcCodePage, CP_UTF16, & Info.nNumberOfSrcToUc ) ;
   Info.pSrcToUc = pSrcToUc ;

   intptr_t nResult = ChnconvStringEx ( szSrcBuffer, szDstBuffer, nSrcCodePage, nDstCodePage, & Info, nMaxLen ) ;

   free ( pSrcToDst ) ;
   free ( pSrcToUc ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI ChnconvChar ( int C, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = dwConversionMode ;

   int nResult = ChnconvCharEx ( C, nSrcCodePage, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI IsAvailableCodePage ( int nSrcCodePage, int nDstCodePage ) {
   return IsChnconvValidCodePage ( nSrcCodePage, nDstCodePage ) ;
}



////////////////////////////////////////////
//               ֐ (2)               //
////////////////////////////////////////////



static CHNFONTINFO FontInfo ;

static volatile long IsFreeBufferProcRegistered ;
void __cdecl FreeBuffer ( void ) ;



// GB ̏o̓tHgݒ
//  0 As 1 Ԃ
int WINAPI SetGbFontName ( const char *szFont, const char *szGothicFont ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = 0 ;

   if ( ! szFont && ! szGothicFont ) {
      free ( (char*) FontInfo.pGbFont ) ;
      free ( (char*) FontInfo.pGbGothicFont ) ;
   }
   else {

      if ( szFont ) {
         free ( (char*) FontInfo.pGbFont ) ;
         FontInfo.pGbFont = strdup ( szFont ) ;
         if ( ! FontInfo.pGbFont ) nResult = 1 ;
      }

      if ( szGothicFont ) {
         free ( (char*) FontInfo.pGbGothicFont ) ;
         FontInfo.pGbGothicFont = strdup ( szGothicFont ) ;
         if ( ! FontInfo.pGbGothicFont ) nResult = 1 ;
      }
   }

   FontInfo.cbSize = sizeof(CHNFONTINFO) ;
   Info.pFontInfo = & FontInfo ;

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   UnlockSemaphore () ;
   return nResult ;
}



// BIG5 ̏o̓tHgݒ
//  0 As 1 Ԃ
int WINAPI SetBig5FontName ( const char *szFont, const char *szGothicFont ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = 0 ;

   if ( ! szFont && ! szGothicFont ) {
      free ( (char*) FontInfo.pBig5Font ) ;
      free ( (char*) FontInfo.pBig5GothicFont ) ;
   }
   else {

      if ( szFont ) {
         free ( (char*) FontInfo.pBig5Font ) ;
         FontInfo.pBig5Font = strdup ( szFont ) ;
         if ( ! FontInfo.pBig5Font ) nResult = 1 ;
      }

      if ( szGothicFont ) {
         free ( (char*) FontInfo.pBig5GothicFont ) ;
         FontInfo.pBig5GothicFont = strdup ( szGothicFont ) ;
         if ( ! FontInfo.pBig5GothicFont ) nResult = 1 ;
      }
   }

   FontInfo.cbSize = sizeof(CHNFONTINFO) ;
   Info.pFontInfo = & FontInfo ;

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   UnlockSemaphore () ;
   return nResult ;
}



// PINYIN ̏o̓tHgݒ
//  0 As 1 Ԃ
int WINAPI SetPinyinFontName ( const char *szFont, const char *szGothicFont ) {

   if ( LockSemaphore () ) return 1 ;

   int nResult = 0 ;

   if ( ! szFont && ! szGothicFont ) {
      free ( (char*) FontInfo.pPinyinFont ) ;
      free ( (char*) FontInfo.pPinyinGothicFont ) ;
   }
   else {

      if ( szFont ) {
         free ( (char*) FontInfo.pPinyinFont ) ;
         FontInfo.pPinyinFont = strdup ( szFont ) ;
         if ( ! FontInfo.pPinyinFont ) nResult = 1 ;
      }

      if ( szGothicFont ) {
         free ( (char*) FontInfo.pPinyinGothicFont ) ;
         FontInfo.pPinyinGothicFont = strdup ( szGothicFont ) ;
         if ( ! FontInfo.pPinyinGothicFont ) nResult = 1 ;
      }
   }

   FontInfo.cbSize = sizeof(CHNFONTINFO) ;
   Info.pFontInfo = & FontInfo ;

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   UnlockSemaphore () ;
   return nResult ;
}



// GB  BIG5 ̏o̓tHgݒ
//  0 As 1 Ԃ
int WINAPI SetFontName ( const char *szGb, const char *szGbGothic, const char *szBig5, const char *szBig5Gothic ) {

   int nResult = 0 ;

   if ( ! szGb && ! szGbGothic && ! szBig5 && ! szBig5Gothic ) {
      nResult += SetGbFontName ( szGb, szGbGothic ) ;
      nResult += SetBig5FontName ( szBig5, szBig5Gothic ) ;
   }
   else {
      if ( szGb || szGbGothic ) nResult += SetGbFontName ( szGb, szGbGothic ) ;
      if ( szBig5 || szBig5Gothic ) nResult += SetBig5FontName ( szBig5, szBig5Gothic ) ;
   }

   return nResult ;
}



////////////////////////////////////////////
//               ֐ (3)               //
////////////////////////////////////////////



typedef struct {
   int nSrcCodePage ;
   int nDstCodePage ;
   int cSrcChar ;
   int cDstChar ;
} USERDEF ;

static USERDEF *pUserDef ;
static size_t nNumberOfUserDef ;



int WINAPI SetUserDefinition ( int nSrcCodePage, int nDstCodePage, int cSrcChar, int cDstChar ) {

   if ( cSrcChar || cDstChar ) {

      USERDEF *pNew = (USERDEF*) realloc ( pUserDef, sizeof(USERDEF) * ( nNumberOfUserDef + 1 ) ) ;
      if ( ! pNew ) return 1 ;

      pUserDef = pNew ;

      ( pUserDef + nNumberOfUserDef )->nSrcCodePage = nSrcCodePage ;
      ( pUserDef + nNumberOfUserDef )->nDstCodePage = nDstCodePage ;
      ( pUserDef + nNumberOfUserDef )->cSrcChar = cSrcChar ;
      ( pUserDef + nNumberOfUserDef )->cDstChar = cDstChar ;

      nNumberOfUserDef ++ ;

   }
   else if ( nSrcCodePage || nDstCodePage ) {

      for ( int nCount = 0 ; nCount < (int) nNumberOfUserDef ; nCount ++ ) {

         if ( ( pUserDef + nCount )->nSrcCodePage != nSrcCodePage ) continue ;
         if ( ( pUserDef + nCount )->nDstCodePage != nDstCodePage ) continue ;

         memmove ( pUserDef + nCount, pUserDef + nCount + 1, ( nNumberOfUserDef - nCount - 1 ) * sizeof(USERDEF) ) ;
         nNumberOfUserDef -- ;
      }

   }
   else {
      free ( pUserDef ) ;
      pUserDef = NULL ;
      nNumberOfUserDef = 0 ;
   }

   if ( ! InterlockedExchange ( & IsFreeBufferProcRegistered, TRUE ) ) atexit ( FreeBuffer ) ;

   return 0 ;
}



static CHNCONV_USERDEF *AllocCurrentUserDefinition ( int nSrcCodePage, int nDstCodePage, size_t *pLength ) {

   CHNCONV_USERDEF *pResult = NULL ;
   size_t nLength = 0 ;

   if ( nSrcCodePage == CP_HZ_CHINESE_SIMPLIFIED ) nSrcCodePage = CP_CHINESE_SIMPLIFIED ;
   if ( nDstCodePage == CP_HZ_CHINESE_SIMPLIFIED ) nDstCodePage = CP_CHINESE_SIMPLIFIED ;

   for ( int nCount = 0 ; nCount < (int) nNumberOfUserDef ; nCount ++ ) {

      if ( ( pUserDef + nCount )->nSrcCodePage != nSrcCodePage ) continue ;
      if ( ( pUserDef + nCount )->nDstCodePage != nDstCodePage ) continue ;

      CHNCONV_USERDEF *pNew = (CHNCONV_USERDEF*) realloc ( pResult, sizeof(CHNCONV_USERDEF) * ( nLength + 1 ) ) ;
      if ( ! pNew ) break ;

      pResult = pNew ;

      ( pResult + nLength )->nSrc = ( pUserDef + nCount )->cSrcChar ;
      ( pResult + nLength )->nDst = ( pUserDef + nCount )->cDstChar ;

      nLength ++ ;
   }

   *pLength = nLength ;
   return pResult ;
}



void __cdecl FreeBuffer ( void ) {

   free ( (char*) FontInfo.pGbFont ) ;
   free ( (char*) FontInfo.pGbGothicFont ) ;
   free ( (char*) FontInfo.pBig5Font ) ;
   free ( (char*) FontInfo.pBig5GothicFont ) ;
   free ( (char*) FontInfo.pPinyinFont ) ;
   free ( (char*) FontInfo.pPinyinGothicFont ) ;

   free ( pUserDef ) ;

}



////////////////////////////////////////////
//               ֐ (4)               //
////////////////////////////////////////////



// v|[Vĩ[hݒifor TM2RTFj
int WINAPI SetTmPropMode ( unsigned long dwMode ) {

   if ( LockSemaphore () ) return 1 ;

   if ( dwMode & TMMODE_WRITE_COMMENT ) Info.dwAdditionalMode |= ADDITIONAL_MODE_WRITE_TM_COMMENT ;
   else                                 Info.dwAdditionalMode &= ~ ADDITIONAL_MODE_WRITE_TM_COMMENT ;

   dwConversionModeTm = 0 ;
   if ( dwMode & TMMODE_ASCII_ALL )   dwConversionModeTm |= CONVMODE_PINYIN_BY_ASCII ;
   if ( dwMode & TMMODE_ASCII_ALPHA ) dwConversionModeTm |= CONVMODE_PINYIN_BY_ASCII ;

   UnlockSemaphore () ;
   return 0 ;
}



// tHgTCYݒifor TM2RTFj
int WINAPI SetTmFontSize ( int nSize ) {

   if ( LockSemaphore () ) return 1 ;

   if ( nSize > 0 )    Info.nFontSize = nSize ;
   else if ( ! nSize ) Info.nFontSize = -1 ;

   UnlockSemaphore () ;
   return 0 ;
}



int WINAPI SetTmAsciiFontChar ( int C ) {
   return 0 ;
}



////////////////////////////////////////////
//               ֐ (5)               //
////////////////////////////////////////////



int WINAPI SetChnconvWindow ( HWND hWnd, int IsDialog ) {
   return 0 ;
}



////////////////////////////////////////////
//               ֐ (6)               //
////////////////////////////////////////////



int WINAPI PrintFileStartA ( int nDstCodePage, const char *szDstFileName ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = ChnconvExA ( NULL, szDstFileName, CP_JAPANESE_TM, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileStartW ( int nDstCodePage, const wchar_t *szDstFileName ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = ChnconvExW ( NULL, szDstFileName, CP_JAPANESE_TM, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileEndA ( int nDstCodePage, const char *szDstFileName ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = ChnconvExA ( NULL, szDstFileName, CP_JAPANESE_TM, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



int WINAPI PrintFileEndW ( int nDstCodePage, const wchar_t *szDstFileName ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   int nResult = ChnconvExW ( NULL, szDstFileName, CP_JAPANESE_TM, nDstCodePage, & Info ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI PrintFileStartString ( int nDstCodePage, char *szDstBuffer, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_START ;
   Info.pReserved = (void*) & ReservedInfo ;

   intptr_t nResult = ChnconvStringEx ( NULL, szDstBuffer, CP_JAPANESE_TM, nDstCodePage, & Info, nMaxLen ) ;

   UnlockSemaphore () ;
   return nResult ;
}



intptr_t WINAPI PrintFileEndString ( int nDstCodePage, char *szDstBuffer, size_t nMaxLen ) {

   if ( LockSemaphore () ) return SetChnconvLastError ( CONV_ERROR_SEMAPHORE ) ;

   Info.cbSize = sizeof(CHNCONVEXINFO) ;
   Info.dwConversionMode = 0 ;

   RESERVEDINFO ReservedInfo = { sizeof(RESERVEDINFO) } ;
   ReservedInfo.dwMode = RESERVEDMODE_ONLY_END ;
   Info.pReserved = (void*) & ReservedInfo ;

   intptr_t nResult = ChnconvStringEx ( NULL, szDstBuffer, CP_JAPANESE_TM, nDstCodePage, & Info, nMaxLen ) ;

   UnlockSemaphore () ;
   return nResult ;
}



////////////////////////////////////////////
//               ֐ (7)               //
////////////////////////////////////////////



int WINAPI Chnconv ( const void *szSrcFileName, const void *szDstFileName, int nSrcCodePage, int nDstCodePage, unsigned long dwConversionMode ) {
   if ( dwConversionMode & CONVMODE_FILENAME_UNICODE ) return ChnconvW ( (const wchar_t*) szSrcFileName, (const wchar_t*) szDstFileName, nSrcCodePage, nDstCodePage, dwConversionMode ) ;
   else                                                return ChnconvA ( (const char*)    szSrcFileName, (const char*)    szDstFileName, nSrcCodePage, nDstCodePage, dwConversionMode ) ;
}



int WINAPI PrintFileStart ( int nDstCodePage, void *pReserved ) {
   return PrintFileStartA ( nDstCodePage, (char*) pReserved ) ;
}



int WINAPI PrintFileEnd ( void *pReserved ) {
   return PrintFileEndA ( 0, (char*) pReserved ) ;
}



