// chnconv_dll.cpp

#include <windows.h>
#include <stdio.h>
#include <ctype.h>
#include <limits.h>
#include "msc.h"
#include "winmain.h"

#include "cjkctype.h"
#include "convchar.h"
#include "codepage.h"
#include "gaiji.h"
#include "nls.h"
#include "res.h"
#include "storechr.h"
#include "stream.h"
#include "stringex.h"

#include "conv_err.h"
#include "chnconv.h"
#include "chnconv_cp.h"
#include "chnconv_dll.h"
#include "conv_abort.h"
#include "conv_pkmsg.h"
#include "fontname.h"

#include "unicode.h"
#include "uninorm.h"
#include "unijtz2ftz.h"
#include "uniftz2jtz.h"

#include "conv_tbl.h"
#include "hkscs.h"



static int nOsCodePage ;         // OS̃R[hy[W
static unsigned long nTlsIndex ;

#define PEEK_MESSAGE_CONUT_MASK  0xFF


#define CP_DEFINE_CODEPAGE    0x0000FFFF     // R[hy[W̒`
#define CP_DEFINE_PSEUDOCODE  0x00FF0000     // [̒`
#define CP_DEFINE_OPTIONAL    0x0F000000     // ɑ̎̒`Ȃ
#define CP_DEFINE_FONT        0xF0000000     // ̒`

#define CP_CHINESE_HZ         ( CP_CHINESE_SIMPLIFIED + 0x100000 )      // old
#define CP_FANTI              0x08000000
#define CP_JAPANESE_CW_FANTI  ( CP_JAPANESE_CW  + CP_FANTI )
#define CP_JAPANESE_CWN_FANTI ( CP_JAPANESE_CWN + CP_FANTI )
#define CP_JAPANESE_NHP       ( CP_JAPANESE_NH  + 0x1000000 )
#define CP_JAPANESE_NH3P      ( CP_JAPANESE_NH  + 0x2000000 )
#define CP_GOTHIC             0x10000000

#if CP_JAPANESE_NHP & CP_FANTI || CP_JAPANESE_NH3P & CP_FANTI
#error Error: Codepage definition illegal 1
#endif

// RTF̃tHg߂ł̂ݎgp
#define CP_CHINESE_TRADITIONAL_HK      ( CP_CHINESE_TRADITIONAL + 0x1000000 )

#if CP_CHINESE_TRADITIONAL_HK & CP_DEFINE_PSEUDOCODE || CP_CHINESE_TRADITIONAL_HK & CP_FANTI
#error Error: Codepage definition illegal 2
#endif



static int InitChnconvData ( CHNCONVDATA *pData, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo ) ;
static int InitConversionMode ( CHNCONVDATA *pData ) ;
static int AllocWritingData ( CHNCONVDATA *pData, const CHNCONVEXINFO *pInfo ) ;
static int FreeWritingData ( CHNCONVDATA *pData ) ;
static int GetCharSize ( int nCodePage ) ;
static int DoChnconv ( CHNCONVDATA *pData ) ;
static int WriteFileStart ( CHNCONVDATA *pData ) ;
static int WriteFileEnd ( CHNCONVDATA *pData ) ;



////////////////////////////////////////////
//    CHNCONV.DLL  LOAD  FREE     //
////////////////////////////////////////////



int WINAPI DllMain ( HINSTANCE hInst, unsigned long dwReason, void *pReserved ) {

   if ( dwReason == DLL_PROCESS_ATTACH ) {

      // LastError p̃CfbNXm
      if ( ( nTlsIndex = TlsAlloc () ) == TLS_OUT_OF_INDEXES ) return FALSE ;

      // ϐ
      nOsCodePage = GetACP () ;

      // \[X̃CX^X
      SetResourceInstance ( hInst ) ;

      // ~tOe[u
      if ( AllocAbortTable () ) return FALSE ;
   }

   if ( dwReason == DLL_PROCESS_DETACH ) {

      FreeAbortTable () ;

      TlsFree ( nTlsIndex ) ;
   }

   return TRUE ;
}



////////////////////////////////////////////
//            CHNCONViANSIŁj           //
////////////////////////////////////////////



// Ƃ CONV_SUCCESS Ԃ
// sƂ CONV_ERROR_UNKNOWN ȏ̒萔Ԃ
int WINAPI ChnconvExA ( const char *szSrcFileName, const char *szDstFileName, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;

   if ( ! pInfo || pInfo->cbSize == sizeof(CHNCONVEXINFO) ) {

      CHNCONVDATA Data = { 0 } ;

      __try {

         // f[^̏
         if ( InitChnconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenFileStreamForReadingA ( szSrcFileName ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenFileStreamForWritingA ( szDstFileName ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            nError = DoChnconv ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, NULL ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   return SetChnconvLastError ( nError ) ;
}



////////////////////////////////////////////
//          CHNCONViUNICODEŁj          //
////////////////////////////////////////////



// Ƃ CONV_SUCCESS Ԃ
// sƂ CONV_ERROR_UNKNOWN ȏ̒萔Ԃ
int WINAPI ChnconvExW ( const wchar_t *szSrcFileName, const wchar_t *szDstFileName, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;

   if ( ! pInfo || pInfo->cbSize == sizeof(CHNCONVEXINFO) ) {

      CHNCONVDATA Data = { 0 } ;

      __try {

         // f[^̏
         if ( InitChnconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenFileStreamForReadingW ( szSrcFileName ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenFileStreamForWritingW ( szDstFileName ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            nError = DoChnconv ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, NULL ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   return SetChnconvLastError ( nError ) ;
}



////////////////////////////////////////////
//           CHNCONViSTRINGŁj          //
////////////////////////////////////////////



// Ƃ CONV_SUCCESS Ԃ
// sƂ CONV_ERROR_UNKNOWN ȏ̒萔Ԃ
intptr_t WINAPI ChnconvStringEx ( const char *szSrcBuffer, char *szDstBuffer, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo, size_t nMaxLen ) {

   int nError = CONV_ERROR_PARAM ;
   size_t nResultLen ;

   if ( ! pInfo || pInfo->cbSize == sizeof(CHNCONVEXINFO) ) {

      CHNCONVDATA Data = { 0 } ;

      __try {

         // f[^̏
         if ( InitChnconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( AllocWritingData ( & Data, pInfo ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }
         if ( RegisterAbortFlag ( & Data.pAbortFlag ) ) { nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

         // ̓Xg[J
         if ( ! ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) ) {
            Data.StreamIn = OpenStringStreamForReading ( szSrcBuffer, (size_t) -1, GetCharSize ( nSrcCodePage ) ) ;
            if ( Data.StreamIn == NULL ) { nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         }

         // o̓Xg[J
         Data.StreamOut = OpenStringStreamForWriting ( szDstBuffer, nMaxLen, GetCharSize ( nDstCodePage ) ) ;
         if ( Data.StreamOut == NULL ) { nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }

         // s
         if ( Data.dwReserved & ( RESERVEDMODE_ONLY_START | RESERVEDMODE_ONLY_END ) ) {
            if      ( Data.dwReserved & RESERVEDMODE_ONLY_START ) nError = WriteFileStart ( & Data ) ;
            else if ( Data.dwReserved & RESERVEDMODE_ONLY_END )   nError = WriteFileEnd ( & Data ) ;
         }
         else {
            nError = DoChnconv ( & Data ) ;
         }

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      __try {

         // Xg[
         if ( Data.StreamIn && CloseStream ( Data.StreamIn, NULL ) && ! nError ) nError = CONV_ERROR_READ ;
         if ( Data.StreamOut && CloseStream ( Data.StreamOut, & nResultLen ) && ! nError ) nError = CONV_ERROR_WRITE ;

         // [̉
         FreeWritingData ( & Data ) ;
         UnregisterAbortFlag ( Data.pAbortFlag ) ;

      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

   }

   if ( SetChnconvLastError ( nError ) ) return -1 ;
   return nResultLen ;
}



////////////////////////////////////////////
//             t@C̔             //
////////////////////////////////////////////



static int CompareFileStart ( STREAM *Stream, const char *szString ) ;
static int ChnconvTxt ( CHNCONVDATA *pData ) ;
static int ChnconvRtf ( CHNCONVDATA *pData ) ;
static int Tm2rtfTxt ( CHNCONVDATA *pData ) ;
static int Tm2rtfTec ( CHNCONVDATA *pData ) ;
static int Fwen2rtf ( CHNCONVDATA *pData ) ;



static int DoChnconv ( CHNCONVDATA *pData ) {

   int nError ;

   if ( pData->nSrcCodePage == CP_JAPANESE_FW ) return Fwen2rtf ( pData ) ;

   if ( pData->nSrcCodePage == CP_JAPANESE_TM ) {

      int IsTec = 0 ;

      if      ( pData->dwConversionMode & CONVMODE_CONVERT_TXT ) IsTec = 0 ;
      else if ( pData->dwConversionMode & CONVMODE_CONVERT_RTF ) IsTec = 1 ;
      else {

         int nResult = CompareFileStart ( pData->StreamIn, szTmSignature ) ;
         if ( nResult < 0 ) return CONV_ERROR_UNKNOWN ;
         if ( ! nResult ) IsTec = 1 ;
      }

      // s
      if ( StreamIsString ( pData->StreamIn ) ) {
         if ( IsTec ) nError = CONV_ERROR_FORMAT ;
         else         nError = Tm2rtfTxt ( pData ) ;
      }
      else {
         if ( IsTec ) nError = Tm2rtfTec ( pData ) ;
         else         nError = Tm2rtfTxt ( pData ) ;
      }
   }
   else {

      int IsRtf = 0 ;

      if      ( pData->dwConversionMode & CONVMODE_CONVERT_TXT ) IsRtf = 0 ;
      else if ( pData->dwConversionMode & CONVMODE_CONVERT_RTF ) IsRtf = 1 ;
      else {

         int nResult = CompareFileStart ( pData->StreamIn, szRtfSignature ) ;
         if ( nResult < 0 ) return CONV_ERROR_UNKNOWN ;
         if ( ! nResult ) IsRtf = 1 ;
      }

      // s
      if ( pData->nSrcCodePage == CP_HZ_CHINESE_SIMPLIFIED ) {
         if ( IsRtf ) nError = CONV_ERROR_FORMAT ;
         else         nError = ChnconvTxt ( pData ) ;
      }
      else {
         if ( IsRtf ) nError = ChnconvRtf ( pData ) ;
         else         nError = ChnconvTxt ( pData ) ;
      }
   }

   return nError ;
}



// v 0 AsvȂ琳AG[Ȃ畉Ԃ
static int CompareFileStart ( STREAM *Stream, const char *szString ) {

   size_t nLength = strlen ( szString ) ;

   if ( nLength >= STREAM_UNREAD_BUFFER_SIZE ) return -1 ;

   char szBuffer [ STREAM_UNREAD_BUFFER_SIZE ] ;
   size_t nCharRead = StreamReadBuffer ( szBuffer, nLength, Stream ) ;

   int nResult ;
   if ( nCharRead == nLength ) nResult = ( strncmp ( szBuffer, szString, nLength ) ) ? 1 : 0 ;
   else                        nResult = 1 ;

   for ( size_t nCount = nCharRead ; nCount ; nCount -- ) {
      int C = (unsigned char) szBuffer [ nCount - 1 ] ;
      if ( StreamUnread ( C, Stream ) == EOF ) return -1 ;
   }

   return nResult ;
}



////////////////////////////////////////////
//              ϊ̑O              //
////////////////////////////////////////////



// ϊf[^
//  0 As炻ȊOԂ
static int InitChnconvData ( CHNCONVDATA *pData, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo ) {

   pData->nSrcCodePage = nSrcCodePage ;
   pData->nDstCodePage = nDstCodePage ;

   if ( pInfo ) {

      pData->dwConversionMode = pInfo->dwConversionMode ;
      pData->dwAdditionalMode = pInfo->dwAdditionalMode ;

      pData->pSrcToDst = pInfo->pSrcToDst ;
      pData->nNumberOfSrcToDst = pInfo->nNumberOfSrcToDst ;
      if ( pData->nNumberOfSrcToDst >=INT_MAX ) return 1 ;

      if ( ( nDstCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE ) {
         pData->pSrcToUc = pInfo->pSrcToUc ;
         pData->nNumberOfSrcToUc = pInfo->nNumberOfSrcToUc ;
         if ( pData->nNumberOfSrcToUc >=INT_MAX ) return 1 ;
      }

      // tHg̏

      pData->szGbFont            = DEFAULT_GB_FONT ;
      pData->szGbGothicFont      = DEFAULT_GB_GOTHIC_FONT ;
      pData->szBig5Font          = DEFAULT_BIG5_FONT ;
      pData->szBig5GothicFont    = DEFAULT_BIG5_GOTHIC_FONT ;
      pData->szPinyinFont        = DEFAULT_PINYIN_FONT ;
      pData->szPinyinGothicFont  = DEFAULT_PINYIN_GOTHIC_FONT ;

      if ( nOsCodePage == CP_CHINESE_SIMPLIFIED ) {
         pData->szGbFont            = DEFAULT_GB_FONT_L ;
         pData->szGbGothicFont      = DEFAULT_GB_GOTHIC_FONT_L ;
      }
      if ( nOsCodePage == CP_CHINESE_TRADITIONAL ) {
         pData->szBig5Font          = DEFAULT_BIG5_FONT_L ;
         pData->szBig5GothicFont    = DEFAULT_BIG5_GOTHIC_FONT_L ;
      }

      if ( pInfo->pFontInfo ) {
         const CHNFONTINFO *pFontInfo = pInfo->pFontInfo ;
         if ( pFontInfo->pGbFont           && *pFontInfo->pGbFont )           pData->szGbFont            = pFontInfo->pGbFont ;
         if ( pFontInfo->pGbGothicFont     && *pFontInfo->pGbGothicFont )     pData->szGbGothicFont      = pFontInfo->pGbGothicFont ;
         if ( pFontInfo->pBig5Font         && *pFontInfo->pBig5Font )         pData->szBig5Font          = pFontInfo->pBig5Font ;
         if ( pFontInfo->pBig5GothicFont   && *pFontInfo->pBig5GothicFont )   pData->szBig5GothicFont    = pFontInfo->pBig5GothicFont ;
         if ( pFontInfo->pPinyinFont       && *pFontInfo->pPinyinFont )       pData->szPinyinFont        = pFontInfo->pPinyinFont ;
         if ( pFontInfo->pPinyinGothicFont && *pFontInfo->pPinyinGothicFont ) pData->szPinyinGothicFont  = pFontInfo->pPinyinGothicFont ;
      }

      // TechnoMate ֘ÃIvV
      if ( pInfo->pReserved ) {
         const RESERVEDINFO *pReserved = (const RESERVEDINFO*) pInfo->pReserved ;
         if ( pReserved->cbSize != sizeof(RESERVEDINFO) ) return 1 ;
         pData->dwReserved = pReserved->dwMode ;
         if ( ! IsNecChineseCodePage ( pData->nSrcCodePage ) && pData->dwReserved ) return 1 ;
      }
   }

   return 0 ;
}



static int InitFontNumber ( CHNCONVDATA *pData ) ;



// ݗpf[^̃[mۂl
//  0 As炻ȊOԂ
static int AllocWritingData ( CHNCONVDATA *pData, const CHNCONVEXINFO *pInfo ) {

   if ( IsNecChineseCodePage ( pData->nSrcCodePage ) ) {

      TMDATA *tm = (TMDATA*) malloc ( sizeof(TMDATA) ) ;
      if ( ! tm ) return 1 ;

      memzero ( tm, sizeof(TMDATA) ) ;
      pData->tm = tm ;

      pData->tm->nFontSize = pInfo->nFontSize ;
      if ( ! pData->tm->nFontSize )   pData->tm->nFontSize = DEFAULT_FONT_SIZE ;
      if ( pData->tm->nFontSize < 0 ) pData->tm->nFontSize = 0 ;

      InitFontNumber ( pData ) ;
   }

   return 0 ;
}



// ݗpf[^̃[
static int FreeWritingData ( CHNCONVDATA *pData ) {

   free ( pData->tm ) ;

   return 0 ;
}



// o̓R[hy[WƃIvV̒
//  0 As 1 Ԃ
static int InitConversionMode ( CHNCONVDATA *pData ) {

   int nSrcCodePage = pData->nSrcCodePage ;
   int nDstCodePage = pData->nDstCodePage ;
   unsigned long dwConversionMode = pData->dwConversionMode ;

   // łƂ̌݊
   if ( nSrcCodePage == CP_CHINESE_HZ ) nSrcCodePage = CP_HZ_CHINESE_SIMPLIFIED ;

   // IvV
   if ( nDstCodePage == CP_CHINESE_SIMPLIFIED && dwConversionMode & CONVMODE_CHNCONV_NO_GBK ) dwConversionMode |= CONVMODE_NO_USE_UNICODE ;
   if ( dwConversionMode & CONVMODE_NO_USE_UNICODE ) dwConversionMode &= ~ CONVMODE_CHNCONV_NO_VARIANT ;

   // GB/BIG5ȊO
   if ( ( nDstCodePage & CP_DEFINE_CODEPAGE ) == CP_JAPANESE ) dwConversionMode &= ~ CONVMODE_PINYIN_BY_ASCII ;

   // PeekMessage
   if ( dwConversionMode & CONVMODE_PEEKMESSAGE ) {
      if ( InitPeekMessage ( & pData->hDialog, & pData->IsMessageUnicode ) ) dwConversionMode &= ~ CONVMODE_PEEKMESSAGE ;
   }

   // li[
   pData->nSrcCodePage = nSrcCodePage ;
   pData->nDstCodePage = nDstCodePage ;
   pData->dwConversionMode = dwConversionMode ;

   return 0 ;
}



////////////////////////////////////////////
//             ϊ[eB             //
////////////////////////////////////////////



typedef int ( CONVERTCHARACTER ) ( int C, unsigned long dwConversionMode ) ;
static CONVERTCHARACTER *GetConvertFunction ( int nSrcCodePage, int nDstCodePage, int IsCwFanti ) ;

static int ConvertByUserDefinition ( int C, int *pUnicode, CHNCONVDATA *pData ) ;

static const char *GetCodeName ( int nCodePage ) ;

static int SjisNec2ibm ( int cSjis ) ;
static int ConvertCharacterEx ( int C, int nSrcCodePage, int nDstCodePage ) ;
static const char *GetDecomposedForm ( int C, int nSrcCodePage, int nDstCodePage , char *szBuffer ) ;

static inline int IsAborted ( CHNCONVDATA *pData ) { return *pData->pAbortFlag ; }



////////////////////////////////////////////
//           TXT ϊ[eB           //
////////////////////////////////////////////



static int Getc ( int C1, CHNCONVDATA *pData ) ;
static int GetcHz ( int C1, CHNCONVDATA *pData ) ;

#define GETC_ONLY_ESCAPE   (EOF-1)

// nEncodeMode
#define  EM_ASCII       0
#define  EM_LOCAL       1

static int Putc ( int C, STREAM *StreamOut ) ;
static int Putwc ( int cUnicode, STREAM *StreamOut ) ;
static int PutUnmappableChar ( int C, CHNCONVDATA *pData ) ;

static int DecodeCharRef ( int C, CHNCONVDATA *pData ) ;
static int UnreadDecodeCharRefBuffer ( int C, CHNCONVDATA *pData ) ;



// eLXgt@Cϊ
// Ƃ 0 Ԃ
// sƂ 1 ȏ̎w肳ꂽ萔Ԃ
static int ChnconvTxt ( CHNCONVDATA *pData ) {

   int nCountChar = 0 ;

   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( pData->nSrcCodePage, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_CW_TXT_FANTIZI ) ;
   if ( ! ConvertChineseCharacter ) return CONV_ERROR_CODEPAGE ;


   // HZ  GB 
   int IsHzEncode = 0 ;
   if ( pData->nSrcCodePage == CP_HZ_CHINESE_SIMPLIFIED ) {
      pData->nSrcCodePage = CP_CHINESE_SIMPLIFIED ;
      IsHzEncode = 1 ;
   }


   while ( 1 ) {

      if ( pData->nError ) goto CAUGHT_ERROR ;

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }


      int C ;

      // 擾
      if ( ( C = PopStoredCharsW ( pData->CharRefUnreadBuffer ) ) == EOF ) {

         int C1 = StreamRead ( pData->StreamIn ) ;
         if ( C1 == EOF ) break ;

         // [ĥGR[hȂ
         if ( IsHzEncode ) {
            C = GetcHz ( C1, pData ) ;
            if ( C == GETC_ONLY_ESCAPE ) continue ;
            if ( C == EOF ) {
               PutUnmappableChar ( C1, pData ) ;
               continue ;
            }
         }
         // [ĥȂGR[hȂ
         else {
            C = Getc ( C1, pData ) ;
            if ( C == EOF ) {
               PutUnmappableChar ( C1, pData ) ;
               continue ;
            }
         }

         // sȃoCg񂩌
         if ( C >= 0x100 && ! ismbbtrail_cp ( C & 0xFF, pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) ) {
            PutUnmappableChar ( C, pData ) ;
            continue ;
         }
      }


      // QƂ͂
      if ( pData->dwConversionMode & CONVMODE_INPUT_CODE_NUMBER ) {

         C = DecodeCharRef ( C, pData ) ;
         if ( C == EOF ) continue ;

         // G[Ȃobt@߂
         if ( *GetStoredCharsA ( pData->CharRefBuffer ) ) {
            C = UnreadDecodeCharRefBuffer ( C, pData ) ;
            if ( pData->CharRefUnreadBuffer == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            if ( C == EOF ) continue ;
         }
      }


      // 䕶EsȂǂ͂̂܂܏o
      if ( isascii ( C ) && iscntrl ( C ) ) {
         StreamWrite ( C, pData->StreamOut ) ;
         continue ;
      }


      int cSaved = C ;

      if ( ( pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) == CP_JAPANESE ) {
         if ( C != 0xEEF9 ) C = SjisNec2ibm ( C ) ;   // NEC->IBM
      }


      int cUnicode ;
      int cResult ;


      // [Uϊ
      cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
      if ( cResult ) {
         Putc ( cResult, pData->StreamOut ) ;
         continue ;
      }
      if ( cUnicode && pData->dwConversionMode & CONVMODE_TXT_UNICODE_HTML ) {
         Putwc ( cUnicode, pData->StreamOut ) ;
         continue ;
      }


      // ϊ̃C
      cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
      if ( ! ( pData->dwConversionMode & CONVMODE_CW_TXT_FANTIZI ) ) {
         switch ( pData->nSrcCodePage ) {
            case CP_JAPANESE_CW :   if ( C >= 0xF040 && C <= 0xF4FF ) { cResult = 0 ; } break ;
            case CP_JAPANESE_CWN :  if ( C >= 0xF090 && C <= 0xF4FF ) { cResult = 0 ; } break ;
         }
      }
      if ( cResult ) {
         Putc ( cResult, pData->StreamOut ) ;
         continue ;
      }


      // O
      if ( ( pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE && ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE ) {
         cResult = UnicodeToGaiji ( GaijiToUnicode ( C, pData->nSrcCodePage & CP_DEFINE_CODEPAGE ), pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ;
         if ( pData->nSrcCodePage == CP_CHINESE_SIMPLIFIED && C >= 0xFE50 && C <= 0xFEA0 ) cResult = 0 ;
         if ( pData->nDstCodePage == CP_CHINESE_SIMPLIFIED && cResult >= 0xFE50 && cResult <= 0xFEA0 ) cResult = 0 ;
      }
      if ( cResult ) {
         Putc ( cResult, pData->StreamOut ) ;
         continue ;
      }


      // ϊłȂ̂ OS oR Unicode ɒڕϊ
      if ( ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE ) ) {
         if ( ( pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE && ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE ) {
            cUnicode = ConvertCharacterEx ( C, pData->nSrcCodePage, pData->nDstCodePage ) ;
            int cTmp = GetNonLinshiDaima ( cUnicode ) ;
            if ( cTmp ) cUnicode = cTmp ;
         }
      }
      if ( cUnicode && pData->dwConversionMode & CONVMODE_TXT_UNICODE_HTML ) {
         Putwc ( cUnicode, pData->StreamOut ) ;
         continue ;
      }


      // ̕ŏo
      if ( ! ( pData->dwConversionMode & CONVMODE_NO_DECOMPOSE ) ) {
         char szBuffer [ 0x20 ] ;
         const char *szString = GetDecomposedForm ( C, pData->nSrcCodePage, pData->nDstCodePage, szBuffer ) ;
         if ( szString ) {
            StreamWriteString ( szString, pData->StreamOut ) ;
            continue ;
         }
      }


      // ϊłȂ̏o
      PutUnmappableChar ( cSaved, pData ) ;
   }


   CAUGHT_ERROR :

   return pData->nError ;
}



// QƂ
// Ԃ
// ŏAEOF Ԃ
static int DecodeCharRef ( int C, CHNCONVDATA *pData ) {

   char *szBuffer = GetStoredCharsA ( pData->CharRefBuffer ) ;

   // [
   if ( C == '[' && ! *szBuffer ||
       *szBuffer == '[' && isascii ( C ) && ( isupper ( C ) || isxdigit ( C ) || C == ':' )
   ) {

      // 
      if ( strlen ( szBuffer ) >= 0x10 ) return C ;

      pData->CharRefBuffer = StoreCharsA ( C, pData->CharRefBuffer ) ;
      if ( pData->CharRefBuffer == STOREDCHARS_ERROR ) pData->nError = CONV_ERROR_MEMORY ;

      return EOF ;
   }

   // l
   if ( C == ']' && *szBuffer == '[' ) {

      szBuffer ++ ;

      // R[hy[WF
      for ( int nCount = 0 ; nCount < CODEPAGE_NAME_LIST_NELEM ; nCount ++ ) {

         int nCodePage = CodePageNameList [ nCount ].nCodePage ;
         const char *nCodeName = CodePageNameList [ nCount ].szCodeName ;

         if ( ! strheadcmp ( szBuffer, nCodeName ) && *( szBuffer + strlen ( nCodeName ) ) == ':' ) {
            char *szString ;
            int cResult = strtoul ( szBuffer + strlen ( nCodeName ) + 1, & szString, 0x10 ) ;
            if ( ! *szString ) {

               if ( nCodePage == pData->nDstCodePage ) {
                  ClearStoredCharsA ( pData->CharRefBuffer ) ;
                  Putc ( cResult, pData->StreamOut ) ;
                  return EOF ;
               }

               if ( nCodePage == pData->nSrcCodePage ) {
                  ClearStoredCharsA ( pData->CharRefBuffer ) ;
                  return cResult ;
               }

               if ( ( nCodePage & CP_DEFINE_CODEPAGE ) == CP_JAPANESE ) {
                  if ( cResult != 0xEEF9 ) cResult = SjisNec2ibm ( cResult ) ;   // NEC->IBM
               }

               CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( nCodePage, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_CW_TXT_FANTIZI ) ;
               if ( ConvertChineseCharacter && ( cResult = ConvertChineseCharacter ( cResult, pData->dwConversionMode ) ) ) {
                  ClearStoredCharsA ( pData->CharRefBuffer ) ;
                  Putc ( cResult, pData->StreamOut ) ;
                  return EOF ;
               }
            }
            break ;
         }
      }
   }

   return C ;
}



// QƂ̃obt@ǂݖ߂AɏׂԂ
static int UnreadDecodeCharRefBuffer ( int C, CHNCONVDATA *pData ) {

   int cStart = PopStoredCharsA ( pData->CharRefBuffer ) ;

   for ( int cTmp ; ( cTmp = PopStoredCharsA ( pData->CharRefBuffer ) ) != EOF ; ) {
      pData->CharRefUnreadBuffer = StoreCharsW ( cTmp, pData->CharRefUnreadBuffer ) ;
   }

   pData->CharRefUnreadBuffer = StoreCharsW ( C, pData->CharRefUnreadBuffer ) ;

   return cStart ;
}



////////////////////////////////////////////
//     TXT ̓[eB̉֐     //
////////////////////////////////////////////



// sȃoCgȂ EOF Ԃ
static int Getc ( int C1, CHNCONVDATA *pData ) {

   STREAM *StreamIn = pData->StreamIn ;
   int nSrcCodePage = pData->nSrcCodePage & CP_DEFINE_CODEPAGE ;

   if ( ismbblead_cp ( C1, nSrcCodePage ) ) {

      int C2 ;
      if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;

      return ( C1 << 8 ) | C2 ;
   }

   return C1 ;
}



// sȃoCgȂ EOF Ԃ
// GXP[vɂ郂[hύX݂̂Ȃ GETC_ONLY_ESCAPE Ԃ
static int GetcHz ( int C1, CHNCONVDATA *pData ) {

   STREAM *StreamIn = pData->StreamIn ;

   // HZ ̏
   if ( pData->nEncodeMode == EM_ASCII ) {
      if ( C1 == '~' ) {
         int C2 ;
         if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
         if ( C2 == 0x7B ) {
            pData->nEncodeMode = EM_LOCAL ;
            return GETC_ONLY_ESCAPE ;
         }
         if ( C2 == '\n' ) {
            return GETC_ONLY_ESCAPE ;
         }
         if ( C2 != '~' ) StreamUnread ( C2, StreamIn ) ;
      }
   }
   else {
      int C2 ;
      if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
      if ( C1 == '~' && C2 == 0x7D ) {
         pData->nEncodeMode = EM_ASCII ;
         return GETC_ONLY_ESCAPE ;
      }
      if ( C1 >= 0x21 && C1 <= 0x7D && C2 >= 0x21 && C2 <= 0x7E ) {
         return ( ( C1 << 8 ) | C2 ) | 0x8080 ;
      }
      // caught illegal
      if ( C1 == 0x20 || C2 == 0x20 ) C1 = 0x20 ;
      else                            StreamUnread ( C2, StreamIn ) ;
   }

   if ( ismbblead_gb ( C1 ) ) {
      int C2 ;
      if ( ( C2 = StreamRead ( StreamIn ) ) == EOF ) return EOF ;
      return ( C1 << 8 ) | C2 ;
   }

   return C1 ;
}



static int Putc ( int C, STREAM *StreamOut ) {
   if ( C >= 0x100 ) StreamWrite ( ( C >> 8 ) & 0xFF, StreamOut ) ;
   StreamWrite ( C & 0xFF, StreamOut ) ;
   return 0 ;
}



static int Putwc ( int cUnicode, STREAM *StreamOut ) {
   StreamWriteFormat ( StreamOut, "&#x%04X;", cUnicode ) ;
   return 0 ;
}



static int PutUnmappableChar ( int C, CHNCONVDATA *pData ) {

   if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
      StreamWriteFormat ( pData->StreamOut, "[%s:%04X]", GetCodeName ( pData->nSrcCodePage ), C ) ;
   }
   else {
      if ( C >= 0x100 && ! ( pData->dwConversionMode & CONVMODE_ONE_QUESTION_MARK ) ) Putc ( '?', pData->StreamOut ) ;
      Putc ( '?', pData->StreamOut ) ;
   }

   return 0 ;
}



////////////////////////////////////////////
//           RTF ϊ[eB           //
////////////////////////////////////////////



#define FIELD_DETECT 0


static int InitRtfFontName ( const char **pDstFont, const char **pDstGothicFont, CHNCONVDATA *pData ) ;
static int WriteFontName ( int nFontNumber, const char *szFontName, int nCodePage, int IsGothic, STREAM *StreamOut ) ;
static int WriteLangInfo ( int nLang, int nLangNp, int nLangFe, int nLangFeNp, STREAM *StreamOut ) ;
static int WriteAltFontName ( const char *szFontName, int nCodePage, STREAM *StreamOut ) ;
static char *EncodeHexaDecimal ( const char *szSrc, char *szDst, int nCodePage ) ;
static const char *GetFirstKeyword ( const char *szString ) ;
static const char *GetLastKeyword ( const char *szString ) ;
static int FontNameToPseudoCodePage ( int nCodePage, const char *szFontName ) ;
static int IsAsciiString ( const char *szString ) ;
static int ConvertFtzToJtz ( int cUnicode, unsigned long dwConversionMode ) ;
static int ConvertJtzToFtz ( int cUnicode, unsigned long dwConversionMode ) ;



// b`eLXg̊e[x̏
typedef struct {
   int nFont ;          // tHgԍ
   int nAltFont ;       // ֘AtHgԍ
   int nCharset ;       // Zbg
   int nLang ;          // ̎w
   int nLangNp ;        // ̎w(no-proofing)
   int nLangFe ;        // ̎w(fareast)
   int nLangFeNp ;      // ̎w(fareast,no-proofing)
   int nIncreaseDepth ; // tHgς邽߂ɐ[x[
   unsigned long dwAttributes ;  // 
   int nDefaultCodePage ;
} DEPTH_INFO ;


// for dwAttributes
#define RTF_FONT_TABLE         1
#define RTF_HIDDEN             2
#define RTF_FIELD           0x10
#define RTF_FIELD_INST      0x20
#define RTF_FIELD_RESULT    0x40
#define RTF_SHAPE          0x100
#define RTF_SHAPE_RESULT   0x200
#define RTF_FONT_ALT      0x1000
#define RTF_LANG          0x2000
#define RTF_STYLE_SHEET  0x10000

// for nIncreaseDepth
#define INCDEPTH_JIANTI          1
#define INCDEPTH_FANTI           2
#define INCDEPTH_PINYIN          3
#define INCDEPTH_NIHAO_PINYIN    4


typedef struct {
   DEPTH_INFO *pTable ;
   int nTableSize ;     // number of items
   int nDepth ;
} DEPTH_INFO_TABLE ;

static DEPTH_INFO *GetDepthInfo ( DEPTH_INFO_TABLE *pDepthInfoTable, int nIncrease ) ;



typedef struct {
   int *pTable ;        // RTF ̃tHgԍɑΉii[鉿͊etHg̃R[hy[Wj
   int nTableSize ;     // number of items
} FONT_TABLE_INFO ;

static int GetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nDefaultCodePage ) ;
static int SetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nCodePage ) ;

static int GetNewFontNumber ( CHNCONVDATA *pData, int nCodePage, int nDstFont, int nDstGothicFont, int nKgFont, int nKgGothicFont, int nCwnfFont, int nCwnfGothicFont, int *pIsToBeBold ) ;
static int WriteNewFontName ( FONT_TABLE_INFO *pFontTableInfo, const char *szFontName, int nCodePage, int IsGothic, STREAM *StreamOut ) ;


static int GetRtfControlValue ( const char *szTag, const char *szKeyword, int *pValue ) ;

#define RTF_TAG_MAXLEN     0x80     // 32{



// b`eLXgeLXgt@Cϊ
// Ƃ 0 Ԃ
// sƂ 1 ȏ̎w肳ꂽ萔Ԃ
static int ChnconvRtf ( CHNCONVDATA *pData ) {

   char szTag [ RTF_TAG_MAXLEN ] ;
   int IsRtfFormat = 0 ;            // 0: ,  1: ς

   DEPTH_INFO_TABLE DepthInfoTable = { 0 } ;
   FONT_TABLE_INFO FontTableInfo = { 0 } ;

   STOREDCHARSA *FontName = NULL ;
   STOREDCHARSA *AltFontName = NULL ;

   int nCurrentCodePage = CP_ANSI_EUROPEAN ;    // tHg̃R[hy[W

   int nDstFont = -1 ;
   int nDstGothicFont = -1 ;
   const char *szDstFont ;
   const char *szDstGothicFont ;
   int nDstLang ;
   int nDstLangNp ;
   int nDstLangFe ;
   int nDstLangFeNp ;

   int nKgFont = -1 ;            // CWô͎ݎgp
   int nKgGothicFont = -1 ;
   int nCwnfFont = -1 ;          // CWNô͎ݎgp
   int nCwnfGothicFont = -1 ;
   int nNh3pFont = -1 ;          // NHô͎ݎgp
   int nPinyinFont = -1 ;        // GBsCp̃tHg
   int nPinyinGothicFont = -1 ;

   int IsExistSrcFont = 0 ;         // nSrcCodePage ̃tHg݂邩H

   int C ;                       // 
   int cLeadChar = 0 ;           // w肳ĂA C ͂QoCg
   int IsCodeDirect = 0 ;        // 𐔎łȂÂ܂ܒځH
   int cUnicode = 0 ;            // Unicodeŏo
   int cUnicodeLead = 0 ;        // Unicode SoCg̐擪QoCg

   int nCountChar = 0 ;

#if FIELD_DETECT
   STOREDCHARSA *Field = NULL ;
   int IsFieldInstHidden = 0 ;
#endif


   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( pData->nSrcCodePage, pData->nDstCodePage, 0 ) ;
   if ( ! ConvertChineseCharacter ) return CONV_ERROR_CODEPAGE ;
   // ϊ֐iɑ̎pj
   CONVERTCHARACTER *ConvertChineseCharacterFanti = GetConvertFunction ( pData->nSrcCodePage, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_SENSE_CW_FANTIZI ) ;
   if ( ! ConvertChineseCharacterFanti ) ConvertChineseCharacterFanti = ConvertChineseCharacter ;
   // ϊ֐iCWj
   CONVERTCHARACTER *ConvertChineseCharacterToCw = GetConvertFunction ( pData->nSrcCodePage, CP_JAPANESE_CW, 0 ) ;


   // tHg
   InitRtfFontName ( & szDstFont, & szDstGothicFont, pData ) ;


   // o͌
   nDstLang = nDstLangNp = MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ) ;
   switch ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) {
      case CP_CHINESE_SIMPLIFIED :  nDstLangFe = nDstLangFeNp = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED ) ; break ;
      case CP_CHINESE_TRADITIONAL : nDstLangFe = nDstLangFeNp = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL ) ; break ;
      case CP_JAPANESE :            nDstLangFe = nDstLangFeNp = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ; break ;
      default :                     nDstLangFe = nDstLangFeNp = MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ) ; break ;
   }


   // [e[u
   DEPTH_INFO *pDepthInfo = GetDepthInfo ( & DepthInfoTable, 0 ) ;
   if ( ! pDepthInfo ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
   pDepthInfo->nFont = -1 ;
   pDepthInfo->nAltFont = -1 ;
   pDepthInfo->nCharset = DEFAULT_CHARSET ;
   pDepthInfo->nDefaultCodePage = CP_ANSI_EUROPEAN ;



   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) {

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( pData->nError ) goto CAUGHT_ERROR ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }


      // [̐ݒ
      if ( ! IsCodeDirect || ! cLeadChar ) {
         if ( C == BRACE_END ) {

            if ( pDepthInfo->nIncreaseDepth ) {
               StreamWrite ( BRACE_END, pData->StreamOut ) ;
               pDepthInfo->nIncreaseDepth = 0 ;
            }

            pDepthInfo = GetDepthInfo ( & DepthInfoTable, -1 ) ;
            if ( ! pDepthInfo ) {
               if ( DepthInfoTable.nDepth >= 0 ) StreamWrite ( BRACE_END, pData->StreamOut ) ;
               break ;
            }

            if ( ! ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) && ( pDepthInfo + 1 )->dwAttributes & RTF_FONT_TABLE ) {
               if ( IsExistSrcFont ) {

                  int IsError = 0 ;

                  // o̓tHg
                  if ( nDstFont < 0 && ( nDstFont = WriteNewFontName ( & FontTableInfo, szDstFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                  if ( ! mbsicmp_cp ( szDstFont, szDstGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ) nDstGothicFont = nDstFont ;
                  if ( nDstGothicFont < 0 && ( nDstGothicFont = WriteNewFontName ( & FontTableInfo, szDstGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 1, pData->StreamOut ) ) < 0 ) IsError = 1 ;

                  // sCpo̓tHg
                  if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) {
                     if ( nPinyinFont < 0 && ( nPinyinFont = WriteNewFontName ( & FontTableInfo, pData->szPinyinFont, CP_ANSI_EUROPEAN, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                     if ( ! stricmp ( pData->szPinyinFont, pData->szPinyinGothicFont ) ) nPinyinGothicFont = nPinyinFont ;
                     if ( nPinyinGothicFont < 0 && ( nPinyinGothicFont = WriteNewFontName ( & FontTableInfo, pData->szPinyinGothicFont, CP_ANSI_EUROPEAN, 1, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                  }

                  // gtHg
                  if ( pData->nDstCodePage == CP_JAPANESE_CW ) {
                     if ( pData->dwConversionMode & CONVMODE_OUTPUT_NEW_FONT ) {
                        if ( nKgFont < 0 && ( nKgFont = WriteNewFontName ( & FontTableInfo, szKg7Font, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                        if ( ! mbsicmp_cp ( szKg7Font, szKg7GothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ) nKgGothicFont = nKgFont ;
                        if ( nKgGothicFont < 0 && ( nKgGothicFont = WriteNewFontName ( & FontTableInfo, szKg7GothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 1, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                     }
                     else {
                        if ( nKgFont < 0 && ( nKgFont = WriteNewFontName ( & FontTableInfo, szKgFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                        if ( ! mbsicmp_cp ( szKgFont, szKgGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ) nKgGothicFont = nKgFont ;
                        if ( nKgGothicFont < 0 && ( nKgGothicFont = WriteNewFontName ( & FontTableInfo, szKgGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 1, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                     }
                  }
                  if ( pData->nDstCodePage == CP_JAPANESE_CWN ) {
                     if ( nCwnfFont < 0 && ( nCwnfFont = WriteNewFontName ( & FontTableInfo, szCwnfFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                     if ( ! mbsicmp_cp ( szCwnfFont, szCwnfGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ) nCwnfGothicFont = nCwnfFont ;
                     if ( nCwnfGothicFont < 0 && ( nCwnfGothicFont = WriteNewFontName ( & FontTableInfo, szCwnfGothicFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 1, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                  }
                  if ( pData->nDstCodePage == CP_JAPANESE_NH ) {
                     if ( nNh3pFont < 0 && ( nNh3pFont = WriteNewFontName ( & FontTableInfo, szNh3pFont, pData->nDstCodePage & CP_DEFINE_CODEPAGE, 0, pData->StreamOut ) ) < 0 ) IsError = 1 ;
                  }

                  if ( IsError ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
               }

               // ʏ͎gȂ
               ClearStoredCharsA ( FontName ) ;
               ClearStoredCharsA ( AltFontName ) ;
            }

            StreamWrite ( C, pData->StreamOut ) ;
            continue ;
         }
         if ( C == BRACE_START ) {

            if ( pDepthInfo->nIncreaseDepth ) {
               StreamWrite ( BRACE_END, pData->StreamOut ) ;
               pDepthInfo->nIncreaseDepth = 0 ;
            }

            pDepthInfo = GetDepthInfo ( & DepthInfoTable, 1 ) ;
            if ( ! pDepthInfo ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }

            StreamWrite ( C, pData->StreamOut ) ;
            continue ;
         }
      }


      int IsCharacterHexa = 0 ;     // ASCII𐔎ŏo


      // GXP[v̏
      if ( C == ESCAPE ) {

         if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;

         // QoCg𒼐ړ͂ꍇ̂QoCgڂ̉
         // ʏ͎gȂ
         if ( IsCodeDirect && cLeadChar && ismbbtrail_cp ( ESCAPE, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ) {
            StreamUnread ( C, pData->StreamIn ) ;
            C = ESCAPE ;
            goto PROCEDURE_OUTPUT_CHAR ;
         }

         // ̎擾i16iR[hj
         if ( C == '\'' ) {
            int C0, C1 ;
            if ( ( C0 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            if ( ( C1 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            szTag [ 0 ] = C0 ;
            szTag [ 1 ] = C1 ;
            szTag [ 2 ] = 0 ;
            C = strtoul ( szTag, NULL, 0x10 ) ;
            if ( ! cLeadChar && isascii ( C ) ) IsCharacterHexa = 1 ;
            IsCodeDirect = 0 ;
            goto PROCEDURE_OUTPUT_CHAR ;
         }

         // ^Oobt@Ɏ擾
         if ( isascii ( C ) && ( ispunct ( C ) || iscntrl ( C ) ) ) {
            szTag [ 0 ] = C ;
            szTag [ 1 ] = 0 ;
         }
         else {
            StreamUnread ( C, pData->StreamIn ) ;
            int nCount = 0 ;
            for ( ; nCount < RTF_TAG_MAXLEN - 1 ; nCount ++ ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( ! isascii ( C ) || ( isspace ( C ) || iscntrl ( C ) || ispunct ( C ) ) && C != '-' ) break ;
               szTag [ nCount ] = C ;
            }
            StreamUnread ( C, pData->StreamIn ) ;
            szTag [ nCount ] = 0 ;
         }

         // ̎擾iLj
         for ( int nCount = 0 ; nCount < RTF_ESCAPE_LIST_NELEM ; nCount ++ ) {
            if ( ! strcmp ( szTag, ( RtfEscapeList [ nCount ] ).szName ) ) {
               C = ( RtfEscapeList [ nCount ] ).nCode ;
               goto PROCEDURE_OUTPUT_CHAR ;
            }
         }

         // tHgԍ̎擾
         if ( ! GetRtfControlValue ( szTag, "f", & pDepthInfo->nFont ) ) {

            // tHgԍ̕ύX
            if ( ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN ) ) ) {
               int nCodePage = GetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, pDepthInfo->nDefaultCodePage ) ;
               if ( ( nCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage ) {
                  int IsToBeBold ;
                  int nNewFont = GetNewFontNumber ( pData, nCodePage, nDstFont, nDstGothicFont, nKgFont, nKgGothicFont, nCwnfFont, nCwnfGothicFont, & IsToBeBold ) ;
                  sprintf ( szTag, "f%d%s", (short) nNewFont, ( IsToBeBold ) ? "\\b" : "" ) ;
                  pDepthInfo->dwAttributes |= RTF_LANG ;
               }
            }

            // ʏ͎gȂ
            if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) ClearStoredCharsA ( FontName ) ;

            goto PROCEDURE_OUTPUT_TAG ;
         }

         // ֘AtHgԍ̎擾
         if ( ! GetRtfControlValue ( szTag, "af", & pDepthInfo->nAltFont ) ) {

            // ֘AtHgԍ̕ύX
            if ( ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN ) ) ) {
               int nCodePage = GetFontTableValue ( & FontTableInfo, pDepthInfo->nAltFont, pDepthInfo->nDefaultCodePage ) ;
               if ( ( nCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage ) {
                  int nNewFont = GetNewFontNumber ( pData, nCodePage, nDstFont, nDstGothicFont, nKgFont, nKgGothicFont, nCwnfFont, nCwnfGothicFont, NULL ) ;
                  sprintf ( szTag, "af%d", (short) nNewFont ) ;
               }
            }

            // ʏ͎gȂ
            if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) ClearStoredCharsA ( AltFontName ) ;

            goto PROCEDURE_OUTPUT_TAG ;
         }

         // ]vȌ폜
         // ^O폜Ƃ͕K̃Xy[X폜邱
         if ( ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN ) ) ) {
            if ( ! GetRtfControlValue ( szTag, "lang", & pDepthInfo->nLang ) ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               pDepthInfo->dwAttributes |= RTF_LANG ;
               continue ;
            }
            if ( ! GetRtfControlValue ( szTag, "langnp", & pDepthInfo->nLangNp ) ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               pDepthInfo->dwAttributes |= RTF_LANG ;
               continue ;
            }
            if ( ! GetRtfControlValue ( szTag, "langfe", & pDepthInfo->nLangFe ) ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               pDepthInfo->dwAttributes |= RTF_LANG ;
               continue ;
            }
            if ( ! GetRtfControlValue ( szTag, "langfenp", & pDepthInfo->nLangFeNp ) ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               pDepthInfo->dwAttributes |= RTF_LANG ;
               continue ;
            }
            // ̎擾iUnicodej
            if ( ! GetRtfControlValue ( szTag, "u", & cUnicode ) ) {
               cUnicode = (wchar_t) cUnicode ;
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               continue ;
            }
            // UnicodeŃXLbv镶̎擾
            if ( ! GetRtfControlValue ( szTag, "uc", NULL ) ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
               continue ;
            }
         }

         // tHg̕Zbg擾
         if ( ! GetRtfControlValue ( szTag, "fcharset", & pDepthInfo->nCharset ) ) goto PROCEDURE_OUTPUT_TAG ;

         // tHgubN̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) ) {
            if ( ! strcmp ( szTag, "fonttbl" ) ) {
               pDepthInfo->dwAttributes |= RTF_FONT_TABLE ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }
         if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE && pDepthInfo->dwAttributes & RTF_HIDDEN ) {
            if ( ! strcmp ( szTag, "falt" ) ) {
               pDepthInfo->dwAttributes |= RTF_FONT_ALT ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }

         // X^CV[gubN̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_STYLE_SHEET ) ) {
            if ( ! strcmp ( szTag, "stylesheet" ) ) {
               pDepthInfo->dwAttributes |= RTF_STYLE_SHEET ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }


         // tB[h̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_FIELD ) ) {
            if ( ! strcmp ( szTag, "field" ) ) {
               pDepthInfo->dwAttributes |= RTF_FIELD ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }
         else {
            if ( ! strcmp ( szTag, "fldinst" ) ) {
               pDepthInfo->dwAttributes |= RTF_FIELD_INST ;
               pDepthInfo->dwAttributes &= ~ RTF_HIDDEN ;
#if FIELD_DETECT
               ClearStoredCharsA ( Field ) ;
               IsFieldInstHidden = 0 ;
#endif
               goto PROCEDURE_OUTPUT_TAG ;
            }
            if ( ! strcmp ( szTag, "fldrslt" ) ) {
               pDepthInfo->dwAttributes &= ~ RTF_FIELD_INST ;
               pDepthInfo->dwAttributes |= RTF_HIDDEN ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }

         // eLXg{bNX̃eLXg
         if ( ! ( pDepthInfo->dwAttributes & RTF_SHAPE ) ) {
            if ( ! strcmp ( szTag, "shp" ) ) {
               pDepthInfo->dwAttributes |= RTF_SHAPE ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
         }
         else {
            if ( ! strcmp ( szTag, "shptxt" ) ) {
               pDepthInfo->dwAttributes &= ~ RTF_HIDDEN ;
               goto PROCEDURE_OUTPUT_TAG ;
            }
            if ( ! ( pDepthInfo->dwAttributes & RTF_SHAPE_RESULT ) ) {
               if ( ! strcmp ( szTag, "shprslt" ) ) {
                  pDepthInfo->dwAttributes |= RTF_SHAPE_RESULT ;
                  goto PROCEDURE_OUTPUT_TAG ;
               }
            }
            else {
               if ( ! strcmp ( szTag, "dptxbxtext" ) ) {
                  pDepthInfo->dwAttributes &= ~ RTF_HIDDEN ;
                  goto PROCEDURE_OUTPUT_TAG ;
               }
            }
         }

         // B̏
         if ( ! ( pDepthInfo->dwAttributes & RTF_HIDDEN ) ) {
            for ( int nCount = 0 ; nCount < HIDDEN_TAG_LIST_NELEM ; nCount ++ ) {
               if ( ! strcmp ( szTag, HiddenTagList [ nCount ] ) ) {
                  pDepthInfo->dwAttributes |= RTF_HIDDEN ;
                  goto PROCEDURE_OUTPUT_TAG ;
               }
            }
         }

         // 
         if ( ! IsRtfFormat ) {
            if ( GetRtfControlValue ( szTag, "rtf", NULL ) ) { pData->nError = CONV_ERROR_FORMAT ; goto CAUGHT_ERROR ; }
            IsRtfFormat = 1 ;
            goto PROCEDURE_OUTPUT_TAG ;
         }

         // ̃R[hy[W̎擾
         if ( ! GetRtfControlValue ( szTag, "ansicpg", & pDepthInfo->nDefaultCodePage ) ) goto PROCEDURE_OUTPUT_TAG ;

         // ^Oo̓vV[W
         PROCEDURE_OUTPUT_TAG :

         if ( pDepthInfo->nIncreaseDepth ) {
            StreamWrite ( BRACE_END, pData->StreamOut ) ;
            pDepthInfo->nIncreaseDepth = 0 ;
         }

         StreamWrite ( ESCAPE, pData->StreamOut ) ;
         StreamWriteString ( szTag, pData->StreamOut ) ;
         // ^Ǒ̃Xy[X
         if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         if ( C != 0x20 ) StreamUnread ( C, pData->StreamIn ) ;
         else             StreamWrite ( C, pData->StreamOut ) ;
         continue ;
      }


      // ڂǂ
      IsCodeDirect = 1 ;


      // o̓vV[W
      PROCEDURE_OUTPUT_CHAR :

      if ( ! IsRtfFormat ) { pData->nError = CONV_ERROR_FORMAT ; goto CAUGHT_ERROR ; }

      // s𑗂
      if ( C == '\r' || C == '\n' ) {
         StreamWrite ( C, pData->StreamOut ) ;
         continue ;
      }


      // Surrogate Pair
      if ( cUnicodeLead ) {
         if ( cUnicode && IsUnicodeLowSurrogate ( cUnicode ) ) {
            cUnicode = MAKE_UNICODE_SCALAR ( cUnicodeLead, cUnicode ) ;
            cUnicodeLead = 0 ;
         }
         else {
            StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d \\\'%02x", (short) cUnicodeLead, '?' ) ;
            cUnicodeLead = 0 ;
         }
      }
      else {
         if ( cUnicode && IsUnicodeHighSurrogate ( cUnicode ) ) {
            cUnicodeLead = cUnicode ;
            cUnicode = 0 ;
            continue ;
         }
      }


      // R[hy[W擾
      if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) nCurrentCodePage = CharsetToCodePage ( pDepthInfo->nCharset ) ;
      else                                             nCurrentCodePage = GetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, pDepthInfo->nDefaultCodePage ) ;


#if FIELD_DETECT
      // tB[h
      if ( pDepthInfo->dwAttributes & RTF_FIELD_INST ) {
         if ( IsCodeDirect && C == '\"' ) {
            int C2 ;
            if ( ( C2 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            StreamUnread ( C2, pData->StreamIn ) ;

            char *szField = GetStoredCharsA ( Field ) ;
            if ( 0 ) {     // for debug
               StreamWriteFormat ( pData->StreamOut, "[%s]", szField ) ;
               StreamWriteFormat ( pData->StreamOut, "[%s]", GetLastKeyword ( szField ) ) ;
            }
            if ( IsFieldInstHidden || ! strheadcmp ( GetFirstKeyword ( szField ), "EQ" ) && ! strheadcmp ( GetLastKeyword ( szField ), "\\*" ) && C2 == 'F' ) {
               IsFieldInstHidden = ! IsFieldInstHidden ;
               continue ;
            }
         }
         StoreCharsA ( C, Field ) ;
         if ( IsFieldInstHidden ) continue ;
      }
#endif


      // tHge[ȕ
      if ( pDepthInfo->dwAttributes & RTF_FONT_TABLE ) {
         if ( C == ';' ) {

            char *szFontName = GetStoredCharsA ( FontName ) ;
            char *szAltFontName = GetStoredCharsA ( AltFontName ) ;

            if ( nCurrentCodePage == ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ) {
               if ( ! strcmp ( szFontName, szDstFont ) )       nDstFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, szDstGothicFont ) ) nDstGothicFont = pDepthInfo->nFont ;
            }
            if ( nCurrentCodePage == CP_JAPANESE ) {
               if ( ! strcmp ( szFontName, szKgFont ) )         nKgFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, szKgGothicFont ) )   nKgGothicFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, szCwnfFont ) )       nCwnfFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, szCwnfGothicFont ) ) nCwnfGothicFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, szNh3pFont ) )       nNh3pFont = pDepthInfo->nFont ;
            }
            if ( nCurrentCodePage == CP_ANSI_EUROPEAN ) {
               if ( ! strcmp ( szFontName, pData->szPinyinFont ) )       nPinyinFont = pDepthInfo->nFont ;
               if ( ! strcmp ( szFontName, pData->szPinyinGothicFont ) ) nPinyinGothicFont = pDepthInfo->nFont ;
            }

            char *szPseudoFontName = szFontName ;
            if ( *szPseudoFontName == '@' ) szPseudoFontName ++ ;

            // tHg̉
            nCurrentCodePage = FontNameToPseudoCodePage ( nCurrentCodePage, szPseudoFontName ) ;
            if ( ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage ) IsExistSrcFont = 1 ;

            // CW7
            if ( ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == CP_JAPANESE_CW ) {
               if ( IsAsciiString ( szPseudoFontName ) ) {
                  const char *szLocalizedName = GetLocalizedFontName ( CodePageToCharset ( CP_JAPANESE ), szPseudoFontName, NULL ) ;
                  if ( szLocalizedName && strcmp ( szAltFontName, szLocalizedName ) ) WriteAltFontName ( szLocalizedName, CP_JAPANESE, pData->StreamOut ) ;
               }
               else {
                  const char *szEnglishName = GetEnglishFontName ( CodePageToCharset ( CP_JAPANESE ), szPseudoFontName, NULL ) ;
                  if ( szEnglishName && strcmp ( szAltFontName, szEnglishName ) ) WriteAltFontName ( szEnglishName, CP_JAPANESE, pData->StreamOut ) ;
               }
            }

            ClearStoredCharsA ( FontName ) ;
            ClearStoredCharsA ( AltFontName ) ;

            // ϊʂ𔽉f
            if ( SetFontTableValue ( & FontTableInfo, pDepthInfo->nFont, nCurrentCodePage ) ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
         }
         // tHg̎擾
         else {
            if ( pDepthInfo->dwAttributes & RTF_HIDDEN ) {
               if ( pDepthInfo->dwAttributes & RTF_FONT_ALT ) {
                  AltFontName = StoreCharsA ( C & 0xFF, AltFontName ) ;
                  if ( AltFontName == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
               }
            }
            else {
               FontName = StoreCharsA ( C & 0xFF, FontName ) ;
               if ( FontName == STOREDCHARS_ERROR ) { pData->nError = CONV_ERROR_MEMORY ; goto CAUGHT_ERROR ; }
            }
         }
      }


      // QoCĝPoCgڂǂ
      if ( ! cLeadChar && ismbblead_cp ( C, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ) {
         cLeadChar = C ;
         continue ;
      }

      // QoCĝQoCgڂ
      if ( cLeadChar ) {
         C = ( cLeadChar << 8 ) | C ;
         cLeadChar = 0 ;
      }


      // o
      if ( ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN | RTF_LANG ) ) == RTF_LANG ) {
         if ( ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage ) {
            pDepthInfo->nLang = nDstLang ;
            pDepthInfo->nLangNp = nDstLangNp ;
            pDepthInfo->nLangFe = nDstLangFe ;
            pDepthInfo->nLangFeNp = nDstLangFeNp ;
         }
         WriteLangInfo ( pDepthInfo->nLang, pDepthInfo->nLangNp, pDepthInfo->nLangFe, pDepthInfo->nLangFeNp, pData->StreamOut ) ;
         pDepthInfo->dwAttributes &= ~ RTF_LANG ;
      }


      // HKSCS ϊ
      if ( ( ( pData->dwConversionMode & CONVMODE_HKSCS_PUA_TO_CJK ) || ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage )
         && ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN ) )
         && ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_OPTIONAL ) ) == CP_CHINESE_TRADITIONAL_HK
         && ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE )
      ) {
         int cUnicodePua = ( cUnicode ) ? cUnicode : GaijiToUnicode ( C, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ;
         if ( cUnicodePua ) {
            cUnicode = HkscsUni2uni ( cUnicodePua, TRUE ) ;
            if ( cUnicode ) C = 0x3F3F ;  // '??'
         }
      }


      // ϊ
      if ( ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN | RTF_STYLE_SHEET ) ) && ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == pData->nSrcCodePage ) {

         // ʏ͎gȂ
         if ( pDepthInfo->nFont < 0 && pDepthInfo->nAltFont > 0 ) {
            StreamWriteFormat ( pData->StreamOut, "\\f%d ", (short) GetNewFontNumber ( pData, nCurrentCodePage, nDstFont, nDstGothicFont, nKgFont, nKgGothicFont, nCwnfFont, nCwnfGothicFont, NULL ) ) ;
            pDepthInfo->nFont = pDepthInfo->nAltFont ;
         }

         // Ô
         if ( cUnicode && ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE ) ) {
            int cTmp = ConvertCharacterEx ( cUnicode, CP_UTF32, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ;
            if ( cTmp ) {
               C = cTmp ;
               cUnicode = 0 ;
            }
         }

         int cSaved = C ;
         if ( ( nCurrentCodePage & CP_DEFINE_CODEPAGE ) == CP_JAPANESE ) {
            cUnicode = 0 ;                               // [R[h Unicode w𖳌ɂ
            if ( C != 0xEEF9 ) C = SjisNec2ibm ( C ) ;   // NEC->IBM
         }

         int nIncreaseDepth = 0 ;

         // ܂ŏɃ[Uw̕ϊ
         int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;

         // e[uŕϊłȂ̂ Unicode ɒڕϊ
         if ( ! cResult && ! cUnicode ) {
            if ( ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE ) ) {
               if ( ConvertChineseCharacter == Cw2gb || ConvertChineseCharacter == Cw2big5 )        cUnicode = Kg2uc_NoGbOnly ( C ) ;
               else if ( ConvertChineseCharacter == Cwn2gb || ConvertChineseCharacter == Cwn2big5 ) cUnicode = Cwn2uc_NoGbOnly ( C ) ;
               if ( cUnicode ) cResult = '?' ;
            }
         }

         // ϊ̃C
         if ( ! cResult && ! cUnicode ) {
            if ( C < 0x100 || ismbbtrail_cp ( C & 0xFF, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ) {
               if ( nCurrentCodePage & CP_FANTI ) cResult = ConvertChineseCharacterFanti ( C, pData->dwConversionMode ) ;
               else                               cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
            }
         }

         // O
         if ( ! cResult && ! cUnicode ) {
            if ( ( pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE && ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) != CP_JAPANESE ) {
               cUnicode = GaijiToUnicode ( C, pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) ;
               cResult = UnicodeToGaiji ( cUnicode, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ;
               if ( pData->nSrcCodePage == CP_CHINESE_SIMPLIFIED && C >= 0xFE50 && C <= 0xFEA0 ) cResult = 0 ;
               if ( pData->nDstCodePage == CP_CHINESE_SIMPLIFIED && cResult >= 0xFE50 && cResult <= 0xFEA0 ) cResult = 0 ;
               if ( cResult ) cUnicode = 0 ;
            }
         }

         // ϊłȂ̂ OS oR Unicode ɒڕϊ
         if ( ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE ) ) {
            if ( pData->nDstCodePage == CP_CHINESE_TRADITIONAL && ! cResult ) {
               if ( ! cUnicode ) {
                  int cGb ;
                  CONVERTCHARACTER *ConvertChineseCharacterTmp = GetConvertFunction ( pData->nSrcCodePage, CP_CHINESE_SIMPLIFIED, nCurrentCodePage & CP_FANTI ) ;
                  if ( ConvertChineseCharacterTmp ) cGb = ConvertChineseCharacterTmp ( C, pData->dwConversionMode ) ;
                  else                              cGb = C ;  // pData->nSrcCodePage == CP_CHINESE_SIMPLIFIED
                  cUnicode = ConvertCharacterEx ( cGb, CP_CHINESE_SIMPLIFIED, CP_UTF32 ) ;
               }
               if ( 1 ) {
                  int cUnicodeResult = ConvertJtzToFtz ( cUnicode, pData->dwConversionMode ) ;
                  if ( cUnicodeResult ) cUnicode = cUnicodeResult ;
               }
            }
            if ( pData->nSrcCodePage == CP_CHINESE_TRADITIONAL && ( ! cResult || pData->dwConversionMode & CONVMODE_CONVERT_CJKEX && IsHaveUniFtzToJtz ( C ) ) ) {
               if ( ! cUnicode ) {
                  cUnicode = ConvertCharacterEx ( C, CP_CHINESE_TRADITIONAL, CP_UTF32 ) ;
               }
               if ( 1 ) {
                  int cUnicodeResult = ConvertFtzToJtz ( cUnicode, pData->dwConversionMode ) ;
                  if ( cUnicodeResult ) cUnicode = cUnicodeResult ;
               }
            }
         }

         // Unicode ʏ̕ɕϊ
         if ( ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE ) ) {
            if ( cUnicode ) {
               if ( pData->nDstCodePage == CP_CHINESE_TRADITIONAL ) {
                  cResult = ConvertCharacterEx ( cUnicode, CP_UTF32, CP_CHINESE_TRADITIONAL ) ;
                  if ( cResult ) cUnicode = 0 ;
                  else           cResult = '?' ;
               }
               else {
                  int cGb = ConvertCharacterEx ( cUnicode, CP_UTF32, CP_CHINESE_SIMPLIFIED ) ;
                  CONVERTCHARACTER *ConvertChineseCharacterTmp = GetConvertFunction ( CP_CHINESE_SIMPLIFIED, pData->nDstCodePage, 0 ) ;
                  if ( ConvertChineseCharacterTmp ) cResult = ConvertChineseCharacterTmp ( cGb, pData->dwConversionMode ) ;
                  else                              cResult = cGb ;
                  if ( cResult || ConvertChineseCharacterTmp ) cUnicode = 0 ;
                  else                                         cResult = '?' ;
               }
            }
         }


         // [R[hŊȑ̎ŕ\łȂGANA
         if ( ! ( nCurrentCodePage & CP_FANTI ) ) {
            switch ( pData->nSrcCodePage ) {
               case CP_JAPANESE_CW :   if ( C >= 0xF040 && C <= 0xF4FF ) { cResult = 0 ; cUnicode = 0 ; } break ;
               case CP_JAPANESE_CWN :  if ( C >= 0xF090 && C <= 0xF4FF ) { cResult = 0 ; cUnicode = 0 ; } break ;
            }
         }

         // sC̏
         if ( C < 0x100 && ! cUnicode || pData->nSrcCodePage == CP_CHINESE_SIMPLIFIED && C >= 0xA8A1 && C <= 0xA8C0 ) {

            int cCw ;
            if ( ConvertChineseCharacterToCw ) cCw = ConvertChineseCharacterToCw ( C, pData->dwConversionMode ) ;
            else                               cCw = C ; // pData->nSrcCodePage == CP_JAPANESE_CW

            if ( pData->nSrcCodePage == CP_JAPANESE_NH ) {
               switch ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE | CP_DEFINE_OPTIONAL ) ) {
                  case CP_JAPANESE_NHP :  cCw = isascii ( C ) ? Nh2cwOneByteChar_0000_007F [ C ] : 0 ; cResult = 0 ; break ;
                  case CP_JAPANESE_NH3P : cCw = isascii ( C ) ? Nh2cwOneByteCharNh3_0000_007F [ C ] : 0 ; cResult = 0 ; break ;
               }
            }

            // GB/BIG5 ̃sCo
            if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) {
               if ( ! isascii ( cCw ) || isprint ( cCw ) ) {
                  int cUnicodeResult = ( isascii ( cCw ) ) ? cCw : CwOneByteCharToUnicode_0080_00FF [ cCw - 0x80 ] ;
                  if ( cUnicodeResult < 0x2000 ) {
                     if ( cUnicodeResult < 0x100 ) {
                        cResult = cUnicodeResult ; // cResult is in CP_ANSI_EUROPEAN
                        cUnicode = 0 ;
                     }
                     else {
                        cResult = '?' ;
                        cUnicode = cUnicodeResult ;
                     }
                     nIncreaseDepth = INCDEPTH_PINYIN ;
                  }
               }
            }

            // Nihao ̃sCo
            if ( pData->nDstCodePage == CP_JAPANESE_NH && cCw >= 0x80 ) {
               cResult = Cw2nhOneByteCharNh3_0080_00FF [ cCw - 0x80 ] ;
               if ( cResult <= 0xFF ) nIncreaseDepth = INCDEPTH_NIHAO_PINYIN ;
            }

            // ʏ Nihao ̃sC͂̂
            if ( ! cResult ) {
               CONVERTCHARACTER *ConvertChineseCharacterTmp = GetConvertFunction ( CP_JAPANESE_CW, pData->nDstCodePage, 0 ) ;
               if ( ConvertChineseCharacterTmp ) cResult = ConvertChineseCharacterTmp ( cCw, pData->dwConversionMode ) ;
               else                              cResult = cCw ;  // pData->nDstCodePage == CP_JAPANESE_CW
            }
         }

         // Chinese Writer ̒̏
         if ( pData->nDstCodePage == CP_JAPANESE_CW ) {
            if ( cResult >= 0xF9D8 && cResult <= 0xF9FC ) nIncreaseDepth = INCDEPTH_FANTI ;
         }

         // ɑ̎ϊłȂƂȑ̎ł̕ϊ݂
         if ( ! cResult && ! cUnicode ) {
            if ( ( nCurrentCodePage & CP_FANTI ) ) {
               if ( pData->nSrcCodePage == CP_JAPANESE_CWN && pData->nDstCodePage == CP_JAPANESE_CW ) {
                  int cGb = Cwnf2gb ( C, pData->dwConversionMode ) ;
                  if ( cGb ) cResult = Gb2cw ( cGb, pData->dwConversionMode ) ;
                  if ( cResult ) nIncreaseDepth = INCDEPTH_JIANTI ;
               }
               if ( pData->nSrcCodePage == CP_JAPANESE_CW && pData->nDstCodePage == CP_JAPANESE_CWN ) {
                  int cGb = Kg2gb ( C, pData->dwConversionMode ) ;
                  if ( cGb ) cResult = Gb2cwn ( cGb, pData->dwConversionMode ) ;
                  if ( cResult ) nIncreaseDepth = INCDEPTH_JIANTI ;
               }
            }
         }

         // Վ̃tHgݒ
         if ( pDepthInfo->nIncreaseDepth != nIncreaseDepth ) {
            if ( pDepthInfo->nIncreaseDepth ) StreamWrite ( BRACE_END, pData->StreamOut ) ;
            if ( ( nCurrentCodePage & CP_DEFINE_FONT ) == CP_GOTHIC ) {
               switch ( nIncreaseDepth ) {
                  case INCDEPTH_JIANTI :        StreamWriteFormat ( pData->StreamOut, "%c\\loch\\af%d\\hich\\af%d\\dbch\\f%d%s ", BRACE_START, (short) nDstGothicFont, (short) nDstGothicFont, (short) nDstGothicFont, ( nDstGothicFont == nDstFont ) ? "\\b" : "" ) ; break ;
                  case INCDEPTH_FANTI :         StreamWriteFormat ( pData->StreamOut, "%c\\loch\\af%d\\hich\\af%d\\dbch\\f%d%s ", BRACE_START, (short) nKgGothicFont, (short) nKgGothicFont, (short) nKgGothicFont, ( nKgGothicFont == nKgFont ) ? "\\b" : "" ) ; break ;
                  case INCDEPTH_PINYIN :        StreamWriteFormat ( pData->StreamOut, "%c\\f%d\\hich\\af%d%s ",                   BRACE_START, (short) nPinyinGothicFont, (short) nPinyinGothicFont, ( nPinyinFont == nPinyinGothicFont ) ? "\\b" : "" ) ; break ;
                  case INCDEPTH_NIHAO_PINYIN :  StreamWriteFormat ( pData->StreamOut, "%c\\f%d%s ",                               BRACE_START, (short) nNh3pFont, "\\b" ) ; break ;
                  default : nIncreaseDepth = 0 ; break ;
               }
            }
            else {
               switch ( nIncreaseDepth ) {
                  case INCDEPTH_JIANTI :        StreamWriteFormat ( pData->StreamOut, "%c\\loch\\af%d\\hich\\af%d\\dbch\\f%d ", BRACE_START, (short) nDstFont, (short) nDstFont, (short) nDstFont ) ; break ;
                  case INCDEPTH_FANTI :         StreamWriteFormat ( pData->StreamOut, "%c\\loch\\af%d\\hich\\af%d\\dbch\\f%d ", BRACE_START, (short) nKgFont, (short) nKgFont, (short) nKgFont ) ; break ;
                  case INCDEPTH_PINYIN :        StreamWriteFormat ( pData->StreamOut, "%c\\f%d\\hich\\af%d ",                   BRACE_START, (short) nPinyinFont, (short) nPinyinFont ) ; break ;
                  case INCDEPTH_NIHAO_PINYIN :  StreamWriteFormat ( pData->StreamOut, "%c\\f%d%s ",                             BRACE_START, (short) nNh3pFont, "" ) ; break ;
                  default : nIncreaseDepth = 0 ; break ;
               }
            }
            pDepthInfo->nIncreaseDepth = nIncreaseDepth ;
         }

         // ϊłȂ̏o
         if ( ! cResult ) {
            if ( ! ( pData->dwConversionMode & CONVMODE_NO_DECOMPOSE ) ) {
               char szBuffer [ 0x20 ] ;
               const char *szString = GetDecomposedForm ( C, pData->nSrcCodePage, pData->nDstCodePage, szBuffer ) ;
               if ( szString ) {
                  char szBuffer [ 0x40 ] ;
                  EncodeHexaDecimal ( szString, szBuffer, pData->nDstCodePage & CP_DEFINE_CODEPAGE ) ;
                  StreamWriteString ( szBuffer, pData->StreamOut ) ;
                  continue ;
               }
            }
         }
         if ( ! cResult && ! cUnicode ) {
            if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
               StreamWriteFormat ( pData->StreamOut, "[%s:%04X]", GetCodeName ( pData->nSrcCodePage ), cSaved ) ;
               continue ;
            }
            if ( C >= 0x100 && ! ( pData->dwConversionMode & CONVMODE_ONE_QUESTION_MARK ) ) cResult = 0x3F3F ; // '??'
            else                                                                            cResult = 0x3F ;   // '?'
         }

         // ϊʂ𔽉f
         C = cResult ;

         // ȍ~ nCurrentCodePage ͕ϊ̌ʂ𔽉f
         nCurrentCodePage = pData->nDstCodePage | ( nCurrentCodePage & CP_DEFINE_FONT ) ;
      }


      // Վ̕ϊ
      if ( ( pData->dwConversionMode & ( CONVMODE_LINGSHIDAIMA_TO_CJKEX | CONVMODE_CJKEX_TO_LINGSHIDAIMA ) )
         && ! ( pDepthInfo->dwAttributes & ( RTF_FONT_TABLE | RTF_HIDDEN ) )
         && ( nCurrentCodePage & ( CP_DEFINE_CODEPAGE | CP_DEFINE_PSEUDOCODE ) ) == CP_CHINESE_SIMPLIFIED
         && ! pDepthInfo->nIncreaseDepth
         && ! ( pData->dwConversionMode & CONVMODE_NO_USE_UNICODE )
      ) {
         int cSrc = ( cUnicode ) ? cUnicode : GaijiToUnicode ( C, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ;
         if ( cSrc ) {
            int cDst = 0 ;
            if ( pData->dwConversionMode & CONVMODE_LINGSHIDAIMA_TO_CJKEX ) cDst = GetNonLinshiDaima ( cSrc ) ;
            if ( pData->dwConversionMode & CONVMODE_CJKEX_TO_LINGSHIDAIMA ) cDst = GetLinshiDaima ( cSrc ) ;
            if ( cDst ) {
               int cResult = UnicodeToGaiji ( cDst, nCurrentCodePage & CP_DEFINE_CODEPAGE ) ;
               if ( cResult ) {
                  C = cResult ;
                  cUnicode = 0 ;
               }
               else {
                  C = '?' ;
                  cUnicode = cDst ;
               }
            }
         }
      }


      // Unicodeŏo
      if ( cUnicode ) {
         if ( IsFourByteLetter ( cUnicode ) ) {
            cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
            StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d \\\'%02x", (short) ( cUnicode >> 16 ), '?' ) ;
         }
         StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d \\\'%02x", (short) cUnicode, '?' ) ;
         cUnicode = 0 ;
         continue ;
      }

      // o
      if ( C >= 0x100 )
         StreamWriteFormat ( pData->StreamOut, "\\\'%02x\\\'%02x", ( C >> 8 ) & 0xFF, C & 0xFF ) ;
      else if ( C >= 0x80 || IsCharacterHexa )
         StreamWriteFormat ( pData->StreamOut, "\\\'%02x", C & 0xFF ) ;
      else if ( C == ESCAPE || C == BRACE_START || C == BRACE_END )
         StreamWriteFormat ( pData->StreamOut, "\\%c", C ) ;
      else if ( C )
         StreamWrite ( C, pData->StreamOut ) ;
   }


   CAUGHT_ERROR :

   free ( FontTableInfo.pTable ) ;
   free ( DepthInfoTable.pTable ) ;

   FreeStoredCharsA ( FontName ) ;
   FreeStoredCharsA ( AltFontName ) ;
#if FIELD_DETECT
   FreeStoredCharsA ( Field ) ;
#endif

   if ( ! pData->nError && DepthInfoTable.nDepth ) pData->nError = CONV_ERROR_BRACE ;
   return pData->nError ;
}



#define INIT_DEPTHINFOTABLE_SIZE  0x20



static DEPTH_INFO *GetDepthInfo ( DEPTH_INFO_TABLE *pDepthInfoTable, int nIncrease ) {

   pDepthInfoTable->nDepth += nIncrease ;

   if ( nIncrease < 0 ) {
      if ( pDepthInfoTable->nDepth <= 0 ) return NULL ;
   }
   else {
      if ( pDepthInfoTable->nDepth >= pDepthInfoTable->nTableSize ) {

         if ( pDepthInfoTable->nDepth > USHRT_MAX ) return NULL ; // too large

         int nNewTableSize = pDepthInfoTable->nTableSize + INIT_DEPTHINFOTABLE_SIZE ;
         DEPTH_INFO *pNewTable = (DEPTH_INFO *) realloc ( pDepthInfoTable->pTable, nNewTableSize * sizeof(DEPTH_INFO) ) ;
         if ( ! pNewTable ) return NULL ;

         if ( ! pDepthInfoTable->nTableSize ) memzero ( pNewTable, sizeof(DEPTH_INFO) ) ;

         pDepthInfoTable->pTable = pNewTable ;
         pDepthInfoTable->nTableSize = nNewTableSize ;
      }
   }

   DEPTH_INFO *pCurrent = pDepthInfoTable->pTable + pDepthInfoTable->nDepth ;
   if ( nIncrease > 0 ) *pCurrent = *( pCurrent - 1 ) ;
   return pCurrent ;
}



// L[[hl擾
// pValue  NULL ł
// 擾 0 Ԃ
static int GetRtfControlValue ( const char *szTag, const char *szKeyword, int *pValue ) {

   int c1, c2 ;

   while ( 1 ) {
      c1 = *szTag ;
      c2 = *szKeyword ;
      if ( ! c2 && ( isdigit ( c1 ) || c1 == '-' || ! c1 ) ) {
         if ( pValue ) *pValue = (unsigned short) strtol ( szTag, NULL, 10 ) ;
         return 0 ;
      }
      if ( c1 != c2 ) break ;
      szTag ++ ;
      szKeyword ++ ;
   }

   return 1 ;
}



////////////////////////////////////////////
//          RTF tHge[u          //
////////////////////////////////////////////



#define INIT_FONTTABLE_SIZE      0x400



// tHg̃R[hy[WԂ
static int GetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nDefaultCodePage ) {

   int nCodePage ;
   if ( (unsigned) nFont < (unsigned) pFontTableInfo->nTableSize ) nCodePage = pFontTableInfo->pTable [ nFont ] ;
   else                                                            nCodePage = 0 ;

   if ( nCodePage ) return nCodePage ;
   if ( nDefaultCodePage ) return nDefaultCodePage ;
   return CP_ANSI_EUROPEAN ;
}



// tHg̃R[hy[Wݒ
//  0 As炻ȊOԂ
static int SetFontTableValue ( FONT_TABLE_INFO *pFontTableInfo, int nFont, int nCodePage ) {

   if ( (unsigned) nFont > SHRT_MAX ) return 0 ;

   if ( nFont >= pFontTableInfo->nTableSize ) {

      int nNewTableSize = ( (unsigned) nFont / INIT_FONTTABLE_SIZE + 1 ) * INIT_FONTTABLE_SIZE ;
      int *pNewTable = (int *) realloc ( pFontTableInfo->pTable, nNewTableSize * sizeof(int) ) ;
      if ( ! pNewTable ) return 1 ;

      for ( int nCount = pFontTableInfo->nTableSize ; nCount < nNewTableSize ; nCount ++ ) pNewTable [ nCount ] = 0 ;

      pFontTableInfo->pTable = pNewTable ;
      pFontTableInfo->nTableSize = nNewTableSize ;
   }

   pFontTableInfo->pTable [ nFont ] = nCodePage ;

   return 0 ;
}



////////////////////////////////////////////
//           tHgԍ̍XV           //
////////////////////////////////////////////



// ϊׂtHgԍVԍԂ
static int GetNewFontNumber ( CHNCONVDATA *pData, int nCodePage, int nDstFont, int nDstGothicFont, int nKgFont, int nKgGothicFont, int nCwnfFont, int nCwnfGothicFont, int *pIsToBeBold ) {

   int nDstFontTmp = nDstFont ;
   int nDstGothicFontTmp = nDstGothicFont ;

   if ( pData->dwConversionMode & CONVMODE_SENSE_CW_FANTIZI && ( nCodePage & CP_FANTI ) ) {
      switch ( pData->nDstCodePage ) {
         case CP_JAPANESE_CW :   nDstFontTmp = nKgFont ; nDstGothicFontTmp = nKgGothicFont ; break ;
         case CP_JAPANESE_CWN :  nDstFontTmp = nCwnfFont ; nDstGothicFontTmp = nCwnfGothicFont ; break ;
      }
   }

   if ( ( nCodePage & CP_DEFINE_FONT ) == CP_GOTHIC ) {
      if ( pIsToBeBold ) *pIsToBeBold = ( nDstFontTmp == nDstGothicFontTmp ) ;
      return nDstGothicFontTmp ;
   }
   else {
      if ( pIsToBeBold ) *pIsToBeBold = 0 ;
      return nDstFontTmp ;
   }
}



// 󂢂ĂtHgԍԂ
// tHgԍAs -1 Ԃ
static int GetUnusedFontNumber ( FONT_TABLE_INFO *pFontTableInfo, int nCodePage ) {

   const int nFontMax = 0x7BC0 ;    // the range to -31680 through 31680

   for ( int nFont = min ( pFontTableInfo->nTableSize - 1, nFontMax / 2 ) ; nFont >= 0 ; nFont -- ) {
      if ( pFontTableInfo->pTable [ nFont ] ) {
         nFont ++ ;
         if ( nFont >= pFontTableInfo->nTableSize || ! pFontTableInfo->pTable [ nFont ] ) {
            if ( ! SetFontTableValue ( pFontTableInfo, nFont, nCodePage ) ) return nFont ;
         }
         break ;
      }
   }

   for ( int nFont = 0 ; nFont < nFontMax ; nFont ++ ) {
      if ( nFont >= pFontTableInfo->nTableSize ) {
         if ( ! SetFontTableValue ( pFontTableInfo, nFont, nCodePage ) ) return nFont ;
         break ;
      }
      if ( ! pFontTableInfo->pTable [ nFont ] ) {
         if ( ! SetFontTableValue ( pFontTableInfo, nFont, nCodePage ) ) return nFont ;
      }
   }

   return -1 ;
}



// 󂢂ĂtHgԍTătHg
// tHgԍAs -1 Ԃ
static int WriteNewFontName ( FONT_TABLE_INFO *pFontTableInfo, const char *szFontName, int nCodePage, int IsGothic, STREAM *StreamOut ) {

   int nFont = GetUnusedFontNumber ( pFontTableInfo, nCodePage ) ;

   if ( nFont >= 0 ) WriteFontName ( nFont, szFontName, nCodePage, IsGothic, StreamOut ) ;

   return nFont ;
}



////////////////////////////////////////////
//     RTF ̓[eB̉֐     //
////////////////////////////////////////////



// tHg
static int InitRtfFontName ( const char **pDstFont, const char **pDstGothicFont, CHNCONVDATA *pData ) {

   switch ( pData->nDstCodePage ) {

      case CP_CHINESE_SIMPLIFIED :
         *pDstFont = pData->szGbFont ;
         *pDstGothicFont = pData->szGbGothicFont ;
         break ;

      case CP_CHINESE_TRADITIONAL :
         *pDstFont = pData->szBig5Font ;
         *pDstGothicFont = pData->szBig5GothicFont ;
         break ;

      case CP_JAPANESE_CW :
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_NEW_FONT ) {
            *pDstFont = szCw7Font ;
            *pDstGothicFont = szCw7GothicFont ;
            break ;
         }
         else {
            *pDstFont = szCwFont ;
            *pDstGothicFont = szCwGothicFont ;
            break ;
         }

      case CP_JAPANESE_NH :
         *pDstFont = szNhFont ;
         *pDstGothicFont = szNhGothicFont ;
         break ;

      case CP_JAPANESE_CWN :
         *pDstFont = szCwnFont ;
         *pDstGothicFont = szCwnGothicFont ;
         break ;

      case CP_JAPANESE_ZQ :
         *pDstFont = szZqFont ;
         *pDstGothicFont = szZqGothicFont ;
         break ;

      case CP_JAPANESE_CX :
         *pDstFont = szCxFont ;
         *pDstGothicFont = szCxGothicFont ;
         break ;

      case CP_JAPANESE_ZJ :
         *pDstFont = szZjFont ;
         *pDstGothicFont = szZjGothicFont ;
         break ;

      default :
         *pDstFont = NULL ;
         *pDstGothicFont = NULL ;
         break ;
   }

   return 0 ;
}



static int FontNameGothic ( int nCodePage, const char *szFontName ) ;



// tHg[R[hEC擾
// YtHgȂ΁AnCodePage ̂܂ܕԂ
static int FontNameToPseudoCodePage ( int nCodePage, const char *szFontName ) {

   switch ( nCodePage ) {

      case CP_CHINESE_SIMPLIFIED :
      case CP_CHINESE_TRADITIONAL :

         if ( strstri ( szFontName, szHkscsName ) ) return FontNameGothic ( CP_CHINESE_TRADITIONAL_HK, szFontName ) ;
         return FontNameGothic ( nCodePage, szFontName ) ;

      case CP_JAPANESE :

         // CP_JAPANESE_CW
         if ( ! strheadcmp ( szFontName, szCwHead ) )       return FontNameGothic ( CP_JAPANESE_CW, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szCwFantiHead ) )  return FontNameGothic ( CP_JAPANESE_CW_FANTI, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szCw7Head ) )      return FontNameGothic ( CP_JAPANESE_CW, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szCw7FantiHead ) ) return FontNameGothic ( CP_JAPANESE_CW_FANTI, szFontName ) ;

         // CP_JAPANESE_NH
         if ( ! strheadcmp ( szFontName, szNhHead ) ) {
            if ( mbsstr_cp ( szFontName, "", CP_JAPANESE ) ) return FontNameGothic ( CP_JAPANESE_NHP, szFontName ) ;
            else                                                 return FontNameGothic ( CP_JAPANESE_NH, szFontName ) ;
         }
         if ( ! strheadcmp ( szFontName, szNh3Head ) ) {
            if ( mbsstr_cp ( szFontName, "", CP_JAPANESE ) ) return FontNameGothic ( CP_JAPANESE_NH3P, szFontName ) ;
            else                                                 return FontNameGothic ( CP_JAPANESE_NH, szFontName ) ;
         }
         if ( ! strheadcmp ( szFontName, szNhOptionHead1 ) )   return FontNameGothic ( CP_JAPANESE_NH, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szNhOptionHead2 ) )   return FontNameGothic ( CP_JAPANESE_NH, szFontName ) ;

         // CP_JAPANESE_CWN
         if ( ! strheadcmp ( szFontName, szCwnHead ) )      return FontNameGothic ( CP_JAPANESE_CWN, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szCwnFantiHead ) ) return FontNameGothic ( CP_JAPANESE_CWN_FANTI, szFontName ) ;

         // CP_JAPANESE_ZQ
         if ( ! strheadcmp ( szFontName, szZqHead ) )       return FontNameGothic ( CP_JAPANESE_ZQ, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szZqFantiHead ) )  return FontNameGothic ( CP_JAPANESE_ZQ, szFontName ) ;

         // CP_JAPANESE_CX :
         if ( ! strheadcmp ( szFontName, szCxHead ) )       return FontNameGothic ( CP_JAPANESE_CX, szFontName ) ;

         // CP_JAPANESE_ZJ :
         if ( ! strheadcmp ( szFontName, szZjHead ) )       return FontNameGothic ( CP_JAPANESE_ZJ, szFontName ) ;
         if ( ! strheadcmp ( szFontName, szZjFantiHead ) )  return FontNameGothic ( CP_JAPANESE_ZJ, szFontName ) ;

         break ;
   }

   return nCodePage ;
}



// [R[h̃tHgSVbNeXgĕԂ
// SVbNłȂ nCodePage ̂܂ܕԂ
static int FontNameGothic ( int nCodePage, const char *szFontName ) {

   switch ( nCodePage ) {

      case CP_CHINESE_SIMPLIFIED :
         if ( mbsstr_cp ( szFontName, szGbGothic1, CP_CHINESE_SIMPLIFIED ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szGbGothic2, CP_CHINESE_SIMPLIFIED ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szGbGothic3, CP_CHINESE_SIMPLIFIED ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_CHINESE_TRADITIONAL :
      case CP_CHINESE_TRADITIONAL_HK :
         if ( mbsstr_cp ( szFontName, szBig5Gothic1, CP_CHINESE_TRADITIONAL ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szBig5Gothic2, CP_CHINESE_TRADITIONAL ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szBig5Gothic3, CP_CHINESE_TRADITIONAL ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_CW :
      case CP_JAPANESE_CW_FANTI :
         if ( mbsstr_cp ( szFontName, szCwGothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szCw7Gothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_NH :
      case CP_JAPANESE_NHP :
      case CP_JAPANESE_NH3P :
         if ( mbsstr_cp ( szFontName, szNhGothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_CWN :
      case CP_JAPANESE_CWN_FANTI :
         if ( mbsstr_cp ( szFontName, szCwnGothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_ZQ :
         if ( mbsstr_cp ( szFontName, szZqGothic1, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szZqGothic2, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         if ( mbsstr_cp ( szFontName, szZqGothic3, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_CX :
         if ( mbsstr_cp ( szFontName, szCxGothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;

      case CP_JAPANESE_ZJ :
         if ( mbsstr_cp ( szFontName, szZjGothic, CP_JAPANESE ) ) return nCodePage | CP_GOTHIC ;
         break ;
   }

   return nCodePage ;
}



static char *EncodeFontName ( const char *szSrc, char *szDst, int nCodePage ) ;



// tHg̏
static int WriteFontName ( int nFontNumber, const char *szFontName, int nCodePage, int IsGothic, STREAM *StreamOut ) {

   char szBuffer [ 0x200 ] ;
   const char *szFontFamily = "froman" ;

   if ( IsGothic ) szFontFamily = "fmodern" ;
   if ( IsGothic && ! strcmp ( szFontName, "Arial") ) szFontFamily = "fswiss" ;

   EncodeFontName ( szFontName, szBuffer, nCodePage ) ;
   StreamWriteFormat ( StreamOut, szFontNameFormat, nFontNumber, szFontFamily, CodePageToCharset ( nCodePage ), szBuffer ) ;

   return 0 ;
}



// փtHg̏
static int WriteAltFontName ( const char *szFontName, int nCodePage, STREAM *StreamOut ) {

   char szBuffer [ 0x100 ] ;

   EncodeHexaDecimal ( szFontName, szBuffer, nCodePage & CP_DEFINE_CODEPAGE ) ;
   StreamWriteFormat ( StreamOut, "%c\\*\\falt %s%c", BRACE_START, szBuffer, BRACE_END ) ;

   return 0 ;
}



// ̏
static int WriteLangInfo ( int nLang, int nLangNp, int nLangFe, int nLangFeNp, STREAM *StreamOut ) {

   if ( nLang ) StreamWriteFormat ( StreamOut, "\\lang%d", (short) nLang ) ;
   if ( nLangNp ) StreamWriteFormat ( StreamOut, "\\langnp%d", (short) nLangNp ) ;
   if ( nLangFe ) StreamWriteFormat ( StreamOut, "\\langfe%d", (short) nLangFe ) ;
   if ( nLangFeNp ) StreamWriteFormat ( StreamOut, "\\langfenp%d", (short) nLangFeNp ) ;

   if ( nLang || nLangNp || nLangFe || nLangFeNp ) StreamWrite ( 0x20, StreamOut ) ;

   return 0 ;
}



// QoCg̃tHg16i\Lɕϊ
// [JȃtHgɂ͉pꖼt
static char *EncodeFontName ( const char *szSrc, char *szDst, int nCodePage ) {

   char szTmp [ FONTNAME_MAX ] ;
   if ( strlen ( szSrc ) >= FONTNAME_MAX ) {
      strlcpy ( szTmp, szSrc, FONTNAME_MAX ) ;
      szSrc = szTmp ;
   }

   EncodeHexaDecimal ( szSrc, szDst, nCodePage ) ;

   if ( ! IsAsciiString ( szSrc ) ) {
      const char *szEnglishName = GetEnglishFontName ( CodePageToCharset ( nCodePage ), szSrc, NULL ) ;
      if ( szEnglishName ) sprintf ( strend ( szDst ), "%c\\*\\falt %s%c", BRACE_START, szEnglishName, BRACE_END ) ;
   }

   if ( ! strheadcmp ( szSrc, szCw7Head ) || ! strheadcmp ( szSrc, szCw7FantiHead ) ) {
      const char *szLocalizedName = GetLocalizedFontName ( CodePageToCharset ( nCodePage ), szSrc, NULL ) ;
      if ( szLocalizedName ) {
         char szBuffer [ 0x100 ] ;
         EncodeHexaDecimal ( szLocalizedName, szBuffer, nCodePage ) ;
         sprintf ( strend ( szDst ), "%c\\*\\falt %s%c", BRACE_START, szBuffer, BRACE_END ) ;
      }
   }

   return szDst ;
}



static int ConvertFtzToJtz ( int cUnicode, unsigned long dwConversionMode ) {

   if ( ! (
      IsCjk ( cUnicode ) || IsCjkCmpt ( cUnicode ) ||
      ( dwConversionMode & CONVMODE_CONVERT_CJKEX ) && ( IsCjkExA ( cUnicode ) || IsCjkExB ( cUnicode ) )
   ) ) return 0 ;

   int cResult = UniFtzToJtz ( cUnicode ) ;

   if ( cResult && ! ( dwConversionMode & CONVMODE_CONVERT_CJKEX ) ) {
      int cTmp = GetLinshiDaima ( cResult ) ;
      if ( cTmp ) cResult = cTmp ;
   }

   if ( ! ( dwConversionMode & CONVMODE_CONVERT_CJKEX ) ) {
      if ( IsCjkExA ( cResult ) || IsCjkExB ( cResult ) ) cResult = 0 ;
   }

   return cResult ;
}



static int ConvertJtzToFtz ( int cUnicode, unsigned long dwConversionMode ) {

   if ( ! (
      IsCjk ( cUnicode ) || IsCjkCmpt ( cUnicode ) || IsUnicodePua ( cUnicode ) ||
      ( dwConversionMode & CONVMODE_CONVERT_CJKEX ) && ( IsCjkExA ( cUnicode ) || IsCjkExB ( cUnicode ) )
   ) ) return 0 ;

   if ( IsUnicodePua ( cUnicode ) ) {
      int cTmp = GetNonLinshiDaima ( cUnicode ) ;
      if ( cTmp ) cUnicode = cTmp ;
   }

   int cResult = UniJtzToFtz ( cUnicode ) ;

   if ( ! ( dwConversionMode & CONVMODE_CONVERT_CJKEX ) ) {
      if ( IsCjkExA ( cResult ) || IsCjkExB ( cResult ) ) cResult = 0 ;
   }

   return cResult ;
}



// 񂪃AXL[ǂ
static int IsAsciiString ( const char *szString ) {
   while ( *szString ) {
      if ( ! isascii ( *szString ) ) return 0 ;
      szString ++ ;
   }
   return 1 ;
}



////////////////////////////////////////////
//               TM -> RTF                //
////////////////////////////////////////////



// RTFo̓Tu[`
static int WriteFontStart ( int nFontNumber, int nLanguageId, CHNCONVDATA *pData ) ;
static int WriteFontAttribStart ( int nFontNumber, int nLanguageId, int nFontMode, int nSizeMode, int nUnderLineMode, int nColorMode, CHNCONVDATA *pData ) ;
static int WriteFontEnd ( CHNCONVDATA *pData ) ;



// eNmCgt@C RTF ɕϊieLXgt@CŁj
// Ƃ    0      Ԃ
// G[̂Ƃ    1 ȏ Ԃ
static int Tm2rtfTxt ( CHNCONVDATA *pData ) {

   int C ;                       // 
   int nCountChar = 0 ;


   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( CP_JAPANESE_TM, pData->nDstCodePage, 0 ) ;
   if ( ! ConvertChineseCharacter ) return CONV_ERROR_CODEPAGE ;
   // ϊ֐iɑ̎pj
   CONVERTCHARACTER *ConvertChineseCharacterFanti = GetConvertFunction ( CP_JAPANESE_TM, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_SENSE_CW_FANTIZI ) ;
   if ( ! ConvertChineseCharacterFanti ) ConvertChineseCharacterFanti = ConvertChineseCharacter ;
   // ϊ֐iCWj
   CONVERTCHARACTER *ConvertChineseCharacterCw = GetConvertFunction ( CP_JAPANESE_CW, pData->nDstCodePage, 0 ) ;


   // ݒ
   int nChineseLanguageId ;
   switch ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) {
      case CP_CHINESE_SIMPLIFIED :  nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED ) ; break ;
      case CP_CHINESE_TRADITIONAL : nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL ) ; break ;
      case CP_JAPANESE :            nChineseLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ; break ;
      default :                     nChineseLanguageId = MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ) ; break ;
   }


   // eNmCg̕ϊe[u
   const unsigned short *pTm2UniTable [ 3 ] ;
   pTm2UniTable [ 0 ] = GetTm2uniTableEx ( 0 ) ;
   pTm2UniTable [ 1 ] = GetTm2uniTableEx ( 1 ) ;
   pTm2UniTable [ 2 ] = GetTm2uniTableEx ( 2 ) ;


   // ȑȌ
   int nFontNumberPrev = -1 ;       // ȑȌo͂tHg
   int nMbcFontNumberPrev = -1 ;    // ȑȎSp̃tHg


   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_START ) ) WriteFileStart ( pData ) ;

   // CW7
   if ( pData->nDstCodePage == CP_JAPANESE_CW && pData->dwConversionMode & CONVMODE_OUTPUT_NEW_FONT ) {
      pData->tm->nChineseFont             = pData->tm->nCw7Font ;
      pData->tm->nChineseGothicFont       = pData->tm->nCw7GothicFont ;
      pData->tm->nChineseFantiFont        = pData->tm->nCw7FantiFont ;
      pData->tm->nChineseFantiGothicFont  = pData->tm->nCw7FantiGothicFont ;
   }



   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) {

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( pData->nError ) goto CAUGHT_ERROR ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }

      int nTmFontNumber = 0 ;       // ͂ꂽtHgi 0 or 1 or 2 j
      int nFontNumber = 0 ;         // o͂tHg
      int IsHankaku = 0 ;           // SpIɔp

      if ( C == TM_IDENTIFIER ) {
         if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         nTmFontNumber = 1 ;
         if ( C == TM_IDENTIFIER ) {
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            nTmFontNumber = 2 ;
         }
      }

      // QoCg
      if ( ismbblead_sjis ( C ) ) {
         int C2 ;
         if ( ( C2 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         C = ( C << 8 ) | C2 ;
      }

      // s̏o
      if ( C == '\r' || C == '\n' ) {
         nMbcFontNumberPrev = pData->tm->nAsciiFont ;
         StreamWriteFormat ( pData->StreamOut, "\r\n\\%s ", szParagraphTag ) ;
         if ( C == '\r' ) {
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            if ( C != '\n' ) StreamUnread ( C, pData->StreamIn ) ;
         }
         continue ;
      }

      // 䕶E_~[󔒂Ȃ
      if ( isascii ( C ) && iscntrl ( C ) || C == 0xA0 ) continue ;

      int cSaved = C ;
      int cUnicode = 0 ;

      // o͂tHgƕw
      switch ( nTmFontNumber ) {

         default :                           // instead of case 0 :
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 0 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            else if ( C < 0x100 ) {    // pJi
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            else {                     // {
               C = SjisNec2ibm ( C ) ;
               if ( ( C >> 8 ) == 0x85 || ( C >> 8 ) == 0x86 ) {     // NECr
                  if ( ( C >> 8 ) == 0x85 ) C = Nec2sjisTable_85 [ ( C & 0xFF ) - 0x40 ] ;
                  else                      C = Nec2sjisTable_86 [ ( C & 0xFF ) - 0x40 ] ;
                  if ( ! C ) C = 0x20 ;
                  if ( C >= 0x100 && cSaved < 0x86A0 ) IsHankaku = 1 ;
               }
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            break ;

         case 1 :                            // Font Number 1 == "_"
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 1 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nAsciiFont ;
            }
            else if ( C < 0x100 ) {    // pЂ炪ȁiSp֕ϊj
               C = TmHanHiraToZenHira_0080_00FF [ C - 0x80 ] ;
               nFontNumber = pData->tm->nJapaneseFont ;
               if ( C >= 0x100 ) IsHankaku = 1 ;
            }
            else {                     // ȑ̎
               int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
               if ( ! cResult && ! cUnicode ) {
                  cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
               }
               C = cResult ;
               nFontNumber = pData->tm->nChineseFont ;
            }
            break ;

         case 2 :                            // Font Number 2 == "__"
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 2 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nAsciiFont ;
            }
            else if ( C < 0x100 ) {    // sC
               if ( ( C & 0xE0 ) == 0xC0 ) {
                  if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) {
                     C = Tm2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
                     cUnicode = ( isascii ( C ) ) ? C : CwOneByteCharToUnicode_0080_00FF [ C - 0x80 ] ;
                     C = 0 ;
                     nFontNumber = pData->tm->nAsciiFont ;
                  }
                  else {
                     C = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
                     nFontNumber = pData->tm->nChineseFont ;
                  }
               }
               else C = 0 ;
            }
            else {                     // ɑ̎
               int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
               if ( ! cResult && ! cUnicode ) {
                  cResult = ConvertChineseCharacterFanti ( C, pData->dwConversionMode ) ;
               }
               C = cResult ;
               nFontNumber = pData->tm->nChineseFantiFont ;
            }
            break ;
      }


      // v|[Vi̎w
      if ( nFontNumber == pData->tm->nJapaneseFont ) {
         if ( isascii ( C ) ) {
            if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) nFontNumber = pData->tm->nAsciiFont ;
         }
      }

      // ɑǓ_͋IɒtHg
      if ( nFontNumber == pData->tm->nJapaneseFont && ( C >> 8 ) == 0x81 ) {
         if ( nMbcFontNumberPrev == pData->tm->nChineseFont || nMbcFontNumberPrev == pData->tm->nChineseFantiFont ) {
            int cResult ;
            if ( ConvertChineseCharacterCw ) cResult = ConvertChineseCharacterCw ( C, pData->dwConversionMode ) ;
            else                             cResult = C ;  // pData->nDstCodePage == CP_JAPANESE_CW
            if ( cResult ) {
               C = cResult ;
               nFontNumber = nMbcFontNumberPrev ;
            }
         }
      }

      // ͋Iɔɑ̎
      if ( nFontNumber == pData->tm->nChineseFont && cSaved >= 0x84C3 && cSaved <= 0x84E8 ) {
         nFontNumber = pData->tm->nChineseFantiFont ;
      }

      // Chinese Writer ȊO͔ɑ̎ȑ̎
      if ( nFontNumber == pData->tm->nChineseFantiFont && pData->nDstCodePage != CP_JAPANESE_CW ) nFontNumber = pData->tm->nChineseFont ;

      // ̎w
      int nDstLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ;
      if ( nFontNumber == pData->tm->nChineseFont || nFontNumber == pData->tm->nChineseFantiFont ) nDstLanguageId = nChineseLanguageId ;

      // tHgw̏o
      if ( nFontNumberPrev != nFontNumber ) {
         if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
         WriteFontStart ( nFontNumber, nDstLanguageId, pData ) ;
      }

      // ݂̏ۑ
      nFontNumberPrev = nFontNumber ;
      if ( ! isascii ( C ) || cUnicode ) nMbcFontNumberPrev = nFontNumber ;


      // ϊłȂꍇ Unicode ŏo
      if ( ! C ) {
         if ( cUnicode ) {
            if ( IsFourByteLetter ( cUnicode ) ) {
               cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
               StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) ( cUnicode >> 16 ) ) ;
               StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            }
            StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) cUnicode ) ;
            StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            continue ;
         }
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
            StreamWriteFormat ( pData->StreamOut, "[%s:%04X]", GetCodeName ( pData->nSrcCodePage ), cSaved ) ;
            continue ;
         }
         if ( cSaved >= 0x100 && ! ( pData->dwConversionMode & CONVMODE_ONE_QUESTION_MARK ) ) C = 0x3F3F ;  // '??'
         else                                                                                 C = 0x3F ;    // '?'
      }

      // L̏o
      if ( C == ESCAPE || C == BRACE_START || C == BRACE_END ) {
         StreamWriteFormat ( pData->StreamOut, "\\%c", C ) ;
         continue ;
      }

      // QoCg̏o
      if ( C >= 0x100 ) {
         if ( IsHankaku ) StreamWriteFormat ( pData->StreamOut, "%c\\charscalex50 ", BRACE_START ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C >> 8 ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         if ( IsHankaku ) StreamWrite ( BRACE_END, pData->StreamOut ) ;
         continue ;
      }

      // PoCgAXL[̏o
      if ( ! isascii ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         continue ;
      }

      // AXL[̏o
      if ( ! iscntrl ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, "%c", C & 0xFF ) ;
         continue ;
      }
   }


   if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_END ) ) WriteFileEnd ( pData ) ;

   CAUGHT_ERROR :

   return pData->nError ;
}



// eNmCgt@C RTF ɕϊi[vt@CŁj
// Ƃ    0      Ԃ
// G[̂Ƃ    1 ȏ Ԃ
static int Tm2rtfTec ( CHNCONVDATA *pData ) {

   int C, CC ;

   int nCountChar = 0 ;


   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( CP_JAPANESE_TM, pData->nDstCodePage, 0 ) ;
   if ( ! ConvertChineseCharacter ) return CONV_ERROR_CODEPAGE ;
   // ϊ֐iɑ̎pj
   CONVERTCHARACTER *ConvertChineseCharacterFanti = GetConvertFunction ( CP_JAPANESE_TM, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_SENSE_CW_FANTIZI ) ;
   if ( ! ConvertChineseCharacterFanti ) ConvertChineseCharacterFanti = ConvertChineseCharacter ;
   // ϊ֐iCWj
   CONVERTCHARACTER *ConvertChineseCharacterCw = GetConvertFunction ( CP_JAPANESE_CW, pData->nDstCodePage, 0 ) ;


   // ݒ
   int nChineseLanguageId ;
   switch ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) {
      case CP_CHINESE_SIMPLIFIED :  nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED ) ; break ;
      case CP_CHINESE_TRADITIONAL : nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL ) ; break ;
      case CP_JAPANESE :            nChineseLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ; break ;
      default :                     nChineseLanguageId = MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ) ; break ;
   }

   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_START ) ) WriteFileStart ( pData ) ;

   // CW7
   if ( pData->nDstCodePage == CP_JAPANESE_CW && pData->dwConversionMode & CONVMODE_OUTPUT_NEW_FONT ) {
      pData->tm->nChineseFont = pData->tm->nCw7Font ;
      pData->tm->nChineseGothicFont = pData->tm->nCw7GothicFont ;
      pData->tm->nChineseFantiFont = pData->tm->nCw7FantiFont ;
      pData->tm->nChineseFantiGothicFont = pData->tm->nCw7FantiGothicFont ;
   }


   // eNmCg̕ϊe[u
   const unsigned short *pTm2UniTable [ 3 ] ;
   pTm2UniTable [ 0 ] = GetTm2uniTableEx ( 0 ) ;
   pTm2UniTable [ 1 ] = GetTm2uniTableEx ( 1 ) ;
   pTm2UniTable [ 2 ] = GetTm2uniTableEx ( 2 ) ;


   // t@Cݒ𑗂
   for ( int nCount = 0 ; nCount < 0x35D ; nCount ++ ) { // 0x35D = tHgJn|Cg
      if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) return CONV_ERROR_FORMAT ;
      if ( nCount > 10 && C == 0xFE ) goto ENCOUNTERED_FE ;
   }
   // tHg𓾂
   for ( int nLoop = 0 ; nLoop < 3 ; nLoop ++ ) {
      char pTmFontName [ 0x10 ] = { 0 } ;
      for ( int nCount = 0 ; nCount < 8 ; nCount ++ ) {     // 0x35D = tHgJn|Cg
         if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) return CONV_ERROR_FORMAT ;
         if ( C == 0xFE ) goto ENCOUNTERED_FE ;
         pTmFontName [ nCount ] = C ;
      }
      pTm2UniTable [ nLoop ] = GetTm2uniTable ( pTmFontName ) ;
   }


   // t@Cݒ𑗂
   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) if ( C == 0xFE ) break ;
   ENCOUNTERED_FE :
   StreamUnread ( C, pData->StreamIn ) ;


   // ȑȌ
   int nFontNumberPrev = -1 ;       // ȑȌo͂tHg
   int nMbcFontNumberPrev = -1 ;    // ȑȎSp̃tHg
   int nFontModePrev = -1 ;         // ȑȎ
   int nSizeModePrev = -1 ;         // ȑÕTCY
   int nUnderLineModePrev = -1 ;    // ȑÕA_[C
   int nColorModePrev = -1 ;        // ȑO̐F
   int IsKeisenLine = 0 ;



   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) {

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( pData->nError ) goto CAUGHT_ERROR ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }

      int nTmFontNumber = 0 ;       // ͂ꂽtHgi 0 or 1 or 2 j
      int nFontNumber = 0 ;         // o͂tHg
      int nFontMode = 0 ;           // 
      int nSizeMode = 0 ;           // TCY
      int nUnderLineMode = 0 ;      // A_[C
      int nColorMode = 0 ;          // F
      int IsHankaku = 0 ;           // SpIɔp

      struct TMLINEINFO TmLineInfo ;

      // 䕶̉
      switch ( C ) {

         case 0xFE :
            StreamUnread ( C, pData->StreamIn ) ;
            if ( StreamReadBuffer ( (char*) & TmLineInfo, sizeof(TmLineInfo), pData->StreamIn ) < sizeof(TmLineInfo) ) break ;

            // Rgs폜
            if ( TmLineInfo.Attrib2 == 0x2A && ! ( pData->dwAdditionalMode & ADDITIONAL_MODE_WRITE_TM_COMMENT ) ) {
               for ( int i = 0 ; i < TmLineInfo.Length - 3 ; i ++ ) StreamRead ( pData->StreamIn ) ;
               continue ;
            }

            // rs
            if ( IsKeisenLine ) {
               StreamWriteFormat ( pData->StreamOut, "\r\n\\%s ", szParagraphTag ) ;
               IsKeisenLine = 0 ;
            }
            if ( TmLineInfo.Attrib1 == 1 ) IsKeisenLine = 1 ;

            // }[W폜
            for ( int nCount = TmLineInfo.LeftMargin - 4 ; nCount ; nCount -- ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               if ( C == 0xA0 || C == 0x20 ) continue ;
               StreamUnread ( C, pData->StreamIn ) ;
               break ;
            }

            continue ;

         case 0x1B :
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;

            if ( C & 3 ) {
               if      ( ( C & 3 ) == 1 ) nFontMode = TM_FONT_ITALIC ;
               else if ( ( C & 3 ) == 2 ) nFontMode = TM_FONT_BOLD ;
               else if ( ( C & 3 ) == 3 ) nFontMode = TM_FONT_COURIER ;
            }

            if      ( C & 4 ) nTmFontNumber = 1 ;
            else if ( C & 8 ) nTmFontNumber = 2 ;

            if ( C & 0x80 ) nSizeMode = TM_SIZE_LARGE ;

            break ;

         case 0x1D :
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;

            if ( C & 3 ) {
               if      ( ( C & 3 ) == 1 ) nSizeMode = TM_SIZE_SUB ;
               else if ( ( C & 3 ) == 2 ) nSizeMode = TM_SIZE_LARGE ;
               else if ( ( C & 3 ) == 3 ) nSizeMode = TM_SIZE_SUPER ;
            }

            if ( ( C & 4 ) && ! nSizeMode ) nFontMode |= TM_FONT_BOLD ;

            if ( C & 0xF0 ) {
               if ( ( C & 0xF0 ) == 0x10 )      nUnderLineMode = TM_ATTRIB_UNDERLINE ;
               else if ( ( C & 0xF0 ) == 0x20 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_DOTTED ;
               else if ( ( C & 0xF0 ) == 0x30 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_THICK ;
               else if ( ( C & 0xF0 ) == 0x40 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_DOUBLE ;
               else if ( ( C & 0xF0 ) == 0x50 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_WAVE ;
               else if ( ( C & 0xF0 ) == 0x60 ) nUnderLineMode = TM_ATTRIB_SHADING_1 ;
               else if ( ( C & 0xF0 ) == 0x70 ) nUnderLineMode = TM_ATTRIB_SHADING_2 ;
               else if ( ( C & 0xF0 ) == 0x80 ) nUnderLineMode = TM_ATTRIB_SHADING_3 ;
               else if ( ( C & 0xF0 ) == 0x90 ) nUnderLineMode = TM_ATTRIB_BORDER ;
               else if ( ( C & 0xF0 ) == 0xA0 ) nUnderLineMode = TM_ATTRIB_STROKE ;
               else if ( ( C & 0xF0 ) == 0xB0 ) nUnderLineMode = TM_ATTRIB_STROKE_DOUBLE ;
            }

            if ( ( C & 0xF0 ) == 0xE0 ) {
               if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
               C = TmKeisenToSjis [ C ] ;
               StreamUnread ( C, pData->StreamIn ) ;
               nTmFontNumber = 0 ;
               nSizeMode = nFontMode = nUnderLineMode = 0 ;
               break ;
            }

            break ;

         case 0x19 :
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            if ( ( CC = StreamRead ( pData->StreamIn ) ) == EOF ) break ;

            if ( C & 3 ) {
               if      ( ( C & 3 ) == 1 ) nFontMode = TM_FONT_ITALIC ;
               else if ( ( C & 3 ) == 2 ) nFontMode = TM_FONT_BOLD ;
               else if ( ( C & 3 ) == 3 ) nFontMode = TM_FONT_COURIER ;
            }

            if      ( C & 4 ) nTmFontNumber = 1 ;
            else if ( C & 8 ) nTmFontNumber = 2 ;

            if ( C & 0x80 ) nSizeMode = TM_SIZE_LARGE ;

            if ( CC & 3 ) {
               if      ( ( CC & 3 ) == 1 ) nSizeMode = TM_SIZE_SUB ;
               else if ( ( CC & 3 ) == 2 ) nSizeMode = TM_SIZE_LARGE ;
               else if ( ( CC & 3 ) == 3 ) nSizeMode = TM_SIZE_SUPER ;
            }
            if ( ( CC & 4 ) && ! nSizeMode ) nFontMode |= TM_FONT_BOLD ;

            if ( CC & 0xF0 ) {
               if ( ( CC & 0xF0 ) == 0x10 )      nUnderLineMode = TM_ATTRIB_UNDERLINE ;
               else if ( ( CC & 0xF0 ) == 0x20 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_DOTTED ;
               else if ( ( CC & 0xF0 ) == 0x30 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_THICK ;
               else if ( ( CC & 0xF0 ) == 0x40 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_DOUBLE ;
               else if ( ( CC & 0xF0 ) == 0x50 ) nUnderLineMode = TM_ATTRIB_UNDERLINE_WAVE ;
               else if ( ( CC & 0xF0 ) == 0x60 ) nUnderLineMode = TM_ATTRIB_SHADING_1 ;
               else if ( ( CC & 0xF0 ) == 0x70 ) nUnderLineMode = TM_ATTRIB_SHADING_2 ;
               else if ( ( CC & 0xF0 ) == 0x80 ) nUnderLineMode = TM_ATTRIB_SHADING_3 ;
               else if ( ( CC & 0xF0 ) == 0x90 ) nUnderLineMode = TM_ATTRIB_BORDER ;
               else if ( ( CC & 0xF0 ) == 0xA0 ) nUnderLineMode = TM_ATTRIB_STROKE ;
               else if ( ( CC & 0xF0 ) == 0xB0 ) nUnderLineMode = TM_ATTRIB_STROKE_DOUBLE ;
            }

            if ( C & 0x20 ) nColorMode += TM_COLOR_BLUE ;
            if ( C & 0x40 ) nColorMode += TM_COLOR_RED ;
            if ( ( CC & 0xF0 ) == 0xF0 ) {
               nColorMode += TM_COLOR_GREED ;
               nColorMode -= TM_COLOR_BLUE ;
            }

            break ;

         default :
            StreamUnread ( C, pData->StreamIn ) ;
            break ;
      }

      if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;

      // QoCg
      if ( ismbblead_sjis ( C ) ) {
         int C2 ;
         if ( ( C2 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         C = ( C << 8 ) | C2 ;
      }

      // s̏o
      if ( C == '\r' ) {
         nMbcFontNumberPrev = pData->tm->nAsciiFont ;
         StreamWriteFormat ( pData->StreamOut, "\r\n\\%s ", szParagraphTag ) ;
         continue ;
      }

      // 䕶E_~[󔒂Ȃ
      if ( isascii ( C ) && iscntrl ( C ) || C == 0xA0 || C == 0xFF ) continue ;

      int cSaved = C ;
      int cUnicode = 0 ;

      // o͂tHgƕw
      switch ( nTmFontNumber ) {

         default :                           // instead of case 0 :
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 0 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            else if ( C < 0x100 ) {    // pJi
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            else {                     // {
               C = SjisNec2ibm ( C ) ;
               if ( ( C >> 8 ) == 0x85 || ( C >> 8 ) == 0x86 ) {     // NECr
                  if ( ( C >> 8 ) == 0x85 ) C = Nec2sjisTable_85 [ ( C & 0xFF ) - 0x40 ] ;
                  else                      C = Nec2sjisTable_86 [ ( C & 0xFF ) - 0x40 ] ;
                  if ( ! C ) C = 0x20 ;
                  if ( C >= 0x100 && cSaved < 0x86A0 ) IsHankaku = 1 ;
               }
               nFontNumber = pData->tm->nJapaneseFont ;
            }
            break ;

         case 1 :                            // Font Number 1 == "_"
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 1 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nAsciiFont ;
            }
            else if ( C < 0x100 ) {    // pЂ炪ȁiSp֕ϊj
               C = TmHanHiraToZenHira_0080_00FF [ C - 0x80 ] ;
               nFontNumber = pData->tm->nJapaneseFont ;
               if ( C >= 0x100 ) IsHankaku = 1 ;
            }
            else {                     // ȑ̎
               int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
               if ( ! cResult && ! cUnicode ) {
                  cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
               }
               C = cResult ;
               nFontNumber = pData->tm->nChineseFont ;
            }
            break ;

         case 2 :                            // Font Number 2 == "__"
            if ( C <= 0x80 ) {
               cUnicode = pTm2UniTable [ 2 ] [ C - 0x20 ] ;
               C = 0 ;
               if ( cUnicode < 0x80 ) {
                  C = cUnicode ;
                  cUnicode = 0 ;
               }
               nFontNumber = pData->tm->nAsciiFont ;
            }
            else if ( C < 0x100 ) {    // sC
               if ( ( C & 0xE0 ) == 0xC0 ) {
                  if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) {
                     C = Tm2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
                     cUnicode = ( isascii ( C ) ) ? C : CwOneByteCharToUnicode_0080_00FF [ C - 0x80 ] ;
                     C = 0 ;
                     nFontNumber = pData->tm->nAsciiFont ;
                  }
                  else {
                     C = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
                     nFontNumber = pData->tm->nChineseFont ;
                  }
               }
               else C = 0 ;
            }
            else {                     // ɑ̎
               int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
               if ( ! cResult && ! cUnicode ) {
                  cResult = ConvertChineseCharacterFanti ( C, pData->dwConversionMode ) ;
               }
               C = cResult ;
               nFontNumber = pData->tm->nChineseFantiFont ;
            }
            break ;
      }


      // v|[Vi̎w
      if ( nFontNumber == pData->tm->nJapaneseFont ) {
         if ( isascii ( C ) ) {
            if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) nFontNumber = pData->tm->nAsciiFont ;
         }
      }

      // ɑǓ_͋IɒtHg
      if ( nFontNumber == pData->tm->nJapaneseFont && ( C >> 8 ) == 0x81 ) {
         if ( nMbcFontNumberPrev == pData->tm->nChineseFont || nMbcFontNumberPrev == pData->tm->nChineseFantiFont ) {
            int cResult ;
            if ( ConvertChineseCharacterCw ) cResult = ConvertChineseCharacterCw ( C, pData->dwConversionMode ) ;
            else                             cResult = C ;  // pData->nDstCodePage == CP_JAPANESE_CW
            if ( cResult ) {
               C = cResult ;
               nFontNumber = nMbcFontNumberPrev ;
            }
         }
      }

      // ͋Iɔɑ̎
      if ( nFontNumber == pData->tm->nChineseFont && cSaved >= 0x84C3 && cSaved <= 0x84E8 ) {
         nFontNumber = pData->tm->nChineseFantiFont ;
      }

      // Chinese Writer ȊO͔ɑ̎ȑ̎
      if ( nFontNumber == pData->tm->nChineseFantiFont && pData->nDstCodePage != CP_JAPANESE_CW ) nFontNumber = pData->tm->nChineseFont ;

      // COURIER  Courier New 
      if ( nFontMode == TM_FONT_COURIER ) {
         if ( nFontNumber == pData->tm->nAsciiFont ) nFontNumber = pData->tm->nCourierFont ;
         nFontMode &= ~ TM_FONT_COURIER ;
      }

      // {[h̓SVbN̂
      if ( nFontMode & TM_FONT_BOLD ) {
         if ( nFontNumber == pData->tm->nAsciiFont ) {
            nFontNumber = pData->tm->nAsciiGothicFont ;
            nFontMode &= ~ TM_FONT_BOLD ;
         }
         else if ( nFontNumber == pData->tm->nJapaneseFont ) {
            nFontNumber = pData->tm->nJapaneseGothicFont ;
            nFontMode &= ~ TM_FONT_BOLD ;
         }
         else if ( nFontNumber == pData->tm->nChineseFont ) {
            nFontNumber = pData->tm->nChineseGothicFont ;
            nFontMode &= ~ TM_FONT_BOLD ;
         }
         else if ( nFontNumber == pData->tm->nChineseFantiFont ) {
            nFontNumber = pData->tm->nChineseFantiGothicFont ;
            nFontMode &= ~ TM_FONT_BOLD ;
         }
      }

      // ̎w
      int nDstLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ;
      if ( nFontNumber == pData->tm->nChineseFont || nFontNumber == pData->tm->nChineseFantiFont ) nDstLanguageId = nChineseLanguageId ;
      if ( nFontNumber == pData->tm->nChineseGothicFont || nFontNumber == pData->tm->nChineseFantiGothicFont ) nDstLanguageId = nChineseLanguageId ;

      // tHgw̏o
      if ( nFontNumberPrev != nFontNumber || nFontModePrev != nFontMode || nSizeModePrev != nSizeMode || nUnderLineModePrev != nUnderLineMode || nColorModePrev != nColorMode ) {
         if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
         WriteFontAttribStart ( nFontNumber, nDstLanguageId, nFontMode, nSizeMode, nUnderLineMode, nColorMode, pData ) ;
      }

      // ݂̏ۑ
      nFontNumberPrev = nFontNumber ;
      if ( ! isascii ( C ) || cUnicode ) nMbcFontNumberPrev = nFontNumber ;
      nFontModePrev = nFontMode ;
      nSizeModePrev = nSizeMode ;
      nUnderLineModePrev = nUnderLineMode ;
      nColorModePrev = nColorMode ;


      // ϊłȂꍇ Unicode ŏo
      if ( ! C ) {
         if ( cUnicode ) {
            if ( IsFourByteLetter ( cUnicode ) ) {
               cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
               StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) ( cUnicode >> 16 ) ) ;
               StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            }
            StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) cUnicode ) ;
            StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            continue ;
         }
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
            StreamWriteFormat ( pData->StreamOut, "[%s:%04X]", GetCodeName ( pData->nSrcCodePage ), cSaved ) ;
            continue ;
         }
         if ( cSaved >= 0x100 && ! ( pData->dwConversionMode & CONVMODE_ONE_QUESTION_MARK ) ) C = 0x3F3F ;  // '??'
         else                                                                                 C = 0x3F ;    // '?'
      }

      // L̏o
      if ( C == ESCAPE || C == BRACE_START || C == BRACE_END ) {
         StreamWriteFormat ( pData->StreamOut, "\\%c", C ) ;
         continue ;
      }

      // QoCg̏o
      if ( C >= 0x100 ) {
         if ( IsHankaku ) StreamWriteFormat ( pData->StreamOut, "%c\\charscalex50 ", BRACE_START ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C >> 8 ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         if ( IsHankaku ) StreamWrite ( BRACE_END, pData->StreamOut ) ;
         continue ;
      }

      // PoCgAXL[̏o
      if ( ! isascii ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         continue ;
      }

      // AXL[̏o
      if ( ! iscntrl ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, "%c", C & 0xFF ) ;
         continue ;
      }
   }


   if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_END ) ) WriteFileEnd ( pData ) ;

   CAUGHT_ERROR :

   return pData->nError ;
}



////////////////////////////////////////////
//           WRITE RTF FILE               //
////////////////////////////////////////////



static int WriteFontTable ( CHNCONVDATA *pData ) ;



static int WriteFileStart ( CHNCONVDATA *pData ) {

   StreamWriteString ( szRtfStartString, pData->StreamOut ) ;
   WriteFontTable ( pData ) ;                               // tHge[ȕo
   StreamWriteString ( szColorTable, pData->StreamOut ) ;   // Fe[ȕo

   return 0 ;
}



static int WriteFileEnd ( CHNCONVDATA *pData ) {

   StreamWriteString ( szRtfEndString, pData->StreamOut ) ;

   return 0 ;
}



// tHgo
static int WriteFontTable ( CHNCONVDATA *pData ) {

   int nDstCodePage = pData->nDstCodePage ;
   STREAM *StreamOut = pData->StreamOut ;

   const char *szDstFont, *szDstGothicFont ;

   // tHg
   InitRtfFontName ( & szDstFont, & szDstGothicFont, pData ) ;

   StreamWrite ( BRACE_START, StreamOut ) ;
   StreamWriteString ( "\\fonttbl", StreamOut ) ;

   // ASCII
   WriteFontName ( pData->tm->nAsciiFont, pData->szPinyinFont, CP_ANSI_EUROPEAN, 0, StreamOut ) ;
   // ASCII (GOTHIC)
   if ( pData->tm->nAsciiFont != pData->tm->nAsciiGothicFont ) {
      WriteFontName ( pData->tm->nAsciiGothicFont, pData->szPinyinGothicFont, CP_ANSI_EUROPEAN, 1, StreamOut ) ;
   }

   // Courier
   WriteFontName ( pData->tm->nCourierFont, szCourierFont, CP_ANSI_EUROPEAN, 1, StreamOut ) ;

   // JAPANESE
   WriteFontName ( pData->tm->nJapaneseFont, szJapaneseFont, CP_JAPANESE, 0, StreamOut ) ;
   // JAPANESE (GOTHIC)
   if ( pData->tm->nJapaneseFont != pData->tm->nJapaneseGothicFont ) {
      WriteFontName ( pData->tm->nJapaneseGothicFont, szJapaneseGothicFont, CP_JAPANESE, 1, StreamOut ) ;
   }

   // CHINESE
   WriteFontName ( pData->tm->nChineseFont, szDstFont, nDstCodePage & CP_DEFINE_CODEPAGE, 0, StreamOut ) ;
   // CHINESE (GOTHIC)
   if ( pData->tm->nChineseFont != pData->tm->nChineseGothicFont ) {
      WriteFontName ( pData->tm->nChineseGothicFont, szDstGothicFont, nDstCodePage & CP_DEFINE_CODEPAGE, 1, StreamOut ) ;
   }

   if ( nDstCodePage == CP_JAPANESE_CW ) {
      // CHINESE FANTI
      WriteFontName ( pData->tm->nChineseFantiFont, szKgFont, CP_JAPANESE, 0, StreamOut ) ;
      // CHINESE FANTI (GOTHIC)
      WriteFontName ( pData->tm->nChineseFantiGothicFont, szKgGothicFont, CP_JAPANESE, 1, StreamOut ) ;
      // CW7
      WriteFontName ( pData->tm->nCw7Font, szCw7Font, CP_JAPANESE, 0, StreamOut ) ;
      WriteFontName ( pData->tm->nCw7GothicFont, szCw7GothicFont, CP_JAPANESE, 0, StreamOut ) ;
      WriteFontName ( pData->tm->nCw7FantiFont, szKg7Font, CP_JAPANESE, 0, StreamOut ) ;
      WriteFontName ( pData->tm->nCw7FantiGothicFont, szKg7GothicFont, CP_JAPANESE, 0, StreamOut ) ;
   }

   StreamWrite ( BRACE_END, StreamOut ) ;
   StreamWriteString ( "\r\n", StreamOut ) ;

   return 0 ;
}



static int WriteFontStart ( int nFontNumber, int nLanguageId, CHNCONVDATA *pData ) {

   StreamWrite ( BRACE_START, pData->StreamOut ) ;
   StreamWriteFormat ( pData->StreamOut, "\\fs%d\\f%d", (short) pData->tm->nFontSize, (short) nFontNumber ) ;
   StreamWriteFormat ( pData->StreamOut, "\\lang%d\\langfe%d\\langfenp%d", (short) MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ), (short) nLanguageId, (short) nLanguageId ) ;
   StreamWriteString ( " ", pData->StreamOut ) ;

   return 0 ;
}



static int WriteFontAttribStart ( int nFontNumber, int nLanguageId, int nFontMode, int nSizeMode, int nUnderLineMode, int nColorMode, CHNCONVDATA *pData ) {

   int nFontSizeTmp ;

   nFontSizeTmp = pData->tm->nFontSize ;
   if ( nSizeMode == TM_SIZE_LARGE ) {
      nFontSizeTmp = ( pData->tm->nFontSize * 3 ) / 2 ;
      nSizeMode = 0 ;
   }

   StreamWrite ( BRACE_START, pData->StreamOut ) ;
   StreamWriteFormat ( pData->StreamOut, "\\fs%d\\f%d", (short) nFontSizeTmp, (short) nFontNumber ) ;
   StreamWriteFormat ( pData->StreamOut, "\\lang%d\\langfe%d\\langfenp%d", (short) MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ), (short) nLanguageId, (short) nLanguageId ) ;
   StreamWriteFormat ( pData->StreamOut, "%s%s%s\\cf%d", FontModeArray [ nFontMode ], SizeModeArray [ nSizeMode ], UnderLineModeArray [ nUnderLineMode ], (short) nColorMode ) ;
   StreamWriteString ( " ", pData->StreamOut ) ;

   return 0 ;
}



static int WriteFontEnd ( CHNCONVDATA *pData ) {
   StreamWrite ( BRACE_END, pData->StreamOut ) ;
   return 0 ;
}



// tHgԍ
static int InitFontNumber ( CHNCONVDATA *pData ) {

   int nDstCodePage = pData->nDstCodePage ;

   int nFontNumber ;
   const char *szDstFont, *szDstGothicFont ;

   // tHg
   InitRtfFontName ( & szDstFont, & szDstGothicFont, pData ) ;

   // ASCII
   pData->tm->nAsciiFont = nFontNumber = 0 ;
   // ASCII (GOTHIC)
   if ( ! strcmp ( pData->szPinyinFont, pData->szPinyinGothicFont ) ) pData->tm->nAsciiGothicFont = nFontNumber ;
   else                                                               pData->tm->nAsciiGothicFont = ++ nFontNumber ;

   // Courier
   pData->tm->nCourierFont = ++ nFontNumber ;

   // JAPANESE
   pData->tm->nJapaneseFont = ++ nFontNumber ;
   // JAPANESE (GOTHIC)
   if ( ! strcmp ( szJapaneseFont, szJapaneseGothicFont ) ) pData->tm->nJapaneseGothicFont = nFontNumber ;
   else                                                     pData->tm->nJapaneseGothicFont = ++ nFontNumber ;

   // CHINESE
   pData->tm->nChineseFont = ++ nFontNumber ;
   // CHINESE (GOTHIC)
   if ( ! strcmp ( szDstFont, szDstGothicFont ) ) pData->tm->nChineseGothicFont = nFontNumber ;
   else                                           pData->tm->nChineseGothicFont = ++ nFontNumber ;

   if ( nDstCodePage == CP_JAPANESE_CW ) {
      // CHINESE FANTI
      pData->tm->nChineseFantiFont = ++ nFontNumber ;
      // CHINESE FANTI (GOTHIC)
      pData->tm->nChineseFantiGothicFont = ++ nFontNumber ;
      // CW7
      pData->tm->nCw7Font = ++ nFontNumber ;
      pData->tm->nCw7GothicFont = ++ nFontNumber ;
      pData->tm->nCw7FantiFont = ++ nFontNumber ;
      pData->tm->nCw7FantiGothicFont = ++ nFontNumber ;
   }
   else {
      pData->tm->nChineseFantiFont = pData->tm->nChineseFont ;
      pData->tm->nChineseFantiGothicFont = pData->tm->nChineseGothicFont ;
   }

   return 0 ;
}



////////////////////////////////////////////
//              FWEN -> RTF               //
////////////////////////////////////////////



static int fwen2tm ( int C ) ;
static int fwen_pinyin2tm ( int C ) ;
static int ismbblead_fwen ( int C ) ;
static int ismbbtrail_fwen ( int C ) ;



// tg؂̃eLXgt@C RTF ɕϊ
// Ƃ    0      Ԃ
// G[̂Ƃ    1 ȏ Ԃ
static int Fwen2rtf ( CHNCONVDATA *pData ) {

   int C ;                       // 

   int nCountChar = 0 ;


   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( CP_JAPANESE_TM, pData->nDstCodePage, 0 ) ;
   if ( ! ConvertChineseCharacter ) return CONV_ERROR_CODEPAGE ;
   // ϊ֐iCWj
   CONVERTCHARACTER *ConvertChineseCharacterCw = GetConvertFunction ( CP_JAPANESE_CW, pData->nDstCodePage, 0 ) ;


   // ݒ
   int nChineseLanguageId ;
   switch ( pData->nDstCodePage & CP_DEFINE_CODEPAGE ) {
      case CP_CHINESE_SIMPLIFIED :  nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED ) ; break ;
      case CP_CHINESE_TRADITIONAL : nChineseLanguageId = MAKELANGID ( LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL ) ; break ;
      case CP_JAPANESE :            nChineseLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ; break ;
      default :                     nChineseLanguageId = MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US ) ; break ;
   }


   // ȑȌ
   int nFontNumberPrev = -1 ;       // ȑȌo͂tHg
   int nMbcFontNumberPrev = -1 ;    // ȑȎSp̃tHg


   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_START ) ) WriteFileStart ( pData ) ;

   // CW7
   if ( pData->nDstCodePage == CP_JAPANESE_CW && pData->dwConversionMode & CONVMODE_OUTPUT_NEW_FONT ) {
      pData->tm->nChineseFont             = pData->tm->nCw7Font ;
      pData->tm->nChineseGothicFont       = pData->tm->nCw7GothicFont ;
      pData->tm->nChineseFantiFont        = pData->tm->nCw7FantiFont ;
      pData->tm->nChineseFantiGothicFont  = pData->tm->nCw7FantiGothicFont ;
   }



   while ( ( C = StreamRead ( pData->StreamIn ) ) != EOF ) {

      // bZ[W
      if ( ! ( nCountChar & PEEK_MESSAGE_CONUT_MASK ) ) {
         if ( pData->dwConversionMode & CONVMODE_PEEKMESSAGE ) PeekMessageProc ( pData->hDialog, pData->IsMessageUnicode ) ;
         if ( pData->nError ) goto CAUGHT_ERROR ;
         if ( StreamError ( pData->StreamIn ) ) { pData->nError = CONV_ERROR_READ ; goto CAUGHT_ERROR ; }
         if ( StreamError ( pData->StreamOut ) ) { pData->nError = CONV_ERROR_WRITE ; goto CAUGHT_ERROR ; }
      }
      nCountChar ++ ;

      // I
      if ( IsAborted ( pData ) ) { pData->nError = CONV_ERROR_ABORTED ; goto CAUGHT_ERROR ; }

      int nFontNumber = 0 ;         // o͂tHg
      int IsHankaku = 0 ;           // SpIɔp

      if ( ismbblead_fwen ( C ) ) {
         int C2 ;
         if ( ( C2 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         C = ( C << 8 ) | C2 ;
      }
      else if ( ismbblead_sjis ( C ) ) {
         int C2 ;
         if ( ( C2 = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
         C = ( C << 8 ) | C2 ;
      }


      // s̏o
      if ( C == '\r' || C == '\n' ) {
         nMbcFontNumberPrev = pData->tm->nAsciiFont ;
         StreamWriteFormat ( pData->StreamOut, "\r\n\\%s ", szParagraphTag ) ;
         if ( C == '\r' ) {
            if ( ( C = StreamRead ( pData->StreamIn ) ) == EOF ) break ;
            if ( C != '\n' ) StreamUnread ( C, pData->StreamIn ) ;
         }
         continue ;
      }

      // 䕶Ȃ
      if ( isascii ( C ) && iscntrl ( C ) ) continue ;

      int cSaved = C ;
      int cUnicode = 0 ;


      // 
      if ( ismbblead_fwen ( C >> 8 ) ) {
         int cResult = ConvertByUserDefinition ( C, & cUnicode, pData ) ;
         if ( ! cResult && ! cUnicode ) {
            C = fwen2tm ( C ) ;
            cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
         }
         C = cResult ;
         nFontNumber = pData->tm->nChineseFont ;

      }
      else {
         int cResult = fwen_pinyin2tm ( C ) ;
         // sC
         if ( cResult ) {
            C = cResult ;
            if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) {
               C = Tm2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
               cUnicode = ( isascii ( C ) ) ? C : CwOneByteCharToUnicode_0080_00FF [ C - 0x80 ] ;
               C = 0 ;
               nFontNumber = pData->tm->nAsciiFont ;
            }
            else {
               C = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
               nFontNumber = pData->tm->nChineseFont ;
            }
         }
         // {
         else {
            C = SjisNec2ibm ( C ) ;
            if ( ( C >> 8 ) == 0x85 || ( C >> 8 ) == 0x86 ) {     // NECr
               if ( ( C >> 8 ) == 0x85 ) C = Nec2sjisTable_85 [ ( C & 0xFF ) - 0x40 ] ;
               else                      C = Nec2sjisTable_86 [ ( C & 0xFF ) - 0x40 ] ;
               if ( ! C ) C = 0x20 ;
               if ( C >= 0x100 && cSaved < 0x86A0 ) IsHankaku = 1 ;
            }
            nFontNumber = pData->tm->nJapaneseFont ;
         }
      }


      // v|[Vi̎w
      if ( nFontNumber == pData->tm->nJapaneseFont ) {
         if ( isascii ( C ) ) {
            if ( pData->dwConversionMode & CONVMODE_PINYIN_BY_ASCII ) nFontNumber = pData->tm->nAsciiFont ;
         }
      }

      // ɑǓ_͋IɒtHg
      if ( nFontNumber == pData->tm->nJapaneseFont && ( C >> 8 ) == 0x81 ) {
         if ( nMbcFontNumberPrev == pData->tm->nChineseFont ) {
            int cResult ;
            if ( ConvertChineseCharacterCw ) cResult = ConvertChineseCharacterCw ( C, pData->dwConversionMode ) ;
            else                             cResult = C ;  // pData->nDstCodePage == CP_JAPANESE_CW
            if ( cResult ) {
               C = cResult ;
               nFontNumber = nMbcFontNumberPrev ;
            }
         }
      }

      // ̎w
      int nDstLanguageId = MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT ) ;
      if ( nFontNumber == pData->tm->nChineseFont ) nDstLanguageId = nChineseLanguageId ;

      // tHgw̏o
      if ( nFontNumberPrev != nFontNumber ) {
         if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
         WriteFontStart ( nFontNumber, nDstLanguageId, pData ) ;
      }

      // ݂̏ۑ
      nFontNumberPrev = nFontNumber ;
      if ( ! isascii ( C ) || cUnicode ) nMbcFontNumberPrev = nFontNumber ;


      // ϊłȂꍇ Unicode ŏo
      if ( ! C ) {
         if ( cUnicode ) {
            if ( IsFourByteLetter ( cUnicode ) ) {
               cUnicode = MAKE_UNICODE_SURROGATE ( cUnicode ) ;
               StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) ( cUnicode >> 16 ) ) ;
               StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            }
            StreamWriteFormat ( pData->StreamOut, "\\uc1\\u%d", (short) cUnicode ) ;
            StreamWriteFormat ( pData->StreamOut, szCharFormat, '?' ) ;
            continue ;
         }
         if ( pData->dwConversionMode & CONVMODE_OUTPUT_CODE_NUMBER ) {
            StreamWriteFormat ( pData->StreamOut, "[%s:%04X]", GetCodeName ( pData->nSrcCodePage ), cSaved ) ;
            continue ;
         }
         if ( cSaved >= 0x100 && ! ( pData->dwConversionMode & CONVMODE_ONE_QUESTION_MARK ) ) C = 0x3F3F ;  // '??'
         else                                                                                 C = 0x3F ;    // '?'
      }

      // L̏o
      if ( C == ESCAPE || C == BRACE_START || C == BRACE_END ) {
         StreamWriteFormat ( pData->StreamOut, "\\%c", C ) ;
         continue ;
      }

      // QoCg̏o
      if ( C >= 0x100 ) {
         if ( IsHankaku ) StreamWriteFormat ( pData->StreamOut, "%c\\charscalex50 ", BRACE_START ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C >> 8 ) ;
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         if ( IsHankaku ) StreamWrite ( BRACE_END, pData->StreamOut ) ;
         continue ;
      }

      // PoCgAXL[̏o
      if ( ! isascii ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, szCharFormat, C & 0xFF ) ;
         continue ;
      }

      // AXL[̏o
      if ( ! iscntrl ( C ) ) {
         StreamWriteFormat ( pData->StreamOut, "%c", C & 0xFF ) ;
         continue ;
      }
   }


   if ( nFontNumberPrev >= 0 ) WriteFontEnd ( pData ) ;
   if ( ! ( pData->dwReserved & RESERVEDMODE_NO_END ) ) WriteFileEnd ( pData ) ;

   CAUGHT_ERROR :

   return pData->nError ;
}



static int ismbblead_fwen ( int C ) {
   if ( C >= 0xB5 && C <= 0xDF ) return 1 ;
   return 0 ;
}



static int ismbbtrail_fwen ( int C ) {
   if ( C >= 0x21 && C <= 0xDF ) return 1 ;
   return 0 ;
}



static int fwen2tm ( int C ) {

   int cHigh = ( C >> 8 ) & 0xFF ;
   int cLow = C & 0xFF ;

   if ( ! ( ismbblead_fwen ( cHigh ) && ismbbtrail_fwen ( cLow ) ) ) return 0 ;

   if ( cLow == 0x7F ) return 0 ;
   if ( cLow >= 0x7F ) cLow -- ;

   if ( cLow >= 0x9F ) cLow -= 0x41 ;
   else                cLow -= 0x21 ;

   int cScalar = ( cHigh - 0xB4 ) * 0x9E + cLow - 0x40 ;
   int cNewHigh = cScalar / 0xBC ;
   int cNewLow = cScalar % 0xBC + 0x40;

   if ( cNewLow >= 0x7F ) cNewLow ++ ;

   int cResult = ( cNewHigh * 0x100 ) + cNewLow + 0x8800 ;
   if ( cResult >= 0xA040 ) cResult += ( 0xE040 - 0xA040 ) ;

   return cResult ;
}



static int fwen_pinyin2tm ( int C ) {

   if ( ! C || C >= 0x100 ) return 0 ;

   static const unsigned char Pinyin2tm [] = {
      0x7B, 0x7D, 0x7E, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC,
      0xAD, 0xAE, 0xAF, 0xB0, 0x60, 0xB1, 0xB2, 0xB3, 0xB4, 0 } ;

   for ( const unsigned char *p = Pinyin2tm ; *p ; p ++ ) {
      if ( *p == C ) return (int) ( p - Pinyin2tm ) + 0xC0 ;
   }

   return 0 ;
}



////////////////////////////////////////////
//              ֐                //
////////////////////////////////////////////



// R[h蓖Ă
static const char *GetCodeName ( int nCodePage ) {

   for ( int nCount = 0 ; nCount < CODEPAGE_NAME_LIST_NELEM ; nCount ++ ) {
      if ( nCodePage == CodePageNameList [ nCount ].nCodePage ) return CodePageNameList [ nCount ].szCodeName ;
   }

   return NULL ;
}



// Ro[^̏ݒ
// ϊ֐ ConvertChineseCharacter 蓖Ă
static CONVERTCHARACTER *GetConvertFunction ( int nSrcCodePage, int nDstCodePage, int IsCwFanti ) {

   if ( IsCwFanti ) {
      switch ( nSrcCodePage ) {
         case CP_JAPANESE_CW :
            if ( nDstCodePage == CP_CHINESE_SIMPLIFIED )    return Kg2gb ;
            if ( nDstCodePage == CP_CHINESE_TRADITIONAL )   return Kg2big5 ;
            break ;
         case CP_JAPANESE_CWN :
            if ( nDstCodePage == CP_CHINESE_SIMPLIFIED )    return Cwnf2gb ;
            if ( nDstCodePage == CP_CHINESE_TRADITIONAL )   return Cwnf2big5 ;
            if ( nDstCodePage == CP_JAPANESE_CW )           return Cwnf2cw ;
            break ;
         case CP_JAPANESE_TM :
            if ( nDstCodePage == CP_CHINESE_SIMPLIFIED )    return Tmf2gb ;
            if ( nDstCodePage == CP_CHINESE_TRADITIONAL )   return Tmf2big5 ;
            break ;
      }
   }

   switch ( nSrcCodePage ) {
      case CP_CHINESE_SIMPLIFIED :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return NULL ;
            case CP_CHINESE_TRADITIONAL :    return Gb2big5 ;
            case CP_JAPANESE_CW :            return Gb2cw ;
            case CP_JAPANESE_NH :            return Gb2nh ;
            case CP_JAPANESE_CWN :           return Gb2cwn ;
            case CP_JAPANESE_ZQ :            return Gb2zq ;
            case CP_JAPANESE_CX :            return Gb2cx ;
            case CP_JAPANESE_ZJ :            return Gb2zj ;
            default :                        break ;
         }
         break ;

      case CP_CHINESE_TRADITIONAL :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Big52gb ;
            case CP_CHINESE_TRADITIONAL :    return NULL ;
            case CP_JAPANESE_CW :            return Big52cw ;
            case CP_JAPANESE_NH :            return Big52nh ;
            case CP_JAPANESE_CWN :           return Big52cwn ;
            case CP_JAPANESE_ZQ :            return Big52zq ;
            case CP_JAPANESE_CX :            return Big52cx ;
            case CP_JAPANESE_ZJ :            return Big52zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_CW :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Cw2gb ;
            case CP_CHINESE_TRADITIONAL :    return Cw2big5 ;
            case CP_JAPANESE_CW :            return NULL ;
            case CP_JAPANESE_NH :            return Cw2nh ;
            case CP_JAPANESE_CWN :           return Cw2cwn ;
            case CP_JAPANESE_ZQ :            return Cw2zq ;
            case CP_JAPANESE_CX :            return Cw2cx ;
            case CP_JAPANESE_ZJ :            return Cw2zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_NH :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Nh2gb ;
            case CP_CHINESE_TRADITIONAL :    return Nh2big5 ;
            case CP_JAPANESE_CW :            return Nh2cw ;
            case CP_JAPANESE_NH :            return NULL ;
            case CP_JAPANESE_CWN :           return Nh2cwn ;
            case CP_JAPANESE_ZQ :            return Nh2zq ;
            case CP_JAPANESE_CX :            return Nh2cx ;
            case CP_JAPANESE_ZJ :            return Nh2zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_CWN :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Cwn2gb ;
            case CP_CHINESE_TRADITIONAL :    return Cwn2big5 ;
            case CP_JAPANESE_CW :            return Cwn2cw ;
            case CP_JAPANESE_NH :            return Cwn2nh ;
            case CP_JAPANESE_CWN :           return NULL ;
            case CP_JAPANESE_ZQ :            return Cwn2zq ;
            case CP_JAPANESE_CX :            return Cwn2cx ;
            case CP_JAPANESE_ZJ :            return Cwn2zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_ZQ :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Zq2gb ;
            case CP_CHINESE_TRADITIONAL :    return Zq2big5 ;
            case CP_JAPANESE_CW :            return Zq2cw ;
            case CP_JAPANESE_NH :            return Zq2nh ;
            case CP_JAPANESE_CWN :           return Zq2cwn ;
            case CP_JAPANESE_ZQ :            return NULL ;
            case CP_JAPANESE_CX :            return Zq2cx ;
            case CP_JAPANESE_ZJ :            return Zq2zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_CX :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Cx2gb ;
            case CP_CHINESE_TRADITIONAL :    return Cx2big5 ;
            case CP_JAPANESE_CW :            return Cx2cw ;
            case CP_JAPANESE_NH :            return Cx2nh ;
            case CP_JAPANESE_CWN :           return Cx2cwn ;
            case CP_JAPANESE_ZQ :            return Cx2zq ;
            case CP_JAPANESE_CX :            return NULL ;
            case CP_JAPANESE_ZJ :            return Cx2zj ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_ZJ :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Zj2gb ;
            case CP_CHINESE_TRADITIONAL :    return Zj2big5 ;
            case CP_JAPANESE_CW :            return Zj2cw ;
            case CP_JAPANESE_NH :            return Zj2nh ;
            case CP_JAPANESE_CWN :           return Zj2cwn ;
            case CP_JAPANESE_ZQ :            return Zj2zq ;
            case CP_JAPANESE_CX :            return Zj2cx ;
            case CP_JAPANESE_ZJ :            return NULL ;
            default :                        break ;
         }
         break ;

      case CP_JAPANESE_TM :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return Tm2gb ;
            case CP_CHINESE_TRADITIONAL :    return Tm2big5 ;
            case CP_JAPANESE_CW :            return Tm2cw ;
            case CP_JAPANESE_NH :            return Tm2nh ;
            case CP_JAPANESE_CWN :           return Tm2cwn ;
            case CP_JAPANESE_ZQ :            return Tm2zq ;
            case CP_JAPANESE_CX :            return Tm2cx ;
            case CP_JAPANESE_ZJ :            return Tm2zj ;
            default :                        break ;
         }
         break ;

      case CP_HZ_CHINESE_SIMPLIFIED :
         switch ( nDstCodePage ) {
            case CP_CHINESE_SIMPLIFIED :     return ConvertChineseCharacterDummy ;
            case CP_CHINESE_TRADITIONAL :    return Gb2big5 ;
            case CP_JAPANESE_CW :            return Gb2cw ;
            case CP_JAPANESE_NH :            return Gb2nh ;
            case CP_JAPANESE_CWN :           return Gb2cwn ;
            case CP_JAPANESE_ZQ :            return Gb2zq ;
            case CP_JAPANESE_CX :            return Gb2cx ;
            case CP_JAPANESE_ZJ :            return Gb2zj ;
            default :                        break ;
         }
         break ;

      default :
         return NULL ;
         break ;
   }

   return NULL ;
}



////////////////////////////////////////////
//               GB -> XX                 //
////////////////////////////////////////////



static int IsGb2312 ( int C ) ;



static int Gb2big5 ( int C, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( C ) ) return 0 ;

   if ( C >= 0x100 ) {
      if ( dwConversionMode & CONVMODE_QUOTATION_MARK ) {
         switch ( C ) {
            case 0xA1AE :  C = 0xA1BA ; break ;
            case 0xA1AF :  C = 0xA1BB ; break ;
            case 0xA1B0 :  C = 0xA1B8 ; break ;
            case 0xA1B1 :  C = 0xA1B9 ; break ;
         }
      }
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_VARIANT ) {
         int cBig5 = ConvertCharacterEx ( C, CP_CHINESE_SIMPLIFIED, CP_CHINESE_TRADITIONAL ) ;
         if ( cBig5 ) return cBig5 ;
      }
      const unsigned short *pTable = Gb2big5Table ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      if ( C == 0x80 ) return 0xA3E1 ;
      return 0 ;
   }

   return C ;
}



static int Gb2cw ( int C, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( C ) ) return 0 ;

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Gb2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) return 0 ;

   return C ;
}



static int Gb2nh ( int C, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( C ) ) return 0 ;

   int cNh = Gb2nh_NoCwOnly ( C ) ;
   if ( cNh ) return cNh ;

   int cCw = Gb2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Gb2cwn ( int C, unsigned long dwConversionMode ) {

   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( C ) ) return 0 ;

   int cCwn = Gb2cwn_NoCwOnly ( C ) ;
   if ( cCwn ) return cCwn ;

   int cCw = Gb2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Gb2zq ( int C, unsigned long dwConversionMode ) {

   int cCw = Gb2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Gb2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Gb2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



static int Gb2zj ( int C, unsigned long dwConversionMode ) {

   int cCw = Gb2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               BIG5 -> XX               //
////////////////////////////////////////////



static int Big52gb ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      if ( dwConversionMode & CONVMODE_QUOTATION_MARK ) {
         switch ( C ) {
            case 0xA175 : case 0xA177 :   C = 0xA1A7 ; break ;
            case 0xA176 : case 0xA178 :   C = 0xA1A8 ; break ;
            case 0xA179 : case 0xA17B :   C = 0xA1A5 ; break ;
            case 0xA17A : case 0xA17C :   C = 0xA1A6 ; break ;
         }
      }
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_VARIANT ) {
         int cGb = ConvertCharacterEx ( C, CP_CHINESE_TRADITIONAL, CP_CHINESE_SIMPLIFIED ) ;
         if ( cGb ) return cGb ;
      }
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK ) {
         int cGb = Big52gb_Gb2312 ( C ) ;
         if ( cGb ) return cGb ;
      }
      const unsigned short *pTable = Big52gbTable ( C >> 8 ) ;
      int cGb = ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
      return cGb ;
   }

   if ( ! isascii ( C ) ) return 0 ;

   return C ;
}



static int Big52cw ( int C, unsigned long dwConversionMode ) {

   int cCw = Big52cw_NoGbOnly ( C ) ;
   if ( cCw ) return cCw ;

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2cw ( cGb, dwConversionMode ) ;
}



static int Big52nh ( int C, unsigned long dwConversionMode ) {

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2nh ( cGb, dwConversionMode ) ;
}



static int Big52cwn ( int C, unsigned long dwConversionMode ) {

   int cCwn = Big52cwn_NoGbOnly ( C ) ;
   if ( cCwn ) return cCwn ;

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2cwn ( cGb, dwConversionMode ) ;
}



static int Big52zq ( int C, unsigned long dwConversionMode ) {

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2zq ( cGb, dwConversionMode ) ;
}



static int Big52cx ( int C, unsigned long dwConversionMode ) {

   int cCx = Big52cx_NoCwOnly ( C ) ;
   if ( cCx ) return cCx ;

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2cx ( cGb, dwConversionMode ) ;
}



static int Big52zj ( int C, unsigned long dwConversionMode ) {

   int cGb = Big52gb ( C, dwConversionMode | CONVMODE_CHNCONV_NO_GBK ) ;
   if ( ! cGb ) cGb = Big52gb ( C, dwConversionMode & ~ CONVMODE_CHNCONV_NO_GBK ) ;
   return Gb2zj ( cGb, dwConversionMode ) ;
}



////////////////////////////////////////////
//               CW -> XX                 //
////////////////////////////////////////////



static int Cw2gb ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2gbTable ( C >> 8 ) ;
      int cGb = ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
      return cGb ;
   }

   if ( ! isascii ( C ) ) {
      int cGb = CwOneByteCharToGb_0080_00FF [ C - 0x80 ] ;
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
      return cGb ;
   }

   return C ;
}



static int Cw2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Cw2gb ( C, dwConversionMode ) ;
   return Gb2big5 ( cGb, dwConversionMode ) ;
}



static int Cw2nh ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2nhTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Cw2nhOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Cw2cwn ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2cwnTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   return C ;
}



static int Cw2zq ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2zqTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Cw2zqOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Cw2cx ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2cxTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Cw2cxOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Cw2zj ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cw2zjTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Cw2zjOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



////////////////////////////////////////////
//               NH -> XX                 //
////////////////////////////////////////////



static int Nh2gb ( int C, unsigned long dwConversionMode ) {

   int cGb = Nh2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Nh2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Nh2gb ( C, dwConversionMode ) ;
   int cBig5 = Gb2big5 ( cGb, dwConversionMode ) ;
   if ( cBig5 ) return cBig5 ;

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2big5 ( cCw, dwConversionMode ) ;
}



static int Nh2cw ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Nh2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return 0 ;
   }

   return C ;
}



static int Nh2cwn ( int C, unsigned long dwConversionMode ) {

   int cGb = Nh2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cCwn = Gb2cwn ( cGb, dwConversionMode ) ;
      if ( cCwn ) return cCwn ;
   }

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Nh2zq ( int C, unsigned long dwConversionMode ) {

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Nh2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



static int Nh2zj ( int C, unsigned long dwConversionMode ) {

   int cCw = Nh2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               CWN -> XX                //
////////////////////////////////////////////



static int Cwn2gb ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0xF540 && C <= 0xF65F ) {
      int cCw = Cwn2cw ( C, dwConversionMode ) ;
      int cGb = Cw2gb ( cCw, dwConversionMode ) ;
      if ( cGb ) return cGb ;
   }

   int cGb = Cwn2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Cwn2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Cwn2gb ( C, dwConversionMode ) ;
   int cBig5 = Gb2big5 ( cGb, dwConversionMode ) ;
   if ( cBig5 ) return cBig5 ;

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2big5 ( cCw, dwConversionMode ) ;
}



static int Cwn2cw ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0xF540 && C <= 0xF65F ) {
      int cCw = 0 ;
      if ( ( C >> 8 ) == 0xF5 ) cCw = Cwn2cwTable_NoFanti_F5 [ ( C & 0xFF ) - 0x40 ] ;
      if ( ( C >> 8 ) == 0xF6 ) cCw = Cwn2cwTable_NoFanti_F6 [ ( C & 0xFF ) - 0x40 ] ;
      if ( cCw ) return cCw ;
   }

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cwn2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      if ( C == (unsigned char) '' ) return 0 ;
      return C ;
   }

   return C ;
}



static int Cwn2nh ( int C, unsigned long dwConversionMode ) {

   int cGb = Cwn2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cCwn = Gb2nh ( cGb, dwConversionMode ) ;
      if ( cCwn ) return cCwn ;
   }

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Cwn2zq ( int C, unsigned long dwConversionMode ) {

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Cwn2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



static int Cwn2zj ( int C, unsigned long dwConversionMode ) {

   int cCw = Cwn2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               ZQ -> XX                 //
////////////////////////////////////////////



static int Zq2gb ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Zq2big5 ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2big5 ( cCw, dwConversionMode ) ;
}



static int Zq2cw ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Zq2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Zq2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Zq2nh ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Zq2cwn ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Zq2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



static int Zq2zj ( int C, unsigned long dwConversionMode ) {

   int cCw = Zq2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               CX -> XX                 //
////////////////////////////////////////////



static int Cx2gb ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Cx2big5 ( int C, unsigned long dwConversionMode ) {

   int cBig5 = Cx2big5_NoCwOnly ( C ) ;
   if ( cBig5 ) return cBig5 ;

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2big5 ( cCw, dwConversionMode ) ;
}



static int Cx2cw ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cCw = Gb2cw ( cGb, dwConversionMode ) ;
      if ( cCw ) return cCw ;
   }

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cx2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Cx2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Cx2nh ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cNh = Gb2nh ( cGb, dwConversionMode ) ;
      if ( cNh ) return cNh ;
   }

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Cx2cwn ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cCwn = Gb2cwn ( cGb, dwConversionMode ) ;
      if ( cCwn ) return cCwn ;
   }

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Cx2zq ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cZq = Gb2zq ( cGb, dwConversionMode ) ;
      if ( cZq ) return cZq ;
   }

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Cx2zj ( int C, unsigned long dwConversionMode ) {

   int cGb = Cx2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) {
      int cZj = Gb2zj ( cGb, dwConversionMode ) ;
      if ( cZj ) return cZj ;
   }

   int cCw = Cx2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               ZJ -> XX                 //
////////////////////////////////////////////



static int Zj2gb ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Zj2big5 ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2big5 ( cCw, dwConversionMode ) ;
}



static int Zj2cw ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Zj2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return 0 ;
   }

   return C ;
}



static int Zj2nh ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Zj2cwn ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Zj2zq ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Zj2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Zj2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               KG -> XX                 //
////////////////////////////////////////////



static int Kg2gb ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Kg2gbTable ( C >> 8 ) ;
      int cGb = ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
      if ( ! cGb ) {
         const unsigned short *pTable = Cw2gbTable ( C >> 8 ) ;
         cGb = ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
      }
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
      return cGb ;
   }

   if ( ! isascii ( C ) ) {
      int cGb = CwOneByteCharToGb_0080_00FF [ C - 0x80 ] ;
      if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
      return cGb ;
   }

   return C ;
}



static int Kg2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Kg2gb ( C, dwConversionMode ) ;
   int cBig5 = Gb2big5 ( cGb, dwConversionMode ) ;
   if ( ! cBig5 ) cBig5 = Cw2big5 ( C, dwConversionMode ) ;

   return cBig5 ;
}



////////////////////////////////////////////
//              CWN F -> XX               //
////////////////////////////////////////////



static int Cwnf2gb ( int C, unsigned long dwConversionMode ) {

   int cGb = Cwn2gbFanti_NoCwOnly ( C ) ;
   if ( ! cGb ) cGb = Cwn2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Cwnf2cw ( C, dwConversionMode ) ;
   return Kg2gb ( cCw, dwConversionMode ) ;
}



static int Cwnf2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Cwnf2gb ( C, dwConversionMode ) ;
   int cBig5 = Gb2big5 ( cGb, dwConversionMode ) ;
   if ( ! cBig5 ) cBig5 = Cwn2big5 ( C, dwConversionMode ) ;

   return cBig5 ;
}



static int Cwnf2cw ( int C, unsigned long dwConversionMode ) {

   int cGb = Cwn2gbFanti_NoCwOnly ( C ) ;
   if ( cGb ) return 0 ;

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Cwn2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      if ( C == (unsigned char) '' ) return 0 ;
      return C ;
   }

   return C ;
}



////////////////////////////////////////////
//               TM -> XX                 //
////////////////////////////////////////////



static int Tm2gb ( int C, unsigned long dwConversionMode ) {

   int cGb = Tm2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2gb ( cCw, dwConversionMode ) ;
}



static int Tm2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Tm2gb ( C, dwConversionMode ) ;
   return Gb2big5 ( cGb, dwConversionMode ) ;
}



static int Tm2cw ( int C, unsigned long dwConversionMode ) {

   if ( C >= 0x100 ) {
      const unsigned short *pTable = Tm2cwTable ( C >> 8 ) ;
      return ( pTable ) ? pTable [ ( C & 0xFF ) - 0x40 ] : 0 ;
   }

   if ( ! isascii ( C ) ) {
      return Tm2cwOneByteChar_0080_00FF [ C - 0x80 ] ;
   }

   return C ;
}



static int Tm2nh ( int C, unsigned long dwConversionMode ) {

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2nh ( cCw, dwConversionMode ) ;
}



static int Tm2cwn ( int C, unsigned long dwConversionMode ) {

   int cGb = Tm2gb ( C, dwConversionMode ) ;
   if ( cGb ) {
      int cCwn = Gb2cwn ( cGb, dwConversionMode ) ;
      if ( cCwn ) return cCwn ;
   }

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2cwn ( cCw, dwConversionMode ) ;
}



static int Tm2zq ( int C, unsigned long dwConversionMode ) {

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2zq ( cCw, dwConversionMode ) ;
}



static int Tm2cx ( int C, unsigned long dwConversionMode ) {

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2cx ( cCw, dwConversionMode ) ;
}



static int Tm2zj ( int C, unsigned long dwConversionMode ) {

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Cw2zj ( cCw, dwConversionMode ) ;
}



////////////////////////////////////////////
//               TM F -> XX               //
////////////////////////////////////////////



static int Tmf2gb ( int C, unsigned long dwConversionMode ) {

   int cGb = Tm2gb_NoCwOnly ( C ) ;
   if ( dwConversionMode & CONVMODE_CHNCONV_NO_GBK && ! IsGb2312 ( cGb ) ) cGb = 0 ;
   if ( cGb ) return cGb ;

   int cCw = Tm2cw ( C, dwConversionMode ) ;
   return Kg2gb ( cCw, dwConversionMode ) ;
}



static int Tmf2big5 ( int C, unsigned long dwConversionMode ) {

   int cGb = Tmf2gb ( C, dwConversionMode ) ;
   int cBig5 = Gb2big5 ( cGb, dwConversionMode ) ;
   if ( ! cBig5 ) cBig5 = Tm2big5 ( C, dwConversionMode ) ;

   return cBig5 ;
}



////////////////////////////////////////////
//                 dummy                  //
////////////////////////////////////////////



static int ConvertChineseCharacterDummy ( int C, unsigned long dwConversionMode ) {
   return C ;
}



////////////////////////////////////////////
//                 GB2312                 //
////////////////////////////////////////////



// GB2312 ̕Ȃ 0 ȊOԂ
static int IsGb2312 ( int cGb ) {

   if ( isascii ( cGb ) ) return 1 ;

   if ( cGb >= 0xA1A1 && cGb <= 0xF7FF && ( cGb & 0xFF ) >= 0xA1 ) {
      if ( cGb >= 0xA2A1 && cGb <= 0xA2AA ) return 0 ;   // GBK
      if ( cGb >= 0xA6D9 && cGb <= 0xA6F5 ) return 0 ;   // GBK
      if ( cGb == 0xA2E3 ) return 0 ;                    // GB18030
      return 1 ;
   }

   return 0 ;
}



////////////////////////////////////////////
//          ̍̏          //
////////////////////////////////////////////



// Q̑gݍ킹Ԃ
// s NULL Ԃ
static const char *GetDecomposedForm ( int C, int nSrcCodePage, int nDstCodePage , char *szBuffer ) {

   int cGb = C ;

   if ( nSrcCodePage != CP_CHINESE_SIMPLIFIED ) {
      CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( nSrcCodePage, CP_CHINESE_SIMPLIFIED, 0 ) ;
      if ( ConvertChineseCharacter ) cGb = ConvertChineseCharacter ( C, 0 ) ;
      else                           return NULL ;
   }

   const char *szDecomposed = GetGbDecomposedForm ( cGb ) ;
   if ( ! szDecomposed ) return NULL ;

   if ( nDstCodePage != CP_CHINESE_SIMPLIFIED ) {

      CONVERTCHARACTER *ConvertChineseCharacter = NULL ;

      const char *szSrc = szDecomposed ;
      char *szDst = szBuffer ;

      while ( *szSrc ) {

         int C = (unsigned char) *szSrc ++ ;
         int cResult ;

         if ( ! isascii ( C ) ) {
            if ( ! ConvertChineseCharacter ) {
               ConvertChineseCharacter = GetConvertFunction ( CP_CHINESE_SIMPLIFIED, nDstCodePage, 0 ) ;
               if ( ! ConvertChineseCharacter ) return NULL ;
            }
            if ( ismbblead_gb ( C ) ) C = ( C << 8 ) + (unsigned char) *szSrc ++ ;
            cResult = ConvertChineseCharacter ( C, 0 ) ;
         }
         else {
            cResult = C ;
         }

         if ( ! cResult ) return NULL ;

         if ( cResult < 0x100 ) *szDst ++ = cResult ;
         else {
            *szDst ++ = ( cResult >> 8 ) & 0xFF ;
            *szDst ++ = cResult & 0xFF ;
         }
      }

      *szDst = 0 ;

      return szBuffer ;
   }

   return szDecomposed ;   // usually never reached
}



////////////////////////////////////////////
//           [U`̕ϊ             //
////////////////////////////////////////////



static int ConvertByUserDefinition ( int C, int *pUnicode, CHNCONVDATA *pData ) {

   if ( 1 ) {
      for ( int nCount = 0 ; nCount < (int) pData->nNumberOfSrcToDst ; nCount ++ ) {
         if ( C == ( pData->pSrcToDst [ nCount ] ).nSrc ) {
            if ( pUnicode ) *pUnicode = 0 ;
            return ( pData->pSrcToDst [ nCount ] ).nDst ;
         }
      }
   }

   if ( pUnicode ) {
      for ( int nCount = 0 ; nCount < (int) pData->nNumberOfSrcToUc ; nCount ++ ) {
         if ( C == ( pData->pSrcToUc [ nCount ] ).nSrc ) {
            *pUnicode = ( pData->pSrcToUc [ nCount ] ).nDst ;
            return 0 ;
         }
      }
   }

   return 0 ;
}



////////////////////////////////////////////
//             {̏               //
////////////////////////////////////////////



// VtgJIS̋mdb@ˑR[hVhal@ˑR[hɕϊ
static int SjisNec2ibm ( int cSjis ) {

   const unsigned short *pTable = SjisNec2ibmTable ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult ) return cResult ;
   }

   return cSjis ;
}



////////////////////////////////////////////
//                TOOLS                   //
////////////////////////////////////////////



// UnicodegĕϊitϊŌ؁j
static int ConvertCharacterEx ( int C, int nSrcCodePage, int nDstCodePage ) {

   int cUnicode ;
   int cResult ;

   if ( nSrcCodePage == CP_UTF32 ) cUnicode = C ;
   else                            cUnicode = ConvertCharacter ( C, nSrcCodePage, CP_UTF32 ) ;

   if ( nDstCodePage == CP_UTF32 ) cResult = cUnicode ;
   else {
      cResult = ConvertCharacter ( cUnicode, CP_UTF32, nDstCodePage ) ;
      if ( cResult && cUnicode != ConvertCharacter ( cResult, nDstCodePage, CP_UTF32 ) ) cResult = 0 ;
   }

   return cResult ;
}



// AXL[16iɕϊ
// ϊ̕ԋp
static char *EncodeHexaDecimal ( const char *szSrc, char *szDst, int nCodePage ) {

   char *szDstSaved = szDst ;

   while ( *szSrc ) {
      if ( ismbblead_cp ( (unsigned char) *szSrc, nCodePage ) ) {
         sprintf ( szDst, "\\\'%02x\\\'%02x", (unsigned char) *szSrc, (unsigned char) *( szSrc + 1 ) ) ;
         szSrc ++ ;
         szSrc ++ ;
         szDst = strend ( szDst ) ;
         continue ;
      }
      if ( ! isascii ( *szSrc ) || iscntrl ( *szSrc ) || *szSrc == ESCAPE || *szSrc == BRACE_START || *szSrc == BRACE_END || *szSrc == SEMICOLON ) {
         sprintf ( szDst, "\\\'%02x", (unsigned char) *szSrc ) ;
         szSrc ++ ;
         szDst = strend ( szDst ) ;
         continue ;
      }
      *szDst = *szSrc ;
      szSrc ++ ;
      szDst ++ ;
      continue ;
   }
   *szDst = 0 ;

   return szDstSaved ;
}



// 񒆂̍ŏ̃L[[ḧʒuԂ
// L[[hȂ΁A̖Ԃ
static const char *GetFirstKeyword ( const char *szString ) {
   if ( szString ) {
      while ( isascii ( *szString ) && isspace ( *szString ) ) szString ++ ;
   }
   return szString ;
}



// 񒆂̍Ō̃L[[ḧʒuԂ
// L[[hȂ΁A̖Ԃ
static const char *GetLastKeyword ( const char *szString ) {
   const char *szKeyword = NULL ;
   if ( szString ) {
      while ( *szString ) {
         if ( isascii ( *szString ) && isspace ( *szString ) ) {
            if ( ! ( isascii ( *( szString + 1 ) ) && isspace ( *( szString + 1 ) ) ) && *( szString + 1 ) ) szKeyword = szString + 1 ;
         }
         szString ++ ;
      }
      if ( szKeyword == NULL ) szKeyword = strend ( szString ) ;
   }
   return szKeyword ;
}



// P̃oCgTCY擾
static int GetCharSize ( int nCodePage ) {

   switch ( nCodePage ) {
      case CP_UTF16 :
      case CP_UTF16_BE :
         return (int) sizeof(wchar16_t) ;

      case CP_UTF32 :
      case CP_UTF32_BE :
         return (int) sizeof(wchar32_t) ;

      default :
         return (int) sizeof(char) ;
   }
}



////////////////////////////////////////////
//         AVAILABLE CODE PAGE            //
////////////////////////////////////////////



// CHNCONV ϊ\ȃR[hy[W
// pȂ 1 AsȂ 0 Ԃ
int WINAPI IsChnconvValidCodePage ( int nSrcCodePage, int nDstCodePage ) {

   if ( nSrcCodePage == CP_JAPANESE_FW ) nSrcCodePage = CP_JAPANESE_TM ;

   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( nSrcCodePage, nDstCodePage, 0 ) ;
   if ( ConvertChineseCharacter ) return 1 ;

   return 0 ;
}



////////////////////////////////////////////
//         CHNCONV for CHARACTER          //
////////////////////////////////////////////



static int ChnconvCharInternal ( int C, CHNCONVDATA *pData, int *pError ) ;



int WINAPI ChnconvCharEx ( int C, int nSrcCodePage, int nDstCodePage, const CHNCONVEXINFO *pInfo ) {

   int nError = CONV_ERROR_PARAM ;
   int cResult = 0 ;

   if ( ! pInfo || pInfo->cbSize == sizeof(CHNCONVEXINFO) ) {

      CHNCONVDATA Data = { 0 } ;

      __try {

         // f[^̏
         if ( InitChnconvData ( & Data, nSrcCodePage, nDstCodePage, pInfo ) ) { nError = CONV_ERROR_PARAM ; goto CAUGHT_ERROR ; }
         if ( InitConversionMode ( & Data ) ) { nError = CONV_ERROR_CODEPAGE ; goto CAUGHT_ERROR ; }

         // s
         nError = CONV_SUCCESS ;
         cResult = ChnconvCharInternal ( C, & Data, & nError ) ;

         CAUGHT_ERROR :;
      }
      __except ( EXCEPTION_EXECUTE_HANDLER ) { nError = CONV_ERROR_UNKNOWN ; }

      if ( C != 0 && cResult == 0 ) cResult = -1 ;
   }

   if ( SetChnconvLastError ( nError ) ) return -1 ;
   return cResult ;
}



static int ChnconvCharInternal ( int C, CHNCONVDATA *pData, int *pError ) {

   int cResult = 0 ;

   if ( pData->nSrcCodePage == CP_JAPANESE_FW ) {
      if ( C >= 0x100 ) C = fwen2tm ( C ) ;
      else {
         int cResult = fwen_pinyin2tm ( C ) ;
         if ( cResult ) C = cResult ;
         else if ( ! isascii ( C ) ) C = 0 ;
      }
      pData->nSrcCodePage = CP_JAPANESE_TM ;
   }

   // ϊ֐
   CONVERTCHARACTER *ConvertChineseCharacter = GetConvertFunction ( pData->nSrcCodePage, pData->nDstCodePage, pData->dwConversionMode & CONVMODE_CW_TXT_FANTIZI ) ;
   if ( ! ConvertChineseCharacter ) {
      *pError = CONV_ERROR_CODEPAGE ;
      return -1 ;
   }

   // s
   if ( (unsigned) C < 0x100 || (unsigned) C < 0x10000 && ismbblead_cp ( ( C >> 8 ) & 0xFF, pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) && ismbbtrail_cp ( C & 0xFF, pData->nSrcCodePage & CP_DEFINE_CODEPAGE ) ) {
      cResult = ConvertChineseCharacter ( C, pData->dwConversionMode ) ;
   }

   return cResult ;
}



////////////////////////////////////////////
//            GET LAST ERROR              //
////////////////////////////////////////////



// G[l擾
// G[l̓XbhƂɊǗ
int WINAPI GetChnconvLastError ( void ) {
   return (int)(intptr_t) TlsGetValue ( nTlsIndex ) ;
}



// G[lݒ
// ݒ肳ꂽG[lԂ
int SetChnconvLastError ( int nError ) {
   TlsSetValue ( nTlsIndex, (void*)(intptr_t) nError ) ;
   return nError ;
}



////////////////////////////////////////////
//                 ABORT                  //
////////////////////////////////////////////



// ~𖽂
int WINAPI AbortChnconv ( void ) {
   SetAbortFlag () ;
   return 0 ;
}



////////////////////////////////////////////
//                VERSION                 //
////////////////////////////////////////////



// o[WԂ
int WINAPI GetDllVersion ( void ) {

   FILEVERSION FileVersion ;
   if ( GetFileVersion ( & FileVersion ) ) return 0 ;

   return TranslateVersionInfo ( & FileVersion ) ;
}



