// uninorm.cpp

#include "msc.h"
#include "codepage.h"
#include "uninorm.h"


#if defined _MSC_VER && _MSC_VER < 1300   // Cannot compile wide char 0x0080-0x00FF with VC6
#pragma setlocale ( "C" )
#endif



////////////////////////////////////////////
//               ݊                 //
////////////////////////////////////////////



#ifndef GetShortArrayBit
#define GetShortArrayBit(Array,Index)  ( ( (Array) [ (Index) >> 4 ] >> ( (Index) & 0xF ) ) & 1 )
#endif



// ݊ʏ̕
int NormalizeUnicodeCompatibility ( int cUnicode ) {


   static const unsigned short Table_3100 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x310x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x311x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x312x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0x1100,  0x1101,  0x11AA,  0x1102,  0x11AC,  0x11AD,  0x1103,  // 0x313x
      0x1104,  0x1105,  0x11B0,  0x11B1,  0x11B2,  0x11B3,  0x11B4,  0x11B5,  
      0x111A,  0x1106,  0x1107,  0x1108,  0x1121,  0x1109,  0x110A,  0x110B,  // 0x314x
      0x110C,  0x110D,  0x110E,  0x110F,  0x1110,  0x1111,  0x1112,  0x1161,  
      0x1162,  0x1163,  0x1164,  0x1165,  0x1166,  0x1167,  0x1168,  0x1169,  // 0x315x
      0x116A,  0x116B,  0x116C,  0x116D,  0x116E,  0x116F,  0x1170,  0x1171,  
      0x1172,  0x1173,  0x1174,  0x1175,  0x1160,  0x1114,  0x1115,  0x11C7,  // 0x316x
      0x11C8,  0x11CC,  0x11CE,  0x11D3,  0x11D7,  0x11D9,  0x111C,  0x11DD,  
      0x11DF,  0x111D,  0x111E,  0x1120,  0x1122,  0x1123,  0x1127,  0x1129,  // 0x317x
      0x112B,  0x112C,  0x112D,  0x112E,  0x112F,  0x1132,  0x1136,  0x1140,  
      0x1147,  0x114C,  0x11F1,  0x11F2,  0x1157,  0x1158,  0x1159,  0x1184,  // 0x318x
      0x1185,  0x1188,  0x1191,  0x1192,  0x1194,  0x119E,  0x11A1,  0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x319x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Ax
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Bx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Cx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Dx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Ex
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x31Fx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   static const unsigned short Table_F900 [] = {
      0x8C48,  0x66F4,  0x8ECA,  0x8CC8,  0x6ED1,  0x4E32,  0x53E5,  0x9F9C,  // 0xF90x
      0x9F9C,  0x5951,  0x91D1,  0x5587,  0x5948,  0x61F6,  0x7669,  0x7F85,  
      0x863F,  0x87BA,  0x88F8,  0x908F,  0x6A02,  0x6D1B,  0x70D9,  0x73DE,  // 0xF91x
      0x843D,  0x916A,  0x99F1,  0x4E82,  0x5375,  0x6B04,  0x721B,  0x862D,  
      0x9E1E,  0x5D50,  0x6FEB,  0x85CD,  0x8964,  0x62C9,  0x81D8,  0x881F,  // 0xF92x
      0x5ECA,  0x6717,  0x6D6A,  0x72FC,  0x90CE,  0x4F86,  0x51B7,  0x52DE,  
      0x64C4,  0x6AD3,  0x7210,  0x76E7,  0x8001,  0x8606,  0x865C,  0x8DEF,  // 0xF93x
      0x9732,  0x9B6F,  0x9DFA,  0x788C,  0x797F,  0x7DA0,  0x83C9,  0x9304,  
      0x9E7F,  0x8AD6,  0x58DF,  0x5F04,  0x7C60,  0x807E,  0x7262,  0x78CA,  // 0xF94x
      0x8CC2,  0x96F7,  0x58D8,  0x5C62,  0x6A13,  0x6DDA,  0x6F0F,  0x7D2F,  
      0x7E37,  0x964B,  0x52D2,  0x808B,  0x51DC,  0x51CC,  0x7A1C,  0x7DBE,  // 0xF95x
      0x83F1,  0x9675,  0x8B80,  0x62CF,  0x6A02,  0x8AFE,  0x4E39,  0x5BE7,  
      0x6012,  0x7387,  0x7570,  0x5317,  0x78FB,  0x4FBF,  0x5FA9,  0x4E0D,  // 0xF96x
      0x6CCC,  0x6578,  0x7D22,  0x53C3,  0x585E,  0x7701,  0x8449,  0x8AAA,  
      0x6BBA,  0x8FB0,  0x6C88,  0x62FE,  0x82E5,  0x63A0,  0x7565,  0x4EAE,  // 0xF97x
      0x5169,  0x51C9,  0x6881,  0x7CE7,  0x826F,  0x8AD2,  0x91CF,  0x52F5,  
      0x5442,  0x5973,  0x5EEC,  0x65C5,  0x6FFE,  0x792A,  0x95AD,  0x9A6A,  // 0xF98x
      0x9E97,  0x9ECE,  0x529B,  0x66C6,  0x6B77,  0x8F62,  0x5E74,  0x6190,  
      0x6200,  0x649A,  0x6F23,  0x7149,  0x7489,  0x79CA,  0x7DF4,  0x806F,  // 0xF99x
      0x8F26,  0x84EE,  0x9023,  0x934A,  0x5217,  0x52A3,  0x54BD,  0x70C8,  
      0x88C2,  0x8AAA,  0x5EC9,  0x5FF5,  0x637B,  0x6BAE,  0x7C3E,  0x7375,  // 0xF9Ax
      0x4EE4,  0x56F9,  0x5BE7,  0x5DBA,  0x601C,  0x73B2,  0x7469,  0x7F9A,  
      0x8046,  0x9234,  0x96F6,  0x9748,  0x9818,  0x4F8B,  0x79AE,  0x91B4,  // 0xF9Bx
      0x96B8,  0x60E1,  0x4E86,  0x50DA,  0x5BEE,  0x5C3F,  0x6599,  0x6A02,  
      0x71CE,  0x7642,  0x84FC,  0x907C,  0x9F8D,  0x6688,  0x962E,  0x5289,  // 0xF9Cx
      0x677B,  0x67F3,  0x6D41,  0x6E9C,  0x7409,  0x7559,  0x786B,  0x7D10,  
      0x985E,  0x516D,  0x622E,  0x9678,  0x502B,  0x5D19,  0x6DEA,  0x8F2A,  // 0xF9Dx
      0x5F8B,  0x6144,  0x6817,  0x7387,  0x9686,  0x5229,  0x540F,  0x5C65,  
      0x6613,  0x674E,  0x68A8,  0x6CE5,  0x7406,  0x75E2,  0x7F79,  0x88CF,  // 0xF9Ex
      0x88E1,  0x91CC,  0x96E2,  0x533F,  0x6EBA,  0x541D,  0x71D0,  0x7498,  
      0x85FA,  0x96A3,  0x9C57,  0x9E9F,  0x6797,  0x6DCB,  0x81E8,  0x7ACB,  // 0xF9Fx
      0x7B20,  0x7C92,  0x72C0,  0x7099,  0x8B58,  0x4EC0,  0x8336,  0x523A,  
   } ;


   static const unsigned short Table_FA00 [] = {
      0x5207,  0x5EA6,  0x62D3,  0x7CD6,  0x5B85,  0x6D1E,  0x66B4,  0x8F3B,  // 0xFA0x
      0x884C,  0x964D,  0x898B,  0x5ED3,  0x5140,  0x55C0,  0,       0,       
      0x585A,  0,       0x6674,  0,       0,       0x51DE,  0x732A,  0x76CA,  // 0xFA1x
      0x793C,  0x795E,  0x7965,  0x798F,  0x9756,  0x7CBE,  0x7FBD,  0,       
      0x8612,  0,       0x8AF8,  0,       0,       0x9038,  0x90FD,  0,       // 0xFA2x
      0,       0,       0x98EF,  0x98FC,  0x9928,  0x9DB4,  0,       0,       
      0x4FAE,  0x50E7,  0x514D,  0x52C9,  0x52E4,  0x5351,  0x559D,  0x5606,  // 0xFA3x
      0x5668,  0x5840,  0x58A8,  0x5C64,  0x5C6E,  0x6094,  0x6168,  0x618E,  
      0x61F2,  0x654F,  0x65E2,  0x6691,  0x6885,  0x6D77,  0x6E1A,  0x6F22,  // 0xFA4x
      0x716E,  0x722B,  0x7422,  0x7891,  0x793E,  0x7949,  0x7948,  0x7950,  
      0x7956,  0x795D,  0x798D,  0x798E,  0x7A40,  0x7A81,  0x7BC0,  0x7DF4,  // 0xFA5x
      0x7E09,  0x7E41,  0x7F72,  0x8005,  0x81ED,  0x8279,  0x8279,  0x8457,  
      0x8910,  0x8996,  0x8B01,  0x8B39,  0x8CD3,  0x8D08,  0x8FB6,  0x9038,  // 0xFA6x
      0x96E3,  0x97FF,  0x983B,  0x6075,  0x42EE,  0x8218,  0,       0,       
      0x4E26,  0x51B5,  0x5168,  0x4F80,  0x5145,  0x5180,  0x52C7,  0x52FA,  // 0xFA7x
      0x559D,  0x5555,  0x5599,  0x55E2,  0x585A,  0x58B3,  0x5944,  0x5954,  
      0x5A62,  0x5B28,  0x5ED2,  0x5ED9,  0x5F69,  0x5FAD,  0x60D8,  0x614E,  // 0xFA8x
      0x6108,  0x618E,  0x6160,  0x61F2,  0x6234,  0x63C4,  0x641C,  0x6452,  
      0x6556,  0x6674,  0x6717,  0x671B,  0x6756,  0x6B79,  0x6BBA,  0x6D41,  // 0xFA9x
      0x6EDB,  0x6ECB,  0x6F22,  0x701E,  0x716E,  0x77A7,  0x7235,  0x72AF,  
      0x732A,  0x7471,  0x7506,  0x753B,  0x761D,  0x761F,  0x76CA,  0x76DB,  // 0xFAAx
      0x76F4,  0x774A,  0x7740,  0x78CC,  0x7AB1,  0x7BC0,  0x7C7B,  0x7D5B,  
      0x7DF4,  0x7F3E,  0x8005,  0x8352,  0x83EF,  0x8779,  0x8941,  0x8986,  // 0xFABx
      0x8996,  0x8ABF,  0x8AF8,  0x8ACB,  0x8B01,  0x8AFE,  0x8AED,  0x8B39,  
      0x8B8A,  0x8D08,  0x8F38,  0x9072,  0x9199,  0x9276,  0x967C,  0x96E3,  // 0xFACx
      0x9756,  0x97DB,  0x97FF,  0x980B,  0x983B,  0x9B12,  0x9F9C,  0x284A,  
      0x2844,  0x33D5,  0x3B9D,  0x4018,  0x4039,  0x5249,  0x5CD0,  0x7ED3,  // 0xFADx
      0x9F43,  0x9F8E,  0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xFAEx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xFAFx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   static const unsigned short Table_1D400 [] = {
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  // 0x1D40x
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  // 0x1D41x
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  // 0x1D42x
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0x0042,  0x0043,  0x0044,  // 0x1D43x
      0x0045,  0x0046,  0x0047,  0x0048,  0x0049,  0x004A,  0x004B,  0x004C,  
      0x004D,  0x004E,  0x004F,  0x0050,  0x0051,  0x0052,  0x0053,  0x0054,  // 0x1D44x
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0x005A,  0x0061,  0x0062,  
      0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  0,       0x0069,  0x006A,  // 0x1D45x
      0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  0x0070,  0x0071,  0x0072,  
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  // 0x1D46x
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  // 0x1D47x
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  // 0x1D48x
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  // 0x1D49x
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0,       0x0043,  0x0044,  
      0,       0,       0x0047,  0,       0,       0x004A,  0x004B,  0,       // 0x1D4Ax
      0,       0x004E,  0x004F,  0x0050,  0x0051,  0,       0x0053,  0x0054,  
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0x005A,  0x0061,  0x0062,  // 0x1D4Bx
      0x0063,  0x0064,  0,       0x0066,  0,       0x0068,  0x0069,  0x006A,  
      0x006B,  0x006C,  0x006D,  0x006E,  0,       0x0070,  0x0071,  0x0072,  // 0x1D4Cx
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  // 0x1D4Dx
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  // 0x1D4Ex
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  // 0x1D4Fx
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  
   } ;


   static const unsigned short Table_1D500 [] = {
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0x0042,  0,       0x0044,  // 0x1D50x
      0x0045,  0x0046,  0x0047,  0,       0,       0x004A,  0x004B,  0x004C,  
      0x004D,  0x004E,  0x004F,  0x0050,  0x0051,  0,       0x0053,  0x0054,  // 0x1D51x
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0,       0x0061,  0x0062,  
      0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  0x0068,  0x0069,  0x006A,  // 0x1D52x
      0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  0x0070,  0x0071,  0x0072,  
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  // 0x1D53x
      0x0041,  0x0042,  0,       0x0044,  0x0045,  0x0046,  0x0047,  0,       
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0,       0x004F,  0,       // 0x1D54x
      0,       0,       0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  
      0x0059,  0,       0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  // 0x1D55x
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  // 0x1D56x
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0x0042,  0x0043,  0x0044,  
      0x0045,  0x0046,  0x0047,  0x0048,  0x0049,  0x004A,  0x004B,  0x004C,  // 0x1D57x
      0x004D,  0x004E,  0x004F,  0x0050,  0x0051,  0x0052,  0x0053,  0x0054,  
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0x005A,  0x0061,  0x0062,  // 0x1D58x
      0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  0x0068,  0x0069,  0x006A,  
      0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  0x0070,  0x0071,  0x0072,  // 0x1D59x
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  // 0x1D5Ax
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  // 0x1D5Bx
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  // 0x1D5Cx
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0x0042,  0x0043,  0x0044,  // 0x1D5Dx
      0x0045,  0x0046,  0x0047,  0x0048,  0x0049,  0x004A,  0x004B,  0x004C,  
      0x004D,  0x004E,  0x004F,  0x0050,  0x0051,  0x0052,  0x0053,  0x0054,  // 0x1D5Ex
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0x005A,  0x0061,  0x0062,  
      0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  0x0068,  0x0069,  0x006A,  // 0x1D5Fx
      0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  0x0070,  0x0071,  0x0072,  
   } ;


   static const unsigned short Table_1D600 [] = {
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  // 0x1D60x
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  // 0x1D61x
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  // 0x1D62x
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  // 0x1D63x
      0x0077,  0x0078,  0x0079,  0x007A,  0x0041,  0x0042,  0x0043,  0x0044,  
      0x0045,  0x0046,  0x0047,  0x0048,  0x0049,  0x004A,  0x004B,  0x004C,  // 0x1D64x
      0x004D,  0x004E,  0x004F,  0x0050,  0x0051,  0x0052,  0x0053,  0x0054,  
      0x0055,  0x0056,  0x0057,  0x0058,  0x0059,  0x005A,  0x0061,  0x0062,  // 0x1D65x
      0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  0x0068,  0x0069,  0x006A,  
      0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  0x0070,  0x0071,  0x0072,  // 0x1D66x
      0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  0x0078,  0x0079,  0x007A,  
      0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  0x0048,  // 0x1D67x
      0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  0x0050,  
      0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  0x0058,  // 0x1D68x
      0x0059,  0x005A,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  
      0x0067,  0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  // 0x1D69x
      0x006F,  0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  
      0x0077,  0x0078,  0x0079,  0x007A,  0x0131,  0x0237,  0,       0,       // 0x1D6Ax
      0x0391,  0x0392,  0x0393,  0x0394,  0x0395,  0x0396,  0x0397,  0x0398,  
      0x0399,  0x039A,  0x039B,  0x039C,  0x039D,  0x039E,  0x039F,  0x03A0,  // 0x1D6Bx
      0x03A1,  0x03F4,  0x03A3,  0x03A4,  0x03A5,  0x03A6,  0x03A7,  0x03A8,  
      0x03A9,  0x2207,  0x03B1,  0x03B2,  0x03B3,  0x03B4,  0x03B5,  0x03B6,  // 0x1D6Cx
      0x03B7,  0x03B8,  0x03B9,  0x03BA,  0x03BB,  0x03BC,  0x03BD,  0x03BE,  
      0x03BF,  0x03C0,  0x03C1,  0x03C2,  0x03C3,  0x03C4,  0x03C5,  0x03C6,  // 0x1D6Dx
      0x03C7,  0x03C8,  0x03C9,  0x2202,  0x03F5,  0x03D1,  0x03F0,  0x03D5,  
      0x03F1,  0x03D6,  0x0391,  0x0392,  0x0393,  0x0394,  0x0395,  0x0396,  // 0x1D6Ex
      0x0397,  0x0398,  0x0399,  0x039A,  0x039B,  0x039C,  0x039D,  0x039E,  
      0x039F,  0x03A0,  0x03A1,  0x03F4,  0x03A3,  0x03A4,  0x03A5,  0x03A6,  // 0x1D6Fx
      0x03A7,  0x03A8,  0x03A9,  0x2207,  0x03B1,  0x03B2,  0x03B3,  0x03B4,  
   } ;


   static const unsigned short Table_1D700 [] = {
      0x03B5,  0x03B6,  0x03B7,  0x03B8,  0x03B9,  0x03BA,  0x03BB,  0x03BC,  // 0x1D70x
      0x03BD,  0x03BE,  0x03BF,  0x03C0,  0x03C1,  0x03C2,  0x03C3,  0x03C4,  
      0x03C5,  0x03C6,  0x03C7,  0x03C8,  0x03C9,  0x2202,  0x03F5,  0x03D1,  // 0x1D71x
      0x03F0,  0x03D5,  0x03F1,  0x03D6,  0x0391,  0x0392,  0x0393,  0x0394,  
      0x0395,  0x0396,  0x0397,  0x0398,  0x0399,  0x039A,  0x039B,  0x039C,  // 0x1D72x
      0x039D,  0x039E,  0x039F,  0x03A0,  0x03A1,  0x03F4,  0x03A3,  0x03A4,  
      0x03A5,  0x03A6,  0x03A7,  0x03A8,  0x03A9,  0x2207,  0x03B1,  0x03B2,  // 0x1D73x
      0x03B3,  0x03B4,  0x03B5,  0x03B6,  0x03B7,  0x03B8,  0x03B9,  0x03BA,  
      0x03BB,  0x03BC,  0x03BD,  0x03BE,  0x03BF,  0x03C0,  0x03C1,  0x03C2,  // 0x1D74x
      0x03C3,  0x03C4,  0x03C5,  0x03C6,  0x03C7,  0x03C8,  0x03C9,  0x2202,  
      0x03F5,  0x03D1,  0x03F0,  0x03D5,  0x03F1,  0x03D6,  0x0391,  0x0392,  // 0x1D75x
      0x0393,  0x0394,  0x0395,  0x0396,  0x0397,  0x0398,  0x0399,  0x039A,  
      0x039B,  0x039C,  0x039D,  0x039E,  0x039F,  0x03A0,  0x03A1,  0x03F4,  // 0x1D76x
      0x03A3,  0x03A4,  0x03A5,  0x03A6,  0x03A7,  0x03A8,  0x03A9,  0x2207,  
      0x03B1,  0x03B2,  0x03B3,  0x03B4,  0x03B5,  0x03B6,  0x03B7,  0x03B8,  // 0x1D77x
      0x03B9,  0x03BA,  0x03BB,  0x03BC,  0x03BD,  0x03BE,  0x03BF,  0x03C0,  
      0x03C1,  0x03C2,  0x03C3,  0x03C4,  0x03C5,  0x03C6,  0x03C7,  0x03C8,  // 0x1D78x
      0x03C9,  0x2202,  0x03F5,  0x03D1,  0x03F0,  0x03D5,  0x03F1,  0x03D6,  
      0x0391,  0x0392,  0x0393,  0x0394,  0x0395,  0x0396,  0x0397,  0x0398,  // 0x1D79x
      0x0399,  0x039A,  0x039B,  0x039C,  0x039D,  0x039E,  0x039F,  0x03A0,  
      0x03A1,  0x03F4,  0x03A3,  0x03A4,  0x03A5,  0x03A6,  0x03A7,  0x03A8,  // 0x1D7Ax
      0x03A9,  0x2207,  0x03B1,  0x03B2,  0x03B3,  0x03B4,  0x03B5,  0x03B6,  
      0x03B7,  0x03B8,  0x03B9,  0x03BA,  0x03BB,  0x03BC,  0x03BD,  0x03BE,  // 0x1D7Bx
      0x03BF,  0x03C0,  0x03C1,  0x03C2,  0x03C3,  0x03C4,  0x03C5,  0x03C6,  
      0x03C7,  0x03C8,  0x03C9,  0x2202,  0x03F5,  0x03D1,  0x03F0,  0x03D5,  // 0x1D7Cx
      0x03F1,  0x03D6,  0x03DC,  0x03DD,  0,       0,       0x0030,  0x0031,  
      0x0032,  0x0033,  0x0034,  0x0035,  0x0036,  0x0037,  0x0038,  0x0039,  // 0x1D7Dx
      0x0030,  0x0031,  0x0032,  0x0033,  0x0034,  0x0035,  0x0036,  0x0037,  
      0x0038,  0x0039,  0x0030,  0x0031,  0x0032,  0x0033,  0x0034,  0x0035,  // 0x1D7Ex
      0x0036,  0x0037,  0x0038,  0x0039,  0x0030,  0x0031,  0x0032,  0x0033,  
      0x0034,  0x0035,  0x0036,  0x0037,  0x0038,  0x0039,  0x0030,  0x0031,  // 0x1D7Fx
      0x0032,  0x0033,  0x0034,  0x0035,  0x0036,  0x0037,  0x0038,  0x0039,  
   } ;


   static const unsigned short Table_2F800 [] = {
      0x4E3D,  0x4E38,  0x4E41,  0x0122,  0x4F60,  0x4FAE,  0x4FBB,  0x5002,  // 0x2F80x
      0x507A,  0x5099,  0x50E7,  0x50CF,  0x349E,  0x063A,  0x514D,  0x5154,  
      0x5164,  0x5177,  0x051C,  0x34B9,  0x5167,  0x518D,  0x054B,  0x5197,  // 0x2F81x
      0x51A4,  0x4ECC,  0x51AC,  0x51B5,  0x91DF,  0x51F5,  0x5203,  0x34DF,  
      0x523B,  0x5246,  0x5272,  0x5277,  0x3515,  0x52C7,  0x52C9,  0x52E4,  // 0x2F82x
      0x52FA,  0x5305,  0x5306,  0x5317,  0x5349,  0x5351,  0x535A,  0x5373,  
      0x537D,  0x537F,  0x537F,  0x537F,  0x0A2C,  0x7070,  0x53CA,  0x53DF,  // 0x2F83x
      0x0B63,  0x53EB,  0x53F1,  0x5406,  0x549E,  0x5438,  0x5448,  0x5468,  
      0x54A2,  0x54F6,  0x5510,  0x5553,  0x5563,  0x5584,  0x5584,  0x5599,  // 0x2F84x
      0x55AB,  0x55B3,  0x55C2,  0x5716,  0x5606,  0x5717,  0x5651,  0x5674,  
      0x5207,  0x58EE,  0x57CE,  0x57F4,  0x580D,  0x578B,  0x5832,  0x5831,  // 0x2F85x
      0x58AC,  0x14E4,  0x58F2,  0x58F7,  0x5906,  0x591A,  0x5922,  0x5962,  
      0x16A8,  0x16EA,  0x59EC,  0x5A1B,  0x5A27,  0x59D8,  0x5A66,  0x36EE,  // 0x2F86x
      0x36FC,  0x5B08,  0x5B3E,  0x5B3E,  0x19C8,  0x5BC3,  0x5BD8,  0x5BE7,  
      0x5BF3,  0x1B18,  0x5BFF,  0x5C06,  0x5F53,  0x5C22,  0x3781,  0x5C60,  // 0x2F87x
      0x5C6E,  0x5CC0,  0x5C8D,  0x1DE4,  0x5D43,  0x1DE6,  0x5D6E,  0x5D6B,  
      0x5D7C,  0x5DE1,  0x5DE2,  0x382F,  0x5DFD,  0x5E28,  0x5E3D,  0x5E69,  // 0x2F88x
      0x3862,  0x2183,  0x387C,  0x5EB0,  0x5EB3,  0x5EB6,  0x5ECA,  0xA392,  
      0x5EFE,  0x2331,  0x2331,  0x8201,  0x5F22,  0x5F22,  0x38C7,  0x32B8,  // 0x2F89x
      0x61DA,  0x5F62,  0x5F6B,  0x38E3,  0x5F9A,  0x5FCD,  0x5FD7,  0x5FF9,  
      0x6081,  0x393A,  0x391C,  0x6094,  0x26D4,  0x60C7,  0x6148,  0x614C,  // 0x2F8Ax
      0x614E,  0x614C,  0x617A,  0x618E,  0x61B2,  0x61A4,  0x61AF,  0x61DE,  
      0x61F2,  0x61F6,  0x6210,  0x621B,  0x625D,  0x62B1,  0x62D4,  0x6350,  // 0x2F8Bx
      0x2B0C,  0x633D,  0x62FC,  0x6368,  0x6383,  0x63E4,  0x2BF1,  0x6422,  
      0x63C5,  0x63A9,  0x3A2E,  0x6469,  0x647E,  0x649D,  0x6477,  0x3A6C,  // 0x2F8Cx
      0x654F,  0x656C,  0x300A,  0x65E3,  0x66F8,  0x6649,  0x3B19,  0x6691,  
      0x3B08,  0x3AE4,  0x5192,  0x5195,  0x6700,  0x669C,  0x80AD,  0x43D9,  // 0x2F8Dx
      0x6717,  0x671B,  0x6721,  0x675E,  0x6753,  0x33C3,  0x3B49,  0x67FA,  
      0x6785,  0x6852,  0x6885,  0x346D,  0x688E,  0x681F,  0x6914,  0x3B9D,  // 0x2F8Ex
      0x6942,  0x69A3,  0x69EA,  0x6AA8,  0x36A3,  0x6ADB,  0x3C18,  0x6B21,  
      0x38A7,  0x6B54,  0x3C4E,  0x6B72,  0x6B9F,  0x6BBA,  0x6BBB,  0x3A8D,  // 0x2F8Fx
      0x1D0B,  0x3AFA,  0x6C4E,  0x3CBC,  0x6CBF,  0x6CCD,  0x6C67,  0x6D16,  
   } ;


   static const unsigned short Table_2F900 [] = {
      0x6D3E,  0x6D77,  0x6D41,  0x6D69,  0x6D78,  0x6D85,  0x3D1E,  0x6D34,  // 0x2F90x
      0x6E2F,  0x6E6E,  0x3D33,  0x6ECB,  0x6EC7,  0x3ED1,  0x6DF9,  0x6F6E,  
      0x3F5E,  0x3F8E,  0x6FC6,  0x7039,  0x701E,  0x701B,  0x3D96,  0x704A,  // 0x2F91x
      0x707D,  0x7077,  0x70AD,  0x0525,  0x7145,  0x4263,  0x719C,  0x43AB,  
      0x7228,  0x7235,  0x7250,  0x4608,  0x7280,  0x7295,  0x4735,  0x4814,  // 0x2F92x
      0x737A,  0x738B,  0x3EAC,  0x73A5,  0x3EB8,  0x3EB8,  0x7447,  0x745C,  
      0x7471,  0x7485,  0x74CA,  0x3F1B,  0x7524,  0x4C36,  0x753E,  0x4C92,  // 0x2F93x
      0x7570,  0x219F,  0x7610,  0x4FA1,  0x4FB8,  0x5044,  0x3FFC,  0x4008,  
      0x76F4,  0x50F3,  0x50F2,  0x5119,  0x5133,  0x771E,  0x771F,  0x771F,  // 0x2F94x
      0x774A,  0x4039,  0x778B,  0x4046,  0x4096,  0x541D,  0x784E,  0x788C,  
      0x78CC,  0x40E3,  0x5626,  0x7956,  0x569A,  0x56C5,  0x798F,  0x79EB,  // 0x2F95x
      0x412F,  0x7A40,  0x7A4A,  0x7A4F,  0x597C,  0x5AA7,  0x5AA7,  0x7AEE,  
      0x4202,  0x5BAB,  0x7BC6,  0x7BC9,  0x4227,  0x5C80,  0x7CD2,  0x42A0,  // 0x2F96x
      0x7CE8,  0x7CE3,  0x7D00,  0x5F86,  0x7D63,  0x4301,  0x7DC7,  0x7E02,  
      0x7E45,  0x4334,  0x6228,  0x6247,  0x4359,  0x62D9,  0x7F7A,  0x633E,  // 0x2F97x
      0x7F95,  0x7FFA,  0x8005,  0x64DA,  0x6523,  0x8060,  0x65A8,  0x8070,  
      0x335F,  0x43D5,  0x80B2,  0x8103,  0x440B,  0x813E,  0x5AB5,  0x67A7,  // 0x2F98x
      0x67B5,  0x3393,  0x339C,  0x8201,  0x8204,  0x8F9E,  0x446B,  0x8291,  
      0x828B,  0x829D,  0x52B3,  0x82B1,  0x82B3,  0x82BD,  0x82E6,  0x6B3C,  // 0x2F99x
      0x82E5,  0x831D,  0x8363,  0x83AD,  0x8323,  0x83BD,  0x83E7,  0x8457,  
      0x8353,  0x83CA,  0x83CC,  0x83DC,  0x6C36,  0x6D6B,  0x6CD5,  0x452B,  // 0x2F9Ax
      0x84F1,  0x84F3,  0x8516,  0x73CA,  0x8564,  0x6F2C,  0x455D,  0x4561,  
      0x6FB1,  0x70D2,  0x456B,  0x8650,  0x865C,  0x8667,  0x8669,  0x86A9,  // 0x2F9Bx
      0x8688,  0x870E,  0x86E2,  0x8779,  0x8728,  0x876B,  0x8786,  0x45D7,  
      0x87E1,  0x8801,  0x45F9,  0x8860,  0x8863,  0x7667,  0x88D7,  0x88DE,  // 0x2F9Cx
      0x4635,  0x88FA,  0x34BB,  0x78AE,  0x7966,  0x46BE,  0x46C7,  0x8AA0,  
      0x8AED,  0x8B8A,  0x8C55,  0x7CA8,  0x8CAB,  0x8CC1,  0x8D1B,  0x8D77,  // 0x2F9Dx
      0x7F2F,  0x0804,  0x8DCB,  0x8DBC,  0x8DF0,  0x08DE,  0x8ED4,  0x8F38,  
      0x85D2,  0x85ED,  0x9094,  0x90F1,  0x9111,  0x872E,  0x911B,  0x9238,  // 0x2F9Ex
      0x92D7,  0x92D8,  0x927C,  0x93F9,  0x9415,  0x8BFA,  0x958B,  0x4995,  
      0x95B7,  0x8D77,  0x49E6,  0x96C3,  0x5DB2,  0x9723,  0x9145,  0x921A,  // 0x2F9Fx
      0x4A6E,  0x4A76,  0x97E0,  0x940A,  0x4AB2,  0x9496,  0x980B,  0x980B,  
   } ;


   static const unsigned short Table_2FA00 [] = {
      0x9829,  0x95B6,  0x98E2,  0x4B33,  0x9929,  0x99A7,  0x99C2,  0x99FE,  // 0x2FA0x
      0x4BCE,  0x9B30,  0x9B12,  0x9C40,  0x9CFD,  0x4CCE,  0x4CED,  0x9D67,  
      0xA0CE,  0x4CF8,  0xA105,  0xA20E,  0xA291,  0x9EBB,  0x4D56,  0x9EF9,  // 0x2FA1x
      0x9EFE,  0x9F05,  0x9F0F,  0x9F16,  0x9F3B,  0xA600,  0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA2x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA3x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA4x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA5x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA6x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA7x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA8x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FA9x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FAAx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FABx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FACx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FADx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FAEx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2FAFx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   static const unsigned short BitTable_FA00 [] = {
      0,       0,       0,       0,       0,       0,       0x1000,  0,       // 0xFAxx
      0,       0,       0,       0,       0x8000,  0x00E3,  0,       0,       
   } ;

   static const unsigned short BitTable_2F800 [] = {
      0x2008,  0x1044,  0,       0x0110,  0,       0x0200,  0x1003,  0x2802,  // 0x2F8xx
      0x8200,  0x0186,  0x0010,  0x4100,  0x0400,  0x2000,  0x1008,  0x0B81,  
   } ;

   static const unsigned short BitTable_2F900 [] = {
      0x2040,  0xA803,  0x00C8,  0x3AA0,  0x201E,  0x7034,  0x0822,  0x58AC,  // 0x2F9xx
      0x0781,  0x0080,  0x2870,  0x0003,  0x1820,  0x2308,  0x2023,  0x28C2,  
   } ;

   static const unsigned short BitTable_2FA00 [] = {
      0x0202,  0x201D,  0,       0,       0,       0,       0,       0,       // 0x2FAxx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   const unsigned short *pTable = NULL ;
   const unsigned short *pBitTable = NULL ;

   switch ( cUnicode >> 8 ) {
      case 0x3100 >> 8 : pTable = Table_3100 ; pBitTable = NULL ; break ;
      case 0xF900 >> 8 : pTable = Table_F900 ; pBitTable = NULL ; break ;
      case 0xFA00 >> 8 : pTable = Table_FA00 ; pBitTable = BitTable_FA00 ; break ;
      case 0x1D400 >> 8 : pTable = Table_1D400 ; pBitTable = NULL ; break ;
      case 0x1D500 >> 8 : pTable = Table_1D500 ; pBitTable = NULL ; break ;
      case 0x1D600 >> 8 : pTable = Table_1D600 ; pBitTable = NULL ; break ;
      case 0x1D700 >> 8 : pTable = Table_1D700 ; pBitTable = NULL ; break ;
      case 0x2F800 >> 8 : pTable = Table_2F800 ; pBitTable = BitTable_2F800 ; break ;
      case 0x2F900 >> 8 : pTable = Table_2F900 ; pBitTable = BitTable_2F900 ; break ;
      case 0x2FA00 >> 8 : pTable = Table_2FA00 ; pBitTable = BitTable_2FA00 ; break ;
   }

   if ( pTable ) {
      if ( pBitTable && GetShortArrayBit ( pBitTable, cUnicode & 0xFF ) )
         return pTable [ cUnicode & 0xFF ] + 0x20000 ;
      else
         return pTable [ cUnicode & 0xFF ] ;
   }

   return 0 ;
}



////////////////////////////////////////////
//              \p                //
////////////////////////////////////////////



// \`ʏ̕
const wchar_t *DecomposeUnicodePresentationForms ( int cUnicode ) {


   static const wchar_t *const Table_FB00 [] = {
      /* 0xFB00 */   L"ff",
      /* 0xFB01 */   L"fi",
      /* 0xFB02 */   L"fl",
      /* 0xFB03 */   L"ffi",
      /* 0xFB04 */   L"ffl",
      /* 0xFB05 */   L"\x017F\x0074",
      /* 0xFB06 */   L"st",
      /* 0xFB07 */   NULL,
      /* 0xFB08 */   NULL,
      /* 0xFB09 */   NULL,
      /* 0xFB0A */   NULL,
      /* 0xFB0B */   NULL,
      /* 0xFB0C */   NULL,
      /* 0xFB0D */   NULL,
      /* 0xFB0E */   NULL,
      /* 0xFB0F */   NULL,
      /* 0xFB10 */   NULL,
      /* 0xFB11 */   NULL,
      /* 0xFB12 */   NULL,
      /* 0xFB13 */   L"\x0574\x0576",
      /* 0xFB14 */   L"\x0574\x0565",
      /* 0xFB15 */   L"\x0574\x056B",
      /* 0xFB16 */   L"\x057E\x0576",
      /* 0xFB17 */   L"\x0574\x056D",
      /* 0xFB18 */   NULL,
      /* 0xFB19 */   NULL,
      /* 0xFB1A */   NULL,
      /* 0xFB1B */   NULL,
      /* 0xFB1C */   NULL,
      /* 0xFB1D */   L"\x05D9\x05B4",
      /* 0xFB1E */   L"\x05BF",
      /* 0xFB1F */   L"\x05F2\x05B7",
      /* 0xFB20 */   L"\x05E2",
      /* 0xFB21 */   L"\x05D0",
      /* 0xFB22 */   L"\x05D3",
      /* 0xFB23 */   L"\x05D4",
      /* 0xFB24 */   L"\x05DB",
      /* 0xFB25 */   L"\x05DC",
      /* 0xFB26 */   L"\x05DD",
      /* 0xFB27 */   L"\x05E8",
      /* 0xFB28 */   L"\x05EA",
      /* 0xFB29 */   L"+",
      /* 0xFB2A */   L"\x05E9\x05C1",
      /* 0xFB2B */   L"\x05E9\x05C2",
      /* 0xFB2C */   L"\x05E9\x05BC\x05C1",
      /* 0xFB2D */   L"\x05E9\x05BC\x05C2",
      /* 0xFB2E */   L"\x05D0\x05B7",
      /* 0xFB2F */   L"\x05D0\x05B8",
      /* 0xFB30 */   L"\x05D0\x05BC",
      /* 0xFB31 */   L"\x05D1\x05BC",
      /* 0xFB32 */   L"\x05D2\x05BC",
      /* 0xFB33 */   L"\x05D3\x05BC",
      /* 0xFB34 */   L"\x05D4\x05BC",
      /* 0xFB35 */   L"\x05D5\x05BC",
      /* 0xFB36 */   L"\x05D6\x05BC",
      /* 0xFB37 */   NULL,
      /* 0xFB38 */   L"\x05D8\x05BC",
      /* 0xFB39 */   L"\x05D9\x05BC",
      /* 0xFB3A */   L"\x05DA\x05BC",
      /* 0xFB3B */   L"\x05DB\x05BC",
      /* 0xFB3C */   L"\x05DC\x05BC",
      /* 0xFB3D */   NULL,
      /* 0xFB3E */   L"\x05DE\x05BC",
      /* 0xFB3F */   NULL,
      /* 0xFB40 */   L"\x05E0\x05BC",
      /* 0xFB41 */   L"\x05E1\x05BC",
      /* 0xFB42 */   NULL,
      /* 0xFB43 */   L"\x05E3\x05BC",
      /* 0xFB44 */   L"\x05E4\x05BC",
      /* 0xFB45 */   NULL,
      /* 0xFB46 */   L"\x05E6\x05BC",
      /* 0xFB47 */   L"\x05E7\x05BC",
      /* 0xFB48 */   L"\x05E8\x05BC",
      /* 0xFB49 */   L"\x05E9\x05BC",
      /* 0xFB4A */   L"\x05EA\x05BC",
      /* 0xFB4B */   L"\x05D5\x05B9",
      /* 0xFB4C */   L"\x05D1\x05BF",
      /* 0xFB4D */   L"\x05DB\x05BF",
      /* 0xFB4E */   L"\x05E4\x05BF",
      /* 0xFB4F */   L"\x05D0\x05DC",
      /* 0xFB50 */   L"\x0671",
      /* 0xFB51 */   L"\x0671",
      /* 0xFB52 */   L"\x067B",
      /* 0xFB53 */   L"\x067B",
      /* 0xFB54 */   L"\x067B",
      /* 0xFB55 */   L"\x067B",
      /* 0xFB56 */   L"\x067E",
      /* 0xFB57 */   L"\x067E",
      /* 0xFB58 */   L"\x067E",
      /* 0xFB59 */   L"\x067E",
      /* 0xFB5A */   L"\x0680",
      /* 0xFB5B */   L"\x0680",
      /* 0xFB5C */   L"\x0680",
      /* 0xFB5D */   L"\x0680",
      /* 0xFB5E */   L"\x067A",
      /* 0xFB5F */   L"\x067A",
      /* 0xFB60 */   L"\x067A",
      /* 0xFB61 */   L"\x067A",
      /* 0xFB62 */   L"\x067F",
      /* 0xFB63 */   L"\x067F",
      /* 0xFB64 */   L"\x067F",
      /* 0xFB65 */   L"\x067F",
      /* 0xFB66 */   L"\x0679",
      /* 0xFB67 */   L"\x0679",
      /* 0xFB68 */   L"\x0679",
      /* 0xFB69 */   L"\x0679",
      /* 0xFB6A */   L"\x06A4",
      /* 0xFB6B */   L"\x06A4",
      /* 0xFB6C */   L"\x06A4",
      /* 0xFB6D */   L"\x06A4",
      /* 0xFB6E */   L"\x06A6",
      /* 0xFB6F */   L"\x06A6",
      /* 0xFB70 */   L"\x06A6",
      /* 0xFB71 */   L"\x06A6",
      /* 0xFB72 */   L"\x0684",
      /* 0xFB73 */   L"\x0684",
      /* 0xFB74 */   L"\x0684",
      /* 0xFB75 */   L"\x0684",
      /* 0xFB76 */   L"\x0683",
      /* 0xFB77 */   L"\x0683",
      /* 0xFB78 */   L"\x0683",
      /* 0xFB79 */   L"\x0683",
      /* 0xFB7A */   L"\x0686",
      /* 0xFB7B */   L"\x0686",
      /* 0xFB7C */   L"\x0686",
      /* 0xFB7D */   L"\x0686",
      /* 0xFB7E */   L"\x0687",
      /* 0xFB7F */   L"\x0687",
      /* 0xFB80 */   L"\x0687",
      /* 0xFB81 */   L"\x0687",
      /* 0xFB82 */   L"\x068D",
      /* 0xFB83 */   L"\x068D",
      /* 0xFB84 */   L"\x068C",
      /* 0xFB85 */   L"\x068C",
      /* 0xFB86 */   L"\x068E",
      /* 0xFB87 */   L"\x068E",
      /* 0xFB88 */   L"\x0688",
      /* 0xFB89 */   L"\x0688",
      /* 0xFB8A */   L"\x0698",
      /* 0xFB8B */   L"\x0698",
      /* 0xFB8C */   L"\x0691",
      /* 0xFB8D */   L"\x0691",
      /* 0xFB8E */   L"\x06A9",
      /* 0xFB8F */   L"\x06A9",
      /* 0xFB90 */   L"\x06A9",
      /* 0xFB91 */   L"\x06A9",
      /* 0xFB92 */   L"\x06AF",
      /* 0xFB93 */   L"\x06AF",
      /* 0xFB94 */   L"\x06AF",
      /* 0xFB95 */   L"\x06AF",
      /* 0xFB96 */   L"\x06B3",
      /* 0xFB97 */   L"\x06B3",
      /* 0xFB98 */   L"\x06B3",
      /* 0xFB99 */   L"\x06B3",
      /* 0xFB9A */   L"\x06B1",
      /* 0xFB9B */   L"\x06B1",
      /* 0xFB9C */   L"\x06B1",
      /* 0xFB9D */   L"\x06B1",
      /* 0xFB9E */   L"\x06BA",
      /* 0xFB9F */   L"\x06BA",
      /* 0xFBA0 */   L"\x06BB",
      /* 0xFBA1 */   L"\x06BB",
      /* 0xFBA2 */   L"\x06BB",
      /* 0xFBA3 */   L"\x06BB",
      /* 0xFBA4 */   L"\x06C0",
      /* 0xFBA5 */   L"\x06C0",
      /* 0xFBA6 */   L"\x06C1",
      /* 0xFBA7 */   L"\x06C1",
      /* 0xFBA8 */   L"\x06C1",
      /* 0xFBA9 */   L"\x06C1",
      /* 0xFBAA */   L"\x06BE",
      /* 0xFBAB */   L"\x06BE",
      /* 0xFBAC */   L"\x06BE",
      /* 0xFBAD */   L"\x06BE",
      /* 0xFBAE */   L"\x06D2",
      /* 0xFBAF */   L"\x06D2",
      /* 0xFBB0 */   L"\x06D3",
      /* 0xFBB1 */   L"\x06D3",
      /* 0xFBB2 */   NULL,
      /* 0xFBB3 */   NULL,
      /* 0xFBB4 */   NULL,
      /* 0xFBB5 */   NULL,
      /* 0xFBB6 */   NULL,
      /* 0xFBB7 */   NULL,
      /* 0xFBB8 */   NULL,
      /* 0xFBB9 */   NULL,
      /* 0xFBBA */   NULL,
      /* 0xFBBB */   NULL,
      /* 0xFBBC */   NULL,
      /* 0xFBBD */   NULL,
      /* 0xFBBE */   NULL,
      /* 0xFBBF */   NULL,
      /* 0xFBC0 */   NULL,
      /* 0xFBC1 */   NULL,
      /* 0xFBC2 */   NULL,
      /* 0xFBC3 */   NULL,
      /* 0xFBC4 */   NULL,
      /* 0xFBC5 */   NULL,
      /* 0xFBC6 */   NULL,
      /* 0xFBC7 */   NULL,
      /* 0xFBC8 */   NULL,
      /* 0xFBC9 */   NULL,
      /* 0xFBCA */   NULL,
      /* 0xFBCB */   NULL,
      /* 0xFBCC */   NULL,
      /* 0xFBCD */   NULL,
      /* 0xFBCE */   NULL,
      /* 0xFBCF */   NULL,
      /* 0xFBD0 */   NULL,
      /* 0xFBD1 */   NULL,
      /* 0xFBD2 */   NULL,
      /* 0xFBD3 */   L"\x06AD",
      /* 0xFBD4 */   L"\x06AD",
      /* 0xFBD5 */   L"\x06AD",
      /* 0xFBD6 */   L"\x06AD",
      /* 0xFBD7 */   L"\x06C7",
      /* 0xFBD8 */   L"\x06C7",
      /* 0xFBD9 */   L"\x06C6",
      /* 0xFBDA */   L"\x06C6",
      /* 0xFBDB */   L"\x06C8",
      /* 0xFBDC */   L"\x06C8",
      /* 0xFBDD */   L"\x0677",
      /* 0xFBDE */   L"\x06CB",
      /* 0xFBDF */   L"\x06CB",
      /* 0xFBE0 */   L"\x06C5",
      /* 0xFBE1 */   L"\x06C5",
      /* 0xFBE2 */   L"\x06C9",
      /* 0xFBE3 */   L"\x06C9",
      /* 0xFBE4 */   L"\x06D0",
      /* 0xFBE5 */   L"\x06D0",
      /* 0xFBE6 */   L"\x06D0",
      /* 0xFBE7 */   L"\x06D0",
      /* 0xFBE8 */   L"\x0649",
      /* 0xFBE9 */   L"\x0649",
      /* 0xFBEA */   L"\x0626\x0627",
      /* 0xFBEB */   L"\x0626\x0627",
      /* 0xFBEC */   L"\x0626\x06D5",
      /* 0xFBED */   L"\x0626\x06D5",
      /* 0xFBEE */   L"\x0626\x0648",
      /* 0xFBEF */   L"\x0626\x0648",
      /* 0xFBF0 */   L"\x0626\x06C7",
      /* 0xFBF1 */   L"\x0626\x06C7",
      /* 0xFBF2 */   L"\x0626\x06C6",
      /* 0xFBF3 */   L"\x0626\x06C6",
      /* 0xFBF4 */   L"\x0626\x06C8",
      /* 0xFBF5 */   L"\x0626\x06C8",
      /* 0xFBF6 */   L"\x0626\x06D0",
      /* 0xFBF7 */   L"\x0626\x06D0",
      /* 0xFBF8 */   L"\x0626\x06D0",
      /* 0xFBF9 */   L"\x0626\x0649",
      /* 0xFBFA */   L"\x0626\x0649",
      /* 0xFBFB */   L"\x0626\x0649",
      /* 0xFBFC */   L"\x06CC",
      /* 0xFBFD */   L"\x06CC",
      /* 0xFBFE */   L"\x06CC",
      /* 0xFBFF */   L"\x06CC",
   } ;


   static const wchar_t *const Table_FC00 [] = {
      /* 0xFC00 */   L"\x0626\x062C",
      /* 0xFC01 */   L"\x0626\x062D",
      /* 0xFC02 */   L"\x0626\x0645",
      /* 0xFC03 */   L"\x0626\x0649",
      /* 0xFC04 */   L"\x0626\x064A",
      /* 0xFC05 */   L"\x0628\x062C",
      /* 0xFC06 */   L"\x0628\x062D",
      /* 0xFC07 */   L"\x0628\x062E",
      /* 0xFC08 */   L"\x0628\x0645",
      /* 0xFC09 */   L"\x0628\x0649",
      /* 0xFC0A */   L"\x0628\x064A",
      /* 0xFC0B */   L"\x062A\x062C",
      /* 0xFC0C */   L"\x062A\x062D",
      /* 0xFC0D */   L"\x062A\x062E",
      /* 0xFC0E */   L"\x062A\x0645",
      /* 0xFC0F */   L"\x062A\x0649",
      /* 0xFC10 */   L"\x062A\x064A",
      /* 0xFC11 */   L"\x062B\x062C",
      /* 0xFC12 */   L"\x062B\x0645",
      /* 0xFC13 */   L"\x062B\x0649",
      /* 0xFC14 */   L"\x062B\x064A",
      /* 0xFC15 */   L"\x062C\x062D",
      /* 0xFC16 */   L"\x062C\x0645",
      /* 0xFC17 */   L"\x062D\x062C",
      /* 0xFC18 */   L"\x062D\x0645",
      /* 0xFC19 */   L"\x062E\x062C",
      /* 0xFC1A */   L"\x062E\x062D",
      /* 0xFC1B */   L"\x062E\x0645",
      /* 0xFC1C */   L"\x0633\x062C",
      /* 0xFC1D */   L"\x0633\x062D",
      /* 0xFC1E */   L"\x0633\x062E",
      /* 0xFC1F */   L"\x0633\x0645",
      /* 0xFC20 */   L"\x0635\x062D",
      /* 0xFC21 */   L"\x0635\x0645",
      /* 0xFC22 */   L"\x0636\x062C",
      /* 0xFC23 */   L"\x0636\x062D",
      /* 0xFC24 */   L"\x0636\x062E",
      /* 0xFC25 */   L"\x0636\x0645",
      /* 0xFC26 */   L"\x0637\x062D",
      /* 0xFC27 */   L"\x0637\x0645",
      /* 0xFC28 */   L"\x0638\x0645",
      /* 0xFC29 */   L"\x0639\x062C",
      /* 0xFC2A */   L"\x0639\x0645",
      /* 0xFC2B */   L"\x063A\x062C",
      /* 0xFC2C */   L"\x063A\x0645",
      /* 0xFC2D */   L"\x0641\x062C",
      /* 0xFC2E */   L"\x0641\x062D",
      /* 0xFC2F */   L"\x0641\x062E",
      /* 0xFC30 */   L"\x0641\x0645",
      /* 0xFC31 */   L"\x0641\x0649",
      /* 0xFC32 */   L"\x0641\x064A",
      /* 0xFC33 */   L"\x0642\x062D",
      /* 0xFC34 */   L"\x0642\x0645",
      /* 0xFC35 */   L"\x0642\x0649",
      /* 0xFC36 */   L"\x0642\x064A",
      /* 0xFC37 */   L"\x0643\x0627",
      /* 0xFC38 */   L"\x0643\x062C",
      /* 0xFC39 */   L"\x0643\x062D",
      /* 0xFC3A */   L"\x0643\x062E",
      /* 0xFC3B */   L"\x0643\x0644",
      /* 0xFC3C */   L"\x0643\x0645",
      /* 0xFC3D */   L"\x0643\x0649",
      /* 0xFC3E */   L"\x0643\x064A",
      /* 0xFC3F */   L"\x0644\x062C",
      /* 0xFC40 */   L"\x0644\x062D",
      /* 0xFC41 */   L"\x0644\x062E",
      /* 0xFC42 */   L"\x0644\x0645",
      /* 0xFC43 */   L"\x0644\x0649",
      /* 0xFC44 */   L"\x0644\x064A",
      /* 0xFC45 */   L"\x0645\x062C",
      /* 0xFC46 */   L"\x0645\x062D",
      /* 0xFC47 */   L"\x0645\x062E",
      /* 0xFC48 */   L"\x0645\x0645",
      /* 0xFC49 */   L"\x0645\x0649",
      /* 0xFC4A */   L"\x0645\x064A",
      /* 0xFC4B */   L"\x0646\x062C",
      /* 0xFC4C */   L"\x0646\x062D",
      /* 0xFC4D */   L"\x0646\x062E",
      /* 0xFC4E */   L"\x0646\x0645",
      /* 0xFC4F */   L"\x0646\x0649",
      /* 0xFC50 */   L"\x0646\x064A",
      /* 0xFC51 */   L"\x0647\x062C",
      /* 0xFC52 */   L"\x0647\x0645",
      /* 0xFC53 */   L"\x0647\x0649",
      /* 0xFC54 */   L"\x0647\x064A",
      /* 0xFC55 */   L"\x064A\x062C",
      /* 0xFC56 */   L"\x064A\x062D",
      /* 0xFC57 */   L"\x064A\x062E",
      /* 0xFC58 */   L"\x064A\x0645",
      /* 0xFC59 */   L"\x064A\x0649",
      /* 0xFC5A */   L"\x064A\x064A",
      /* 0xFC5B */   L"\x0630\x0670",
      /* 0xFC5C */   L"\x0631\x0670",
      /* 0xFC5D */   L"\x0649\x0670",
      /* 0xFC5E */   L" \x064C\x0651",
      /* 0xFC5F */   L" \x064D\x0651",
      /* 0xFC60 */   L" \x064E\x0651",
      /* 0xFC61 */   L" \x064F\x0651",
      /* 0xFC62 */   L" \x0650\x0651",
      /* 0xFC63 */   L" \x0651\x0670",
      /* 0xFC64 */   L"\x0626\x0631",
      /* 0xFC65 */   L"\x0626\x0632",
      /* 0xFC66 */   L"\x0626\x0645",
      /* 0xFC67 */   L"\x0626\x0646",
      /* 0xFC68 */   L"\x0626\x0649",
      /* 0xFC69 */   L"\x0626\x064A",
      /* 0xFC6A */   L"\x0628\x0631",
      /* 0xFC6B */   L"\x0628\x0632",
      /* 0xFC6C */   L"\x0628\x0645",
      /* 0xFC6D */   L"\x0628\x0646",
      /* 0xFC6E */   L"\x0628\x0649",
      /* 0xFC6F */   L"\x0628\x064A",
      /* 0xFC70 */   L"\x062A\x0631",
      /* 0xFC71 */   L"\x062A\x0632",
      /* 0xFC72 */   L"\x062A\x0645",
      /* 0xFC73 */   L"\x062A\x0646",
      /* 0xFC74 */   L"\x062A\x0649",
      /* 0xFC75 */   L"\x062A\x064A",
      /* 0xFC76 */   L"\x062B\x0631",
      /* 0xFC77 */   L"\x062B\x0632",
      /* 0xFC78 */   L"\x062B\x0645",
      /* 0xFC79 */   L"\x062B\x0646",
      /* 0xFC7A */   L"\x062B\x0649",
      /* 0xFC7B */   L"\x062B\x064A",
      /* 0xFC7C */   L"\x0641\x0649",
      /* 0xFC7D */   L"\x0641\x064A",
      /* 0xFC7E */   L"\x0642\x0649",
      /* 0xFC7F */   L"\x0642\x064A",
      /* 0xFC80 */   L"\x0643\x0627",
      /* 0xFC81 */   L"\x0643\x0644",
      /* 0xFC82 */   L"\x0643\x0645",
      /* 0xFC83 */   L"\x0643\x0649",
      /* 0xFC84 */   L"\x0643\x064A",
      /* 0xFC85 */   L"\x0644\x0645",
      /* 0xFC86 */   L"\x0644\x0649",
      /* 0xFC87 */   L"\x0644\x064A",
      /* 0xFC88 */   L"\x0645\x0627",
      /* 0xFC89 */   L"\x0645\x0645",
      /* 0xFC8A */   L"\x0646\x0631",
      /* 0xFC8B */   L"\x0646\x0632",
      /* 0xFC8C */   L"\x0646\x0645",
      /* 0xFC8D */   L"\x0646\x0646",
      /* 0xFC8E */   L"\x0646\x0649",
      /* 0xFC8F */   L"\x0646\x064A",
      /* 0xFC90 */   L"\x0649\x0670",
      /* 0xFC91 */   L"\x064A\x0631",
      /* 0xFC92 */   L"\x064A\x0632",
      /* 0xFC93 */   L"\x064A\x0645",
      /* 0xFC94 */   L"\x064A\x0646",
      /* 0xFC95 */   L"\x064A\x0649",
      /* 0xFC96 */   L"\x064A\x064A",
      /* 0xFC97 */   L"\x0626\x062C",
      /* 0xFC98 */   L"\x0626\x062D",
      /* 0xFC99 */   L"\x0626\x062E",
      /* 0xFC9A */   L"\x0626\x0645",
      /* 0xFC9B */   L"\x0626\x0647",
      /* 0xFC9C */   L"\x0628\x062C",
      /* 0xFC9D */   L"\x0628\x062D",
      /* 0xFC9E */   L"\x0628\x062E",
      /* 0xFC9F */   L"\x0628\x0645",
      /* 0xFCA0 */   L"\x0628\x0647",
      /* 0xFCA1 */   L"\x062A\x062C",
      /* 0xFCA2 */   L"\x062A\x062D",
      /* 0xFCA3 */   L"\x062A\x062E",
      /* 0xFCA4 */   L"\x062A\x0645",
      /* 0xFCA5 */   L"\x062A\x0647",
      /* 0xFCA6 */   L"\x062B\x0645",
      /* 0xFCA7 */   L"\x062C\x062D",
      /* 0xFCA8 */   L"\x062C\x0645",
      /* 0xFCA9 */   L"\x062D\x062C",
      /* 0xFCAA */   L"\x062D\x0645",
      /* 0xFCAB */   L"\x062E\x062C",
      /* 0xFCAC */   L"\x062E\x0645",
      /* 0xFCAD */   L"\x0633\x062C",
      /* 0xFCAE */   L"\x0633\x062D",
      /* 0xFCAF */   L"\x0633\x062E",
      /* 0xFCB0 */   L"\x0633\x0645",
      /* 0xFCB1 */   L"\x0635\x062D",
      /* 0xFCB2 */   L"\x0635\x062E",
      /* 0xFCB3 */   L"\x0635\x0645",
      /* 0xFCB4 */   L"\x0636\x062C",
      /* 0xFCB5 */   L"\x0636\x062D",
      /* 0xFCB6 */   L"\x0636\x062E",
      /* 0xFCB7 */   L"\x0636\x0645",
      /* 0xFCB8 */   L"\x0637\x062D",
      /* 0xFCB9 */   L"\x0638\x0645",
      /* 0xFCBA */   L"\x0639\x062C",
      /* 0xFCBB */   L"\x0639\x0645",
      /* 0xFCBC */   L"\x063A\x062C",
      /* 0xFCBD */   L"\x063A\x0645",
      /* 0xFCBE */   L"\x0641\x062C",
      /* 0xFCBF */   L"\x0641\x062D",
      /* 0xFCC0 */   L"\x0641\x062E",
      /* 0xFCC1 */   L"\x0641\x0645",
      /* 0xFCC2 */   L"\x0642\x062D",
      /* 0xFCC3 */   L"\x0642\x0645",
      /* 0xFCC4 */   L"\x0643\x062C",
      /* 0xFCC5 */   L"\x0643\x062D",
      /* 0xFCC6 */   L"\x0643\x062E",
      /* 0xFCC7 */   L"\x0643\x0644",
      /* 0xFCC8 */   L"\x0643\x0645",
      /* 0xFCC9 */   L"\x0644\x062C",
      /* 0xFCCA */   L"\x0644\x062D",
      /* 0xFCCB */   L"\x0644\x062E",
      /* 0xFCCC */   L"\x0644\x0645",
      /* 0xFCCD */   L"\x0644\x0647",
      /* 0xFCCE */   L"\x0645\x062C",
      /* 0xFCCF */   L"\x0645\x062D",
      /* 0xFCD0 */   L"\x0645\x062E",
      /* 0xFCD1 */   L"\x0645\x0645",
      /* 0xFCD2 */   L"\x0646\x062C",
      /* 0xFCD3 */   L"\x0646\x062D",
      /* 0xFCD4 */   L"\x0646\x062E",
      /* 0xFCD5 */   L"\x0646\x0645",
      /* 0xFCD6 */   L"\x0646\x0647",
      /* 0xFCD7 */   L"\x0647\x062C",
      /* 0xFCD8 */   L"\x0647\x0645",
      /* 0xFCD9 */   L"\x0647\x0670",
      /* 0xFCDA */   L"\x064A\x062C",
      /* 0xFCDB */   L"\x064A\x062D",
      /* 0xFCDC */   L"\x064A\x062E",
      /* 0xFCDD */   L"\x064A\x0645",
      /* 0xFCDE */   L"\x064A\x0647",
      /* 0xFCDF */   L"\x0626\x0645",
      /* 0xFCE0 */   L"\x0626\x0647",
      /* 0xFCE1 */   L"\x0628\x0645",
      /* 0xFCE2 */   L"\x0628\x0647",
      /* 0xFCE3 */   L"\x062A\x0645",
      /* 0xFCE4 */   L"\x062A\x0647",
      /* 0xFCE5 */   L"\x062B\x0645",
      /* 0xFCE6 */   L"\x062B\x0647",
      /* 0xFCE7 */   L"\x0633\x0645",
      /* 0xFCE8 */   L"\x0633\x0647",
      /* 0xFCE9 */   L"\x0634\x0645",
      /* 0xFCEA */   L"\x0634\x0647",
      /* 0xFCEB */   L"\x0643\x0644",
      /* 0xFCEC */   L"\x0643\x0645",
      /* 0xFCED */   L"\x0644\x0645",
      /* 0xFCEE */   L"\x0646\x0645",
      /* 0xFCEF */   L"\x0646\x0647",
      /* 0xFCF0 */   L"\x064A\x0645",
      /* 0xFCF1 */   L"\x064A\x0647",
      /* 0xFCF2 */   L"\x0640\x064E\x0651",
      /* 0xFCF3 */   L"\x0640\x064F\x0651",
      /* 0xFCF4 */   L"\x0640\x0650\x0651",
      /* 0xFCF5 */   L"\x0637\x0649",
      /* 0xFCF6 */   L"\x0637\x064A",
      /* 0xFCF7 */   L"\x0639\x0649",
      /* 0xFCF8 */   L"\x0639\x064A",
      /* 0xFCF9 */   L"\x063A\x0649",
      /* 0xFCFA */   L"\x063A\x064A",
      /* 0xFCFB */   L"\x0633\x0649",
      /* 0xFCFC */   L"\x0633\x064A",
      /* 0xFCFD */   L"\x0634\x0649",
      /* 0xFCFE */   L"\x0634\x064A",
      /* 0xFCFF */   L"\x062D\x0649",
   } ;


   static const wchar_t *const Table_FD00 [] = {
      /* 0xFD00 */   L"\x062D\x064A",
      /* 0xFD01 */   L"\x062C\x0649",
      /* 0xFD02 */   L"\x062C\x064A",
      /* 0xFD03 */   L"\x062E\x0649",
      /* 0xFD04 */   L"\x062E\x064A",
      /* 0xFD05 */   L"\x0635\x0649",
      /* 0xFD06 */   L"\x0635\x064A",
      /* 0xFD07 */   L"\x0636\x0649",
      /* 0xFD08 */   L"\x0636\x064A",
      /* 0xFD09 */   L"\x0634\x062C",
      /* 0xFD0A */   L"\x0634\x062D",
      /* 0xFD0B */   L"\x0634\x062E",
      /* 0xFD0C */   L"\x0634\x0645",
      /* 0xFD0D */   L"\x0634\x0631",
      /* 0xFD0E */   L"\x0633\x0631",
      /* 0xFD0F */   L"\x0635\x0631",
      /* 0xFD10 */   L"\x0636\x0631",
      /* 0xFD11 */   L"\x0637\x0649",
      /* 0xFD12 */   L"\x0637\x064A",
      /* 0xFD13 */   L"\x0639\x0649",
      /* 0xFD14 */   L"\x0639\x064A",
      /* 0xFD15 */   L"\x063A\x0649",
      /* 0xFD16 */   L"\x063A\x064A",
      /* 0xFD17 */   L"\x0633\x0649",
      /* 0xFD18 */   L"\x0633\x064A",
      /* 0xFD19 */   L"\x0634\x0649",
      /* 0xFD1A */   L"\x0634\x064A",
      /* 0xFD1B */   L"\x062D\x0649",
      /* 0xFD1C */   L"\x062D\x064A",
      /* 0xFD1D */   L"\x062C\x0649",
      /* 0xFD1E */   L"\x062C\x064A",
      /* 0xFD1F */   L"\x062E\x0649",
      /* 0xFD20 */   L"\x062E\x064A",
      /* 0xFD21 */   L"\x0635\x0649",
      /* 0xFD22 */   L"\x0635\x064A",
      /* 0xFD23 */   L"\x0636\x0649",
      /* 0xFD24 */   L"\x0636\x064A",
      /* 0xFD25 */   L"\x0634\x062C",
      /* 0xFD26 */   L"\x0634\x062D",
      /* 0xFD27 */   L"\x0634\x062E",
      /* 0xFD28 */   L"\x0634\x0645",
      /* 0xFD29 */   L"\x0634\x0631",
      /* 0xFD2A */   L"\x0633\x0631",
      /* 0xFD2B */   L"\x0635\x0631",
      /* 0xFD2C */   L"\x0636\x0631",
      /* 0xFD2D */   L"\x0634\x062C",
      /* 0xFD2E */   L"\x0634\x062D",
      /* 0xFD2F */   L"\x0634\x062E",
      /* 0xFD30 */   L"\x0634\x0645",
      /* 0xFD31 */   L"\x0633\x0647",
      /* 0xFD32 */   L"\x0634\x0647",
      /* 0xFD33 */   L"\x0637\x0645",
      /* 0xFD34 */   L"\x0633\x062C",
      /* 0xFD35 */   L"\x0633\x062D",
      /* 0xFD36 */   L"\x0633\x062E",
      /* 0xFD37 */   L"\x0634\x062C",
      /* 0xFD38 */   L"\x0634\x062D",
      /* 0xFD39 */   L"\x0634\x062E",
      /* 0xFD3A */   L"\x0637\x0645",
      /* 0xFD3B */   L"\x0638\x0645",
      /* 0xFD3C */   L"\x0627\x064B",
      /* 0xFD3D */   L"\x0627\x064B",
      /* 0xFD3E */   NULL,
      /* 0xFD3F */   NULL,
      /* 0xFD40 */   NULL,
      /* 0xFD41 */   NULL,
      /* 0xFD42 */   NULL,
      /* 0xFD43 */   NULL,
      /* 0xFD44 */   NULL,
      /* 0xFD45 */   NULL,
      /* 0xFD46 */   NULL,
      /* 0xFD47 */   NULL,
      /* 0xFD48 */   NULL,
      /* 0xFD49 */   NULL,
      /* 0xFD4A */   NULL,
      /* 0xFD4B */   NULL,
      /* 0xFD4C */   NULL,
      /* 0xFD4D */   NULL,
      /* 0xFD4E */   NULL,
      /* 0xFD4F */   NULL,
      /* 0xFD50 */   L"\x062A\x062C\x0645",
      /* 0xFD51 */   L"\x062A\x062D\x062C",
      /* 0xFD52 */   L"\x062A\x062D\x062C",
      /* 0xFD53 */   L"\x062A\x062D\x0645",
      /* 0xFD54 */   L"\x062A\x062E\x0645",
      /* 0xFD55 */   L"\x062A\x0645\x062C",
      /* 0xFD56 */   L"\x062A\x0645\x062D",
      /* 0xFD57 */   L"\x062A\x0645\x062E",
      /* 0xFD58 */   L"\x062C\x0645\x062D",
      /* 0xFD59 */   L"\x062C\x0645\x062D",
      /* 0xFD5A */   L"\x062D\x0645\x064A",
      /* 0xFD5B */   L"\x062D\x0645\x0649",
      /* 0xFD5C */   L"\x0633\x062D\x062C",
      /* 0xFD5D */   L"\x0633\x062C\x062D",
      /* 0xFD5E */   L"\x0633\x062C\x0649",
      /* 0xFD5F */   L"\x0633\x0645\x062D",
      /* 0xFD60 */   L"\x0633\x0645\x062D",
      /* 0xFD61 */   L"\x0633\x0645\x062C",
      /* 0xFD62 */   L"\x0633\x0645\x0645",
      /* 0xFD63 */   L"\x0633\x0645\x0645",
      /* 0xFD64 */   L"\x0635\x062D\x062D",
      /* 0xFD65 */   L"\x0635\x062D\x062D",
      /* 0xFD66 */   L"\x0635\x0645\x0645",
      /* 0xFD67 */   L"\x0634\x062D\x0645",
      /* 0xFD68 */   L"\x0634\x062D\x0645",
      /* 0xFD69 */   L"\x0634\x062C\x064A",
      /* 0xFD6A */   L"\x0634\x0645\x062E",
      /* 0xFD6B */   L"\x0634\x0645\x062E",
      /* 0xFD6C */   L"\x0634\x0645\x0645",
      /* 0xFD6D */   L"\x0634\x0645\x0645",
      /* 0xFD6E */   L"\x0636\x062D\x0649",
      /* 0xFD6F */   L"\x0636\x062E\x0645",
      /* 0xFD70 */   L"\x0636\x062E\x0645",
      /* 0xFD71 */   L"\x0637\x0645\x062D",
      /* 0xFD72 */   L"\x0637\x0645\x062D",
      /* 0xFD73 */   L"\x0637\x0645\x0645",
      /* 0xFD74 */   L"\x0637\x0645\x064A",
      /* 0xFD75 */   L"\x0639\x062C\x0645",
      /* 0xFD76 */   L"\x0639\x0645\x0645",
      /* 0xFD77 */   L"\x0639\x0645\x0645",
      /* 0xFD78 */   L"\x0639\x0645\x0649",
      /* 0xFD79 */   L"\x063A\x0645\x0645",
      /* 0xFD7A */   L"\x063A\x0645\x064A",
      /* 0xFD7B */   L"\x063A\x0645\x0649",
      /* 0xFD7C */   L"\x0641\x062E\x0645",
      /* 0xFD7D */   L"\x0641\x062E\x0645",
      /* 0xFD7E */   L"\x0642\x0645\x062D",
      /* 0xFD7F */   L"\x0642\x0645\x0645",
      /* 0xFD80 */   L"\x0644\x062D\x0645",
      /* 0xFD81 */   L"\x0644\x062D\x064A",
      /* 0xFD82 */   L"\x0644\x062D\x0649",
      /* 0xFD83 */   L"\x0644\x062C\x062C",
      /* 0xFD84 */   L"\x0644\x062C\x062C",
      /* 0xFD85 */   L"\x0644\x062E\x0645",
      /* 0xFD86 */   L"\x0644\x062E\x0645",
      /* 0xFD87 */   L"\x0644\x0645\x062D",
      /* 0xFD88 */   L"\x0644\x0645\x062D",
      /* 0xFD89 */   L"\x0645\x062D\x062C",
      /* 0xFD8A */   L"\x0645\x062D\x0645",
      /* 0xFD8B */   L"\x0645\x062D\x064A",
      /* 0xFD8C */   L"\x0645\x062C\x062D",
      /* 0xFD8D */   L"\x0645\x062C\x0645",
      /* 0xFD8E */   L"\x0645\x062E\x062C",
      /* 0xFD8F */   L"\x0645\x062E\x0645",
      /* 0xFD90 */   NULL,
      /* 0xFD91 */   NULL,
      /* 0xFD92 */   L"\x0645\x062C\x062E",
      /* 0xFD93 */   L"\x0647\x0645\x062C",
      /* 0xFD94 */   L"\x0647\x0645\x0645",
      /* 0xFD95 */   L"\x0646\x062D\x0645",
      /* 0xFD96 */   L"\x0646\x062D\x0649",
      /* 0xFD97 */   L"\x0646\x062C\x0645",
      /* 0xFD98 */   L"\x0646\x062C\x0645",
      /* 0xFD99 */   L"\x0646\x062C\x0649",
      /* 0xFD9A */   L"\x0646\x0645\x064A",
      /* 0xFD9B */   L"\x0646\x0645\x0649",
      /* 0xFD9C */   L"\x064A\x0645\x0645",
      /* 0xFD9D */   L"\x064A\x0645\x0645",
      /* 0xFD9E */   L"\x0628\x062E\x064A",
      /* 0xFD9F */   L"\x062A\x062C\x064A",
      /* 0xFDA0 */   L"\x062A\x062C\x0649",
      /* 0xFDA1 */   L"\x062A\x062E\x064A",
      /* 0xFDA2 */   L"\x062A\x062E\x0649",
      /* 0xFDA3 */   L"\x062A\x0645\x064A",
      /* 0xFDA4 */   L"\x062A\x0645\x0649",
      /* 0xFDA5 */   L"\x062C\x0645\x064A",
      /* 0xFDA6 */   L"\x062C\x062D\x0649",
      /* 0xFDA7 */   L"\x062C\x0645\x0649",
      /* 0xFDA8 */   L"\x0633\x062E\x0649",
      /* 0xFDA9 */   L"\x0635\x062D\x064A",
      /* 0xFDAA */   L"\x0634\x062D\x064A",
      /* 0xFDAB */   L"\x0636\x062D\x064A",
      /* 0xFDAC */   L"\x0644\x062C\x064A",
      /* 0xFDAD */   L"\x0644\x0645\x064A",
      /* 0xFDAE */   L"\x064A\x062D\x064A",
      /* 0xFDAF */   L"\x064A\x062C\x064A",
      /* 0xFDB0 */   L"\x064A\x0645\x064A",
      /* 0xFDB1 */   L"\x0645\x0645\x064A",
      /* 0xFDB2 */   L"\x0642\x0645\x064A",
      /* 0xFDB3 */   L"\x0646\x062D\x064A",
      /* 0xFDB4 */   L"\x0642\x0645\x062D",
      /* 0xFDB5 */   L"\x0644\x062D\x0645",
      /* 0xFDB6 */   L"\x0639\x0645\x064A",
      /* 0xFDB7 */   L"\x0643\x0645\x064A",
      /* 0xFDB8 */   L"\x0646\x062C\x062D",
      /* 0xFDB9 */   L"\x0645\x062E\x064A",
      /* 0xFDBA */   L"\x0644\x062C\x0645",
      /* 0xFDBB */   L"\x0643\x0645\x0645",
      /* 0xFDBC */   L"\x0644\x062C\x0645",
      /* 0xFDBD */   L"\x0646\x062C\x062D",
      /* 0xFDBE */   L"\x062C\x062D\x064A",
      /* 0xFDBF */   L"\x062D\x062C\x064A",
      /* 0xFDC0 */   L"\x0645\x062C\x064A",
      /* 0xFDC1 */   L"\x0641\x0645\x064A",
      /* 0xFDC2 */   L"\x0628\x062D\x064A",
      /* 0xFDC3 */   L"\x0643\x0645\x0645",
      /* 0xFDC4 */   L"\x0639\x062C\x0645",
      /* 0xFDC5 */   L"\x0635\x0645\x0645",
      /* 0xFDC6 */   L"\x0633\x062E\x064A",
      /* 0xFDC7 */   L"\x0646\x062C\x064A",
      /* 0xFDC8 */   NULL,
      /* 0xFDC9 */   NULL,
      /* 0xFDCA */   NULL,
      /* 0xFDCB */   NULL,
      /* 0xFDCC */   NULL,
      /* 0xFDCD */   NULL,
      /* 0xFDCE */   NULL,
      /* 0xFDCF */   NULL,
      /* 0xFDD0 */   NULL,
      /* 0xFDD1 */   NULL,
      /* 0xFDD2 */   NULL,
      /* 0xFDD3 */   NULL,
      /* 0xFDD4 */   NULL,
      /* 0xFDD5 */   NULL,
      /* 0xFDD6 */   NULL,
      /* 0xFDD7 */   NULL,
      /* 0xFDD8 */   NULL,
      /* 0xFDD9 */   NULL,
      /* 0xFDDA */   NULL,
      /* 0xFDDB */   NULL,
      /* 0xFDDC */   NULL,
      /* 0xFDDD */   NULL,
      /* 0xFDDE */   NULL,
      /* 0xFDDF */   NULL,
      /* 0xFDE0 */   NULL,
      /* 0xFDE1 */   NULL,
      /* 0xFDE2 */   NULL,
      /* 0xFDE3 */   NULL,
      /* 0xFDE4 */   NULL,
      /* 0xFDE5 */   NULL,
      /* 0xFDE6 */   NULL,
      /* 0xFDE7 */   NULL,
      /* 0xFDE8 */   NULL,
      /* 0xFDE9 */   NULL,
      /* 0xFDEA */   NULL,
      /* 0xFDEB */   NULL,
      /* 0xFDEC */   NULL,
      /* 0xFDED */   NULL,
      /* 0xFDEE */   NULL,
      /* 0xFDEF */   NULL,
      /* 0xFDF0 */   L"\x0635\x0644\x06D2",
      /* 0xFDF1 */   L"\x0642\x0644\x06D2",
      /* 0xFDF2 */   L"\x0627\x0644\x0644\x0647",
      /* 0xFDF3 */   L"\x0627\x0643\x0628\x0631",
      /* 0xFDF4 */   L"\x0645\x062D\x0645\x062F",
      /* 0xFDF5 */   L"\x0635\x0644\x0639\x0645",
      /* 0xFDF6 */   L"\x0631\x0633\x0648\x0644",
      /* 0xFDF7 */   L"\x0639\x0644\x064A\x0647",
      /* 0xFDF8 */   L"\x0648\x0633\x0644\x0645",
      /* 0xFDF9 */   L"\x0635\x0644\x0649",
      /* 0xFDFA */   L"\x0635\x0644\x0649 \x0627\x0644\x0644\x0647 \x0639\x0644\x064A\x0647 \x0648\x0633\x0644\x0645",
      /* 0xFDFB */   L"\x062C\x0644 \x062C\x0644\x0627\x0644\x0647",
      /* 0xFDFC */   L"\x0631\x06CC\x0627\x0644",
      /* 0xFDFD */   NULL,
      /* 0xFDFE */   NULL,
      /* 0xFDFF */   NULL,
   } ;


   static const wchar_t *const Table_FE00 [] = {
      /* 0xFE00 */   NULL,
      /* 0xFE01 */   NULL,
      /* 0xFE02 */   NULL,
      /* 0xFE03 */   NULL,
      /* 0xFE04 */   NULL,
      /* 0xFE05 */   NULL,
      /* 0xFE06 */   NULL,
      /* 0xFE07 */   NULL,
      /* 0xFE08 */   NULL,
      /* 0xFE09 */   NULL,
      /* 0xFE0A */   NULL,
      /* 0xFE0B */   NULL,
      /* 0xFE0C */   NULL,
      /* 0xFE0D */   NULL,
      /* 0xFE0E */   NULL,
      /* 0xFE0F */   NULL,
      /* 0xFE10 */   L",",
      /* 0xFE11 */   L"\x3001",     // "A"
      /* 0xFE12 */   L"\x3002",     // "B"
      /* 0xFE13 */   L":",
      /* 0xFE14 */   L";",
      /* 0xFE15 */   L"!",
      /* 0xFE16 */   L"?",
      /* 0xFE17 */   L"\x3016",
      /* 0xFE18 */   L"\x3017",
      /* 0xFE19 */   L"\x2026",     // "c"
      /* 0xFE1A */   NULL,
      /* 0xFE1B */   NULL,
      /* 0xFE1C */   NULL,
      /* 0xFE1D */   NULL,
      /* 0xFE1E */   NULL,
      /* 0xFE1F */   NULL,
      /* 0xFE20 */   NULL,
      /* 0xFE21 */   NULL,
      /* 0xFE22 */   NULL,
      /* 0xFE23 */   NULL,
      /* 0xFE24 */   NULL,
      /* 0xFE25 */   NULL,
      /* 0xFE26 */   NULL,
      /* 0xFE27 */   NULL,
      /* 0xFE28 */   NULL,
      /* 0xFE29 */   NULL,
      /* 0xFE2A */   NULL,
      /* 0xFE2B */   NULL,
      /* 0xFE2C */   NULL,
      /* 0xFE2D */   NULL,
      /* 0xFE2E */   NULL,
      /* 0xFE2F */   NULL,
      /* 0xFE30 */   L"\x2025",     // "d"
      /* 0xFE31 */   L"\x2014",
      /* 0xFE32 */   L"\x2013",
      /* 0xFE33 */   L"_",
      /* 0xFE34 */   NULL,
      /* 0xFE35 */   L"(",
      /* 0xFE36 */   L")",
      /* 0xFE37 */   L"{",
      /* 0xFE38 */   L"}",
      /* 0xFE39 */   L"\x3014",     // "k"
      /* 0xFE3A */   L"\x3015",     // "l"
      /* 0xFE3B */   L"\x3010",     // "y"
      /* 0xFE3C */   L"\x3011",     // "z"
      /* 0xFE3D */   L"\x300A",     // "s"
      /* 0xFE3E */   L"\x300B",     // "t"
      /* 0xFE3F */   L"\x3008",     // "q"
      /* 0xFE40 */   L"\x3009",     // "r"
      /* 0xFE41 */   L"\x300C",     // "u"
      /* 0xFE42 */   L"\x300D",     // "v"
      /* 0xFE43 */   L"\x300E",     // "w"
      /* 0xFE44 */   L"\x300F",     // "x"
      /* 0xFE45 */   NULL,
      /* 0xFE46 */   NULL,
      /* 0xFE47 */   L"[",
      /* 0xFE48 */   L"]",
      /* 0xFE49 */   NULL,
      /* 0xFE4A */   NULL,
      /* 0xFE4B */   NULL,
      /* 0xFE4C */   NULL,
      /* 0xFE4D */   NULL,
      /* 0xFE4E */   NULL,
      /* 0xFE4F */   NULL,
      /* 0xFE50 */   L",",
      /* 0xFE51 */   L"\x3001",     // "A"
      /* 0xFE52 */   L".",
      /* 0xFE53 */   NULL,
      /* 0xFE54 */   L";",
      /* 0xFE55 */   L":",
      /* 0xFE56 */   L"?",
      /* 0xFE57 */   L"!",
      /* 0xFE58 */   L"\x2014",
      /* 0xFE59 */   L"(",
      /* 0xFE5A */   L")",
      /* 0xFE5B */   L"{",
      /* 0xFE5C */   L"}",
      /* 0xFE5D */   L"\x3014",     // "k"
      /* 0xFE5E */   L"\x3015",     // "l"
      /* 0xFE5F */   L"#",
      /* 0xFE60 */   L"&",
      /* 0xFE61 */   L"*",
      /* 0xFE62 */   L"+",
      /* 0xFE63 */   L"-",
      /* 0xFE64 */   L"<",
      /* 0xFE65 */   L">",
      /* 0xFE66 */   L"=",
      /* 0xFE67 */   NULL,
      /* 0xFE68 */   L"\\",
      /* 0xFE69 */   L"$",
      /* 0xFE6A */   L"%",
      /* 0xFE6B */   L"@",
      /* 0xFE6C */   NULL,
      /* 0xFE6D */   NULL,
      /* 0xFE6E */   NULL,
      /* 0xFE6F */   NULL,
      /* 0xFE70 */   L" \x064B",
      /* 0xFE71 */   L"\x0640\x064B",
      /* 0xFE72 */   L" \x064C",
      /* 0xFE73 */   NULL,
      /* 0xFE74 */   L" \x064D",
      /* 0xFE75 */   NULL,
      /* 0xFE76 */   L" \x064E",
      /* 0xFE77 */   L"\x0640\x064E",
      /* 0xFE78 */   L" \x064F",
      /* 0xFE79 */   L"\x0640\x064F",
      /* 0xFE7A */   L" \x0650",
      /* 0xFE7B */   L"\x0640\x0650",
      /* 0xFE7C */   L" \x0651",
      /* 0xFE7D */   L"\x0640\x0651",
      /* 0xFE7E */   L" \x0652",
      /* 0xFE7F */   L"\x0640\x0652",
      /* 0xFE80 */   L"\x0621",
      /* 0xFE81 */   L"\x0622",
      /* 0xFE82 */   L"\x0622",
      /* 0xFE83 */   L"\x0623",
      /* 0xFE84 */   L"\x0623",
      /* 0xFE85 */   L"\x0624",
      /* 0xFE86 */   L"\x0624",
      /* 0xFE87 */   L"\x0625",
      /* 0xFE88 */   L"\x0625",
      /* 0xFE89 */   L"\x0626",
      /* 0xFE8A */   L"\x0626",
      /* 0xFE8B */   L"\x0626",
      /* 0xFE8C */   L"\x0626",
      /* 0xFE8D */   L"\x0627",
      /* 0xFE8E */   L"\x0627",
      /* 0xFE8F */   L"\x0628",
      /* 0xFE90 */   L"\x0628",
      /* 0xFE91 */   L"\x0628",
      /* 0xFE92 */   L"\x0628",
      /* 0xFE93 */   L"\x0629",
      /* 0xFE94 */   L"\x0629",
      /* 0xFE95 */   L"\x062A",
      /* 0xFE96 */   L"\x062A",
      /* 0xFE97 */   L"\x062A",
      /* 0xFE98 */   L"\x062A",
      /* 0xFE99 */   L"\x062B",
      /* 0xFE9A */   L"\x062B",
      /* 0xFE9B */   L"\x062B",
      /* 0xFE9C */   L"\x062B",
      /* 0xFE9D */   L"\x062C",
      /* 0xFE9E */   L"\x062C",
      /* 0xFE9F */   L"\x062C",
      /* 0xFEA0 */   L"\x062C",
      /* 0xFEA1 */   L"\x062D",
      /* 0xFEA2 */   L"\x062D",
      /* 0xFEA3 */   L"\x062D",
      /* 0xFEA4 */   L"\x062D",
      /* 0xFEA5 */   L"\x062E",
      /* 0xFEA6 */   L"\x062E",
      /* 0xFEA7 */   L"\x062E",
      /* 0xFEA8 */   L"\x062E",
      /* 0xFEA9 */   L"\x062F",
      /* 0xFEAA */   L"\x062F",
      /* 0xFEAB */   L"\x0630",
      /* 0xFEAC */   L"\x0630",
      /* 0xFEAD */   L"\x0631",
      /* 0xFEAE */   L"\x0631",
      /* 0xFEAF */   L"\x0632",
      /* 0xFEB0 */   L"\x0632",
      /* 0xFEB1 */   L"\x0633",
      /* 0xFEB2 */   L"\x0633",
      /* 0xFEB3 */   L"\x0633",
      /* 0xFEB4 */   L"\x0633",
      /* 0xFEB5 */   L"\x0634",
      /* 0xFEB6 */   L"\x0634",
      /* 0xFEB7 */   L"\x0634",
      /* 0xFEB8 */   L"\x0634",
      /* 0xFEB9 */   L"\x0635",
      /* 0xFEBA */   L"\x0635",
      /* 0xFEBB */   L"\x0635",
      /* 0xFEBC */   L"\x0635",
      /* 0xFEBD */   L"\x0636",
      /* 0xFEBE */   L"\x0636",
      /* 0xFEBF */   L"\x0636",
      /* 0xFEC0 */   L"\x0636",
      /* 0xFEC1 */   L"\x0637",
      /* 0xFEC2 */   L"\x0637",
      /* 0xFEC3 */   L"\x0637",
      /* 0xFEC4 */   L"\x0637",
      /* 0xFEC5 */   L"\x0638",
      /* 0xFEC6 */   L"\x0638",
      /* 0xFEC7 */   L"\x0638",
      /* 0xFEC8 */   L"\x0638",
      /* 0xFEC9 */   L"\x0639",
      /* 0xFECA */   L"\x0639",
      /* 0xFECB */   L"\x0639",
      /* 0xFECC */   L"\x0639",
      /* 0xFECD */   L"\x063A",
      /* 0xFECE */   L"\x063A",
      /* 0xFECF */   L"\x063A",
      /* 0xFED0 */   L"\x063A",
      /* 0xFED1 */   L"\x0641",
      /* 0xFED2 */   L"\x0641",
      /* 0xFED3 */   L"\x0641",
      /* 0xFED4 */   L"\x0641",
      /* 0xFED5 */   L"\x0642",
      /* 0xFED6 */   L"\x0642",
      /* 0xFED7 */   L"\x0642",
      /* 0xFED8 */   L"\x0642",
      /* 0xFED9 */   L"\x0643",
      /* 0xFEDA */   L"\x0643",
      /* 0xFEDB */   L"\x0643",
      /* 0xFEDC */   L"\x0643",
      /* 0xFEDD */   L"\x0644",
      /* 0xFEDE */   L"\x0644",
      /* 0xFEDF */   L"\x0644",
      /* 0xFEE0 */   L"\x0644",
      /* 0xFEE1 */   L"\x0645",
      /* 0xFEE2 */   L"\x0645",
      /* 0xFEE3 */   L"\x0645",
      /* 0xFEE4 */   L"\x0645",
      /* 0xFEE5 */   L"\x0646",
      /* 0xFEE6 */   L"\x0646",
      /* 0xFEE7 */   L"\x0646",
      /* 0xFEE8 */   L"\x0646",
      /* 0xFEE9 */   L"\x0647",
      /* 0xFEEA */   L"\x0647",
      /* 0xFEEB */   L"\x0647",
      /* 0xFEEC */   L"\x0647",
      /* 0xFEED */   L"\x0648",
      /* 0xFEEE */   L"\x0648",
      /* 0xFEEF */   L"\x0649",
      /* 0xFEF0 */   L"\x0649",
      /* 0xFEF1 */   L"\x064A",
      /* 0xFEF2 */   L"\x064A",
      /* 0xFEF3 */   L"\x064A",
      /* 0xFEF4 */   L"\x064A",
      /* 0xFEF5 */   L"\x0644\x0622",
      /* 0xFEF6 */   L"\x0644\x0622",
      /* 0xFEF7 */   L"\x0644\x0623",
      /* 0xFEF8 */   L"\x0644\x0623",
      /* 0xFEF9 */   L"\x0644\x0625",
      /* 0xFEFA */   L"\x0644\x0625",
      /* 0xFEFB */   L"\x0644\x0627",
      /* 0xFEFC */   L"\x0644\x0627",
      /* 0xFEFD */   NULL,
      /* 0xFEFE */   NULL,
      /* 0xFEFF */   NULL,
   } ;


   const wchar_t *const *pTable = NULL ;

   switch ( cUnicode >> 8 ) {
      case 0xFB00 >> 8 : pTable = Table_FB00 ; break ;
      case 0xFC00 >> 8 : pTable = Table_FC00 ; break ;
      case 0xFD00 >> 8 : pTable = Table_FD00 ; break ;
      case 0xFE00 >> 8 : pTable = Table_FE00 ; break ;
   }

   if ( pTable ) return pTable [ cUnicode & 0xFF ] ;
   return NULL ;
}



////////////////////////////////////////////
//                                //
////////////////////////////////////////////



// ʏ̕
const wchar_t *DecomposeUnicodeCharacter ( int cUnicode ) {


   static const wchar_t *const Table_00A0 [] = {
      /* 0x00A0 */   NULL,
      /* 0x00A1 */   NULL,
      /* 0x00A2 */   NULL,
      /* 0x00A3 */   NULL,
      /* 0x00A4 */   NULL,
      /* 0x00A5 */   NULL,
      /* 0x00A6 */   NULL,
      /* 0x00A7 */   NULL,
      /* 0x00A8 */   NULL,
      /* 0x00A9 */   L"(C)",
      /* 0x00AA */   NULL,
      /* 0x00AB */   NULL,
      /* 0x00AC */   NULL,
      /* 0x00AD */   L"",
      /* 0x00AE */   L"(R)",
      /* 0x00AF */   NULL,
   } ;


   static const wchar_t *const Table_00B0 [] = {
      /* 0x00B0 */   NULL,
      /* 0x00B1 */   NULL,
      /* 0x00B2 */   NULL,
      /* 0x00B3 */   NULL,
      /* 0x00B4 */   NULL,
      /* 0x00B5 */   NULL,
      /* 0x00B6 */   NULL,
      /* 0x00B7 */   NULL,
      /* 0x00B8 */   NULL,
      /* 0x00B9 */   NULL,
      /* 0x00BA */   NULL,
      /* 0x00BB */   NULL,
      /* 0x00BC */   L"1/4",
      /* 0x00BD */   L"1/2",
      /* 0x00BE */   L"3/4",
      /* 0x00BF */   NULL,
   } ;


   static const wchar_t *const Table_00C0 [] = {
      /* 0x00C0 */   NULL,
      /* 0x00C1 */   NULL,
      /* 0x00C2 */   NULL,
      /* 0x00C3 */   NULL,
      /* 0x00C4 */   NULL,
      /* 0x00C5 */   NULL,
      /* 0x00C6 */   L"AE",
      /* 0x00C7 */   NULL,
      /* 0x00C8 */   NULL,
      /* 0x00C9 */   NULL,
      /* 0x00CA */   NULL,
      /* 0x00CB */   NULL,
      /* 0x00CC */   NULL,
      /* 0x00CD */   NULL,
      /* 0x00CE */   NULL,
      /* 0x00CF */   NULL,
   } ;


   static const wchar_t *const Table_00D0 [] = {
      /* 0x00D0 */   NULL,
      /* 0x00D1 */   NULL,
      /* 0x00D2 */   NULL,
      /* 0x00D3 */   NULL,
      /* 0x00D4 */   NULL,
      /* 0x00D5 */   NULL,
      /* 0x00D6 */   NULL,
      /* 0x00D7 */   NULL,
      /* 0x00D8 */   NULL,
      /* 0x00D9 */   NULL,
      /* 0x00DA */   NULL,
      /* 0x00DB */   NULL,
      /* 0x00DC */   NULL,
      /* 0x00DD */   NULL,
      /* 0x00DE */   NULL,
      /* 0x00DF */   L"ss",
   } ;


   static const wchar_t *const Table_00E0 [] = {
      /* 0x00E0 */   NULL,
      /* 0x00E1 */   NULL,
      /* 0x00E2 */   NULL,
      /* 0x00E3 */   NULL,
      /* 0x00E4 */   NULL,
      /* 0x00E5 */   NULL,
      /* 0x00E6 */   L"ae",
      /* 0x00E7 */   NULL,
      /* 0x00E8 */   NULL,
      /* 0x00E9 */   NULL,
      /* 0x00EA */   NULL,
      /* 0x00EB */   NULL,
      /* 0x00EC */   NULL,
      /* 0x00ED */   NULL,
      /* 0x00EE */   NULL,
      /* 0x00EF */   NULL,
   } ;


   static const wchar_t *const Table_0130 [] = {
      /* 0x0130 */   NULL,
      /* 0x0131 */   NULL,
      /* 0x0132 */   L"IJ",
      /* 0x0133 */   L"ij",
      /* 0x0134 */   NULL,
      /* 0x0135 */   NULL,
      /* 0x0136 */   NULL,
      /* 0x0137 */   NULL,
      /* 0x0138 */   NULL,
      /* 0x0139 */   NULL,
      /* 0x013A */   NULL,
      /* 0x013B */   NULL,
      /* 0x013C */   NULL,
      /* 0x013D */   NULL,
      /* 0x013E */   NULL,
      /* 0x013F */   NULL,
   } ;


   static const wchar_t *const Table_0150 [] = {
      /* 0x0150 */   NULL,
      /* 0x0151 */   NULL,
      /* 0x0152 */   L"OE",
      /* 0x0153 */   L"oe",
      /* 0x0154 */   NULL,
      /* 0x0155 */   NULL,
      /* 0x0156 */   NULL,
      /* 0x0157 */   NULL,
      /* 0x0158 */   NULL,
      /* 0x0159 */   NULL,
      /* 0x015A */   NULL,
      /* 0x015B */   NULL,
      /* 0x015C */   NULL,
      /* 0x015D */   NULL,
      /* 0x015E */   NULL,
      /* 0x015F */   NULL,
   } ;


   static const wchar_t *const Table_01C0 [] = {
      /* 0x01C0 */   NULL,
      /* 0x01C1 */   NULL,
      /* 0x01C2 */   NULL,
      /* 0x01C3 */   NULL,
      /* 0x01C4 */   L"\x0044\x017D",
      /* 0x01C5 */   L"\x0044\x017E",
      /* 0x01C6 */   L"\x0064\x017E",
      /* 0x01C7 */   L"LJ",
      /* 0x01C8 */   L"Lj",
      /* 0x01C9 */   L"lj",
      /* 0x01CA */   L"NJ",
      /* 0x01CB */   L"Nj",
      /* 0x01CC */   L"nj",
      /* 0x01CD */   NULL,
      /* 0x01CE */   NULL,
      /* 0x01CF */   NULL,
   } ;


   static const wchar_t *const Table_01E0 [] = {
      /* 0x01E0 */   NULL,
      /* 0x01E1 */   NULL,
      /* 0x01E2 */   L"AE",
      /* 0x01E3 */   L"ae",
      /* 0x01E4 */   NULL,
      /* 0x01E5 */   NULL,
      /* 0x01E6 */   NULL,
      /* 0x01E7 */   NULL,
      /* 0x01E8 */   NULL,
      /* 0x01E9 */   NULL,
      /* 0x01EA */   NULL,
      /* 0x01EB */   NULL,
      /* 0x01EC */   NULL,
      /* 0x01ED */   NULL,
      /* 0x01EE */   NULL,
      /* 0x01EF */   NULL,
   } ;


   static const wchar_t *const Table_01F0 [] = {
      /* 0x01F0 */   NULL,
      /* 0x01F1 */   L"DZ",
      /* 0x01F2 */   L"Dz",
      /* 0x01F3 */   L"dz",
      /* 0x01F4 */   NULL,
      /* 0x01F5 */   NULL,
      /* 0x01F6 */   NULL,
      /* 0x01F7 */   NULL,
      /* 0x01F8 */   NULL,
      /* 0x01F9 */   NULL,
      /* 0x01FA */   NULL,
      /* 0x01FB */   NULL,
      /* 0x01FC */   L"AE",
      /* 0x01FD */   L"ae",
      /* 0x01FE */   NULL,
      /* 0x01FF */   NULL,
   } ;


   static const wchar_t *const Table_0400 [] = {
      /* 0x0400 */   NULL,
      /* 0x0401 */   NULL,
      /* 0x0402 */   NULL,
      /* 0x0403 */   NULL,
      /* 0x0404 */   NULL,
      /* 0x0405 */   NULL,
      /* 0x0406 */   NULL,
      /* 0x0407 */   NULL,
      /* 0x0408 */   NULL,
      /* 0x0409 */   L"\x041B\x042C",     // "L]"
      /* 0x040A */   L"\x041D\x042C",     // "N]"
      /* 0x040B */   NULL,
      /* 0x040C */   NULL,
      /* 0x040D */   NULL,
      /* 0x040E */   NULL,
      /* 0x040F */   NULL,
   } ;


   static const wchar_t *const Table_0450 [] = {
      /* 0x0450 */   NULL,
      /* 0x0451 */   NULL,
      /* 0x0452 */   NULL,
      /* 0x0453 */   NULL,
      /* 0x0454 */   NULL,
      /* 0x0455 */   NULL,
      /* 0x0456 */   NULL,
      /* 0x0457 */   NULL,
      /* 0x0458 */   NULL,
      /* 0x0459 */   L"\x043B\x044C",     // "|"
      /* 0x045A */   L"\x043D\x044C",     // "~"
      /* 0x045B */   NULL,
      /* 0x045C */   NULL,
      /* 0x045D */   NULL,
      /* 0x045E */   NULL,
      /* 0x045F */   NULL,
   } ;


   static const wchar_t *const Table_04D0 [] = {
      /* 0x04D0 */   NULL,
      /* 0x04D1 */   NULL,
      /* 0x04D2 */   NULL,
      /* 0x04D3 */   NULL,
      /* 0x04D4 */   L"\x0410\x0415",     // "@E"
      /* 0x04D5 */   L"\x0430\x0435",     // "pu"
      /* 0x04D6 */   NULL,
      /* 0x04D7 */   NULL,
      /* 0x04D8 */   NULL,
      /* 0x04D9 */   NULL,
      /* 0x04DA */   NULL,
      /* 0x04DB */   NULL,
      /* 0x04DC */   NULL,
      /* 0x04DD */   NULL,
      /* 0x04DE */   NULL,
      /* 0x04DF */   NULL,
   } ;


   static const wchar_t *const Table_1D00 [] = {
      /* 0x1D00 */   NULL,
      /* 0x1D01 */   L"AE",
      /* 0x1D02 */   NULL,
      /* 0x1D03 */   NULL,
      /* 0x1D04 */   NULL,
      /* 0x1D05 */   NULL,
      /* 0x1D06 */   NULL,
      /* 0x1D07 */   NULL,
      /* 0x1D08 */   NULL,
      /* 0x1D09 */   NULL,
      /* 0x1D0A */   NULL,
      /* 0x1D0B */   NULL,
      /* 0x1D0C */   NULL,
      /* 0x1D0D */   NULL,
      /* 0x1D0E */   NULL,
      /* 0x1D0F */   NULL,
   } ;


   static const wchar_t *const Table_1D20 [] = {
      /* 0x1D20 */   NULL,
      /* 0x1D21 */   NULL,
      /* 0x1D22 */   NULL,
      /* 0x1D23 */   NULL,
      /* 0x1D24 */   NULL,
      /* 0x1D25 */   NULL,
      /* 0x1D26 */   NULL,
      /* 0x1D27 */   NULL,
      /* 0x1D28 */   NULL,
      /* 0x1D29 */   NULL,
      /* 0x1D2A */   NULL,
      /* 0x1D2B */   NULL,
      /* 0x1D2C */   NULL,
      /* 0x1D2D */   L"AE",
      /* 0x1D2E */   NULL,
      /* 0x1D2F */   NULL,
   } ;


   static const wchar_t *const Table_1D40 [] = {
      /* 0x1D40 */   NULL,
      /* 0x1D41 */   NULL,
      /* 0x1D42 */   NULL,
      /* 0x1D43 */   NULL,
      /* 0x1D44 */   NULL,
      /* 0x1D45 */   NULL,
      /* 0x1D46 */   L"ae",
      /* 0x1D47 */   NULL,
      /* 0x1D48 */   NULL,
      /* 0x1D49 */   NULL,
      /* 0x1D4A */   NULL,
      /* 0x1D4B */   NULL,
      /* 0x1D4C */   NULL,
      /* 0x1D4D */   NULL,
      /* 0x1D4E */   NULL,
      /* 0x1D4F */   NULL,
   } ;


   static const wchar_t *const Table_1D60 [] = {
      /* 0x1D60 */   NULL,
      /* 0x1D61 */   NULL,
      /* 0x1D62 */   NULL,
      /* 0x1D63 */   NULL,
      /* 0x1D64 */   NULL,
      /* 0x1D65 */   NULL,
      /* 0x1D66 */   NULL,
      /* 0x1D67 */   NULL,
      /* 0x1D68 */   NULL,
      /* 0x1D69 */   NULL,
      /* 0x1D6A */   NULL,
      /* 0x1D6B */   L"ue",
      /* 0x1D6C */   NULL,
      /* 0x1D6D */   NULL,
      /* 0x1D6E */   NULL,
      /* 0x1D6F */   NULL,
   } ;


   static const wchar_t *const Table_2000 [] = {
      /* 0x2000 */   NULL,
      /* 0x2001 */   NULL,
      /* 0x2002 */   NULL,
      /* 0x2003 */   NULL,
      /* 0x2004 */   NULL,
      /* 0x2005 */   NULL,
      /* 0x2006 */   NULL,
      /* 0x2007 */   NULL,
      /* 0x2008 */   NULL,
      /* 0x2009 */   NULL,
      /* 0x200A */   NULL,
      /* 0x200B */   L"",
      /* 0x200C */   L"",
      /* 0x200D */   L"",
      /* 0x200E */   NULL,
      /* 0x200F */   NULL,
   } ;


   static const wchar_t *const Table_2030 [] = {
      /* 0x2030 */   NULL,
      /* 0x2031 */   NULL,
      /* 0x2032 */   NULL,
      /* 0x2033 */   NULL,
      /* 0x2034 */   NULL,
      /* 0x2035 */   NULL,
      /* 0x2036 */   NULL,
      /* 0x2037 */   NULL,
      /* 0x2038 */   NULL,
      /* 0x2039 */   NULL,
      /* 0x203A */   NULL,
      /* 0x203B */   NULL,
      /* 0x203C */   L"!!",
      /* 0x203D */   NULL,
      /* 0x203E */   NULL,
      /* 0x203F */   NULL,
   } ;


   static const wchar_t *const Table_2040 [] = {
      /* 0x2040 */   NULL,
      /* 0x2041 */   NULL,
      /* 0x2042 */   NULL,
      /* 0x2043 */   NULL,
      /* 0x2044 */   NULL,
      /* 0x2045 */   NULL,
      /* 0x2046 */   NULL,
      /* 0x2047 */   L"??",
      /* 0x2048 */   L"?!",
      /* 0x2049 */   L"!?",
      /* 0x204A */   NULL,
      /* 0x204B */   NULL,
      /* 0x204C */   NULL,
      /* 0x204D */   NULL,
      /* 0x204E */   NULL,
      /* 0x204F */   NULL,
   } ;


   static const wchar_t *const Table_2100 [] = {
      /* 0x2100 */   L"a/c",
      /* 0x2101 */   L"a/s",
      /* 0x2102 */   NULL,
      /* 0x2103 */   L"\x00B0\x0043",     // "C"
      /* 0x2104 */   NULL,
      /* 0x2105 */   L"c/o",
      /* 0x2106 */   L"c/u",
      /* 0x2107 */   NULL,
      /* 0x2108 */   NULL,
      /* 0x2109 */   L"\x00B0\x0046",     // "F"
      /* 0x210A */   NULL,
      /* 0x210B */   NULL,
      /* 0x210C */   NULL,
      /* 0x210D */   NULL,
      /* 0x210E */   NULL,
      /* 0x210F */   NULL,
   } ;


   static const wchar_t *const Table_2110 [] = {
      /* 0x2110 */   NULL,
      /* 0x2111 */   NULL,
      /* 0x2112 */   NULL,
      /* 0x2113 */   NULL,
      /* 0x2114 */   NULL,
      /* 0x2115 */   NULL,
      /* 0x2116 */   L"No.",
      /* 0x2117 */   NULL,
      /* 0x2118 */   NULL,
      /* 0x2119 */   NULL,
      /* 0x211A */   NULL,
      /* 0x211B */   NULL,
      /* 0x211C */   NULL,
      /* 0x211D */   NULL,
      /* 0x211E */   NULL,
      /* 0x211F */   NULL,
   } ;


   static const wchar_t *const Table_2120 [] = {
      /* 0x2120 */   L"SM",
      /* 0x2121 */   L"TEL",
      /* 0x2122 */   L"TM",
      /* 0x2123 */   NULL,
      /* 0x2124 */   NULL,
      /* 0x2125 */   NULL,
      /* 0x2126 */   NULL,
      /* 0x2127 */   NULL,
      /* 0x2128 */   NULL,
      /* 0x2129 */   NULL,
      /* 0x212A */   NULL,
      /* 0x212B */   NULL,
      /* 0x212C */   NULL,
      /* 0x212D */   NULL,
      /* 0x212E */   NULL,
      /* 0x212F */   NULL,
   } ;


   static const wchar_t *const Table_2130 [] = {
      /* 0x2130 */   NULL,
      /* 0x2131 */   NULL,
      /* 0x2132 */   NULL,
      /* 0x2133 */   NULL,
      /* 0x2134 */   NULL,
      /* 0x2135 */   NULL,
      /* 0x2136 */   NULL,
      /* 0x2137 */   NULL,
      /* 0x2138 */   NULL,
      /* 0x2139 */   NULL,
      /* 0x213A */   NULL,
      /* 0x213B */   L"FAX",
      /* 0x213C */   NULL,
      /* 0x213D */   NULL,
      /* 0x213E */   NULL,
      /* 0x213F */   NULL,
   } ;


   static const wchar_t *const Table_2150 [] = {
      /* 0x2150 */   L"1/7",
      /* 0x2151 */   L"1/9",
      /* 0x2152 */   L"1/10",
      /* 0x2153 */   L"1/3",
      /* 0x2154 */   L"2/3",
      /* 0x2155 */   L"1/5",
      /* 0x2156 */   L"2/5",
      /* 0x2157 */   L"3/5",
      /* 0x2158 */   L"4/5",
      /* 0x2159 */   L"1/6",
      /* 0x215A */   L"5/6",
      /* 0x215B */   L"1/8",
      /* 0x215C */   L"3/8",
      /* 0x215D */   L"5/8",
      /* 0x215E */   L"7/8",
      /* 0x215F */   L"1/",
   } ;


   static const wchar_t *const Table_2160 [] = {
      /* 0x2160 */   L"I",
      /* 0x2161 */   L"II",
      /* 0x2162 */   L"III",
      /* 0x2163 */   L"IV",
      /* 0x2164 */   L"V",
      /* 0x2165 */   L"VI",
      /* 0x2166 */   L"VII",
      /* 0x2167 */   L"VIII",
      /* 0x2168 */   L"IX",
      /* 0x2169 */   L"X",
      /* 0x216A */   L"XI",
      /* 0x216B */   L"XII",
      /* 0x216C */   NULL,
      /* 0x216D */   NULL,
      /* 0x216E */   NULL,
      /* 0x216F */   NULL,
   } ;


   static const wchar_t *const Table_2170 [] = {
      /* 0x2170 */   L"i",
      /* 0x2171 */   L"ii",
      /* 0x2172 */   L"iii",
      /* 0x2173 */   L"iv",
      /* 0x2174 */   L"v",
      /* 0x2175 */   L"vi",
      /* 0x2176 */   L"vii",
      /* 0x2177 */   L"viii",
      /* 0x2178 */   L"ix",
      /* 0x2179 */   L"x",
      /* 0x217A */   L"xi",
      /* 0x217B */   L"xii",
      /* 0x217C */   NULL,
      /* 0x217D */   NULL,
      /* 0x217E */   NULL,
      /* 0x217F */   NULL,
   } ;


   static const wchar_t *const Table_2180 [] = {
      /* 0x2180 */   NULL,
      /* 0x2181 */   NULL,
      /* 0x2182 */   NULL,
      /* 0x2183 */   NULL,
      /* 0x2184 */   NULL,
      /* 0x2185 */   NULL,
      /* 0x2186 */   NULL,
      /* 0x2187 */   NULL,
      /* 0x2188 */   NULL,
      /* 0x2189 */   L"0/3",
      /* 0x218A */   NULL,
      /* 0x218B */   NULL,
      /* 0x218C */   NULL,
      /* 0x218D */   NULL,
      /* 0x218E */   NULL,
      /* 0x218F */   NULL,
   } ;


   static const wchar_t *const Table_2400 [] = {
      /* 0x2400 */   L"(NUL)",
      /* 0x2401 */   L"(SOH)",
      /* 0x2402 */   L"(STX)",
      /* 0x2403 */   L"(ETX)",
      /* 0x2404 */   L"(EOT)",
      /* 0x2405 */   L"(ENQ)",
      /* 0x2406 */   L"(ACK)",
      /* 0x2407 */   L"(BEL)",
      /* 0x2408 */   L"(BS)",
      /* 0x2409 */   L"(HT)",
      /* 0x240A */   L"(LF)",
      /* 0x240B */   L"(VT)",
      /* 0x240C */   L"(FF)",
      /* 0x240D */   L"(CR)",
      /* 0x240E */   L"(SO)",
      /* 0x240F */   L"(SI)",
   } ;


   static const wchar_t *const Table_2410 [] = {
      /* 0x2410 */   L"(DLE)",
      /* 0x2411 */   L"(DC1)",
      /* 0x2412 */   L"(DC2)",
      /* 0x2413 */   L"(DC3)",
      /* 0x2414 */   L"(DC4)",
      /* 0x2415 */   L"(NAK)",
      /* 0x2416 */   L"(SYN)",
      /* 0x2417 */   L"(ETB)",
      /* 0x2418 */   L"(CAN)",
      /* 0x2419 */   L"(EM)",
      /* 0x241A */   L"(SUB)",
      /* 0x241B */   L"(ESC)",
      /* 0x241C */   L"(FS)",
      /* 0x241D */   L"(GS)",
      /* 0x241E */   L"(RS)",
      /* 0x241F */   L"(US)",
   } ;


   static const wchar_t *const Table_2420 [] = {
      /* 0x2420 */   L"(SP)",
      /* 0x2421 */   L"(DEL)",
      /* 0x2422 */   NULL,
      /* 0x2423 */   NULL,
      /* 0x2424 */   L"(NL)",
      /* 0x2425 */   NULL,
      /* 0x2426 */   NULL,
      /* 0x2427 */   NULL,
      /* 0x2428 */   NULL,
      /* 0x2429 */   NULL,
      /* 0x242A */   NULL,
      /* 0x242B */   NULL,
      /* 0x242C */   NULL,
      /* 0x242D */   NULL,
      /* 0x242E */   NULL,
      /* 0x242F */   NULL,
   } ;


   static const wchar_t *const Table_2460 [] = {
      /* 0x2460 */   L"(1)",
      /* 0x2461 */   L"(2)",
      /* 0x2462 */   L"(3)",
      /* 0x2463 */   L"(4)",
      /* 0x2464 */   L"(5)",
      /* 0x2465 */   L"(6)",
      /* 0x2466 */   L"(7)",
      /* 0x2467 */   L"(8)",
      /* 0x2468 */   L"(9)",
      /* 0x2469 */   L"(10)",
      /* 0x246A */   L"(11)",
      /* 0x246B */   L"(12)",
      /* 0x246C */   L"(13)",
      /* 0x246D */   L"(14)",
      /* 0x246E */   L"(15)",
      /* 0x246F */   L"(16)",
   } ;


   static const wchar_t *const Table_2470 [] = {
      /* 0x2470 */   L"(17)",
      /* 0x2471 */   L"(18)",
      /* 0x2472 */   L"(19)",
      /* 0x2473 */   L"(20)",
      /* 0x2474 */   L"(1)",
      /* 0x2475 */   L"(2)",
      /* 0x2476 */   L"(3)",
      /* 0x2477 */   L"(4)",
      /* 0x2478 */   L"(5)",
      /* 0x2479 */   L"(6)",
      /* 0x247A */   L"(7)",
      /* 0x247B */   L"(8)",
      /* 0x247C */   L"(9)",
      /* 0x247D */   L"(10)",
      /* 0x247E */   L"(11)",
      /* 0x247F */   L"(12)",
   } ;


   static const wchar_t *const Table_2480 [] = {
      /* 0x2480 */   L"(13)",
      /* 0x2481 */   L"(14)",
      /* 0x2482 */   L"(15)",
      /* 0x2483 */   L"(16)",
      /* 0x2484 */   L"(17)",
      /* 0x2485 */   L"(18)",
      /* 0x2486 */   L"(19)",
      /* 0x2487 */   L"(20)",
      /* 0x2488 */   L"1.",
      /* 0x2489 */   L"2.",
      /* 0x248A */   L"3.",
      /* 0x248B */   L"4.",
      /* 0x248C */   L"5.",
      /* 0x248D */   L"6.",
      /* 0x248E */   L"7.",
      /* 0x248F */   L"8.",
   } ;


   static const wchar_t *const Table_2490 [] = {
      /* 0x2490 */   L"9.",
      /* 0x2491 */   L"10.",
      /* 0x2492 */   L"11.",
      /* 0x2493 */   L"12.",
      /* 0x2494 */   L"13.",
      /* 0x2495 */   L"14.",
      /* 0x2496 */   L"15.",
      /* 0x2497 */   L"16.",
      /* 0x2498 */   L"17.",
      /* 0x2499 */   L"18.",
      /* 0x249A */   L"19.",
      /* 0x249B */   L"20.",
      /* 0x249C */   L"(a)",
      /* 0x249D */   L"(b)",
      /* 0x249E */   L"(c)",
      /* 0x249F */   L"(d)",
   } ;


   static const wchar_t *const Table_24A0 [] = {
      /* 0x24A0 */   L"(e)",
      /* 0x24A1 */   L"(f)",
      /* 0x24A2 */   L"(g)",
      /* 0x24A3 */   L"(h)",
      /* 0x24A4 */   L"(i)",
      /* 0x24A5 */   L"(j)",
      /* 0x24A6 */   L"(k)",
      /* 0x24A7 */   L"(l)",
      /* 0x24A8 */   L"(m)",
      /* 0x24A9 */   L"(n)",
      /* 0x24AA */   L"(o)",
      /* 0x24AB */   L"(p)",
      /* 0x24AC */   L"(q)",
      /* 0x24AD */   L"(r)",
      /* 0x24AE */   L"(s)",
      /* 0x24AF */   L"(t)",
   } ;


   static const wchar_t *const Table_24B0 [] = {
      /* 0x24B0 */   L"(u)",
      /* 0x24B1 */   L"(v)",
      /* 0x24B2 */   L"(w)",
      /* 0x24B3 */   L"(x)",
      /* 0x24B4 */   L"(y)",
      /* 0x24B5 */   L"(z)",
      /* 0x24B6 */   L"(A)",
      /* 0x24B7 */   L"(B)",
      /* 0x24B8 */   L"(C)",
      /* 0x24B9 */   L"(D)",
      /* 0x24BA */   L"(E)",
      /* 0x24BB */   L"(F)",
      /* 0x24BC */   L"(G)",
      /* 0x24BD */   L"(H)",
      /* 0x24BE */   L"(I)",
      /* 0x24BF */   L"(J)",
   } ;


   static const wchar_t *const Table_24C0 [] = {
      /* 0x24C0 */   L"(K)",
      /* 0x24C1 */   L"(L)",
      /* 0x24C2 */   L"(M)",
      /* 0x24C3 */   L"(N)",
      /* 0x24C4 */   L"(O)",
      /* 0x24C5 */   L"(P)",
      /* 0x24C6 */   L"(Q)",
      /* 0x24C7 */   L"(R)",
      /* 0x24C8 */   L"(S)",
      /* 0x24C9 */   L"(T)",
      /* 0x24CA */   L"(U)",
      /* 0x24CB */   L"(V)",
      /* 0x24CC */   L"(W)",
      /* 0x24CD */   L"(X)",
      /* 0x24CE */   L"(Y)",
      /* 0x24CF */   L"(Z)",
   } ;


   static const wchar_t *const Table_24D0 [] = {
      /* 0x24D0 */   L"(a)",
      /* 0x24D1 */   L"(b)",
      /* 0x24D2 */   L"(c)",
      /* 0x24D3 */   L"(d)",
      /* 0x24D4 */   L"(e)",
      /* 0x24D5 */   L"(f)",
      /* 0x24D6 */   L"(g)",
      /* 0x24D7 */   L"(h)",
      /* 0x24D8 */   L"(i)",
      /* 0x24D9 */   L"(j)",
      /* 0x24DA */   L"(k)",
      /* 0x24DB */   L"(l)",
      /* 0x24DC */   L"(m)",
      /* 0x24DD */   L"(n)",
      /* 0x24DE */   L"(o)",
      /* 0x24DF */   L"(p)",
   } ;


   static const wchar_t *const Table_24E0 [] = {
      /* 0x24E0 */   L"(q)",
      /* 0x24E1 */   L"(r)",
      /* 0x24E2 */   L"(s)",
      /* 0x24E3 */   L"(t)",
      /* 0x24E4 */   L"(u)",
      /* 0x24E5 */   L"(v)",
      /* 0x24E6 */   L"(w)",
      /* 0x24E7 */   L"(x)",
      /* 0x24E8 */   L"(y)",
      /* 0x24E9 */   L"(z)",
      /* 0x24EA */   L"(0)",
      /* 0x24EB */   L"(11)",
      /* 0x24EC */   L"(12)",
      /* 0x24ED */   L"(13)",
      /* 0x24EE */   L"(14)",
      /* 0x24EF */   L"(15)",
   } ;


   static const wchar_t *const Table_24F0 [] = {
      /* 0x24F0 */   L"(16)",
      /* 0x24F1 */   L"(17)",
      /* 0x24F2 */   L"(18)",
      /* 0x24F3 */   L"(19)",
      /* 0x24F4 */   L"(20)",
      /* 0x24F5 */   L"(1)",
      /* 0x24F6 */   L"(2)",
      /* 0x24F7 */   L"(3)",
      /* 0x24F8 */   L"(4)",
      /* 0x24F9 */   L"(5)",
      /* 0x24FA */   L"(6)",
      /* 0x24FB */   L"(7)",
      /* 0x24FC */   L"(8)",
      /* 0x24FD */   L"(9)",
      /* 0x24FE */   L"(10)",
      /* 0x24FF */   L"(0)",
   } ;


   static const wchar_t *const Table_2770 [] = {
      /* 0x2770 */   NULL,
      /* 0x2771 */   NULL,
      /* 0x2772 */   NULL,
      /* 0x2773 */   NULL,
      /* 0x2774 */   NULL,
      /* 0x2775 */   NULL,
      /* 0x2776 */   L"(1)",
      /* 0x2777 */   L"(2)",
      /* 0x2778 */   L"(3)",
      /* 0x2779 */   L"(4)",
      /* 0x277A */   L"(5)",
      /* 0x277B */   L"(6)",
      /* 0x277C */   L"(7)",
      /* 0x277D */   L"(8)",
      /* 0x277E */   L"(9)",
      /* 0x277F */   L"(10)",
   } ;


   static const wchar_t *const Table_2780 [] = {
      /* 0x2780 */   L"(1)",
      /* 0x2781 */   L"(2)",
      /* 0x2782 */   L"(3)",
      /* 0x2783 */   L"(4)",
      /* 0x2784 */   L"(5)",
      /* 0x2785 */   L"(6)",
      /* 0x2786 */   L"(7)",
      /* 0x2787 */   L"(8)",
      /* 0x2788 */   L"(9)",
      /* 0x2789 */   L"(10)",
      /* 0x278A */   L"(1)",
      /* 0x278B */   L"(2)",
      /* 0x278C */   L"(3)",
      /* 0x278D */   L"(4)",
      /* 0x278E */   L"(5)",
      /* 0x278F */   L"(6)",
   } ;


   static const wchar_t *const Table_2790 [] = {
      /* 0x2790 */   L"(7)",
      /* 0x2791 */   L"(8)",
      /* 0x2792 */   L"(9)",
      /* 0x2793 */   L"(10)",
      /* 0x2794 */   NULL,
      /* 0x2795 */   NULL,
      /* 0x2796 */   NULL,
      /* 0x2797 */   NULL,
      /* 0x2798 */   NULL,
      /* 0x2799 */   NULL,
      /* 0x279A */   NULL,
      /* 0x279B */   NULL,
      /* 0x279C */   NULL,
      /* 0x279D */   NULL,
      /* 0x279E */   NULL,
      /* 0x279F */   NULL,
   } ;


   static const wchar_t *const Table_30F0 [] = {
      /* 0x30F0 */   NULL,
      /* 0x30F1 */   NULL,
      /* 0x30F2 */   NULL,
      /* 0x30F3 */   NULL,
      /* 0x30F4 */   NULL,
      /* 0x30F5 */   NULL,
      /* 0x30F6 */   NULL,
      /* 0x30F7 */   L"\x30EF\x309B",     // "J"
      /* 0x30F8 */   L"\x30F0\x309B",     // "J"
      /* 0x30F9 */   L"\x30F1\x309B",     // "J"
      /* 0x30FA */   L"\x30F2\x309B",     // "J"
      /* 0x30FB */   NULL,
      /* 0x30FC */   NULL,
      /* 0x30FD */   NULL,
      /* 0x30FE */   NULL,
      /* 0x30FF */   L"\x30B3\x30C8",     // "Rg"
   } ;


   static const wchar_t *const Table_3190 [] = {
      /* 0x3190 */   L"\xFF5C",     // "b"
      /* 0x3191 */   L"\x30EC",     // ""
      /* 0x3192 */   NULL,
      /* 0x3193 */   NULL,
      /* 0x3194 */   NULL,
      /* 0x3195 */   NULL,
      /* 0x3196 */   NULL,
      /* 0x3197 */   NULL,
      /* 0x3198 */   NULL,
      /* 0x3199 */   NULL,
      /* 0x319A */   NULL,
      /* 0x319B */   NULL,
      /* 0x319C */   NULL,
      /* 0x319D */   NULL,
      /* 0x319E */   NULL,
      /* 0x319F */   NULL,
   } ;


   static const wchar_t *const Table_3200 [] = {
      /* 0x3200 */   L"(\x3131)",
      /* 0x3201 */   L"(\x3134)",
      /* 0x3202 */   L"(\x3137)",
      /* 0x3203 */   L"(\x3139)",
      /* 0x3204 */   L"(\x3141)",
      /* 0x3205 */   L"(\x3142)",
      /* 0x3206 */   L"(\x3145)",
      /* 0x3207 */   L"(\x3147)",
      /* 0x3208 */   L"(\x3148)",
      /* 0x3209 */   L"(\x314A)",
      /* 0x320A */   L"(\x314B)",
      /* 0x320B */   L"(\x314C)",
      /* 0x320C */   L"(\x314D)",
      /* 0x320D */   L"(\x314E)",
      /* 0x320E */   L"(\xAC00)",
      /* 0x320F */   L"(\xB098)",
   } ;


   static const wchar_t *const Table_3210 [] = {
      /* 0x3210 */   L"(\xB2E4)",
      /* 0x3211 */   L"(\xB77C)",
      /* 0x3212 */   L"(\xB9C8)",
      /* 0x3213 */   L"(\xBC14)",
      /* 0x3214 */   L"(\xC0AC)",
      /* 0x3215 */   L"(\xC544)",
      /* 0x3216 */   L"(\xC790)",
      /* 0x3217 */   L"(\xCC28)",
      /* 0x3218 */   L"(\xCE74)",
      /* 0x3219 */   L"(\xD0C0)",
      /* 0x321A */   L"(\xD30C)",
      /* 0x321B */   L"(\xD558)",
      /* 0x321C */   L"(\xC8FC)",
      /* 0x321D */   L"(\xC624\xC804)",
      /* 0x321E */   L"(\xC624\xD6C4)",
      /* 0x321F */   NULL,
   } ;


   static const wchar_t *const Table_3220 [] = {
      /* 0x3220 */   L"(\x4E00)",      // "()"
      /* 0x3221 */   L"(\x4E8C)",      // "()"
      /* 0x3222 */   L"(\x4E09)",      // "(O)"
      /* 0x3223 */   L"(\x56DB)",      // "(l)"
      /* 0x3224 */   L"(\x4E94)",      // "()"
      /* 0x3225 */   L"(\x516D)",      // "(Z)"
      /* 0x3226 */   L"(\x4E03)",      // "()"
      /* 0x3227 */   L"(\x516B)",      // "()"
      /* 0x3228 */   L"(\x4E5D)",      // "()"
      /* 0x3229 */   L"(\x5341)",      // "(\)"
      /* 0x322A */   L"(\x6708)",      // "()"
      /* 0x322B */   L"(\x706B)",      // "()"
      /* 0x322C */   L"(\x6C34)",      // "()"
      /* 0x322D */   L"(\x6728)",      // "()"
      /* 0x322E */   L"(\x91D1)",      // "()"
      /* 0x322F */   L"(\x571F)",      // "(y)"
   } ;


   static const wchar_t *const Table_3230 [] = {
      /* 0x3230 */   L"(\x65E5)",      // "()"
      /* 0x3231 */   L"(\x682A)",      // "()"
      /* 0x3232 */   L"(\x6709)",      // "(L)"
      /* 0x3233 */   L"(\x793E)",      // "()"
      /* 0x3234 */   L"(\x540D)",      // "()"
      /* 0x3235 */   L"(\x7279)",      // "()"
      /* 0x3236 */   L"(\x8CA1)",      // "()"
      /* 0x3237 */   L"(\x795D)",      // "(j)"
      /* 0x3238 */   L"(\x52B4)",      // "(J)"
      /* 0x3239 */   L"(\x4EE3)",      // "()"
      /* 0x323A */   L"(\x547C)",      // "()"
      /* 0x323B */   L"(\x5B66)",      // "(w)"
      /* 0x323C */   L"(\x76E3)",      // "()"
      /* 0x323D */   L"(\x4F01)",      // "()"
      /* 0x323E */   L"(\x8CC7)",      // "()"
      /* 0x323F */   L"(\x5354)",      // "()"
   } ;


   static const wchar_t *const Table_3240 [] = {
      /* 0x3240 */   L"(\x796D)",      // "()"
      /* 0x3241 */   L"(\x4F11)",      // "(x)"
      /* 0x3242 */   L"(\x81EA)",      // "()"
      /* 0x3243 */   L"(\x81F3)",      // "()"
      /* 0x3244 */   L"(\x554F)",      // "()"
      /* 0x3245 */   L"(\x5E7C)",      // "(c)"
      /* 0x3246 */   L"(\x6587)",      // "()"
      /* 0x3247 */   L"(\x7B8F)",      // "()"
      /* 0x3248 */   L"(10)",
      /* 0x3249 */   L"(20)",
      /* 0x324A */   L"(30)",
      /* 0x324B */   L"(40)",
      /* 0x324C */   L"(50)",
      /* 0x324D */   L"(60)",
      /* 0x324E */   L"(70)",
      /* 0x324F */   L"(80)",
   } ;


   static const wchar_t *const Table_3250 [] = {
      /* 0x3250 */   L"PTE",
      /* 0x3251 */   L"(21)",
      /* 0x3252 */   L"(22)",
      /* 0x3253 */   L"(23)",
      /* 0x3254 */   L"(24)",
      /* 0x3255 */   L"(25)",
      /* 0x3256 */   L"(26)",
      /* 0x3257 */   L"(27)",
      /* 0x3258 */   L"(28)",
      /* 0x3259 */   L"(29)",
      /* 0x325A */   L"(30)",
      /* 0x325B */   L"(31)",
      /* 0x325C */   L"(32)",
      /* 0x325D */   L"(33)",
      /* 0x325E */   L"(34)",
      /* 0x325F */   L"(35)",
   } ;


   static const wchar_t *const Table_3260 [] = {
      /* 0x3260 */   L"(\x3131)",
      /* 0x3261 */   L"(\x3134)",
      /* 0x3262 */   L"(\x3137)",
      /* 0x3263 */   L"(\x3139)",
      /* 0x3264 */   L"(\x3141)",
      /* 0x3265 */   L"(\x3142)",
      /* 0x3266 */   L"(\x3145)",
      /* 0x3267 */   L"(\x3147)",
      /* 0x3268 */   L"(\x3148)",
      /* 0x3269 */   L"(\x314A)",
      /* 0x326A */   L"(\x314B)",
      /* 0x326B */   L"(\x314C)",
      /* 0x326C */   L"(\x314D)",
      /* 0x326D */   L"(\x314E)",
      /* 0x326E */   L"(\xAC00)",
      /* 0x326F */   L"(\xB098)",
   } ;


   static const wchar_t *const Table_3270 [] = {
      /* 0x3270 */   L"(\xB2E4)",
      /* 0x3271 */   L"(\xB77C)",
      /* 0x3272 */   L"(\xB9C8)",
      /* 0x3273 */   L"(\xBC14)",
      /* 0x3274 */   L"(\xC0AC)",
      /* 0x3275 */   L"(\xC544)",
      /* 0x3276 */   L"(\xC790)",
      /* 0x3277 */   L"(\xCC28)",
      /* 0x3278 */   L"(\xCE74)",
      /* 0x3279 */   L"(\xD0C0)",
      /* 0x327A */   L"(\xD30C)",
      /* 0x327B */   L"(\xD558)",
      /* 0x327C */   L"(\xCC38\xACE0)",
      /* 0x327D */   L"(\xC8FC\xC758)",
      /* 0x327E */   L"(\xC6B0)",
      /* 0x327F */   L"KS",
   } ;


   static const wchar_t *const Table_3280 [] = {
      /* 0x3280 */   L"(\x4E00)",      // "()"
      /* 0x3281 */   L"(\x4E8C)",      // "()"
      /* 0x3282 */   L"(\x4E09)",      // "(O)"
      /* 0x3283 */   L"(\x56DB)",      // "(l)"
      /* 0x3284 */   L"(\x4E94)",      // "()"
      /* 0x3285 */   L"(\x516D)",      // "(Z)"
      /* 0x3286 */   L"(\x4E03)",      // "()"
      /* 0x3287 */   L"(\x516B)",      // "()"
      /* 0x3288 */   L"(\x4E5D)",      // "()"
      /* 0x3289 */   L"(\x5341)",      // "(\)"
      /* 0x328A */   L"(\x6708)",      // "()"
      /* 0x328B */   L"(\x706B)",      // "()"
      /* 0x328C */   L"(\x6C34)",      // "()"
      /* 0x328D */   L"(\x6728)",      // "()"
      /* 0x328E */   L"(\x91D1)",      // "()"
      /* 0x328F */   L"(\x571F)",      // "(y)"
   } ;


   static const wchar_t *const Table_3290 [] = {
      /* 0x3290 */   L"(\x65E5)",      // "()"
      /* 0x3291 */   L"(\x682A)",      // "()"
      /* 0x3292 */   L"(\x6709)",      // "(L)"
      /* 0x3293 */   L"(\x793E)",      // "()"
      /* 0x3294 */   L"(\x540D)",      // "()"
      /* 0x3295 */   L"(\x7279)",      // "()"
      /* 0x3296 */   L"(\x8CA1)",      // "()"
      /* 0x3297 */   L"(\x795D)",      // "(j)"
      /* 0x3298 */   L"(\x52B4)",      // "(J)"
      /* 0x3299 */   L"(\x79D8)",      // "()"
      /* 0x329A */   L"(\x7537)",      // "(j)"
      /* 0x329B */   L"(\x5973)",      // "()"
      /* 0x329C */   L"(\x9069)",      // "(K)"
      /* 0x329D */   L"(\x512A)",      // "(D)"
      /* 0x329E */   L"(\x5370)",      // "()"
      /* 0x329F */   L"(\x6CE8)",      // "()"
   } ;


   static const wchar_t *const Table_32A0 [] = {
      /* 0x32A0 */   L"(\x9805)",      // "()"
      /* 0x32A1 */   L"(\x4F11)",      // "(x)"
      /* 0x32A2 */   L"(\x5199)",      // "()"
      /* 0x32A3 */   L"(\x6B63)",      // "()"
      /* 0x32A4 */   L"(\x4E0A)",      // "()"
      /* 0x32A5 */   L"(\x4E2D)",      // "()"
      /* 0x32A6 */   L"(\x4E0B)",      // "()"
      /* 0x32A7 */   L"(\x5DE6)",      // "()"
      /* 0x32A8 */   L"(\x53F3)",      // "(E)"
      /* 0x32A9 */   L"(\x533B)",      // "()"
      /* 0x32AA */   L"(\x5B97)",      // "(@)"
      /* 0x32AB */   L"(\x5B66)",      // "(w)"
      /* 0x32AC */   L"(\x76E3)",      // "()"
      /* 0x32AD */   L"(\x4F01)",      // "()"
      /* 0x32AE */   L"(\x8CC7)",      // "()"
      /* 0x32AF */   L"(\x5354)",      // "()"
   } ;


   static const wchar_t *const Table_32B0 [] = {
      /* 0x32B0 */   L"(\x591C)",      // "()"
      /* 0x32B1 */   L"(36)",
      /* 0x32B2 */   L"(37)",
      /* 0x32B3 */   L"(38)",
      /* 0x32B4 */   L"(39)",
      /* 0x32B5 */   L"(40)",
      /* 0x32B6 */   L"(41)",
      /* 0x32B7 */   L"(42)",
      /* 0x32B8 */   L"(43)",
      /* 0x32B9 */   L"(44)",
      /* 0x32BA */   L"(45)",
      /* 0x32BB */   L"(46)",
      /* 0x32BC */   L"(47)",
      /* 0x32BD */   L"(48)",
      /* 0x32BE */   L"(49)",
      /* 0x32BF */   L"(50)",
   } ;


   static const wchar_t *const Table_32C0 [] = {
      /* 0x32C0 */   L"\x0031\x6708",     // "1"
      /* 0x32C1 */   L"\x0032\x6708",     // "2"
      /* 0x32C2 */   L"\x0033\x6708",     // "3"
      /* 0x32C3 */   L"\x0034\x6708",     // "4"
      /* 0x32C4 */   L"\x0035\x6708",     // "5"
      /* 0x32C5 */   L"\x0036\x6708",     // "6"
      /* 0x32C6 */   L"\x0037\x6708",     // "7"
      /* 0x32C7 */   L"\x0038\x6708",     // "8"
      /* 0x32C8 */   L"\x0039\x6708",     // "9"
      /* 0x32C9 */   L"\x0031\x0030\x6708",     // "10"
      /* 0x32CA */   L"\x0031\x0031\x6708",     // "11"
      /* 0x32CB */   L"\x0031\x0032\x6708",     // "12"
      /* 0x32CC */   L"Hg",
      /* 0x32CD */   L"erg",
      /* 0x32CE */   L"eV",
      /* 0x32CF */   L"LTD",
   } ;


   static const wchar_t *const Table_32D0 [] = {
      /* 0x32D0 */   L"(\x30A2)",      // "(A)"
      /* 0x32D1 */   L"(\x30A4)",      // "(C)"
      /* 0x32D2 */   L"(\x30A6)",      // "(E)"
      /* 0x32D3 */   L"(\x30A8)",      // "(G)"
      /* 0x32D4 */   L"(\x30AA)",      // "(I)"
      /* 0x32D5 */   L"(\x30AB)",      // "(J)"
      /* 0x32D6 */   L"(\x30AD)",      // "(L)"
      /* 0x32D7 */   L"(\x30AF)",      // "(N)"
      /* 0x32D8 */   L"(\x30B1)",      // "(P)"
      /* 0x32D9 */   L"(\x30B3)",      // "(R)"
      /* 0x32DA */   L"(\x30B5)",      // "(T)"
      /* 0x32DB */   L"(\x30B7)",      // "(V)"
      /* 0x32DC */   L"(\x30B9)",      // "(X)"
      /* 0x32DD */   L"(\x30BB)",      // "(Z)"
      /* 0x32DE */   L"(\x30BD)",      // "(\)"
      /* 0x32DF */   L"(\x30BF)",      // "(^)"
   } ;


   static const wchar_t *const Table_32E0 [] = {
      /* 0x32E0 */   L"(\x30C1)",      // "(`)"
      /* 0x32E1 */   L"(\x30C4)",      // "(c)"
      /* 0x32E2 */   L"(\x30C6)",      // "(e)"
      /* 0x32E3 */   L"(\x30C8)",      // "(g)"
      /* 0x32E4 */   L"(\x30CA)",      // "(i)"
      /* 0x32E5 */   L"(\x30CB)",      // "(j)"
      /* 0x32E6 */   L"(\x30CC)",      // "(k)"
      /* 0x32E7 */   L"(\x30CD)",      // "(l)"
      /* 0x32E8 */   L"(\x30CE)",      // "(m)"
      /* 0x32E9 */   L"(\x30CF)",      // "(n)"
      /* 0x32EA */   L"(\x30D2)",      // "(q)"
      /* 0x32EB */   L"(\x30D5)",      // "(t)"
      /* 0x32EC */   L"(\x30D8)",      // "(w)"
      /* 0x32ED */   L"(\x30DB)",      // "(z)"
      /* 0x32EE */   L"(\x30DE)",      // "(})"
      /* 0x32EF */   L"(\x30DF)",      // "(~)"
   } ;


   static const wchar_t *const Table_32F0 [] = {
      /* 0x32F0 */   L"(\x30E0)",      // "()"
      /* 0x32F1 */   L"(\x30E1)",      // "()"
      /* 0x32F2 */   L"(\x30E2)",      // "()"
      /* 0x32F3 */   L"(\x30E4)",      // "()"
      /* 0x32F4 */   L"(\x30E6)",      // "()"
      /* 0x32F5 */   L"(\x30E8)",      // "()"
      /* 0x32F6 */   L"(\x30E9)",      // "()"
      /* 0x32F7 */   L"(\x30EA)",      // "()"
      /* 0x32F8 */   L"(\x30EB)",      // "()"
      /* 0x32F9 */   L"(\x30EC)",      // "()"
      /* 0x32FA */   L"(\x30ED)",      // "()"
      /* 0x32FB */   L"(\x30EF)",      // "()"
      /* 0x32FC */   L"(\x30F0)",      // "()"
      /* 0x32FD */   L"(\x30F1)",      // "()"
      /* 0x32FE */   L"(\x30F2)",      // "()"
      /* 0x32FF */   NULL,
   } ;


   static const wchar_t *const Table_3300 [] = {
      /* 0x3300 */   L"\x30A2\x30D1\x30FC\x30C8",     // "Ap[g"
      /* 0x3301 */   L"\x30A2\x30EB\x30D5\x30A1",     // "At@"
      /* 0x3302 */   L"\x30A2\x30F3\x30DA\x30A2",     // "AyA"
      /* 0x3303 */   L"\x30A2\x30FC\x30EB",     // "A["
      /* 0x3304 */   L"\x30A4\x30CB\x30F3\x30B0",     // "CjO"
      /* 0x3305 */   L"\x30A4\x30F3\x30C1",     // "C`"
      /* 0x3306 */   L"\x30A6\x30A9\x30F3",     // "EH"
      /* 0x3307 */   L"\x30A8\x30B9\x30AF\x30FC\x30C9",     // "GXN[h"
      /* 0x3308 */   L"\x30A8\x30FC\x30AB\x30FC",     // "G[J["
      /* 0x3309 */   L"\x30AA\x30F3\x30B9",     // "IX"
      /* 0x330A */   L"\x30AA\x30FC\x30E0",     // "I["
      /* 0x330B */   L"\x30AB\x30A4\x30EA",     // "JC"
      /* 0x330C */   L"\x30AB\x30E9\x30C3\x30C8",     // "Jbg"
      /* 0x330D */   L"\x30AB\x30ED\x30EA\x30FC",     // "J["
      /* 0x330E */   L"\x30AC\x30ED\x30F3",     // "K"
      /* 0x330F */   L"\x30AC\x30F3\x30DE",     // "K}"
   } ;


   static const wchar_t *const Table_3310 [] = {
      /* 0x3310 */   L"\x30AE\x30AC",     // "MK"
      /* 0x3311 */   L"\x30AE\x30CB\x30FC",     // "Mj["
      /* 0x3312 */   L"\x30AD\x30E5\x30EA\x30FC",     // "L["
      /* 0x3313 */   L"\x30AE\x30EB\x30C0\x30FC",     // "M_["
      /* 0x3314 */   L"\x30AD\x30ED",     // "L"
      /* 0x3315 */   L"\x30AD\x30ED\x30B0\x30E9\x30E0",     // "LO"
      /* 0x3316 */   L"\x30AD\x30ED\x30E1\x30FC\x30C8\x30EB",     // "L[g"
      /* 0x3317 */   L"\x30AD\x30ED\x30EF\x30C3\x30C8",     // "Lbg"
      /* 0x3318 */   L"\x30B0\x30E9\x30E0",     // "O"
      /* 0x3319 */   L"\x30B0\x30E9\x30E0\x30C8\x30F3",     // "Og"
      /* 0x331A */   L"\x30AF\x30EB\x30BC\x30A4\x30ED",     // "N[C"
      /* 0x331B */   L"\x30AF\x30ED\x30FC\x30CD",     // "N[l"
      /* 0x331C */   L"\x30B1\x30FC\x30B9",     // "P[X"
      /* 0x331D */   L"\x30B3\x30EB\x30CA",     // "Ri"
      /* 0x331E */   L"\x30B3\x30FC\x30DD",     // "R[|"
      /* 0x331F */   L"\x30B5\x30A4\x30AF\x30EB",     // "TCN"
   } ;


   static const wchar_t *const Table_3320 [] = {
      /* 0x3320 */   L"\x30B5\x30F3\x30C1\x30FC\x30E0",     // "T`["
      /* 0x3321 */   L"\x30B7\x30EA\x30F3\x30B0",     // "VO"
      /* 0x3322 */   L"\x30BB\x30F3\x30C1",     // "Z`"
      /* 0x3323 */   L"\x30BB\x30F3\x30C8",     // "Zg"
      /* 0x3324 */   L"\x30C0\x30FC\x30B9",     // "_[X"
      /* 0x3325 */   L"\x30C7\x30B7",     // "fV"
      /* 0x3326 */   L"\x30C9\x30EB",     // "h"
      /* 0x3327 */   L"\x30C8\x30F3",     // "g"
      /* 0x3328 */   L"\x30CA\x30CE",     // "im"
      /* 0x3329 */   L"\x30CE\x30C3\x30C8",     // "mbg"
      /* 0x332A */   L"\x30CF\x30A4\x30C4",     // "nCc"
      /* 0x332B */   L"\x30D1\x30FC\x30BB\x30F3\x30C8",     // "p[Zg"
      /* 0x332C */   L"\x30D1\x30FC\x30C4",     // "p[c"
      /* 0x332D */   L"\x30D0\x30FC\x30EC\x30EB",     // "o["
      /* 0x332E */   L"\x30D4\x30A2\x30B9\x30C8\x30EB",     // "sAXg"
      /* 0x332F */   L"\x30D4\x30AF\x30EB",     // "sN"
   } ;


   static const wchar_t *const Table_3330 [] = {
      /* 0x3330 */   L"\x30D4\x30B3",     // "sR"
      /* 0x3331 */   L"\x30D3\x30EB",     // "r"
      /* 0x3332 */   L"\x30D5\x30A1\x30E9\x30C3\x30C9",     // "t@bh"
      /* 0x3333 */   L"\x30D5\x30A3\x30FC\x30C8",     // "tB[g"
      /* 0x3334 */   L"\x30D6\x30C3\x30B7\x30A7\x30EB",     // "ubVF"
      /* 0x3335 */   L"\x30D5\x30E9\x30F3",     // "t"
      /* 0x3336 */   L"\x30D8\x30AF\x30BF\x30FC\x30EB",     // "wN^["
      /* 0x3337 */   L"\x30DA\x30BD",     // "y\"
      /* 0x3338 */   L"\x30DA\x30CB\x30D2",     // "yjq"
      /* 0x3339 */   L"\x30D8\x30EB\x30C4",     // "wc"
      /* 0x333A */   L"\x30DA\x30F3\x30B9",     // "yX"
      /* 0x333B */   L"\x30DA\x30FC\x30B8",     // "y[W"
      /* 0x333C */   L"\x30D9\x30FC\x30BF",     // "x[^"
      /* 0x333D */   L"\x30DD\x30A4\x30F3\x30C8",     // "|Cg"
      /* 0x333E */   L"\x30DC\x30EB\x30C8",     // "{g"
      /* 0x333F */   L"\x30DB\x30F3",     // "z"
   } ;


   static const wchar_t *const Table_3340 [] = {
      /* 0x3340 */   L"\x30DD\x30F3\x30C9",     // "|h"
      /* 0x3341 */   L"\x30DB\x30FC\x30EB",     // "z["
      /* 0x3342 */   L"\x30DB\x30FC\x30F3",     // "z["
      /* 0x3343 */   L"\x30DE\x30A4\x30AF\x30ED",     // "}CN"
      /* 0x3344 */   L"\x30DE\x30A4\x30EB",     // "}C"
      /* 0x3345 */   L"\x30DE\x30C3\x30CF",     // "}bn"
      /* 0x3346 */   L"\x30DE\x30EB\x30AF",     // "}N"
      /* 0x3347 */   L"\x30DE\x30F3\x30B7\x30E7\x30F3",     // "}V"
      /* 0x3348 */   L"\x30DF\x30AF\x30ED\x30F3",     // "~N"
      /* 0x3349 */   L"\x30DF\x30EA",     // "~"
      /* 0x334A */   L"\x30DF\x30EA\x30D0\x30FC\x30EB",     // "~o["
      /* 0x334B */   L"\x30E1\x30AC",     // "K"
      /* 0x334C */   L"\x30E1\x30AC\x30C8\x30F3",     // "Kg"
      /* 0x334D */   L"\x30E1\x30FC\x30C8\x30EB",     // "[g"
      /* 0x334E */   L"\x30E4\x30FC\x30C9",     // "[h"
      /* 0x334F */   L"\x30E4\x30FC\x30EB",     // "["
   } ;


   static const wchar_t *const Table_3350 [] = {
      /* 0x3350 */   L"\x30E6\x30A2\x30F3",     // "A"
      /* 0x3351 */   L"\x30EA\x30C3\x30C8\x30EB",     // "bg"
      /* 0x3352 */   L"\x30EA\x30E9",     // ""
      /* 0x3353 */   L"\x30EB\x30D4\x30FC",     // "s["
      /* 0x3354 */   L"\x30EB\x30FC\x30D6\x30EB",     // "[u"
      /* 0x3355 */   L"\x30EC\x30E0",     // ""
      /* 0x3356 */   L"\x30EC\x30F3\x30C8\x30B2\x30F3",     // "gQ"
      /* 0x3357 */   L"\x30EF\x30C3\x30C8",     // "bg"
      /* 0x3358 */   L"\x0030\x70B9",     // "0_"
      /* 0x3359 */   L"\x0031\x70B9",     // "1_"
      /* 0x335A */   L"\x0032\x70B9",     // "2_"
      /* 0x335B */   L"\x0033\x70B9",     // "3_"
      /* 0x335C */   L"\x0034\x70B9",     // "4_"
      /* 0x335D */   L"\x0035\x70B9",     // "5_"
      /* 0x335E */   L"\x0036\x70B9",     // "6_"
      /* 0x335F */   L"\x0037\x70B9",     // "7_"
   } ;


   static const wchar_t *const Table_3360 [] = {
      /* 0x3360 */   L"\x0038\x70B9",     // "8_"
      /* 0x3361 */   L"\x0039\x70B9",     // "9_"
      /* 0x3362 */   L"\x0031\x0030\x70B9",     // "10_"
      /* 0x3363 */   L"\x0031\x0031\x70B9",     // "11_"
      /* 0x3364 */   L"\x0031\x0032\x70B9",     // "12_"
      /* 0x3365 */   L"\x0031\x0033\x70B9",     // "13_"
      /* 0x3366 */   L"\x0031\x0034\x70B9",     // "14_"
      /* 0x3367 */   L"\x0031\x0035\x70B9",     // "15_"
      /* 0x3368 */   L"\x0031\x0036\x70B9",     // "16_"
      /* 0x3369 */   L"\x0031\x0037\x70B9",     // "17_"
      /* 0x336A */   L"\x0031\x0038\x70B9",     // "18_"
      /* 0x336B */   L"\x0031\x0039\x70B9",     // "19_"
      /* 0x336C */   L"\x0032\x0030\x70B9",     // "20_"
      /* 0x336D */   L"\x0032\x0031\x70B9",     // "21_"
      /* 0x336E */   L"\x0032\x0032\x70B9",     // "22_"
      /* 0x336F */   L"\x0032\x0033\x70B9",     // "23_"
   } ;


   static const wchar_t *const Table_3370 [] = {
      /* 0x3370 */   L"\x0032\x0034\x70B9",     // "24_"
      /* 0x3371 */   L"hPa",
      /* 0x3372 */   L"da",
      /* 0x3373 */   L"AU",
      /* 0x3374 */   L"bar",
      /* 0x3375 */   L"oV",
      /* 0x3376 */   L"pc",
      /* 0x3377 */   L"dm",
      /* 0x3378 */   L"\x0064\x006D\x00B2",     // "dm2"
      /* 0x3379 */   L"\x0064\x006D\x00B3",     // "dm3"
      /* 0x337A */   L"IU",
      /* 0x337B */   L"\x5E73\x6210",     // ""
      /* 0x337C */   L"\x662D\x548C",     // "a"
      /* 0x337D */   L"\x5927\x6B63",     // "吳"
      /* 0x337E */   L"\x660E\x6CBB",     // ""
      /* 0x337F */   L"\x682A\x5F0F\x4F1A\x793E",     // ""
   } ;


   static const wchar_t *const Table_3380 [] = {
      /* 0x3380 */   L"pA",
      /* 0x3381 */   L"nA",
      /* 0x3382 */   L"\x03BC\x0041",     // "A"
      /* 0x3383 */   L"mA",
      /* 0x3384 */   L"kA",
      /* 0x3385 */   L"KB",
      /* 0x3386 */   L"MB",
      /* 0x3387 */   L"GB",
      /* 0x3388 */   L"cal",
      /* 0x3389 */   L"kcal",
      /* 0x338A */   L"pF",
      /* 0x338B */   L"nF",
      /* 0x338C */   L"\x03BC\x0046",     // "F"
      /* 0x338D */   L"\x03BC\x0067",     // "g"
      /* 0x338E */   L"mg",
      /* 0x338F */   L"kg",
   } ;


   static const wchar_t *const Table_3390 [] = {
      /* 0x3390 */   L"Hz",
      /* 0x3391 */   L"kHz",
      /* 0x3392 */   L"MHz",
      /* 0x3393 */   L"GHz",
      /* 0x3394 */   L"THz",
      /* 0x3395 */   L"\x03BC\x2113",
      /* 0x3396 */   L"\x006D\x2113",
      /* 0x3397 */   L"\x0064\x2113",
      /* 0x3398 */   L"\x006B\x2113",
      /* 0x3399 */   L"fm",
      /* 0x339A */   L"nm",
      /* 0x339B */   L"\x03BC\x006D",     // "m"
      /* 0x339C */   L"mm",
      /* 0x339D */   L"cm",
      /* 0x339E */   L"km",
      /* 0x339F */   L"\x006D\x006D\x00B2",     // "mm2"
   } ;


   static const wchar_t *const Table_33A0 [] = {
      /* 0x33A0 */   L"\x0063\x006D\x00B2",     // "cm2"
      /* 0x33A1 */   L"\x006D\x00B2",     // "m2"
      /* 0x33A2 */   L"\x006B\x006D\x00B2",     // "km2"
      /* 0x33A3 */   L"\x006D\x006D\x00B3",     // "mm3"
      /* 0x33A4 */   L"\x0063\x006D\x00B3",     // "cm3"
      /* 0x33A5 */   L"\x006D\x00B3",     // "m3"
      /* 0x33A6 */   L"\x006B\x006D\x00B3",     // "km3"
      /* 0x33A7 */   L"\x006D\x2215\x0073",
      /* 0x33A8 */   L"\x006D\x2215\x0073\x00B2",
      /* 0x33A9 */   L"Pa",
      /* 0x33AA */   L"kPa",
      /* 0x33AB */   L"MPa",
      /* 0x33AC */   L"GPa",
      /* 0x33AD */   L"rad",
      /* 0x33AE */   L"\x0072\x0061\x0064\x2215\x0073",
      /* 0x33AF */   L"\x0072\x0061\x0064\x2215\x0073\x00B2",
   } ;


   static const wchar_t *const Table_33B0 [] = {
      /* 0x33B0 */   L"ps",
      /* 0x33B1 */   L"ns",
      /* 0x33B2 */   L"\x03BC\x0073",     // "s"
      /* 0x33B3 */   L"ms",
      /* 0x33B4 */   L"pV",
      /* 0x33B5 */   L"nV",
      /* 0x33B6 */   L"\x03BC\x0056",     // "V"
      /* 0x33B7 */   L"mV",
      /* 0x33B8 */   L"kV",
      /* 0x33B9 */   L"MV",
      /* 0x33BA */   L"pW",
      /* 0x33BB */   L"nW",
      /* 0x33BC */   L"\x03BC\x0057",     // "W"
      /* 0x33BD */   L"mW",
      /* 0x33BE */   L"kW",
      /* 0x33BF */   L"MW",
   } ;


   static const wchar_t *const Table_33C0 [] = {
      /* 0x33C0 */   L"\x006B\x03A9",     // "k"
      /* 0x33C1 */   L"\x004D\x03A9",     // "M"
      /* 0x33C2 */   L"a.m.",
      /* 0x33C3 */   L"Bq",
      /* 0x33C4 */   L"cc",
      /* 0x33C5 */   L"cd",
      /* 0x33C6 */   L"\x0043\x2215\x006B\x0067",
      /* 0x33C7 */   L"Co.",
      /* 0x33C8 */   L"dB",
      /* 0x33C9 */   L"Gy",
      /* 0x33CA */   L"ha",
      /* 0x33CB */   L"HP",
      /* 0x33CC */   L"in",
      /* 0x33CD */   L"KK",
      /* 0x33CE */   L"KM",
      /* 0x33CF */   L"kt",
   } ;


   static const wchar_t *const Table_33D0 [] = {
      /* 0x33D0 */   L"lm",
      /* 0x33D1 */   L"ln",
      /* 0x33D2 */   L"log",
      /* 0x33D3 */   L"lx",
      /* 0x33D4 */   L"mb",
      /* 0x33D5 */   L"mil",
      /* 0x33D6 */   L"mol",
      /* 0x33D7 */   L"pH",
      /* 0x33D8 */   L"p.m.",
      /* 0x33D9 */   L"PPM",
      /* 0x33DA */   L"PR",
      /* 0x33DB */   L"sr",
      /* 0x33DC */   L"Sv",
      /* 0x33DD */   L"Wb",
      /* 0x33DE */   L"\x0056\x2215\x006D",
      /* 0x33DF */   L"\x0041\x2215\x006D",
   } ;


   static const wchar_t *const Table_33E0 [] = {
      /* 0x33E0 */   L"\x0031\x65E5",     // "1"
      /* 0x33E1 */   L"\x0032\x65E5",     // "2"
      /* 0x33E2 */   L"\x0033\x65E5",     // "3"
      /* 0x33E3 */   L"\x0034\x65E5",     // "4"
      /* 0x33E4 */   L"\x0035\x65E5",     // "5"
      /* 0x33E5 */   L"\x0036\x65E5",     // "6"
      /* 0x33E6 */   L"\x0037\x65E5",     // "7"
      /* 0x33E7 */   L"\x0038\x65E5",     // "8"
      /* 0x33E8 */   L"\x0039\x65E5",     // "9"
      /* 0x33E9 */   L"\x0031\x0030\x65E5",     // "10"
      /* 0x33EA */   L"\x0031\x0031\x65E5",     // "11"
      /* 0x33EB */   L"\x0031\x0032\x65E5",     // "12"
      /* 0x33EC */   L"\x0031\x0033\x65E5",     // "13"
      /* 0x33ED */   L"\x0031\x0034\x65E5",     // "14"
      /* 0x33EE */   L"\x0031\x0035\x65E5",     // "15"
      /* 0x33EF */   L"\x0031\x0036\x65E5",     // "16"
   } ;


   static const wchar_t *const Table_33F0 [] = {
      /* 0x33F0 */   L"\x0031\x0037\x65E5",     // "17"
      /* 0x33F1 */   L"\x0031\x0038\x65E5",     // "18"
      /* 0x33F2 */   L"\x0031\x0039\x65E5",     // "19"
      /* 0x33F3 */   L"\x0032\x0030\x65E5",     // "20"
      /* 0x33F4 */   L"\x0032\x0031\x65E5",     // "21"
      /* 0x33F5 */   L"\x0032\x0032\x65E5",     // "22"
      /* 0x33F6 */   L"\x0032\x0033\x65E5",     // "23"
      /* 0x33F7 */   L"\x0032\x0034\x65E5",     // "24"
      /* 0x33F8 */   L"\x0032\x0035\x65E5",     // "25"
      /* 0x33F9 */   L"\x0032\x0036\x65E5",     // "26"
      /* 0x33FA */   L"\x0032\x0037\x65E5",     // "27"
      /* 0x33FB */   L"\x0032\x0038\x65E5",     // "28"
      /* 0x33FC */   L"\x0032\x0039\x65E5",     // "29"
      /* 0x33FD */   L"\x0033\x0030\x65E5",     // "30"
      /* 0x33FE */   L"\x0033\x0031\x65E5",     // "31"
      /* 0x33FF */   L"gal",
   } ;


   static const wchar_t *const Table_FE00 [] = {
      /* 0xFE00 */   L"",
      /* 0xFE01 */   L"",
      /* 0xFE02 */   L"",
      /* 0xFE03 */   L"",
      /* 0xFE04 */   L"",
      /* 0xFE05 */   L"",
      /* 0xFE06 */   L"",
      /* 0xFE07 */   L"",
      /* 0xFE08 */   L"",
      /* 0xFE09 */   L"",
      /* 0xFE0A */   L"",
      /* 0xFE0B */   L"",
      /* 0xFE0C */   L"",
      /* 0xFE0D */   L"",
      /* 0xFE0E */   L"",
      /* 0xFE0F */   L"",
   } ;


   static const wchar_t *const Table_FEF0 [] = {
      /* 0xFEF0 */   NULL,
      /* 0xFEF1 */   NULL,
      /* 0xFEF2 */   NULL,
      /* 0xFEF3 */   NULL,
      /* 0xFEF4 */   NULL,
      /* 0xFEF5 */   NULL,
      /* 0xFEF6 */   NULL,
      /* 0xFEF7 */   NULL,
      /* 0xFEF8 */   NULL,
      /* 0xFEF9 */   NULL,
      /* 0xFEFA */   NULL,
      /* 0xFEFB */   NULL,
      /* 0xFEFC */   NULL,
      /* 0xFEFD */   NULL,
      /* 0xFEFE */   NULL,
      /* 0xFEFF */   L"",
   } ;


   static const wchar_t *const Table_1F100 [] = {
      /* 0x1F100 */  L"0.",
      /* 0x1F101 */  L"0,",
      /* 0x1F102 */  L"1,",
      /* 0x1F103 */  L"2,",
      /* 0x1F104 */  L"3,",
      /* 0x1F105 */  L"4,",
      /* 0x1F106 */  L"5,",
      /* 0x1F107 */  L"6,",
      /* 0x1F108 */  L"7,",
      /* 0x1F109 */  L"8,",
      /* 0x1F10A */  L"9,",
      /* 0x1F10B */  NULL,
      /* 0x1F10C */  NULL,
      /* 0x1F10D */  NULL,
      /* 0x1F10E */  NULL,
      /* 0x1F10F */  NULL,
   } ;


   static const wchar_t *const Table_1F110 [] = {
      /* 0x1F110 */  L"(A)",
      /* 0x1F111 */  L"(B)",
      /* 0x1F112 */  L"(C)",
      /* 0x1F113 */  L"(D)",
      /* 0x1F114 */  L"(E)",
      /* 0x1F115 */  L"(F)",
      /* 0x1F116 */  L"(G)",
      /* 0x1F117 */  L"(H)",
      /* 0x1F118 */  L"(I)",
      /* 0x1F119 */  L"(J)",
      /* 0x1F11A */  L"(K)",
      /* 0x1F11B */  L"(L)",
      /* 0x1F11C */  L"(M)",
      /* 0x1F11D */  L"(N)",
      /* 0x1F11E */  L"(O)",
      /* 0x1F11F */  L"(P)",
   } ;


   static const wchar_t *const Table_1F120 [] = {
      /* 0x1F120 */  L"(Q)",
      /* 0x1F121 */  L"(R)",
      /* 0x1F122 */  L"(S)",
      /* 0x1F123 */  L"(T)",
      /* 0x1F124 */  L"(U)",
      /* 0x1F125 */  L"(V)",
      /* 0x1F126 */  L"(W)",
      /* 0x1F127 */  L"(X)",
      /* 0x1F128 */  L"(Y)",
      /* 0x1F129 */  L"(Z)",
      /* 0x1F12A */  L"[S]",
      /* 0x1F12B */  L"(C)",
      /* 0x1F12C */  L"(R)",
      /* 0x1F12D */  L"(CD)",
      /* 0x1F12E */  L"(WZ)",
      /* 0x1F12F */  NULL,
   } ;


   static const wchar_t *const Table_1F130 [] = {
      /* 0x1F130 */  NULL,
      /* 0x1F131 */  L"[B]",
      /* 0x1F132 */  NULL,
      /* 0x1F133 */  NULL,
      /* 0x1F134 */  NULL,
      /* 0x1F135 */  NULL,
      /* 0x1F136 */  NULL,
      /* 0x1F137 */  NULL,
      /* 0x1F138 */  NULL,
      /* 0x1F139 */  NULL,
      /* 0x1F13A */  NULL,
      /* 0x1F13B */  NULL,
      /* 0x1F13C */  NULL,
      /* 0x1F13D */  L"[N]",
      /* 0x1F13E */  NULL,
      /* 0x1F13F */  L"[P]",
   } ;


   static const wchar_t *const Table_1F140 [] = {
      /* 0x1F140 */  NULL,
      /* 0x1F141 */  NULL,
      /* 0x1F142 */  L"[S]",
      /* 0x1F143 */  NULL,
      /* 0x1F144 */  NULL,
      /* 0x1F145 */  NULL,
      /* 0x1F146 */  L"[W]",
      /* 0x1F147 */  NULL,
      /* 0x1F148 */  NULL,
      /* 0x1F149 */  NULL,
      /* 0x1F14A */  L"[HV]",
      /* 0x1F14B */  L"[MV]",
      /* 0x1F14C */  L"[SD]",
      /* 0x1F14D */  L"[SS]",
      /* 0x1F14E */  L"[PPV]",
      /* 0x1F14F */  NULL,
   } ;


   static const wchar_t *const Table_1F190 [] = {
      /* 0x1F190 */  L"DJ",
      /* 0x1F191 */  NULL,
      /* 0x1F192 */  NULL,
      /* 0x1F193 */  NULL,
      /* 0x1F194 */  NULL,
      /* 0x1F195 */  NULL,
      /* 0x1F196 */  NULL,
      /* 0x1F197 */  NULL,
      /* 0x1F198 */  NULL,
      /* 0x1F199 */  NULL,
      /* 0x1F19A */  NULL,
      /* 0x1F19B */  NULL,
      /* 0x1F19C */  NULL,
      /* 0x1F19D */  NULL,
      /* 0x1F19E */  NULL,
      /* 0x1F19F */  NULL,
   } ;


   static const wchar_t *const Table_1F200 [] = {
      /* 0x1F200 */  L"\x307B\x304B",     // "ق"
      /* 0x1F201 */  NULL,
      /* 0x1F202 */  NULL,
      /* 0x1F203 */  NULL,
      /* 0x1F204 */  NULL,
      /* 0x1F205 */  NULL,
      /* 0x1F206 */  NULL,
      /* 0x1F207 */  NULL,
      /* 0x1F208 */  NULL,
      /* 0x1F209 */  NULL,
      /* 0x1F20A */  NULL,
      /* 0x1F20B */  NULL,
      /* 0x1F20C */  NULL,
      /* 0x1F20D */  NULL,
      /* 0x1F20E */  NULL,
      /* 0x1F20F */  NULL,
   } ;


   static const wchar_t *const Table_1F210 [] = {
      /* 0x1F210 */  L"[\x624B]",      // "[]"
      /* 0x1F211 */  L"[\x5B57]",      // "[]"
      /* 0x1F212 */  L"[\x53CC]",      // "[o]"
      /* 0x1F213 */  L"[\x30C7]",      // "[f]"
      /* 0x1F214 */  L"[\x4E8C]",      // "[]"
      /* 0x1F215 */  L"[\x591A]",      // "[]"
      /* 0x1F216 */  L"[\x89E3]",      // "[]"
      /* 0x1F217 */  L"[\x5929]",      // "[V]"
      /* 0x1F218 */  L"[\x4EA4]",      // "[]"
      /* 0x1F219 */  L"[\x6620]",      // "[f]"
      /* 0x1F21A */  L"[\x7121]",      // "[]"
      /* 0x1F21B */  L"[\x6599]",      // "[]"
      /* 0x1F21C */  L"[\x524D]",      // "[O]"
      /* 0x1F21D */  L"[\x5F8C]",      // "[]"
      /* 0x1F21E */  L"[\x518D]",      // "[]"
      /* 0x1F21F */  L"[\x65B0]",      // "[V]"
   } ;


   static const wchar_t *const Table_1F220 [] = {
      /* 0x1F220 */  L"[\x521D]",      // "[]"
      /* 0x1F221 */  L"[\x7D42]",      // "[I]"
      /* 0x1F222 */  L"[\x751F]",      // "[]"
      /* 0x1F223 */  L"[\x8CA9]",      // "[]"
      /* 0x1F224 */  L"[\x58F0]",      // "[]"
      /* 0x1F225 */  L"[\x5439]",      // "[]"
      /* 0x1F226 */  L"[\x6F14]",      // "[]"
      /* 0x1F227 */  L"[\x6295]",      // "[]"
      /* 0x1F228 */  L"[\x6355]",      // "[]"
      /* 0x1F229 */  L"[\x4E00]",      // "[]"
      /* 0x1F22A */  L"[\x4E09]",      // "[O]"
      /* 0x1F22B */  L"[\x904A]",      // "[V]"
      /* 0x1F22C */  L"[\x5DE6]",      // "[]"
      /* 0x1F22D */  L"[\x4E2D]",      // "[]"
      /* 0x1F22E */  L"[\x53F3]",      // "[E]"
      /* 0x1F22F */  L"[\x6307]",      // "[w]"
   } ;


   static const wchar_t *const Table_1F230 [] = {
      /* 0x1F230 */  L"[\x8D70]",      // "[]"
      /* 0x1F231 */  L"[\x6253]",      // "[]"
      /* 0x1F232 */  NULL,
      /* 0x1F233 */  NULL,
      /* 0x1F234 */  NULL,
      /* 0x1F235 */  NULL,
      /* 0x1F236 */  NULL,
      /* 0x1F237 */  NULL,
      /* 0x1F238 */  NULL,
      /* 0x1F239 */  NULL,
      /* 0x1F23A */  NULL,
      /* 0x1F23B */  NULL,
      /* 0x1F23C */  NULL,
      /* 0x1F23D */  NULL,
      /* 0x1F23E */  NULL,
      /* 0x1F23F */  NULL,
   } ;


   static const wchar_t *const Table_1F240 [] = {
      /* 0x1F240 */  L"[\x672C]",      // "[{]"
      /* 0x1F241 */  L"[\x4E09]",      // "[O]"
      /* 0x1F242 */  L"[\x4E8C]",      // "[]"
      /* 0x1F243 */  L"[\x5B89]",      // "[]"
      /* 0x1F244 */  L"[\x70B9]",      // "[_]"
      /* 0x1F245 */  L"[\x6253]",      // "[]"
      /* 0x1F246 */  L"[\x76D7]",      // "[]"
      /* 0x1F247 */  L"[\x52DD]",      // "[]"
      /* 0x1F248 */  L"[\x6557]",      // "[s]"
      /* 0x1F249 */  NULL,
      /* 0x1F24A */  NULL,
      /* 0x1F24B */  NULL,
      /* 0x1F24C */  NULL,
      /* 0x1F24D */  NULL,
      /* 0x1F24E */  NULL,
      /* 0x1F24F */  NULL,
   } ;

   const wchar_t *const *Table_E0100 = Table_FE00 ;
   const wchar_t *const *Table_E0110 = Table_FE00 ;
   const wchar_t *const *Table_E0120 = Table_FE00 ;
   const wchar_t *const *Table_E0130 = Table_FE00 ;
   const wchar_t *const *Table_E0140 = Table_FE00 ;
   const wchar_t *const *Table_E0150 = Table_FE00 ;
   const wchar_t *const *Table_E0160 = Table_FE00 ;
   const wchar_t *const *Table_E0170 = Table_FE00 ;
   const wchar_t *const *Table_E0180 = Table_FE00 ;
   const wchar_t *const *Table_E0190 = Table_FE00 ;
   const wchar_t *const *Table_E01A0 = Table_FE00 ;
   const wchar_t *const *Table_E01B0 = Table_FE00 ;
   const wchar_t *const *Table_E01C0 = Table_FE00 ;
   const wchar_t *const *Table_E01D0 = Table_FE00 ;
   const wchar_t *const *Table_E01E0 = Table_FE00 ;


   const wchar_t *const *pTable = NULL ;

   switch ( cUnicode >> 4 ) {
      case 0xA0 >> 4 : pTable = Table_00A0 ; break ;
      case 0xB0 >> 4 : pTable = Table_00B0 ; break ;
      case 0xC0 >> 4 : pTable = Table_00C0 ; break ;
      case 0xD0 >> 4 : pTable = Table_00D0 ; break ;
      case 0xE0 >> 4 : pTable = Table_00E0 ; break ;
      case 0x130 >> 4 : pTable = Table_0130 ; break ;
      case 0x150 >> 4 : pTable = Table_0150 ; break ;
      case 0x1C0 >> 4 : pTable = Table_01C0 ; break ;
      case 0x1E0 >> 4 : pTable = Table_01E0 ; break ;
      case 0x1F0 >> 4 : pTable = Table_01F0 ; break ;
      case 0x400 >> 4 : pTable = Table_0400 ; break ;
      case 0x450 >> 4 : pTable = Table_0450 ; break ;
      case 0x4D0 >> 4 : pTable = Table_04D0 ; break ;
      case 0x1D00 >> 4 : pTable = Table_1D00 ; break ;
      case 0x1D20 >> 4 : pTable = Table_1D20 ; break ;
      case 0x1D40 >> 4 : pTable = Table_1D40 ; break ;
      case 0x1D60 >> 4 : pTable = Table_1D60 ; break ;
      case 0x2000 >> 4 : pTable = Table_2000 ; break ;
      case 0x2030 >> 4 : pTable = Table_2030 ; break ;
      case 0x2040 >> 4 : pTable = Table_2040 ; break ;
      case 0x2100 >> 4 : pTable = Table_2100 ; break ;
      case 0x2110 >> 4 : pTable = Table_2110 ; break ;
      case 0x2120 >> 4 : pTable = Table_2120 ; break ;
      case 0x2130 >> 4 : pTable = Table_2130 ; break ;
      case 0x2150 >> 4 : pTable = Table_2150 ; break ;
      case 0x2160 >> 4 : pTable = Table_2160 ; break ;
      case 0x2170 >> 4 : pTable = Table_2170 ; break ;
      case 0x2180 >> 4 : pTable = Table_2180 ; break ;
      case 0x2400 >> 4 : pTable = Table_2400 ; break ;
      case 0x2410 >> 4 : pTable = Table_2410 ; break ;
      case 0x2420 >> 4 : pTable = Table_2420 ; break ;
      case 0x2460 >> 4 : pTable = Table_2460 ; break ;
      case 0x2470 >> 4 : pTable = Table_2470 ; break ;
      case 0x2480 >> 4 : pTable = Table_2480 ; break ;
      case 0x2490 >> 4 : pTable = Table_2490 ; break ;
      case 0x24A0 >> 4 : pTable = Table_24A0 ; break ;
      case 0x24B0 >> 4 : pTable = Table_24B0 ; break ;
      case 0x24C0 >> 4 : pTable = Table_24C0 ; break ;
      case 0x24D0 >> 4 : pTable = Table_24D0 ; break ;
      case 0x24E0 >> 4 : pTable = Table_24E0 ; break ;
      case 0x24F0 >> 4 : pTable = Table_24F0 ; break ;
      case 0x2770 >> 4 : pTable = Table_2770 ; break ;
      case 0x2780 >> 4 : pTable = Table_2780 ; break ;
      case 0x2790 >> 4 : pTable = Table_2790 ; break ;
      case 0x30F0 >> 4 : pTable = Table_30F0 ; break ;
      case 0x3190 >> 4 : pTable = Table_3190 ; break ;
      case 0x3200 >> 4 : pTable = Table_3200 ; break ;
      case 0x3210 >> 4 : pTable = Table_3210 ; break ;
      case 0x3220 >> 4 : pTable = Table_3220 ; break ;
      case 0x3230 >> 4 : pTable = Table_3230 ; break ;
      case 0x3240 >> 4 : pTable = Table_3240 ; break ;
      case 0x3250 >> 4 : pTable = Table_3250 ; break ;
      case 0x3260 >> 4 : pTable = Table_3260 ; break ;
      case 0x3270 >> 4 : pTable = Table_3270 ; break ;
      case 0x3280 >> 4 : pTable = Table_3280 ; break ;
      case 0x3290 >> 4 : pTable = Table_3290 ; break ;
      case 0x32A0 >> 4 : pTable = Table_32A0 ; break ;
      case 0x32B0 >> 4 : pTable = Table_32B0 ; break ;
      case 0x32C0 >> 4 : pTable = Table_32C0 ; break ;
      case 0x32D0 >> 4 : pTable = Table_32D0 ; break ;
      case 0x32E0 >> 4 : pTable = Table_32E0 ; break ;
      case 0x32F0 >> 4 : pTable = Table_32F0 ; break ;
      case 0x3300 >> 4 : pTable = Table_3300 ; break ;
      case 0x3310 >> 4 : pTable = Table_3310 ; break ;
      case 0x3320 >> 4 : pTable = Table_3320 ; break ;
      case 0x3330 >> 4 : pTable = Table_3330 ; break ;
      case 0x3340 >> 4 : pTable = Table_3340 ; break ;
      case 0x3350 >> 4 : pTable = Table_3350 ; break ;
      case 0x3360 >> 4 : pTable = Table_3360 ; break ;
      case 0x3370 >> 4 : pTable = Table_3370 ; break ;
      case 0x3380 >> 4 : pTable = Table_3380 ; break ;
      case 0x3390 >> 4 : pTable = Table_3390 ; break ;
      case 0x33A0 >> 4 : pTable = Table_33A0 ; break ;
      case 0x33B0 >> 4 : pTable = Table_33B0 ; break ;
      case 0x33C0 >> 4 : pTable = Table_33C0 ; break ;
      case 0x33D0 >> 4 : pTable = Table_33D0 ; break ;
      case 0x33E0 >> 4 : pTable = Table_33E0 ; break ;
      case 0x33F0 >> 4 : pTable = Table_33F0 ; break ;
      case 0xFE00 >> 4 : pTable = Table_FE00 ; break ;
      case 0xFEF0 >> 4 : pTable = Table_FEF0 ; break ;
      case 0x1F100 >> 4 : pTable = Table_1F100 ; break ;
      case 0x1F110 >> 4 : pTable = Table_1F110 ; break ;
      case 0x1F120 >> 4 : pTable = Table_1F120 ; break ;
      case 0x1F130 >> 4 : pTable = Table_1F130 ; break ;
      case 0x1F140 >> 4 : pTable = Table_1F140 ; break ;
      case 0x1F190 >> 4 : pTable = Table_1F190 ; break ;
      case 0x1F200 >> 4 : pTable = Table_1F200 ; break ;
      case 0x1F210 >> 4 : pTable = Table_1F210 ; break ;
      case 0x1F220 >> 4 : pTable = Table_1F220 ; break ;
      case 0x1F230 >> 4 : pTable = Table_1F230 ; break ;
      case 0x1F240 >> 4 : pTable = Table_1F240 ; break ;
      case 0xE0100 >> 4 : pTable = Table_E0100 ; break ;
      case 0xE0110 >> 4 : pTable = Table_E0110 ; break ;
      case 0xE0120 >> 4 : pTable = Table_E0120 ; break ;
      case 0xE0130 >> 4 : pTable = Table_E0130 ; break ;
      case 0xE0140 >> 4 : pTable = Table_E0140 ; break ;
      case 0xE0150 >> 4 : pTable = Table_E0150 ; break ;
      case 0xE0160 >> 4 : pTable = Table_E0160 ; break ;
      case 0xE0170 >> 4 : pTable = Table_E0170 ; break ;
      case 0xE0180 >> 4 : pTable = Table_E0180 ; break ;
      case 0xE0190 >> 4 : pTable = Table_E0190 ; break ;
      case 0xE01A0 >> 4 : pTable = Table_E01A0 ; break ;
      case 0xE01B0 >> 4 : pTable = Table_E01B0 ; break ;
      case 0xE01C0 >> 4 : pTable = Table_E01C0 ; break ;
      case 0xE01D0 >> 4 : pTable = Table_E01D0 ; break ;
      case 0xE01E0 >> 4 : pTable = Table_E01E0 ; break ;
   }

   if ( pTable ) return pTable [ cUnicode & 0xF ] ;
   return NULL ;
}



////////////////////////////////////////////
//          nO̕ƍ          //
////////////////////////////////////////////



// nO𕪉
// Ԃ
const wchar_t *DecomposeUnicodeHangul ( int cUnicode, wchar_t *szBuffer ) {

   if ( ! ( cUnicode >= 0xAC00 && cUnicode <= 0xD7A3 ) ) return NULL ;

   int C0 =   ( ( (unsigned) cUnicode - 0xAC00 ) / 0x24C )          + 0x1100 ;
   int C1 = ( ( ( (unsigned) cUnicode - 0xAC00 ) % 0x24C ) / 0x1C ) + 0x1161 ;
   int C2 = ( ( ( (unsigned) cUnicode - 0xAC00 ) % 0x24C ) % 0x1C ) + 0x11A7 ;

   if ( C2 == 0x11A7 ) C2 = 0 ;

   szBuffer [ 0 ] = C0 ;
   szBuffer [ 1 ] = C1 ;
   szBuffer [ 2 ] = C2 ;
   if ( C2 ) szBuffer [ 3 ] = 0 ;

   return szBuffer ;
}



// nO
// ͂̒ pLength Ɋi[
// Ԃ
int ComposeUnicodeHangul ( const wchar_t *szString, size_t *pLength ) {

   int cResult = 0 ;
   if ( pLength ) *pLength = 0 ;

   int C0 = szString [ 0 ] ;
   if ( C0 >= 0x1100 && C0 <= 0x1112 ) {

      cResult = 0xAC00 + ( C0 - 0x1100 ) * 0x24C ;

      int C1 = szString [ 1 ] ;
      if ( C1 >= 0x1161 && C1 <= 0x1175 ) {

         cResult += ( C1 - 0x1161 ) * 0x1C ;
         if ( pLength ) *pLength = 2 ;

         int C2 = szString [ 2 ] ;
         if ( C2 >= 0x11A8 && C2 <= 0x11C2 ) {

            cResult += ( C2 - 0x11A8 ) + 1 ;
            if ( pLength ) *pLength = 3 ;
         }
      }
      else {
         return 0 ;
      }
   }

   return cResult ;
}



////////////////////////////////////////////
//             SpEp             //
////////////////////////////////////////////



// SpEpʏ̕
int NormalizeUnicodeWidthForms ( int cUnicode ) {

   static const unsigned short Table_FF00 [] = {
      0,       0x0021,  0x0022,  0x0023,  0x0024,  0x0025,  0x0026,  0x0027,  // 0xFF0x
      0x0028,  0x0029,  0x002A,  0x002B,  0x002C,  0x002D,  0x002E,  0x002F,  
      0x0030,  0x0031,  0x0032,  0x0033,  0x0034,  0x0035,  0x0036,  0x0037,  // 0xFF1x
      0x0038,  0x0039,  0x003A,  0x003B,  0x003C,  0x003D,  0x003E,  0x003F,  
      0x0040,  0x0041,  0x0042,  0x0043,  0x0044,  0x0045,  0x0046,  0x0047,  // 0xFF2x
      0x0048,  0x0049,  0x004A,  0x004B,  0x004C,  0x004D,  0x004E,  0x004F,  
      0x0050,  0x0051,  0x0052,  0x0053,  0x0054,  0x0055,  0x0056,  0x0057,  // 0xFF3x
      0x0058,  0x0059,  0x005A,  0x005B,  0x005C,  0x005D,  0x005E,  0x005F,  
      0x0060,  0x0061,  0x0062,  0x0063,  0x0064,  0x0065,  0x0066,  0x0067,  // 0xFF4x
      0x0068,  0x0069,  0x006A,  0x006B,  0x006C,  0x006D,  0x006E,  0x006F,  
      0x0070,  0x0071,  0x0072,  0x0073,  0x0074,  0x0075,  0x0076,  0x0077,  // 0xFF5x
      0x0078,  0x0079,  0x007A,  0x007B,  0x007C,  0x007D,  0x007E,  0x2985,  
      0x2986,  0x3002,  0x300C,  0x300D,  0x3001,  0x30FB,  0x30F2,  0x30A1,  // 0xFF6x
      0x30A3,  0x30A5,  0x30A7,  0x30A9,  0x30E3,  0x30E5,  0x30E7,  0x30C3,  
      0x30FC,  0x30A2,  0x30A4,  0x30A6,  0x30A8,  0x30AA,  0x30AB,  0x30AD,  // 0xFF7x
      0x30AF,  0x30B1,  0x30B3,  0x30B5,  0x30B7,  0x30B9,  0x30BB,  0x30BD,  
      0x30BF,  0x30C1,  0x30C4,  0x30C6,  0x30C8,  0x30CA,  0x30CB,  0x30CC,  // 0xFF8x
      0x30CD,  0x30CE,  0x30CF,  0x30D2,  0x30D5,  0x30D8,  0x30DB,  0x30DE,  
      0x30DF,  0x30E0,  0x30E1,  0x30E2,  0x30E4,  0x30E6,  0x30E8,  0x30E9,  // 0xFF9x
      0x30EA,  0x30EB,  0x30EC,  0x30ED,  0x30EF,  0x30F3,  0x309B,  0x309C,  
      0x3164,  0x3131,  0x3132,  0x3133,  0x3134,  0x3135,  0x3136,  0x3137,  // 0xFFAx
      0x3138,  0x3139,  0x313A,  0x313B,  0x313C,  0x313D,  0x313E,  0x313F,  
      0x3140,  0x3141,  0x3142,  0x3143,  0x3144,  0x3145,  0x3146,  0x3147,  // 0xFFBx
      0x3148,  0x3149,  0x314A,  0x314B,  0x314C,  0x314D,  0x314E,  0,       
      0,       0,       0x314F,  0x3150,  0x3151,  0x3152,  0x3153,  0x3154,  // 0xFFCx
      0,       0,       0x3155,  0x3156,  0x3157,  0x3158,  0x3159,  0x315A,  
      0,       0,       0x315B,  0x315C,  0x315D,  0x315E,  0x315F,  0x3160,  // 0xFFDx
      0,       0,       0x3161,  0x3162,  0x3163,  0,       0,       0,       
      0x00A2,  0x00A3,  0x00AC,  0x00AF,  0x00A6,  0x00A5,  0x20A9,  0,       // 0xFFEx
      0x2502,  0x2190,  0x2191,  0x2192,  0x2193,  0x25A0,  0x25CB,  0,       
   } ;

   if ( cUnicode >= 0xFF00 && cUnicode <= 0xFFEF ) return Table_FF00 [ cUnicode - 0xFF00 ] ;

   return 0 ;
}



////////////////////////////////////////////
//                                //
////////////////////////////////////////////



//  CJK 
int CjkRadicalsToCjk ( int cUnicode ) {

   static const unsigned long Table_2E80 [] = {
      0,       0,       0x4E5B,  0x4E5A,  0,       0x4EBB,  0,       0,       // 0x2E8x
      0,       0x5202,  0,       0x353E,  0,       0,       0x5140,  0x5C23,  
      0x5C22,  0x5C23,  0x5DF3,  0x5E7A,  0x5F51,  0x5F50,  0x5FC4,  0,       // 0x2E9x
      0x624C,  0x6535,  0,       0x65E1,  0,       0,       0x6B7A,  0x6BCD,  
      0x6C11,  0x6C35,  0x6C3A,  0x706C,  0x722B,  0,       0x4E2C,  0,       // 0x2EAx
      0x72AD,  0x738B,  0,       0x7F52,  0x793A,  0x793B,  0,       0x7CF9,  
      0x7E9F,  0x7F53,  0x7F52,  0,       0,       0x2626B, 0,       0,       // 0x2EBx
      0x7F8B,  0x8002,  0x8080,  0,       0,       0,       0x8279,  0x8279,  
      0,       0x864E,  0x8864,  0x8980,  0x897F,  0x89C1,  0x89D2,  0x278B2, // 0x2ECx
      0x8BA0,  0x8D1D,  0,       0x8F66,  0x8FB6,  0,       0,       0,       
      0x9485,  0x9577,  0x9578,  0x957F,  0x95E8,  0x28E0F, 0x961D,  0,       // 0x2EDx
      0x9752,  0x97E6,  0x9875,  0x98CE,  0x98DE,  0x98DF,  0x2967F, 0x98E0,  
      0x9963,  0x29810, 0x9A6C,  0x9AA8,  0x9B3C,  0x9C7C,  0x9E1F,  0x5364,  // 0x2EEx
      0x9EA6,  0x9EC4,  0x9EFE,  0x6589,  0x9F50,  0x6B6F,  0x9F7F,  0x7ADC,  
      0x9F99,  0x9F9C,  0x4E80,  0x9F9F,  0,       0,       0,       0,       // 0x2EFx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0x4E00,  0x4E28,  0x4E36,  0x4E3F,  0x4E59,  0x4E85,  0x4E8C,  0x4EA0,  // 0x2F0x
      0x4EBA,  0x513F,  0x5165,  0x516B,  0x5182,  0x5196,  0x51AB,  0x51E0,  
      0x51F5,  0x5200,  0x529B,  0x52F9,  0x5315,  0x531A,  0x5338,  0x5341,  // 0x2F1x
      0x535C,  0x5369,  0x5382,  0x53B6,  0x53C8,  0x53E3,  0x56D7,  0x571F,  
      0x58EB,  0x5902,  0x590A,  0x5915,  0x5927,  0x5973,  0x5B50,  0x5B80,  // 0x2F2x
      0x5BF8,  0x5C0F,  0x5C22,  0x5C38,  0x5C6E,  0x5C71,  0x5DDB,  0x5DE5,  
      0x5DF1,  0x5DFE,  0x5E72,  0x5E7A,  0x5E7F,  0x5EF4,  0x5EFE,  0x5F0B,  // 0x2F3x
      0x5F13,  0x5F50,  0x5F61,  0x5F73,  0x5FC3,  0x6208,  0x6236,  0x624B,  
      0x652F,  0x6534,  0x6587,  0x6597,  0x65A4,  0x65B9,  0x65E0,  0x65E5,  // 0x2F4x
      0x66F0,  0x6708,  0x6728,  0x6B20,  0x6B62,  0x6B79,  0x6BB3,  0x6BCB,  
      0x6BD4,  0x6BDB,  0x6C0F,  0x6C14,  0x6C34,  0x706B,  0x722A,  0x7236,  // 0x2F5x
      0x723B,  0x723F,  0x7247,  0x7259,  0x725B,  0x72AC,  0x7384,  0x7389,  
      0x74DC,  0x74E6,  0x7518,  0x751F,  0x7528,  0x7530,  0x758B,  0x7592,  // 0x2F6x
      0x7676,  0x767D,  0x76AE,  0x76BF,  0x76EE,  0x77DB,  0x77E2,  0x77F3,  
      0x793A,  0x79B8,  0x79BE,  0x7A74,  0x7ACB,  0x7AF9,  0x7C73,  0x7CF8,  // 0x2F7x
      0x7F36,  0x7F51,  0x7F8A,  0x7FBD,  0x8001,  0x800C,  0x8012,  0x8033,  
      0x807F,  0x8089,  0x81E3,  0x81EA,  0x81F3,  0x81FC,  0x820C,  0x821B,  // 0x2F8x
      0x821F,  0x826E,  0x8272,  0x8278,  0x864D,  0x866B,  0x8840,  0x884C,  
      0x8863,  0x897E,  0x898B,  0x89D2,  0x8A00,  0x8C37,  0x8C46,  0x8C55,  // 0x2F9x
      0x8C78,  0x8C9D,  0x8D64,  0x8D70,  0x8DB3,  0x8EAB,  0x8ECA,  0x8F9B,  
      0x8FB0,  0x8FB5,  0x9091,  0x9149,  0x91C6,  0x91CC,  0x91D1,  0x9577,  // 0x2FAx
      0x9580,  0x961C,  0x96B6,  0x96B9,  0x96E8,  0x9751,  0x975E,  0x9762,  
      0x9769,  0x97CB,  0x97ED,  0x97F3,  0x9801,  0x98A8,  0x98DB,  0x98DF,  // 0x2FBx
      0x9996,  0x9999,  0x99AC,  0x9AA8,  0x9AD8,  0x9ADF,  0x9B25,  0x9B2F,  
      0x9B32,  0x9B3C,  0x9B5A,  0x9CE5,  0x9E75,  0x9E7F,  0x9EA5,  0x9EBB,  // 0x2FCx
      0x9EC3,  0x9ECD,  0x9ED1,  0x9EF9,  0x9EFD,  0x9F0E,  0x9F13,  0x9F20,  
      0x9F3B,  0x9F4A,  0x9F52,  0x9F8D,  0x9F9C,  0x9FA0,  0,       0,       // 0x2FDx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   if ( cUnicode >= 0x2E80 && cUnicode <= 0x2FDF ) return Table_2E80 [ cUnicode - 0x2E80 ] ;

   return 0 ;
}



////////////////////////////////////////////
//            GB13000Վ             //
////////////////////////////////////////////



#ifndef GetShortArrayBit
#define GetShortArrayBit(Array,Index)  ( ( (Array) [ (Index) >> 4 ] >> ( (Index) & 0xF ) ) & 1 )
#endif



// GB13000Վ̃e[u擾
static const unsigned long *GetLinshiDaimaTable ( int cUnicode ) {


   static const unsigned long Table_E740 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE74x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE75x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE76x
      0,       0,       0,       0,       0x20AC,  0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE77x
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   static const unsigned long Table_E7C0 [] = {
      0,       0,       0,       0,       0,       0,       0,       0x1E3F,  // 0xE7Cx
      0x01F9,  0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE7Dx
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0x303E,  // 0xE7Ex
      0x2FF0,  0x2FF1,  0x2FF2,  0x2FF3,  0x2FF4,  0x2FF5,  0x2FF6,  0x2FF7,  
      0x2FF8,  0x2FF9,  0x2FFA,  0x2FFB,  0,       0,       0,       0,       // 0xE7Fx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   static const unsigned long Table_E800 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE80x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0x2E81,  0x20087, 0x20089, // 0xE81x
      0x200CC, 0x2E84,  0x3473,  0x3447,  0x2E88,  0x2E8B,  0x9FB4,  0x359E,  
      0x361A,  0x360E,  0x2E8C,  0x2E97,  0x396E,  0x3918,  0x9FB5,  0x39CF,  // 0xE82x
      0x39DF,  0x3A73,  0x39D0,  0x9FB6,  0x9FB7,  0x3B4E,  0x3C6E,  0x3CE0,  
      0x2EA7,  0x215D7, 0x9FB8,  0x2EAA,  0x4056,  0x415F,  0x2EAE,  0x4337,  // 0xE83x
      0x2EB3,  0x2EB6,  0x2EB7,  0x2298F, 0x43B1,  0x43AC,  0x2EBB,  0x43DD,  
   } ;


   static const unsigned long Table_E840 [] = {
      0x44D6,  0x4661,  0x464C,  0x9FB9,  0x4723,  0x4729,  0x477C,  0x478D,  // 0xE84x
      0x2ECA,  0x4947,  0x497A,  0x497D,  0x4982,  0x4983,  0x4985,  0x4986,  
      0x499F,  0x499B,  0x49B7,  0x49B6,  0x9FBA,  0x241FE, 0x4CA3,  0x4C9F,  // 0xE85x
      0x4CA0,  0x4CA1,  0x4C77,  0x4CA2,  0x4D13,  0x4D14,  0x4D15,  0x4D16,  
      0x4D17,  0x4D18,  0x4D19,  0x4DAE,  0x9FBB,  0,       0,       0,       // 0xE86x
      0,       0,       0,       0,       0,       0,       0,       0,       
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE87x
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;


   switch ( cUnicode >> 6 ) {
      case 0xE740 >> 6 : return Table_E740 ;
      case 0xE7C0 >> 6 : return Table_E7C0 ;
      case 0xE800 >> 6 : return Table_E800 ;
      case 0xE840 >> 6 : return Table_E840 ;
   }

   return NULL ;
}



// GB13000ՎUnicodeԂ
int GetNonLinshiDaima ( int cUnicode ) {

   const unsigned long *pTable = GetLinshiDaimaTable ( cUnicode ) ;
   if ( pTable ) return pTable [ (unsigned) cUnicode % 0x40 ] ;

   return 0 ;
}



// GB13000ՎǂԂ
int IsLinshiDaima ( int cUnicode ) {

   if ( GetNonLinshiDaima ( cUnicode ) ) return 1 ;

   return 0 ;
}



static int GetLinshiDaimaTableBase ( int nCount ) {

   switch ( nCount ) {
      case 0 : return 0xE840 ;
      case 1 : return 0xE800 ;
      case 2 : return 0xE7C0 ;
      case 3 : return 0xE740 ;
   }

   return 0 ;
}



// UnicodeGB13000ՎԂ
int GetLinshiDaima ( int cUnicode ) {

   if ( IsHaveLinshiDaima ( cUnicode ) ) {

      for ( int nCount = 0 ; ; nCount ++ ) {

         int cBase = GetLinshiDaimaTableBase ( nCount ) ;
         if ( ! cBase ) break ;

         const unsigned long *pTable = GetLinshiDaimaTable ( cBase ) ;
         if ( ! pTable ) continue ;

         for ( int C = 0 ; C < 0x40 ; C ++ ) {
            if ( (long) pTable [ C ] == cUnicode ) return cBase + C ;
         }
      }
   }

   return 0 ;
}



// UnicodeɑΉGB13000Վ邩Ԃ
int IsHaveLinshiDaima ( int cUnicode ) {

   static const unsigned short BitTable_0100 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x01xx
      0,       0,       0,       0,       0,       0,       0,       0x0200,  
   } ;

   static const unsigned short BitTable_1E00 [] = {
      0,       0,       0,       0x8000,  0,       0,       0,       0,       // 0x1Exx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_2000 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x20xx
      0,       0,       0x1000,  0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_2E00 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2Exx
      0x1912,  0x0080,  0x4480,  0x08C8,  0x0400,  0,       0,       0,       
   } ;

   static const unsigned short BitTable_2F00 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x2Fxx
      0,       0,       0,       0,       0,       0,       0,       0x0FFF,  
   } ;

   static const unsigned short BitTable_3000 [] = {
      0,       0,       0,       0x4000,  0,       0,       0,       0,       // 0x30xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3400 [] = {
      0,       0,       0,       0,       0x0080,  0,       0,       0x0008,  // 0x34xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3500 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x35xx
      0,       0x4000,  0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3600 [] = {
      0x4000,  0x0400,  0,       0,       0,       0,       0,       0,       // 0x36xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3900 [] = {
      0,       0x0100,  0,       0,       0,       0,       0x4000,  0,       // 0x39xx
      0,       0,       0,       0,       0x8000,  0x8001,  0,       0,       
   } ;

   static const unsigned short BitTable_3A00 [] = {
      0,       0,       0,       0,       0,       0,       0,       0x0008,  // 0x3Axx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3B00 [] = {
      0,       0,       0,       0,       0x4000,  0,       0,       0,       // 0x3Bxx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_3C00 [] = {
      0,       0,       0,       0,       0,       0,       0x4000,  0,       // 0x3Cxx
      0,       0,       0,       0,       0,       0,       0x0001,  0,       
   } ;

   static const unsigned short BitTable_4000 [] = {
      0,       0,       0,       0,       0,       0x0040,  0,       0,       // 0x40xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4100 [] = {
      0,       0,       0,       0,       0,       0x8000,  0,       0,       // 0x41xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4300 [] = {
      0,       0,       0,       0x0080,  0,       0,       0,       0,       // 0x43xx
      0,       0,       0x1000,  0x0002,  0,       0x2000,  0,       0,       
   } ;

   static const unsigned short BitTable_4400 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x44xx
      0,       0,       0,       0,       0,       0x0040,  0,       0,       
   } ;

   static const unsigned short BitTable_4600 [] = {
      0,       0,       0,       0,       0x1000,  0,       0x0002,  0,       // 0x46xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4700 [] = {
      0,       0,       0x0208,  0,       0,       0,       0,       0x1000,  // 0x47xx
      0x2000,  0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4900 [] = {
      0,       0,       0,       0,       0x0080,  0,       0,       0x2400,  // 0x49xx
      0x006C,  0x8800,  0,       0x00C0,  0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4C00 [] = {
      0,       0,       0,       0,       0,       0,       0,       0x0080,  // 0x4Cxx
      0,       0x8000,  0x000F,  0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_4D00 [] = {
      0,       0x03F8,  0,       0,       0,       0,       0,       0,       // 0x4Dxx
      0,       0,       0x4000,  0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_9F00 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x9Fxx
      0,       0,       0,       0x0FF0,  0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_20000 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x200xx
      0x0280,  0,       0,       0,       0x1000,  0,       0,       0,       
   } ;

   static const unsigned short BitTable_21500 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x215xx
      0,       0,       0,       0,       0,       0x0080,  0,       0,       
   } ;

   static const unsigned short BitTable_22900 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x229xx
      0x8000,  0,       0,       0,       0,       0,       0,       0,       
   } ;

   static const unsigned short BitTable_24100 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0x241xx
      0,       0,       0,       0,       0,       0,       0,       0x4000,  
   } ;

   const unsigned short *pTable ;

   switch ( cUnicode >> 8 ) {
      case 0x100 >> 8 : pTable = BitTable_0100 ; break ;
      case 0x1E00 >> 8 : pTable = BitTable_1E00 ; break ;
      case 0x2000 >> 8 : pTable = BitTable_2000 ; break ;
      case 0x2E00 >> 8 : pTable = BitTable_2E00 ; break ;
      case 0x2F00 >> 8 : pTable = BitTable_2F00 ; break ;
      case 0x3000 >> 8 : pTable = BitTable_3000 ; break ;
      case 0x3400 >> 8 : pTable = BitTable_3400 ; break ;
      case 0x3500 >> 8 : pTable = BitTable_3500 ; break ;
      case 0x3600 >> 8 : pTable = BitTable_3600 ; break ;
      case 0x3900 >> 8 : pTable = BitTable_3900 ; break ;
      case 0x3A00 >> 8 : pTable = BitTable_3A00 ; break ;
      case 0x3B00 >> 8 : pTable = BitTable_3B00 ; break ;
      case 0x3C00 >> 8 : pTable = BitTable_3C00 ; break ;
      case 0x4000 >> 8 : pTable = BitTable_4000 ; break ;
      case 0x4100 >> 8 : pTable = BitTable_4100 ; break ;
      case 0x4300 >> 8 : pTable = BitTable_4300 ; break ;
      case 0x4400 >> 8 : pTable = BitTable_4400 ; break ;
      case 0x4600 >> 8 : pTable = BitTable_4600 ; break ;
      case 0x4700 >> 8 : pTable = BitTable_4700 ; break ;
      case 0x4900 >> 8 : pTable = BitTable_4900 ; break ;
      case 0x4C00 >> 8 : pTable = BitTable_4C00 ; break ;
      case 0x4D00 >> 8 : pTable = BitTable_4D00 ; break ;
      case 0x9F00 >> 8 : pTable = BitTable_9F00 ; break ;
      case 0x20000 >> 8 : pTable = BitTable_20000 ; break ;
      case 0x21500 >> 8 : pTable = BitTable_21500 ; break ;
      case 0x22900 >> 8 : pTable = BitTable_22900 ; break ;
      case 0x24100 >> 8 : pTable = BitTable_24100 ; break ;
      default : return 0 ;
   }

   if ( GetShortArrayBit ( pTable, cUnicode & 0xFF ) ) return 1 ;

   return 0 ;
}



// GB13000ՎǂԂ
int IsLinshiDaimaGb18030 ( int cUnicode ) {

   static const unsigned short BitTable_E700 [] = {
      0,       0,       0,       0,       0,       0,       0,       0,       // 0xE7xx
      0,       0,       0,       0,       0x0080,  0,       0,       0,       
   } ;

   static const unsigned short BitTable_E800 [] = {
      0,       0x41C0,  0x1840,  0x0806,  0x0008,  0x0030,  0x0010,  0,       // 0xE8xx
      0,       0,       0,       0,       0,       0,       0,       0,       
   } ;

   const unsigned short *pTable ;

   switch ( cUnicode >> 8 ) {
      case 0xE700 >> 8 : pTable = BitTable_E700 ; break ;
      case 0xE800 >> 8 : pTable = BitTable_E800 ; break ;
      default : return 0 ;
   }

   if ( GetShortArrayBit ( pTable, cUnicode & 0xFF ) ) return 1 ;

   return 0 ;
}



////////////////////////////////////////////
//                ISCII                   //
////////////////////////////////////////////



// ISCII + NUKTA -> UNICODE
int GetNuktaChar ( int C, int nCodePage ) {

   switch ( nCodePage ) {
      case CP_ISCII_DEVANAGARI :
         switch ( C ) {
            case 0xA1 : return 0x0950 ;
            case 0xA6 : return 0x090C ;
            case 0xA7 : return 0x0961 ;
            case 0xAA : return 0x0960 ;
            case 0xB3 : return 0x0958 ;
            case 0xB4 : return 0x0959 ;
            case 0xB5 : return 0x095A ;
            case 0xBA : return 0x095B ;
            case 0xBF : return 0x095C ;
            case 0xC0 : return 0x095D ;
            case 0xC9 : return 0x095E ;
            case 0xDB : return 0x0962 ;
            case 0xDC : return 0x0963 ;
            case 0xDF : return 0x0944 ;
            case 0xEA : return 0x093D ;
         }
         break ;
      case CP_ISCII_BENGALI :
      case CP_ISCII_ASSAMESE :
         switch ( C ) {
            case 0xA6 : return 0x098C ;
            case 0xA7 : return 0x09E1 ;
            case 0xAA : return 0x09E0 ;
            case 0xBF : return 0x09DC ;
            case 0xC0 : return 0x09DD ;
            case 0xDB : return 0x09E2 ;
            case 0xDC : return 0x09E3 ;
            case 0xDF : return 0x09C4 ;
            case 0xEA : return 0x09BD ;
         }
         break ;
      case CP_ISCII_TAMIL :
         switch ( C ) {
            case 0xA1 : return 0x0BD0 ;
         }
         break ;
      case CP_ISCII_TELUGU :
         switch ( C ) {
            case 0xA6 : return 0x0C0C ;
            case 0xA7 : return 0x0C61 ;
            case 0xAA : return 0x0C60 ;
            case 0xB3 : return 0x0C58 ;
            case 0xB4 : return 0x0C59 ;
            case 0xDB : return 0x0C62 ;
            case 0xDC : return 0x0C63 ;
            case 0xDF : return 0x0C44 ;
            case 0xEA : return 0x0C3D ;
         }
         break ;
      case CP_ISCII_ORIYA :
         switch ( C ) {
            case 0xA6 : return 0x0B0C ;
            case 0xA7 : return 0x0B61 ;
            case 0xAA : return 0x0B60 ;
            case 0xBF : return 0x0B5C ;
            case 0xC0 : return 0x0B5D ;
            case 0xDB : return 0x0B62 ;
            case 0xDC : return 0x0B63 ;
            case 0xDF : return 0x0B44 ;
            case 0xEA : return 0x0B3D ;
         }
         break ;
      case CP_ISCII_KANNADA :
         switch ( C ) {
            case 0xA6 : return 0x0C8C ;
            case 0xA7 : return 0x0CE1 ;
            case 0xAA : return 0x0CE0 ;
            case 0xC9 : return 0x0CDE ;
            case 0xDB : return 0x0CE2 ;
            case 0xDC : return 0x0CE3 ;
            case 0xDF : return 0x0CC4 ;
            case 0xEA : return 0x0CBD ;
         }
         break ;
      case CP_ISCII_MALAYALAM :
         switch ( C ) {
            case 0xA6 : return 0x0D0C ;
            case 0xA7 : return 0x0D61 ;
            case 0xAA : return 0x0D60 ;
            case 0xDB : return 0x0D62 ;
            case 0xDC : return 0x0D63 ;
            case 0xDF : return 0x0D44 ;
            case 0xEA : return 0x0D3D ;
         }
         break ;
      case CP_ISCII_GUJARATHI :
         switch ( C ) {
            case 0xA1 : return 0x0AD0 ;
            case 0xA6 : return 0x0A8C ;
            case 0xA7 : return 0x0AE1 ;
            case 0xAA : return 0x0AE0 ;
            case 0xDB : return 0x0AE2 ;
            case 0xDC : return 0x0AE3 ;
            case 0xDF : return 0x0AC4 ;
            case 0xEA : return 0x0ABD ;
         }
         break ;
      case CP_ISCII_PANJABI :
         switch ( C ) {
            case 0xB4 : return 0x0A59 ;
            case 0xB5 : return 0x0A5A ;
            case 0xBA : return 0x0A5B ;
            case 0xBF : return 0x0A5C ;
            case 0xC9 : return 0x0A5E ;
         }
         break ;
   }

   return 0 ;
}



// UNICODE -> ISCII + NUKTA
int GetNonNuktaChar ( int C, int nCodePage ) {

   switch ( nCodePage ) {
      case CP_ISCII_DEVANAGARI :
         switch ( C ) {
            case 0x0950 : return 0xA1 ;
            case 0x090C : return 0xA6 ;
            case 0x0961 : return 0xA7 ;
            case 0x0960 : return 0xAA ;
            case 0x0958 : return 0xB3 ;
            case 0x0959 : return 0xB4 ;
            case 0x095A : return 0xB5 ;
            case 0x095B : return 0xBA ;
            case 0x095C : return 0xBF ;
            case 0x095D : return 0xC0 ;
            case 0x095E : return 0xC9 ;
            case 0x0962 : return 0xDB ;
            case 0x0963 : return 0xDC ;
            case 0x0944 : return 0xDF ;
            case 0x093D : return 0xEA ;
         }
         break ;
      case CP_ISCII_BENGALI :
      case CP_ISCII_ASSAMESE :
         switch ( C ) {
            case 0x098C : return 0xA6 ;
            case 0x09E1 : return 0xA7 ;
            case 0x09E0 : return 0xAA ;
            case 0x09DC : return 0xBF ;
            case 0x09DD : return 0xC0 ;
            case 0x09E2 : return 0xDB ;
            case 0x09E3 : return 0xDC ;
            case 0x09C4 : return 0xDF ;
            case 0x09BD : return 0xEA ;
         }
         break ;
      case CP_ISCII_TAMIL :
         switch ( C ) {
            case 0x0BD0 : return 0xA1 ;
         }
         break ;
      case CP_ISCII_TELUGU :
         switch ( C ) {
            case 0x0C0C : return 0xA6 ;
            case 0x0C61 : return 0xA7 ;
            case 0x0C60 : return 0xAA ;
            case 0x0C58 : return 0xB3 ;
            case 0x0C59 : return 0xB4 ;
            case 0x0C62 : return 0xDB ;
            case 0x0C63 : return 0xDC ;
            case 0x0C44 : return 0xDF ;
            case 0x0C3D : return 0xEA ;
         }
         break ;
      case CP_ISCII_ORIYA :
         switch ( C ) {
            case 0x0B0C : return 0xA6 ;
            case 0x0B61 : return 0xA7 ;
            case 0x0B60 : return 0xAA ;
            case 0x0B5C : return 0xBF ;
            case 0x0B5D : return 0xC0 ;
            case 0x0B62 : return 0xDB ;
            case 0x0B63 : return 0xDC ;
            case 0x0B44 : return 0xDF ;
            case 0x0B3D : return 0xEA ;
         }
         break ;
      case CP_ISCII_KANNADA :
         switch ( C ) {
            case 0x0C8C : return 0xA6 ;
            case 0x0CE1 : return 0xA7 ;
            case 0x0CE0 : return 0xAA ;
            case 0x0CDE : return 0xC9 ;
            case 0x0CE2 : return 0xDB ;
            case 0x0CE3 : return 0xDC ;
            case 0x0CC4 : return 0xDF ;
            case 0x0CBD : return 0xEA ;
         }
         break ;
      case CP_ISCII_MALAYALAM :
         switch ( C ) {
            case 0x0D0C : return 0xA6 ;
            case 0x0D61 : return 0xA7 ;
            case 0x0D60 : return 0xAA ;
            case 0x0D62 : return 0xDB ;
            case 0x0D63 : return 0xDC ;
            case 0x0D44 : return 0xDF ;
            case 0x0D3D : return 0xEA ;
         }
         break ;
      case CP_ISCII_GUJARATHI :
         switch ( C ) {
            case 0x0AD0 : return 0xA1 ;
            case 0x0A8C : return 0xA6 ;
            case 0x0AE1 : return 0xA7 ;
            case 0x0AE0 : return 0xAA ;
            case 0x0AE2 : return 0xDB ;
            case 0x0AE3 : return 0xDC ;
            case 0x0AC4 : return 0xDF ;
            case 0x0ABD : return 0xEA ;
         }
         break ;
      case CP_ISCII_PANJABI :
         switch ( C ) {
            case 0x0A59 : return 0xB4 ;
            case 0x0A5A : return 0xB5 ;
            case 0x0A5B : return 0xBA ;
            case 0x0A5C : return 0xBF ;
            case 0x0A5E : return 0xC9 ;
         }
         break ;
   }

   return 0 ;
}



